/**
 *   @file  test_acc.c
 *
 *   @brief   
 *      Accumulator test file (limited to new features)
 *
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2014, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  \par
*/

/* XDC includes */ 
#include <xdc/std.h>
#include <xdc/runtime/IHeap.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/Error.h>
#include <xdc/cfg/global.h>

#include <string.h>

/* sysbios includes */ 
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

/* QMSS LLD include */
#include <ti/drv/qmss/qmss_drv.h>
#include <ti/drv/qmss/qmss_firmware.h>

/* CSL RL includes */
#include <ti/csl/csl_chip.h>
#include <ti/csl/csl_cacheAux.h>
/************************ USER DEFINES ********************/

#define NUM_HOST_DESC               32
#define SIZE_HOST_DESC              48
#define NUM_MONOLITHIC_DESC         32
#define SIZE_MONOLITHIC_DESC        160
#define NUM_DATA_BUFFER             32
#define SIZE_DATA_BUFFER            64

#define MAPPED_VIRTUAL_ADDRESS      0x81000000

/* MPAX segment 2 registers */
#define XMPAXL2                     0x08000010 
#define XMPAXH2                     0x08000014

/************************ GLOBAL VARIABLES ********************/
/* QMSS configuration */
Qmss_InitCfg            qmssInitConfig;
/* Memory region configuration information */
Qmss_MemRegInfo         memInfo;

/* Queue handles for close */
Qmss_QueueHnd           handles[QMSS_MAX_QUEUES_SS_0];
uint32_t                handle_idx = 0;

/* Accumulator list */
#pragma DATA_ALIGN (hiPrioList, 16)
#define MAX_LIST_SIZE 80
Uint32                  hiPrioList[MAX_LIST_SIZE];

/************************ EXTERN VARIABLES ********************/

/* QMSS device specific configuration */
extern Qmss_GlobalConfigParams  qmssGblCfgParams;

/*************************** FUNCTIONS ************************/

/**
 *  @b Description
 *  @n  
 *      Utility function which converts a local GEM L2 memory address 
 *      to global memory address.
 *
 *  @param[in]  addr
 *      Local address to be converted
 *
 *  @retval
 *      Computed L2 global Address
 */
static uint32_t l2_global_address (uint32_t addr)
{
    uint32_t corenum;

    /* Get the core number. */
    corenum = CSL_chipReadReg (CSL_CHIP_DNUM); 

    /* Compute the global address. */
    return (addr + (0x10000000 + (corenum * 0x1000000)));
}

/**
 *  @b Description
 *  @n  
 *      Entry point for the test code.
 *      This is an testcase for new features added to the accumulator
 *      after 2014.
 *
 *      It performs the following
 *          - Initialize QM
 *          - Downloads accumulator
 *          - Verifies bounds check for MaxEntries
 *          - Deinitialize QM
 *
 *      For an example that uses the accumulator, see
 *      example/InfraDmaSC.
 *
 *  @retval
 *      Not Applicable.
 */
void accTest (void)
{
    int32_t                 result;
    extern uint32_t         errorCount;
    Qmss_QueueHnd           rxQueHnd;
    Qmss_AccCmdCfg          cfg;
    uint8_t                 isAllocated;

    System_printf ("**************************************************\n");
    System_printf ("************* QMSS accumulator testing ***********\n");
    System_printf ("**************************************************\n");

    /* Reset the variable to indicate to other cores system init is not yet done */
    System_printf ("L1D cache size %d. L2 cache size %d.\n", CACHE_getL1DSize(), CACHE_getL2Size());
    
    memset ((void *) &qmssInitConfig, 0, sizeof (Qmss_InitCfg));

    /* Set up the linking RAM. Use the internal Linking RAM. 
     * LLD will configure the internal linking RAM address and default size if a value of zero is specified.
     * Linking RAM1 is not used */
    qmssInitConfig.linkingRAM0Base = 0;
    qmssInitConfig.linkingRAM0Size = 0;
    qmssInitConfig.linkingRAM1Base = 0;
    qmssInitConfig.maxDescNum      = NUM_HOST_DESC + NUM_MONOLITHIC_DESC;

    /* Select endian dependent firmware version */
#ifdef xdc_target__bigEndian
    qmssInitConfig.pdspFirmware[0].pdspId = Qmss_PdspId_PDSP1;
    qmssInitConfig.pdspFirmware[0].firmware = &acc48_be;
    qmssInitConfig.pdspFirmware[0].size = sizeof (acc48_be);
#else
    qmssInitConfig.pdspFirmware[0].pdspId = Qmss_PdspId_PDSP1;
    qmssInitConfig.pdspFirmware[0].firmware = &acc48_le;
    qmssInitConfig.pdspFirmware[0].size = sizeof (acc48_le);
#endif

    /* Initialize Queue Manager SubSystem */
    result = Qmss_init (&qmssInitConfig, &qmssGblCfgParams);
    if (result != QMSS_SOK)
    {
        System_printf ("Error: Initializing Queue Manager SubSystem error code : %d\n", result);
        return;
    }
    
    /* Start Queue Manager SubSystem */
    result = Qmss_start ();
    if (result != QMSS_SOK)
    {
        System_printf ("Error starting Queue Manager error code : %d\n", result);
    }
    else
    {
        System_printf ("\nQMSS initialization done\n");
    }

    /* Opens receive queue in first QM */
    if ((rxQueHnd = Qmss_queueOpenInGroup (0, Qmss_QueueType_HIGH_PRIORITY_QUEUE, QMSS_PARAM_NOT_SPECIFIED, &isAllocated)) < 0)
    {
        System_printf ("Opening Receive Queue: %d \n", rxQueHnd);
        errorCount++;
        return;
    }
    else
    {
        System_printf ("Receive Queue Number : %d opened\n", rxQueHnd);
    }

    /* program the high priority accumulator */
    memset ((void *) &hiPrioList, 0, sizeof (hiPrioList));
    cfg.channel = 0;
    cfg.command = Qmss_AccCmd_ENABLE_CHANNEL;
    cfg.queueEnMask = 0;
    cfg.listAddress = l2_global_address ((uint32_t) hiPrioList); /* Should be global if reading on another core */
    /* Get queue manager and queue number from handle */
    cfg.queMgrIndex = Qmss_getQIDFromHandle (rxQueHnd);
    cfg.maxPageEntries = MAX_LIST_SIZE + 1; /* should fail */
    cfg.timerLoadCount = 0;
    cfg.interruptPacingMode = Qmss_AccPacingMode_NONE;
    cfg.listEntrySize = Qmss_AccEntrySize_REG_D;
    cfg.listCountMode = Qmss_AccCountMode_ENTRY_COUNT;
    cfg.multiQueueMode = Qmss_AccQueueMode_SINGLE_QUEUE;
    
    /* Negative test case */
    if ((result = Qmss_programAccumulator (Qmss_PdspId_PDSP1, &cfg)) != QMSS_ACC_INVALID_MAXENTRIES)
    {
        System_printf ("Programming high priority accumulator for channel : %d queue : %d "
                       "should have failed %d but got error code : %d\n",
                       cfg.channel, cfg.queMgrIndex, QMSS_ACC_INVALID_MAXENTRIES, result);
        errorCount++;
    }
    else
    {
        System_printf ("high priority accumulator negative test passed for channel : %d queue : %d\n", 
                       cfg.channel, cfg.queMgrIndex);
    }

    /* Postive test case */
    cfg.maxPageEntries = MAX_LIST_SIZE; /* should fail */
    if ((result = Qmss_programAccumulator (Qmss_PdspId_PDSP1, &cfg)) != QMSS_ACC_SOK)
    {
        System_printf ("Error programming high priority accumulator for channel : %d queue : %d error code : %d\n",
                       cfg.channel, cfg.queMgrIndex, result);
        errorCount++;
    }
    else
    {
        System_printf ("high priority accumulator positive test passed for channel : %d queue : %d\n", 
                       cfg.channel, cfg.queMgrIndex);
    }

    /* unprogram accumulator */
    if ( (result = Qmss_disableAccumulator (Qmss_PdspId_PDSP1, cfg.channel)) != QMSS_ACC_SOK)
    {
        errorCount++;
        System_printf ("Error disabling accumulator: %d \n", result);
    }
    /* close queues */
    if ( (result = Qmss_queueClose(rxQueHnd)) != QMSS_SOK)
    {
        errorCount++;
        System_printf ("Error closing queue %d QM: %d \n", rxQueHnd, result);
    }

    System_printf ("exit QM\n");
    if ( (result = Qmss_exit()) != QMSS_SOK)
    {
        errorCount++;
        System_printf ("Error exiting QM: %d \n", result);
    }

    if (errorCount)
    {
        System_printf ("test failed with %d errors\n", errorCount);
    }
    else 
    {
        System_printf ("*******************************************************\n");
        System_printf ("********* QMSS Accumulator testing Done (PASS) ********\n");
        System_printf ("*******************************************************\n");
    }
} /* accTest */

void run_test (void)
{
    accTest ();
}

