/** 
 *   @file  bcp_lte_dl.c
 *
 *   @brief  
 *      Runs LTE Downlink (DL) test case using BCP driver APIs.
 * 
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2010, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "bcp_example.h"
#include "bcp_lte.h"

#define     TX_DATA_BUFFER_SIZE             2048 
#define     RX_DATA_BUFFER_SIZE             2048
#define     TX_NUM_DESC                     2
#define     RX_NUM_DESC                     (BCP_EXAMPLE_NUM_HOST_DESC/4 - TX_NUM_DESC)
#define     RX_Q_NUM                        900 + CSL_chipReadReg (CSL_CHIP_DNUM)
#define     RX_FLOW_ID                      0 + CSL_chipReadReg (CSL_CHIP_DNUM)

/* High Priority Accumulator configuration . */
#define     ACC_CHANNEL_NUM                 4 + CSL_chipReadReg (CSL_CHIP_DNUM)
#define     ACC_LIST_ENTRY_CNT              1
#define     ACC_LIST_SIZE                   ((ACC_LIST_ENTRY_CNT + 1) * 2)

/* Allocate accumulator list from LL2 */
#pragma DATA_SECTION (gHiPriAccumList, ".testData");
#pragma DATA_ALIGN (gHiPriAccumList, 16)
static UInt32       gHiPriAccumList[ACC_LIST_SIZE];

#pragma DATA_SECTION (bIsAccPingListUsed, ".testData");
static UInt8        bIsAccPingListUsed;
#pragma DATA_SECTION (hRxSem, ".testData");
static Void*        hRxSem;

/* Software list to buffer packets received from ISR for application processing */
#pragma DATA_SECTION (gRxPktList, ".testData");
static UInt32       gRxPktList[ACC_LIST_SIZE];

/* Tracks number of packets got from BCP in the ISR */
#pragma DATA_SECTION (gRxPktListCtr, ".testData");
static UInt32       gRxPktListCtr;

static Void register_rx_interrupts (Void);

/* Rx High Priority Accumulation ISR */
Void Bcp_HiPriorityAccumISR (Void);

extern UInt32   totalNumTestsPass, totalNumTestsFail;

#pragma DATA_SECTION (tbSize, ".testData");
static UInt32      tbSize;
#pragma DATA_SECTION (ueCategory, ".testData");
static UInt8       ueCategory;
#pragma DATA_SECTION (ueTransMode, ".testData");
static UInt8       ueTransMode;
#pragma DATA_SECTION (maxNumHarqProcess, ".testData");
static UInt8       maxNumHarqProcess;
#pragma DATA_SECTION (Nir, ".testData");
static UInt32      Nir;
#pragma DATA_SECTION (numBitsG, ".testData");
static UInt32      numBitsG;
#pragma DATA_SECTION (rnti, ".testData");
static UInt16      rnti;
#pragma DATA_SECTION (rvIdx, ".testData");
static UInt8       rvIdx;
#pragma DATA_SECTION (codeWordIdx, ".testData");
static UInt8       codeWordIdx;
#pragma DATA_SECTION (cellID, ".testData");
static UInt16      cellID;
#pragma DATA_SECTION (modulation, ".testData");
static UInt8       modulation;
#pragma DATA_SECTION (ns, ".testData");
static UInt8       ns; 
#pragma DATA_SECTION (numLayers, ".testData");
static UInt8       numLayers;

/* Turbo Code Internal Interleaver Parameters */
#pragma DATA_SECTION (TurboInterTable, ".testData");
static const UInt16 TurboInterTable[189][3] = {
/*K=Data Size  f1      f2 */     
    {0	    , 0	     ,     0  },
    {40	    , 3	     ,    10  },
    {48	    , 7	     ,    12  },
    {56	    , 19	 ,    42  },
    {64	    , 7	     ,    16  },
    {72	    , 7	     ,    18  },
    {80	    , 11	 ,    20  },
    {88	    , 5	     ,    22  },
    {96	    , 11	 ,    24  },
    {104	, 7	     ,    26  },
    {112	, 41	 ,    84  },
    {120	, 103	 ,    90  },
    {128	, 15	 ,    32  },
    {136	, 9	     ,    34  },
    {144	, 17	 ,    108 },
    {152	, 9	     ,    38  },
    {160	, 21	 ,    120 },
    {168	, 101	 ,    84  },
    {176	, 21	 ,    44  },
    {184	, 57	 ,    46  },
    {192	, 23	 ,    48  },
    {200	, 13	 ,    50  },
    {208	, 27	 ,    52  },
    {216	, 11	 ,    36  },
    {224	, 27	 ,    56  },
    {232	, 85	 ,    58  },
    {240	, 29	 ,    60  },
    {248	, 33	 ,    62  },
    {256	, 15	 ,    32  },
    {264	, 17	 ,    198 },
    {272	, 33	 ,    68  },
    {280	, 103	 ,    210 },
    {288	, 19	 ,    36  },
    {296	, 19	 ,    74  },
    {304	, 37	 ,    76  },
    {312	, 19	 ,    78  },
    {320	, 21	 ,    120 },
    {328	, 21	 ,    82  },
    {336	, 115	 ,    84  },
    {344	, 193	 ,    86  },
    {352	, 21	 ,    44  },
    {360	, 133	 ,    90  },
    {368	, 81	 ,    46  },
    {376	, 45	 ,    94  },
    {384	, 23	 ,    48  },
    {392	, 243	 ,    98  },
    {400	, 151	 ,    40  },
    {408	, 155	 ,    102 },
    {416	, 25	 ,    52  },
    {424	, 51	 ,    106 },
    {432	, 47	 ,    72  },
    {440	, 91	 ,    110 },
    {448	, 29	 ,    168 },
    {456	, 29	 ,    114 },
    {464	, 247	 ,    58  },
    {472	, 29	 ,    118 },
    {480	, 89	 ,    180 },
    {488	, 91	 ,    122 },
    {496	, 157	 ,    62  },
    {504	, 55	 ,    84  },
    {512	, 31	 ,    64  },
    {528	, 17	 ,    66  },
    {544	, 35	 ,    68  },
    {560	, 227	 ,    420 },
    {576	, 65	 ,    96  },
    {592	, 19	 ,    74  },
    {608	, 37	 ,    76  },
    {624	, 41	 ,    234 },
    {640	, 39	 ,    80  },
    {656	, 185	 ,    82  },
    {672	, 43	 ,    252 },
    {688	, 21	 ,    86  },
    {704	, 155	 ,    44  },
    {720	, 79	 ,    120 },
    {736	, 139	 ,    92  },
    {752	, 23	 ,    94  },
    {768	, 217	 ,    48  },
    {784	, 25	 ,    98  },
    {800	, 17	 ,    80  },
    {816	, 127	 ,    102 },
    {832	, 25	 ,    52  },
    {848	, 239	 ,    106 },
    {864	, 17	 ,    48  },
    {880	, 137	 ,    110 },
    {896	, 215	 ,    112 },
    {912	, 29	 ,    114 },
    {928	, 15	 ,    58  },
    {944	, 147	 ,    118 },
    {960	, 29	 ,    60  },
    {976	, 59	 ,    122 },
    {992	, 65	 ,    124 },
    {1008	, 55	 ,    84  },
    {1024	, 31	 ,    64  },
    {1056	, 17	 ,    66  },
    {1088	, 171	 ,    204 },
    {1120	, 67	 ,    140 },
    {1152	, 35	 ,    72  },
    {1184	, 19	 ,    74  },
    {1216	, 39	 ,    76  },
    {1248	, 19	 ,    78  },
    {1280	, 199	 ,    240 },
    {1312	, 21	 ,    82  },
    {1344	, 211	 ,    252 },
    {1376	, 21	 ,    86  },
    {1408	, 43	 ,    88  },
    {1440	, 149	 ,    60  },
    {1472	, 45	 ,    92  },
    {1504	, 49	 ,    846 },
    {1536	, 71	 ,    48  },
    {1568	, 13	 ,    28  },
    {1600	, 17	 ,    80  },
    {1632	, 25	 ,    102 },
    {1664	, 183	 ,    104 },
    {1696	, 55	 ,    954 },
    {1728	, 127	 ,    96  },
    {1760	, 27	 ,    110 },
    {1792	, 29	 ,    112 },
    {1824	, 29	 ,    114 },
    {1856	, 57	 ,    116 },
    {1888	, 45	 ,    354 },
    {1920	, 31	 ,    120 },
    {1952	, 59	 ,    610 },
    {1984	, 185	 ,    124 },
    {2016	, 113	 ,    420 },
    {2048	, 31	 ,    64  },
    {2112	, 17	 ,    66  },
    {2176	, 171	 ,    136 },
    {2240	, 209	 ,    420 },
    {2304	, 253	 ,    216 },
    {2368	, 367	 ,    444 },
    {2432	, 265	 ,    456 },
    {2496	, 181	 ,    468 },
    {2560	, 39	 ,    80  },
    {2624	, 27	 ,    164 },
    {2688	, 127	 ,    504 },
    {2752	, 143	 ,    172 },
    {2816	, 43	 ,    88  },
    {2880	, 29	 ,    300 },
    {2944	, 45	 ,    92  },
    {3008	, 157	 ,    188 },
    {3072	, 47	 ,    96  },
    {3136	, 13	 ,    28  },
    {3200	, 111	 ,    240 },
    {3264	, 443	 ,    204 },
    {3328	, 51	 ,    104 },
    {3392	, 51	 ,    212 },
    {3456	, 451	 ,    192 },
    {3520	, 257	 ,    220 },
    {3584	, 57	 ,    336 },
    {3648	, 313	 ,    228 },
    {3712	, 271	 ,    232 },
    {3776	, 179	 ,    236 },
    {3840	, 331	 ,    120 },
    {3904	, 363	 ,    244 },
    {3968	, 375	 ,    248 },
    {4032	, 127	 ,    168 },
    {4096	, 31	 ,    64  },
    {4160	, 33	 ,    130 },
    {4224	, 43	 ,    264 },
    {4288	, 33	 ,    134 },
    {4352	, 477	 ,    408 },
    {4416	, 35	 ,    138 },
    {4480	, 233	 ,    280 },
    {4544	, 357	 ,    142 },
    {4608	, 337	 ,    480 },
    {4672	, 37	 ,    146 },
    {4736	, 71	 ,    444 },
    {4800	, 71	 ,    120 },
    {4864	, 37	 ,    152 },
    {4928	, 39	 ,    462 },
    {4992	, 127	 ,    234 },
    {5056	, 39	 ,    158 },
    {5120	, 39	 ,    80  },
    {5184	, 31	 ,    96  },
    {5248	, 113	 ,    902 },
    {5312	, 41	 ,    166 },
    {5376	, 251	 ,    336 },
    {5440	, 43	 ,    170 },
    {5504	, 21	 ,    86  },
    {5568	, 43	 ,    174 },
    {5632	, 45	 ,    176 },
    {5696	, 45	 ,    178 },
    {5760	, 161	 ,    120 },
    {5824	, 89	 ,    182 },
    {5888	, 323	 ,    184 },
    {5952	, 47	 ,    186 },
    {6016	, 23	 ,    94  },
    {6080	, 47	 ,    190 },
    {6144	, 263	 ,    480 } 
};                           

/* Reference Output Data for the test */
#define LTE_DL_OUTPUT_PKT_1_WRD_SIZE  12600

#pragma DATA_SECTION (lte_dl_out_packet_1, ".testData");
static UInt32 lte_dl_out_packet_1[LTE_DL_OUTPUT_PKT_1_WRD_SIZE] = {
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c};

/** ============================================================================
 *   @n@b read_bcp_config
 *
 *   @b Description
 *   @n Utility function that reads the input test vector params from a given
 *      file.
 *
 *   @param[in]  
 *   @n fp              Test configuration file handle.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void read_bcp_config(FILE* fp)
{
	Char            lineBuf[300];
	Char*           ptrBuf;
	Int32           i;
	
    while (fgets (lineBuf, 300, fp) != NULL)
	{
        if ((ptrBuf = strstr(lineBuf, "="))!=NULL)
		{
			ptrBuf ++;
			if (strstr(lineBuf,"TBS")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				tbSize = i;
				continue;
			}
		
			if (strstr(lineBuf,"UeCat")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ueCategory = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"UeTM")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ueTransMode = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"MaxNumHarqProcess")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				maxNumHarqProcess = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"Nir")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				Nir = i;
				continue;
			}


			if (strstr(lineBuf,"G")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				numBitsG = i;
				continue;
			}

			if (strstr(lineBuf,"RV")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				rvIdx = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"RNTI")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				rnti = (UInt16) i;
				continue;
			}

			if (strstr(lineBuf,"CodeWord")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				codeWordIdx = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"ns")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ns = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"CellID")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				cellID = (UInt16) i;
				continue;
			}

			if (strstr(lineBuf,"Modulation")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				modulation = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"NumLayers")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				numLayers = (UInt8) i;
				continue;

			}
		}
	}

    return;
}

/** ============================================================================
 *   @n@b compute_cbparams_ref
 *
 *   @b Description
 *   @n Given a small subset of 3GPP Params, this function calculates and maps
 *      them out to actual BCP H/w parameters.
 *
 *   @param[in]  
 *   @n tbSize          Transport block size
 * 
 *   @param[out]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void compute_cbparams_ref (UInt32 tbSize, BcpTest_LteCBParams* pCodeBlkParams)
{
	UInt32          numCodeBlocks;
	UInt32          outputbitsNoFiller;
	UInt32          firstSegmtSize;
	UInt32          secondSegmtSize;
	UInt32          firstSegCount;
	UInt32          secondSegCount;
	Int16           idx;
	UInt8           crcOrder = 24;
	UInt8           delaK;
	UInt16          numFillerBits;
	UInt32          numInputBits = tbSize + 24;  // tb size + 24 bits crc
		
	/* Initialize the code block parameters. */
    memset (pCodeBlkParams, 0, sizeof (BcpTest_LteCBParams));

	/* calculate the number of code blocks to be generated from imput bit stream */
	if( numInputBits <= MAX_CODE_BLOCK_SIZE )
	{
		numCodeBlocks = 1;
		outputbitsNoFiller = numInputBits;
	}
	else
	{ 
      	numCodeBlocks = (UInt32)ceil((double)numInputBits/( MAX_CODE_BLOCK_SIZE - crcOrder));
		outputbitsNoFiller = numInputBits + ( numCodeBlocks * crcOrder) ;
	}
	
	/* calculate number of bits in each block only when numCodeBlocks!=0 */
	if( numCodeBlocks != 0)
	{

		/* K+ = is minimum K in 3GPP TS 36.212 table 5.1.3-3 C * K >= B' */
		for(idx=0; idx < MAX_BLOCK_INDEX; idx++)
		{
			firstSegmtSize = TurboInterTable[idx][0];
			
			if( (numCodeBlocks * firstSegmtSize) >= outputbitsNoFiller)
			{
				firstSegCount = 1;
				break;
			}
		}
		pCodeBlkParams->f1Kp = TurboInterTable[idx][1];
		pCodeBlkParams->f2Kp = TurboInterTable[idx][2];

		if(numCodeBlocks == 1)
		{
			secondSegmtSize	= 0;
			secondSegCount  = 0;
			/* no kminues */
			pCodeBlkParams->f1Km = 0;
			pCodeBlkParams->f2Km = 0;
		}
		else if(numCodeBlocks > 1)
		{
			for(idx = MAX_BLOCK_INDEX - 1; idx >=0; idx--)
			{
				secondSegmtSize = TurboInterTable[idx][0];
				if( secondSegmtSize < firstSegmtSize )
				{
					break;
				}
			}
			pCodeBlkParams->f1Km = TurboInterTable[idx][1];
			pCodeBlkParams->f2Km = TurboInterTable[idx][2];
	
			delaK = firstSegmtSize - secondSegmtSize;
			secondSegCount = ( numCodeBlocks * firstSegmtSize ) - outputbitsNoFiller;
			secondSegCount = (UInt32)floor( secondSegCount / delaK );
	
			firstSegCount =  numCodeBlocks - secondSegCount;

		}
		
		numFillerBits = firstSegCount * firstSegmtSize + secondSegCount * secondSegmtSize - outputbitsNoFiller;
    
		/* Update the code block parameters with values calculated */ 
		pCodeBlkParams->numCodeBks          =	numCodeBlocks;
		pCodeBlkParams->outputbitsNoFiller  =   outputbitsNoFiller;
		pCodeBlkParams->numCodeBksKp        =   firstSegCount;
		pCodeBlkParams->codeBkSizeKp        =   firstSegmtSize;
		pCodeBlkParams->numCodeBksKm        =   secondSegCount;
		pCodeBlkParams->codeBkSizeKm        =   secondSegmtSize;
		pCodeBlkParams->numFillerBits       =   numFillerBits;
	}

    return;
}

/** ============================================================================
 *   @n@b compute_rmGamma_ref
 *
 *   @b Description
 *   @n Computes rate matching parameters for the test. 
 *
 *   @param[in]  
 *   @n numChannelBits        
 * 
 *   @param[in]  
 *   @n mod             Modulation type
 * 
 *   @param[in]  
 *   @n numLayers       Number of layers
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @param[in]  
 *   @n rvIndex         Redundancy Version (RV) Index.
 * 
 *   @param[in]  
 *   @n ratioKm     
 * 
 *   @param[in]  
 *   @n ratioKp     
 *
 *   @param[in]  
 *   @n Nir             
 * 
 *   @param[in]  
 *   @n testFlag        LTE channel type
 * 
 *   @param[out]  
 *   @n pRmParams       Rate matching params thus calculated for the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void compute_rmGamma_ref
(
    UInt32                      numChannelBits,
    UInt8                       mod, 
    UInt8                       numLayers,
    BcpTest_LteCBParams*        pCodeBlkParams, 
    UInt8                       rvIndex,
    float                       ratioKm, 
    float                       ratioKp,
    UInt32                      Nir,  
    UInt8                       testFlag, 
    BcpTest_RateMatchParams*    pRmParams
)
{
	UInt16 Gprime;
    UInt32 Gamma, E0, E1;   
	UInt32 G; 
	UInt8 modulationType , blkMap, numCodeBks;
	UInt16 numRows;
	UInt32 vBufferSize;
	
	G = numChannelBits;
	modulationType = mod;
	blkMap = numLayers;
	numCodeBks = pCodeBlkParams->numCodeBks;

               
	Gprime = G / (blkMap * modulationType);

    Gamma = Gprime % numCodeBks ;      
   
    //E0 = blkMap * modulationType * ((UInt32)floor( Gprime / numCodeBks));   
    //E1 = blkMap * modulationType * ((UInt32)ceil( Gprime / numCodeBks ));       

	E0 = blkMap *modulationType * (Gprime /numCodeBks);
	if (Gprime % numCodeBks==0)
		E1 = 0;
	else
		E1 = blkMap *modulationType * (Gprime /numCodeBks+1);
       
	pRmParams->Gamma   = Gamma;
	pRmParams->E0      = E0;
	pRmParams->E1      = E1;

	if (testFlag==LTE_PDSCH) //PDSCH
	{
		if(pRmParams->E0 > 64000) //satuarate to maximum 
			pRmParams->E0 = 64000/modulationType * modulationType;
		if (pRmParams->E1 > 64000)
			pRmParams->E1 = 64000/modulationType * modulationType;;

	}

	if (pCodeBlkParams->numCodeBksKm)
	{
		numRows = ((pCodeBlkParams->codeBkSizeKm + 4) + 31)/32;
		vBufferSize = numRows * 32 * 3;

		if (testFlag==LTE_PDSCH) //PDSCH
		{
			if (ratioKm != 0)
				pRmParams->NcbKm = (UInt32) (ratioKm * numRows * 32.f);
			else //Nir is the input
			{
				if (Nir/numCodeBks < vBufferSize)
					pRmParams->NcbKm =Nir/numCodeBks;
				else
					pRmParams->NcbKm = vBufferSize;
			}
		}
		else //PUSCH_SIC
			pRmParams->NcbKm = vBufferSize;
	
		if (pRmParams->NcbKm == numRows * 32)
			pRmParams->NcbKm ++; //not in the end of systematic bits

		pRmParams->rvKmCol = (UInt32) (2 * ceil(pRmParams->NcbKm/(8.f * numRows)) * rvIndex + 2);
		pRmParams->NcbKmCol = ( pRmParams->NcbKm - numRows * 32 - 1)/( 2 * numRows);
		pRmParams->NcbKmRow = ( pRmParams->NcbKm - numRows * 32 - 1)% (2 * numRows);

		if (pRmParams->rvKmCol > 31)
		{
			pRmParams->rvKm = (pRmParams->rvKmCol - 32)/ 2 + 32;

		}
		else
			pRmParams->rvKm = pRmParams->rvKmCol;

	}
	else
	{
		pRmParams->NcbKm = 0;
		pRmParams->rvKm = 0;
		pRmParams->rvKmCol = 0;
		pRmParams->NcbKmCol = 0;
		pRmParams->NcbKmRow = 0;
	}


	if (pCodeBlkParams->numCodeBksKp)
	{
		numRows = ((pCodeBlkParams->codeBkSizeKp + 4) + 31)/32;
		vBufferSize = numRows * 32 * 3;

		if (testFlag==LTE_PDSCH) //PDSCH
		{
			if (ratioKp != 0)
				pRmParams->NcbKp = (UInt32) (ratioKp * numRows * 32.f);
			else
			{
				if (Nir/numCodeBks < vBufferSize)
					pRmParams->NcbKp =Nir/numCodeBks;
				else
					pRmParams->NcbKp = vBufferSize;
			}
		}

		else
			pRmParams->NcbKp = vBufferSize;

		if (pRmParams->NcbKp == numRows * 32)
			pRmParams->NcbKp++;
	
		pRmParams->rvKpCol = (UInt32) (2 * ceil(pRmParams->NcbKp/(8.f * numRows)) * rvIndex + 2);
		pRmParams->NcbKpCol = ( pRmParams->NcbKp - numRows * 32 - 1)/( 2 * numRows);
		pRmParams->NcbKpRow = ( pRmParams->NcbKp - numRows * 32 - 1)% (2 * numRows);

		if (pRmParams->rvKpCol > 31)
		{
			pRmParams->rvKp = (pRmParams->rvKpCol - 32)/ 2 + 32;

		}
		else
			pRmParams->rvKp = pRmParams->rvKpCol;
	}
	else
	{
		pRmParams->NcbKp = 0;
		pRmParams->rvKp = 0;
		pRmParams->rvKpCol = 0;
		pRmParams->NcbKpCol = 0;
		pRmParams->NcbKpRow = 0;

	}

    return;
}

/** ============================================================================
 *   @n@b prepare_lte_enchdr_cfg
 *
 *   @b Description
 *   @n Sets up the Encoder engine header for the test. 
 *
 *   @param[out]  
 *   @n pEncHdrCfg      Encoder engine Header configuration thus populated for 
 *                      the test.
 * 
 *   @param[in]  
 *   @n radioStd        Radio standard to use for the test.
 * 
 *   @param[in]  
 *   @n lteChanType     LTE channel type.
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_lte_enchdr_cfg 
(
    Bcp_EncHdrCfg*              pEncHdrCfg, 
    Bcp_RadioStd                radioStd, 
    UInt32                      lteChanType, 
    BcpTest_LteCBParams*        pCodeBlkParams
)
{
    UInt32                      i;

    /* Setup the Encoder header as per inputs passed. */        
    pEncHdrCfg->local_hdr_len       =   3;    

    if (lteChanType == LTE_PDSCH || lteChanType == LTE_PUSCH_SIC_HARD)
    {
        pEncHdrCfg->turbo_conv_sel  =   1;

    }
    else
    {
        pEncHdrCfg->turbo_conv_sel  =   0;
    }

    if (lteChanType == LTE_PDSCH)
    {
        if (pCodeBlkParams->numCodeBks == 1)
            pEncHdrCfg->scr_crc_en  =   0;  //Dont add CRC
        else
            pEncHdrCfg->scr_crc_en  =   1;  //Add CRC per FEC block
    }
    else
    {
        pEncHdrCfg->scr_crc_en      =   0;  //Dont add CRC
    }

    pEncHdrCfg->code_rate_flag      =   0;

    if (pCodeBlkParams->numCodeBksKm == 0)
    {
        pEncHdrCfg->blockCfg[0].block_size      =   pCodeBlkParams->codeBkSizeKp;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   pCodeBlkParams->numCodeBksKp;

        pEncHdrCfg->blockCfg[1].block_size      =   0;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   0;

        pEncHdrCfg->blockCfg[0].intvpar0        =   pCodeBlkParams->f1Kp;
        pEncHdrCfg->blockCfg[0].intvpar1        =   pCodeBlkParams->f2Kp;

        pEncHdrCfg->blockCfg[1].intvpar0        =   0;
        pEncHdrCfg->blockCfg[1].intvpar1        =   0;
    }
    else
    {
        pEncHdrCfg->blockCfg[0].block_size      =   pCodeBlkParams->codeBkSizeKm;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   pCodeBlkParams->numCodeBksKm;

        pEncHdrCfg->blockCfg[1].block_size      =   pCodeBlkParams->codeBkSizeKp;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   pCodeBlkParams->numCodeBksKp;

        pEncHdrCfg->blockCfg[0].intvpar0        =   pCodeBlkParams->f1Km;
        pEncHdrCfg->blockCfg[0].intvpar1        =   pCodeBlkParams->f2Km;

        pEncHdrCfg->blockCfg[1].intvpar0        =   pCodeBlkParams->f1Kp;
        pEncHdrCfg->blockCfg[1].intvpar1        =   pCodeBlkParams->f2Kp;
    }
    pEncHdrCfg->blockCfg[2].block_size          =   0;
    pEncHdrCfg->blockCfg[2].num_code_blks       =   0;

    pEncHdrCfg->blockCfg[2].intvpar0            =   0;
    pEncHdrCfg->blockCfg[2].intvpar1            =   0;

    for (i = 0; i < 3; i ++)
    {
        pEncHdrCfg->blockCfg[i].intvpar2        =   0;
        pEncHdrCfg->blockCfg[i].intvpar3        =   0;
    }

    return;        
}

/** ============================================================================
 *   @n@b prepare_lte_rmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the LTE Rate modulation header for the test. 
 *
 *   @param[out]  
 *   @n pLteRmHdrCfg    Rate modulation Header configuration thus populated for 
 *                      the test.
 * 
 *   @param[in]  
 *   @n lteChanType     LTE channel type.
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @param[in]  
 *   @n pRmParams       Input Rate matching parameters for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_lte_rmhdr_cfg 
(
    Bcp_RmHdr_LteCfg*           pLteRmHdrCfg, 
    UInt32                      lteChanType,
    BcpTest_LteCBParams*        pCodeBlkParams, 
    BcpTest_RateMatchParams*    pRmParams
)
{
    /* Setup the LTE RM header as per the inputs specified */        
    pLteRmHdrCfg->local_hdr_len             =   6;        
    
    if (lteChanType == LTE_PDSCH || lteChanType == LTE_PUSCH_SIC)
        pLteRmHdrCfg->channel_type          =   0;        
    else
        pLteRmHdrCfg->channel_type          =   1;        

    if (lteChanType == LTE_PDSCH || lteChanType == LTE_PDCCH)
    {
        pLteRmHdrCfg->input_bit_format      =   0;        
        pLteRmHdrCfg->output_bit_format     =   0;
    }
    else
    {
        pLteRmHdrCfg->input_bit_format      =   1;        
        pLteRmHdrCfg->output_bit_format     =   1;
    }

    pLteRmHdrCfg->num_filler_bits_f         =   pCodeBlkParams->numFillerBits;        

    if (pCodeBlkParams->numCodeBksKm == 0)
    {
        pLteRmHdrCfg->num_code_blocks_c1    =   pCodeBlkParams->numCodeBksKp;        
        pLteRmHdrCfg->block_size_k1         =   pCodeBlkParams->codeBkSizeKp;        
        pLteRmHdrCfg->num_code_blocks_c2    =   0;        
        pLteRmHdrCfg->block_size_k2         =   0;        

        pLteRmHdrCfg->rv_start_column1      =   pRmParams->rvKp;        
        pLteRmHdrCfg->rv_start_column2      =   0;        
        pLteRmHdrCfg->param_ncb1_column     =   pRmParams->NcbKpCol;        
        pLteRmHdrCfg->param_ncb1_row        =   pRmParams->NcbKpRow;        
        pLteRmHdrCfg->param_ncb2_column     =   0;        
        pLteRmHdrCfg->param_ncb2_row        =   0;        
    }
    else
    {
        pLteRmHdrCfg->num_code_blocks_c1    =   pCodeBlkParams->numCodeBksKm;        
        pLteRmHdrCfg->block_size_k1         =   pCodeBlkParams->codeBkSizeKm;        
        pLteRmHdrCfg->num_code_blocks_c2    =   pCodeBlkParams->numCodeBksKp;        

        pLteRmHdrCfg->rv_start_column1      =   pRmParams->rvKm;        
        pLteRmHdrCfg->rv_start_column2      =   pRmParams->rvKp;        
        pLteRmHdrCfg->param_ncb1_column     =   pRmParams->NcbKmCol;        
        pLteRmHdrCfg->param_ncb1_row        =   pRmParams->NcbKmRow;        
        pLteRmHdrCfg->param_ncb2_column     =   pRmParams->NcbKpCol;        
        pLteRmHdrCfg->param_ncb2_row        =   pRmParams->NcbKpRow;        
        pLteRmHdrCfg->block_size_k2         =   pCodeBlkParams->codeBkSizeKp;        
    }
    pLteRmHdrCfg->num_code_blocks_ce1       =   (pCodeBlkParams->numCodeBks - pRmParams->Gamma);        
    pLteRmHdrCfg->block_size_e1             =   pRmParams->E0;        
    pLteRmHdrCfg->num_code_blocks_ce2       =   pRmParams->Gamma;        
    pLteRmHdrCfg->block_size_e2             =   pRmParams->E1;        

    return;        
}

/** ============================================================================
 *   @n@b prepare_modhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Modulation engine header for the test. 
 *
 *   @param[out]  
 *   @n pModHdrCfg      Modulation engine header configuration thus populated for the test.
 * 
 *   @param[in]  
 *   @n radioStd        Radio standard to use.
 * 
 *   @param[in]  
 *   @n Rprime          Number of allocation subcarriers.
 * 
 *   @param[in]  
 *   @n mod             Modulation type.
 * 
 *   @param[in]  
 *   @n operationMode   Modulation mode, i.e., hard/soft.
 * 
 *   @param[in]  
 *   @n rms             
 * 
 *   @param[in]  
 *   @n numOFDMsymPerSubfrm     Number of SC-FDMA symbols per sub-frame
 * 
 *   @param[in]  
 *   @n numCqiSubcs     Number of CQI sub-carriers per sub-frame
 * 
 *   @param[in]  
 *   @n numRiSubcs      Number of RI sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n numAckSubcs     Number of ACK/NACK sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n cinit           Scrambler second shift register initial value
 * 
 *   @param[in]  
 *   @n qFmt            Input LLR Q format
 * 
 *   @return        
 *   @n None
 * =============================================================================
 */
static Void prepare_modhdr_cfg 
(
    Bcp_ModHdrCfg*      pModHdrCfg,
    Bcp_RadioStd        radioStd,
    UInt32              Rprime, 
    UInt8               mod,   
    UInt8               operationMode, 
    UInt16              rms,
    UInt8               numOFDMsymPerSubfrm,
    UInt16              numCqiSubcs,
    UInt16              numRiSubcs,
    UInt16              numAckSubcs,
    UInt32              cinit,
    UInt8               qFmt
)
{
	UInt32              Rprime1;
	UInt8               prbList[45]={1, 2, 3, 4, 5, 6, 8, 9, 10,12,15,16, 18,20,24,25, 27,30,32,36, 40,45,48,50, 54,60,64,72, 75,80,
81,90,96,100,108,120,128,144,150,160,162,180,192,200,216};
        
    /* Prepare modulation header as per inputs provided */        
    pModHdrCfg->local_hdr_len   =   4;        
    if (operationMode == 3)
        pModHdrCfg->sh_mod_sel      =   Bcp_ModulationMode_Soft;        
    else
        pModHdrCfg->sh_mod_sel      =   (Bcp_ModulationMode)operationMode;        
    pModHdrCfg->jack_bit        =   0;        
    if (mod == 1)
        pModHdrCfg->mod_type_sel=   Bcp_ModulationType_BPSK;
    else
        pModHdrCfg->mod_type_sel=   (Bcp_ModulationType)(mod/2);

    if (operationMode == 2 || operationMode == 3) // soft modulator
    {
	    pModHdrCfg->uva_val     =	rms;          
    }
    else
    {
        if (operationMode == 0) // uncompressed mode hard modulator
        {
            if (mod == 1 || mod == 2) // BPSK
                pModHdrCfg->uva_val =   (rms * 23170 + 0x4000) >> 15;        
            else if (mod == 4) //16 QAM
                pModHdrCfg->uva_val =   (rms * 10362 + 0x4000) >> 15;        
            else if (mod == 6) //64 QAM 
                pModHdrCfg->uva_val =   (rms * 5056 + 0x4000) >> 15;        
            else //256 QAM
                pModHdrCfg->uva_val =   (rms * 2513 + 0x4000) >> 15;
        }
        else //compressed mode hard modulator
        {
            pModHdrCfg->uva_val =       0;
        }
    }

    if (radioStd == Bcp_RadioStd_LTE)
        pModHdrCfg->scr_en      =   1;        
    else
        pModHdrCfg->scr_en      =   0;        

    if (operationMode == 2 || operationMode == 3)
    {
        pModHdrCfg->rmux_ln     =   Rprime;        

        if (operationMode == 2) // single layer
            Rprime1 =   12 * prbList[Rprime];                
        else // 2 layers
            Rprime1 =   12 * prbList[Rprime] * 2;                

        pModHdrCfg->b_table_index=  gind_row_index(Rprime1 * mod);        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) (numOFDMsymPerSubfrm-9);        
        pModHdrCfg->q_format    =   (Bcp_QFormat) qFmt;        
        pModHdrCfg->cqi_ln      =   numCqiSubcs;        
        pModHdrCfg->ri_ln       =   numRiSubcs;        
        pModHdrCfg->ack_ln      =   numAckSubcs;        
    }
    else
    {
        pModHdrCfg->rmux_ln     =   0;        
        pModHdrCfg->b_table_index=  0;        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) 0;        
        pModHdrCfg->q_format    =   (Bcp_QFormat) 0;        
        pModHdrCfg->cqi_ln      =   0;        
        pModHdrCfg->ri_ln       =   0;        
        pModHdrCfg->ack_ln      =   0;        
    }
    
    pModHdrCfg->cinit_p2        =   cinit;        
    if (operationMode == 2)
        pModHdrCfg->split_mode_en   =   0;        
    else
        pModHdrCfg->split_mode_en   =   1;        

    return;
}

/** ============================================================================
 *   @n@b prepare_tmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Traffic Manager header for the test. 
 *
 *   @param[out]  
 *   @n pTmHdrCfg   TM Header configuration thus populated for the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_tmhdr_cfg (Bcp_TmHdrCfg* pTmHdrCfg)
{
    pTmHdrCfg->ps_data_size     =   0;
    pTmHdrCfg->info_data_size   =   0;

    return;
}

/** ============================================================================
 *   @n@b add_bcp_config_data
 *
 *   @b Description
 *   @n Given a data buffer, this API sets up the packet contents for the test.
 *      It adds all the BCP configuration params (header) and the payload to the 
 *      data buffer. On success, returns the number of bytes of configuration 
 *      and payload data added to the data buffer passed. 
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n pDataBuffer Data Buffer handle to which the payload and BCP configuration
 *                  params need to be added.
 * 
 *   @return        Int32
 *   @n >0      -   Number of bytes of data (payload + header configuration) added 
 *                  to the data buffer.
 *
 *   @n -1      -   Error populating the data buffer.
 * =============================================================================
 */
static Int32 add_bcp_config_data (Bcp_DrvHandle hBcp, UInt8*  pDataBuffer)
{
	FILE*                       pTestCfgFile;
    UInt32                      dataBufferLen, tmpLen;
    BcpTest_LteCBParams         codeBlkParams;
    UInt32                      lteChanType, cInit;
    Bcp_RadioStd                radioStd;
    Bcp_GlobalHdrCfg            bcpGlblHdrCfg;
    Bcp_CrcHdrCfg               crcHdrCfg;
    Bcp_EncHdrCfg               encHdrCfg;
    BcpTest_RateMatchParams     rmParams;
    Bcp_RmHdr_LteCfg            lteRmHdrCfg;
    Bcp_ModHdrCfg               modHdrCfg;
    Bcp_TmHdrCfg                tmHdrCfg;
    UInt8*  					pStartDataBuffer;

    /* Get the test configuration from the file. */
    if ((pTestCfgFile = fopen("..\\..\\testcases\\lte\\pdsch_parameters_DV021.dat","r")) == NULL)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to open test configuration file: ..\..\testcases\lte\pdsch_parameters_DV021.dat\n");
#endif
        return -1;
    }
#ifdef BCP_EXAMPLE_DEBUG            
    System_printf ("Reading test configuration ... \n");
#endif
    read_bcp_config(pTestCfgFile);
    fclose (pTestCfgFile);

    /* Get the code block params */
    compute_cbparams_ref (tbSize, &codeBlkParams);

    /* To begin with, lets test LTE PSDCH channel */
    radioStd    =   Bcp_RadioStd_LTE; 
    lteChanType =   LTE_PDSCH;
    
    /* Start adding BCP Packet headers based on the test configuration we read. */

    /* Initialize our data buffer length running counter */
    dataBufferLen   =   0;
    tmpLen			=	0;

    /* Save the start pointer and set running pointer to CRC header */
    pStartDataBuffer = pDataBuffer;
    pDataBuffer 	+=	8;
    dataBufferLen	+=	8;

    /* Header 2: CRC Header */
    prepare_crchdr_cfg (&crcHdrCfg, radioStd, tbSize, codeBlkParams.numFillerBits, 0, 0, 0, 0, 0, 0, 0, NULL);
    if (Bcp_addCRCHeader (&crcHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to add CRC Header to packet \n");
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 3: Encoder Header */
    prepare_lte_enchdr_cfg (&encHdrCfg, radioStd, lteChanType, &codeBlkParams);
    if (Bcp_addEncoderHeader (&encHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to add Encoder Header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 4: Rate matching header */
    compute_rmGamma_ref (numBitsG,
                         modulation,
                         numLayers, //assumed 1 layer
                         &codeBlkParams,
                         rvIdx,
                         0, //ratio is 0
                         0,
                         Nir,
                         lteChanType,
                         &rmParams);
    prepare_lte_rmhdr_cfg (&lteRmHdrCfg, lteChanType, &codeBlkParams, &rmParams);
    if (Bcp_addLte_RMHeader (&lteRmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to add lte rate modulation header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 5: Modulation header */
    cInit = rnti * (1<<14) + codeWordIdx * (1<<13) + ns/2  * (1<<9) + cellID;
    prepare_modhdr_cfg (&modHdrCfg, 
                        radioStd, 
				        0, // number of subcarrier, not used,
				        modulation,
				        0, //uncompressed mode
				        1024,//rms
				        0, //numOFDMsymPerSubfrm, not used
				        0, //numCqiSubcs
				        0, //numRiSubcs
				        0, //numAckSubcs
				        cInit,
				        0); //qformat
    if (Bcp_addModulatorHeader (&modHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to add Modulation header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 6: Traffic Manager header */
    prepare_tmhdr_cfg (&tmHdrCfg);
    if (Bcp_addTMHeader (&tmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Failed to add Traffic Manager header to packet \n");            
#endif
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 1: Global Header */
    bcpGlblHdrCfg.pkt_type          =   Bcp_PacketType_Normal;
    bcpGlblHdrCfg.flush             =   0;
    bcpGlblHdrCfg.drop              =   0;
    bcpGlblHdrCfg.halt              =   0;
    bcpGlblHdrCfg.radio_standard    =   radioStd;
    bcpGlblHdrCfg.hdr_end_ptr       =   ((dataBufferLen + 3) >> 2);
    bcpGlblHdrCfg.flow_id           =   RX_FLOW_ID;
    bcpGlblHdrCfg.destn_tag         =   0xDEAD;
    if (Bcp_addGlobalHeader (&bcpGlblHdrCfg, pStartDataBuffer, &tmpLen) < 0)
    {
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Failed to add Global Header to packet \n");
#endif
        return -1;
    }

    /* No Info words/PS to be added for now */

	/* Check and Add padding to align data on 128 bits (16 bytes) */
    tmpLen = (dataBufferLen % 16)? (16 - (dataBufferLen % 16)): 0;

	/* Add padding to align data on 128 bits */
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;

    /* Finally add the data to the packet in the same order as the headers */
    if ((pTestCfgFile = fopen("..\\..\\testcases\\lte\\pdsch_inforBits_0002.dat","r")) == NULL)
    {
#ifdef BCP_EXAMPLE_DEBUG            
        System_printf ("Cannot open data input file:  ..\..\testcases\lte\pdsch_inforBits_0002.dat\n");
#endif
        return -1;
    }
    read_data_from_file (pTestCfgFile, pDataBuffer, &dataBufferLen);
    fclose (pTestCfgFile);
       
    /* Successfully read the test configuration */        
    return dataBufferLen;
}

/** ============================================================================
 *   @n@b test_lte_dl
 *
 *   @b Description
 *   @n Sets up the parameters required and runs an LTE Downlink (DL) test case.
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n hGlblFDQ    Global free descriptor queue handle. The test will allocate all its
 *                  free descriptors from this queue for its run and release them
 *                  back once done to this queue.
 * 
 *   @return
 *   @n None
 *
 * =============================================================================
 */
Void test_lte_dl (Bcp_DrvHandle hBcp, Qmss_QueueHnd hGlblFDQ)
{
    Bcp_TxCfg           txCfg;
    Bcp_TxHandle        hTx = NULL;
    Bcp_RxCfg           rxCfg;
    Bcp_RxHandle        hRx = NULL;
    Qmss_QueueHnd       hTxFDQ, hRxFDQ;
    UInt8               rxFlowId, rxSrcId;
    UInt32              i, testFail = 0, numTestPkts;
    UInt32              dataBufferLen, rxDataBufferLen, rxPsInfoLen, rxDataTotalLen, key;
    UInt16              rxDestnTag;
    Cppi_Desc*          pCppiDesc;
    UInt8*              pDataBuffer;
    UInt8*              pRxDataBuffer;
    UInt8*              pRxPsInfo;
    Bcp_DrvBufferHandle hRxDrvBuffer;
    Bcp_DrvBufferHandle hVoid;
    Bcp_DrvBufferHandle hTmp;
    Int32               dataBufferLenUsed;
    Qmss_Queue          tmpQ;
#ifdef USE_POLL_MODE
    UInt32				numRxPackets;
#endif

    ((Void)maxNumHarqProcess);
    ((Void)ueCategory);
    ((Void)ueTransMode);

    /* Setup Rx side:
     *  -   Open BCP Rx queue on which BCP results are to be received
     *  -   Setup a Rx FDQ, Rx flow to receive data
     *  -   Set the BCP Tunnel Rx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, 0, &hRxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Tx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Rx FDQ %d successfully setup with %d descriptors\n", hRxFDQ, RX_NUM_DESC);
    }
    
    tmpQ = Qmss_getQueueNumber(hRxFDQ);

    rxCfg.rxQNum                        =   RX_Q_NUM;       
#ifdef USE_POLL_MODE
    rxCfg.bUseInterrupts                =   0;              // Use High Priority Accumulation Interrupts
#else
    rxCfg.bUseInterrupts                =   1;              // Use High Priority Accumulation Interrupts
#endif
    memset (&rxCfg.flowCfg, 0, sizeof(Cppi_RxFlowCfg));
    rxCfg.flowCfg.flowIdNum             =   RX_FLOW_ID;  
    rxCfg.flowCfg.rx_dest_qmgr          =   0;    
    rxCfg.flowCfg.rx_dest_qnum          =   RX_Q_NUM;  
    rxCfg.flowCfg.rx_desc_type          =   Cppi_DescType_HOST; 
    rxCfg.flowCfg.rx_ps_location        =   Cppi_PSLoc_PS_IN_DESC;  
    rxCfg.flowCfg.rx_psinfo_present     =   0;           
    rxCfg.flowCfg.rx_error_handling     =   0;              //  Drop the packet, do not retry on starvation
    rxCfg.flowCfg.rx_einfo_present      =   0;              //  By default no EPIB info
    
    rxCfg.flowCfg.rx_dest_tag_lo_sel    =   4;              //  Pick dest tag 7:0 bits from the PD dest_tag        
    rxCfg.flowCfg.rx_dest_tag_hi_sel    =   5;              //  Pick the dest tag 15:8 bits from the PD dest_tag
    rxCfg.flowCfg.rx_src_tag_lo_sel     =   2;              //  Pick the src tag 7:0 bits from the PD flow_id 7:0
    rxCfg.flowCfg.rx_src_tag_hi_sel     =   4;              //  Pick the src tag 15:8 bits from the PD src_tag 7:0 

    rxCfg.flowCfg.rx_size_thresh0_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh1_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh2_en    =   0;    

    rxCfg.flowCfg.rx_size_thresh0       =   0x0;
    rxCfg.flowCfg.rx_size_thresh1       =   0x0;
    rxCfg.flowCfg.rx_size_thresh2       =   0x0;

    rxCfg.flowCfg.rx_fdq0_sz0_qmgr      =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq0_sz0_qnum      =   tmpQ.qNum;    
    rxCfg.flowCfg.rx_fdq0_sz1_qnum      =   0x0; 
    rxCfg.flowCfg.rx_fdq0_sz1_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qmgr      =   0x0;

    rxCfg.flowCfg.rx_fdq1_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq1_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq2_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq2_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq3_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq3_qmgr          =   tmpQ.qMgr;

    /* Setup TM flow entry configuration for BCP */
#ifdef SIMULATOR_SUPPORT
    /* Simulator bug workaround: BCP TM module endian handling not correct. */
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_128;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_128;
#else
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_32;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_32;
#endif
    rxCfg.tmFlowCfg.format_in           =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.pkt_type            =   0;      // No special pkt assignments
    rxCfg.tmFlowCfg.dsp_int_sel         =   0;      // Interrupt core 0
    rxCfg.tmFlowCfg.format_out          =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.qfifo_out           =   RX_FLOW_ID;      // Use Rx QFIFO 3
    rxCfg.tmFlowCfg.ps_flags            =   0;      // No PS for now

    /* Setup High Priority Accumulation interrupt configuration for this Rx object */
    rxCfg.accumCfg.channel              =    ACC_CHANNEL_NUM;
    rxCfg.accumCfg.command              =    Qmss_AccCmd_ENABLE_CHANNEL;
    rxCfg.accumCfg.queueEnMask          =    0;
    rxCfg.accumCfg.queMgrIndex          =    RX_Q_NUM;
    rxCfg.accumCfg.maxPageEntries       =    ACC_LIST_ENTRY_CNT + 1;     // Add an extra entry for holding the entry count
    rxCfg.accumCfg.timerLoadCount       =    0;     // Disable pacing for now.
    rxCfg.accumCfg.interruptPacingMode  =    Qmss_AccPacingMode_NONE;
    rxCfg.accumCfg.listEntrySize        =    Qmss_AccEntrySize_REG_D;
    rxCfg.accumCfg.listCountMode        =    Qmss_AccCountMode_ENTRY_COUNT;
    rxCfg.accumCfg.multiQueueMode       =    Qmss_AccQueueMode_SINGLE_QUEUE;

    memset ((Void *)&gHiPriAccumList[0], 0, sizeof(gHiPriAccumList));
    rxCfg.accumCfg.listAddress          =    convert_coreLocalToGlobalAddr((UInt32)&gHiPriAccumList[0]);

    if ((hRx = Bcp_rxOpen (hBcp, &rxCfg, NULL)) == NULL)
    {
        System_printf ("BCP Rx Open failed \n");
		goto cleanup_and_return;
    }
    else
    {
        System_printf ("Flow %d opened to send data to RxQ: %d \n", RX_FLOW_ID, RX_Q_NUM);
    }

#ifndef USE_POLL_MODE
    /* Register ISR handler for the BCP interrupt */
    register_rx_interrupts ();
#endif

    /* Create a Rx semaphore. Will be used for communication between Rx ISR and the 
     * current task
     */
    if ((hRxSem = (Void *)Semaphore_create (0, NULL, NULL)) == NULL)
    {
        System_printf ("Error creating Rx semaphore \n");
		goto cleanup_and_return;
    }
    bIsAccPingListUsed = 0;

    /* Setup Tx side:
     *  -   Open BCP Tx queue using which data would be sent 
     *  -   Setup a Tx FDQ and initialize it with some Tx FDs
     *  -   Set the tunnel Tx endpoint configuration to NULL since we
     *      are processing BCP packets locally.
     */
    txCfg.txQNum    =   Bcp_QueueId_0;
    if ((hTx = Bcp_txOpen (hBcp, &txCfg, NULL)) == NULL)
    {
        System_printf ("BCP Tx Open failed \n");
        return;
    }

    if (allocate_fdq (hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, 0, &hTxFDQ, NULL) < 0)
    {
        System_printf ("Error opening Tx FDQ \n");            
        return;
    }
    else
    {
        System_printf ("Tx FDQ %d successfully setup with %d descriptors\n", hTxFDQ, TX_NUM_DESC);
    }

    for (numTestPkts = 0; numTestPkts < BCP_TEST_NUM_PACKETS; numTestPkts ++)
    {
        /* Build and Send a packet with LTE DL parameters for BCP Processing */
        if ((pCppiDesc = (Cppi_Desc*) Qmss_queuePop (hTxFDQ)) == NULL)
        {
#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("Out of Tx FDs! \n");
#endif
            testFail ++;
		    goto cleanup_and_return;
        }
        pCppiDesc = (Cppi_Desc *) (QMSS_DESC_PTR (pCppiDesc));    
        Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pDataBuffer, &dataBufferLen);
        
	    memset (pDataBuffer, 0, dataBufferLen); 

        /* Read test configuration */
        if ((dataBufferLenUsed = add_bcp_config_data (hBcp, pDataBuffer)) <= 0)
        {
            System_printf ("Error reading test vectors/populating packet \n");
            testFail ++;
            goto cleanup_and_return;
        }

        /* Setup the data length in the descriptor */
        Cppi_setDataLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);
        Cppi_setPacketLen (Cppi_DescType_HOST, pCppiDesc, dataBufferLenUsed);

#ifdef BCP_EXAMPLE_DEBIG
   	    System_printf ("Sending a packet of len %d to BCP ...\n", dataBufferLenUsed);
#endif

        /* Since BCP is local to the device, set destination address to NULL */
        Bcp_send (hTx, (Bcp_DrvBufferHandle) pCppiDesc, BCP_EXAMPLE_SIZE_HOST_DESC, NULL);

#ifdef USE_POLL_MODE
        /* Wait on data to be received from BCP and validate it */
        for (i = 0; i < 400; i ++);
        while ((numRxPackets = Bcp_rxGetNumOutputEntries (hRx)) == 0);
#else
        /* Wait on BCP output from Rx ISR */
#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Waiting on output from BCP ... \n");
#endif
        Semaphore_pend (hRxSem, BIOS_WAIT_FOREVER);

        /* Disable interrupts while we are reading BCP output from the Rx software list. */
        key = Hwi_disable();
#endif //USE_POLL_MODE

#ifdef BCP_EXAMPLE_DEBUG
        System_printf ("Got %d packet(s) from BCP \n", gRxPktListCtr);
#endif

#ifdef USE_POLL_MODE
		for (i = 0; i < numRxPackets; i ++)
        {
            /* Data could arrive scattered across multiple linked descriptors.
             * Collect data from all linked descriptors and validate it.
             */
            rxDataTotalLen  =   0;

            Bcp_recv  (hRx,
                                &hRxDrvBuffer,
                                &pRxDataBuffer,
                                &rxDataBufferLen,
                                &pRxPsInfo,
                                &rxPsInfoLen,
                                &rxFlowId,
                                &rxSrcId,
                                &rxDestnTag);

            if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1,
                                LTE_DL_OUTPUT_PKT_1_WRD_SIZE*4,
                                pRxDataBuffer,
                                rxDataBufferLen,
                                rxDataTotalLen) != 0)
                testFail ++;

            rxDataTotalLen  +=  rxDataBufferLen;

            /* Check if there are any descriptors linked to this Rx desc */
            while (hRxDrvBuffer)
            {
                /* Save original Rx desc handle. */
                hTmp = hRxDrvBuffer;

                if ((hRxDrvBuffer = Cppi_getNextBD (Cppi_getDescType (hRxDrvBuffer), hRxDrvBuffer)))
                {
                    Bcp_rxProcessDesc  (hRx,
                                        hRxDrvBuffer,
                                        &hVoid,
                                        &pRxDataBuffer,
                                        &rxDataBufferLen,
                                        &pRxPsInfo,
                                        &rxPsInfoLen,
                                        &rxFlowId,
                                        &rxSrcId,
                                        &rxDestnTag);

                    if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1,
                                        LTE_DL_OUTPUT_PKT_1_WRD_SIZE*4,
                                        pRxDataBuffer,
                                        rxDataBufferLen,
                                        rxDataTotalLen) != 0)
                        testFail ++;

                    rxDataTotalLen  +=  rxDataBufferLen;

                }

                Bcp_rxFreeRecvBuffer (hRx, hTmp, BCP_EXAMPLE_SIZE_HOST_DESC);
            }

#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("[Pkt %d]: Total Len: %d DestnTag: 0x%x SrcId: 0x%x (testFail: %d)\n", i, rxDataTotalLen, rxDestnTag, rxSrcId, testFail);
#endif

            if (rxDataTotalLen != LTE_DL_OUTPUT_PKT_1_WRD_SIZE * 4)
                testFail ++;
        }
#else
		for (i = 0; i < gRxPktListCtr; i ++)
        {
            /* Data could arrive scattered across multiple linked descriptors.
             * Collect data from all linked descriptors and validate it.
             */
            rxDataTotalLen  =   0;
            hRxDrvBuffer    =   (Bcp_DrvBufferHandle) gRxPktList [i];

            Bcp_rxProcessDesc  (hRx,
                                hRxDrvBuffer,
                                &hVoid,
                                &pRxDataBuffer,
                                &rxDataBufferLen,
                                &pRxPsInfo,
                                &rxPsInfoLen,
                                &rxFlowId,
                                &rxSrcId,
                                &rxDestnTag);   

            if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1, 
                                LTE_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                pRxDataBuffer, 
                                rxDataBufferLen, 
                                rxDataTotalLen) != 0)
                testFail ++;                

            rxDataTotalLen  +=  rxDataBufferLen;

            /* Check if there are any descriptors linked to this Rx desc */
            while (hRxDrvBuffer)
            {
                /* Save original Rx desc handle. */
                hTmp = hRxDrvBuffer;

                if ((hRxDrvBuffer = Cppi_getNextBD (Cppi_getDescType (hRxDrvBuffer), hRxDrvBuffer)))
                {
                    Bcp_rxProcessDesc  (hRx,
                                        hRxDrvBuffer,
                                        &hVoid,
                                        &pRxDataBuffer,
                                        &rxDataBufferLen,
                                        &pRxPsInfo,
                                        &rxPsInfoLen,
                                        &rxFlowId,
                                        &rxSrcId,
                                        &rxDestnTag);            

                    if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1, 
                                        LTE_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                        pRxDataBuffer, 
                                        rxDataBufferLen, 
                                        rxDataTotalLen) != 0)
                        testFail ++;                        

                    rxDataTotalLen  +=  rxDataBufferLen;

                }
        
                Bcp_rxFreeRecvBuffer (hRx, hTmp, BCP_EXAMPLE_SIZE_HOST_DESC);
            }

#ifdef BCP_EXAMPLE_DEBUG
            System_printf ("[Pkt %d]: Total Len: %d DestnTag: 0x%x SrcId: 0x%x (testFail: %d)\n", i, rxDataTotalLen, rxDestnTag, rxSrcId, testFail);
#endif

            if (rxDataTotalLen != LTE_DL_OUTPUT_PKT_1_WRD_SIZE * 4)
                testFail ++;                    
        }

        /* Done processing results received from ISR. Enable interrupts again. */
        gRxPktListCtr = 0;
        Hwi_restore (key);
#endif
    }

cleanup_and_return:
    if (testFail > 0)
    {
        System_printf ("LTE DL Test:    FAILED\n");                
        totalNumTestsFail ++;
    }
    else
    {
        System_printf ("LTE DL Test:    PASS\n");                
        totalNumTestsPass ++;
    }
    
    if (hRxSem)
        Semaphore_delete (hRxSem);

    if (hRx)
        Bcp_rxClose (hRx);

    deallocate_fdq (hRxFDQ, hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, NULL);

    if (hTx)
        Bcp_txClose (hTx);

    deallocate_fdq (hTxFDQ, hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, NULL);

    return;
}

/* ============================================================================
 *  @n@b register_rx_interrupts
 *
 *  @b  brief
 *  @n  Utility function to register interrupts using BIOS APIs.
 * ============================================================================
 */
static Void register_rx_interrupts (Void)
{
    Int32       		vectId;
    Int16               eventId;

    eventId     =   49 + CSL_chipReadReg (CSL_CHIP_DNUM);
    vectId     	=   8;

    /* Register the ISR handle for this event */
    EventCombiner_dispatchPlug (eventId, (EventCombiner_FuncPtr)Bcp_HiPriorityAccumISR, (UArg)NULL, TRUE);
	EventCombiner_enableEvent(eventId);

	/* Map the event id to hardware interrupt. */
    Hwi_eventMap(vectId, eventId); 

    /* Enable interrupt. */
    Hwi_enableInterrupt(vectId);

    return;
}

/** ============================================================================
 *   @n@b Bcp_HiPriorityAccumISR
 *
 *   @b Description
 *   @n High priority accumulation ISR. Gets all the BCP output packets
 *      from accumulator list and wakes up the Rx task to process the pending
 *      packet.
 *
 *   @param[in] Void
 *
 *   @return
 *   @n None
 *
 * =============================================================================
 */
Void Bcp_HiPriorityAccumISR (Void)
{
    UInt8                     count, i;
    
    /* Process ISR. 
     *
     * Get the number of entries in accumulator list. 
	 * The hardware enqueues data alternatively to Ping/Pong buffer lists in
     * the accumulator. Hence, we need to track which list (Ping/Pong)
     * we serviced the last time and accordingly process the other one
     * this time around.
     */ 
     if (!bIsAccPingListUsed)
     {
		/* Serviced Pong list last time. So read off the Ping list now */     	
    	count   =   gHiPriAccumList[0];     	
     }
     else
     {
		/* Serviced Pong list last time. So read off the Ping list now */     	
    	count   =   gHiPriAccumList[ACC_LIST_ENTRY_CNT + 1];     	
     }
    
    /* Process all the Results received 
     *
     * Skip the first entry in the list that contains the 
     * entry count and proceed processing results.
     */
    for (i = 1; i <= count; i ++)
    {
        /* Get the result descriptor.
         *
         * The hardware enqueues data alternatively to Ping/Pong buffer lists in
         * the accumulator. Hence, we need to track which list (Ping/Pong)
         * we serviced the last time and accordingly process the other one
         * this time around.
         */
        if (!bIsAccPingListUsed)
        {
            /* Serviced Pong list last time. So read off the Ping list now */                
            gRxPktList [i - 1]   =   gHiPriAccumList [i];
            gHiPriAccumList [i] = NULL;
        }
        else
        {
            /* Serviced Ping list last time. So read off the Pong list now 
             *
             * Skip over Ping list length to arrive at Pong list start.
             */                
            gRxPktList [i - 1]   =   gHiPriAccumList [i + ACC_LIST_ENTRY_CNT + 1];
            gHiPriAccumList [i + ACC_LIST_ENTRY_CNT + 1] = NULL;
        }

        /* Increment the number of packets received and saved for application processing */
        gRxPktListCtr ++;
    }

    /* Wake up any application blocked on this result. */
    if (hRxSem)
    {
        Semaphore_post (hRxSem);                
    }

    /* Clear the accumulator list and save whether we used Ping/Pong
     * list information for next time around.
     */
    if (!bIsAccPingListUsed)
    {
        /* Just processed Ping list */            
        bIsAccPingListUsed  =   1;
    }
    else
    {
        /* Just processed Pong list */            
        bIsAccPingListUsed  =   0;
    }

    /* Clear INTD */
    Qmss_ackInterrupt (ACC_CHANNEL_NUM, 1);
    Qmss_setEoiVector (Qmss_IntdInterruptType_HIGH, ACC_CHANNEL_NUM);

    /* Done processing interrupt. Return */
    return;
}
