/** 
 *   @file  test_lte_dl_remote.c
 *
 *   @brief  
 *      Runs LTE Downlink (DL) test case on remote BCP using BCP driver APIs.
 * 
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2010, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/
#include "bcp_test.h"
#include "bcp_test_lte.h"

/* CSL SRIO Functional Layer */
#include <ti/csl/csl_srio.h>
#include <ti/csl/csl_srioAux.h>
#include <ti/csl/csl_srioAuxPhyLayer.h>

#define     TX_DATA_BUFFER_SIZE             2048 
#define     RX_DATA_BUFFER_SIZE             2048
#define     TX_NUM_DESC                     8
#define     RX_NUM_DESC                     8
#define     RX_Q_NUM                        900
#define     RX_FLOW_ID                      0
#define     BCP_TX_QUEUE(i)                 (QMSS_BCP_QUEUE_BASE + i)
#define     SRIO_TX_QUEUE(i)                (QMSS_SRIO_QUEUE_BASE + i)

extern const uint32_t DEVICE_ID1_16BIT;
extern const uint32_t DEVICE_ID1_8BIT;
extern const uint32_t DEVICE_ID_BCP_16BIT;
extern const uint32_t DEVICE_ID_BCP_8BIT;

extern Int32 srio_freeRecvBuffer (Void* hVoid, Void* hDrvBuffer);

extern UInt32   totalNumTestsPass, totalNumTestsFail;

static UInt32      tbSize;
static UInt8       ueCategory;
static UInt8       ueTransMode;
static UInt8       maxNumHarqProcess;
static UInt32      Nir;
static UInt32      numBitsG;
static UInt16      rnti;
static UInt8       rvIdx;
static UInt8       codeWordIdx;
static UInt16      cellID;
static UInt8       modulation;
static UInt8       ns; 
static UInt8       numLayers;

/* Turbo Code Internal Interleaver Parameters */
#pragma DATA_SECTION (TurboInterTable, ".testData");
static const UInt16 TurboInterTable[189][3] = {
/*K=Data Size  f1      f2 */     
    {0	    , 0	     ,     0  },
    {40	    , 3	     ,    10  },
    {48	    , 7	     ,    12  },
    {56	    , 19	 ,    42  },
    {64	    , 7	     ,    16  },
    {72	    , 7	     ,    18  },
    {80	    , 11	 ,    20  },
    {88	    , 5	     ,    22  },
    {96	    , 11	 ,    24  },
    {104	, 7	     ,    26  },
    {112	, 41	 ,    84  },
    {120	, 103	 ,    90  },
    {128	, 15	 ,    32  },
    {136	, 9	     ,    34  },
    {144	, 17	 ,    108 },
    {152	, 9	     ,    38  },
    {160	, 21	 ,    120 },
    {168	, 101	 ,    84  },
    {176	, 21	 ,    44  },
    {184	, 57	 ,    46  },
    {192	, 23	 ,    48  },
    {200	, 13	 ,    50  },
    {208	, 27	 ,    52  },
    {216	, 11	 ,    36  },
    {224	, 27	 ,    56  },
    {232	, 85	 ,    58  },
    {240	, 29	 ,    60  },
    {248	, 33	 ,    62  },
    {256	, 15	 ,    32  },
    {264	, 17	 ,    198 },
    {272	, 33	 ,    68  },
    {280	, 103	 ,    210 },
    {288	, 19	 ,    36  },
    {296	, 19	 ,    74  },
    {304	, 37	 ,    76  },
    {312	, 19	 ,    78  },
    {320	, 21	 ,    120 },
    {328	, 21	 ,    82  },
    {336	, 115	 ,    84  },
    {344	, 193	 ,    86  },
    {352	, 21	 ,    44  },
    {360	, 133	 ,    90  },
    {368	, 81	 ,    46  },
    {376	, 45	 ,    94  },
    {384	, 23	 ,    48  },
    {392	, 243	 ,    98  },
    {400	, 151	 ,    40  },
    {408	, 155	 ,    102 },
    {416	, 25	 ,    52  },
    {424	, 51	 ,    106 },
    {432	, 47	 ,    72  },
    {440	, 91	 ,    110 },
    {448	, 29	 ,    168 },
    {456	, 29	 ,    114 },
    {464	, 247	 ,    58  },
    {472	, 29	 ,    118 },
    {480	, 89	 ,    180 },
    {488	, 91	 ,    122 },
    {496	, 157	 ,    62  },
    {504	, 55	 ,    84  },
    {512	, 31	 ,    64  },
    {528	, 17	 ,    66  },
    {544	, 35	 ,    68  },
    {560	, 227	 ,    420 },
    {576	, 65	 ,    96  },
    {592	, 19	 ,    74  },
    {608	, 37	 ,    76  },
    {624	, 41	 ,    234 },
    {640	, 39	 ,    80  },
    {656	, 185	 ,    82  },
    {672	, 43	 ,    252 },
    {688	, 21	 ,    86  },
    {704	, 155	 ,    44  },
    {720	, 79	 ,    120 },
    {736	, 139	 ,    92  },
    {752	, 23	 ,    94  },
    {768	, 217	 ,    48  },
    {784	, 25	 ,    98  },
    {800	, 17	 ,    80  },
    {816	, 127	 ,    102 },
    {832	, 25	 ,    52  },
    {848	, 239	 ,    106 },
    {864	, 17	 ,    48  },
    {880	, 137	 ,    110 },
    {896	, 215	 ,    112 },
    {912	, 29	 ,    114 },
    {928	, 15	 ,    58  },
    {944	, 147	 ,    118 },
    {960	, 29	 ,    60  },
    {976	, 59	 ,    122 },
    {992	, 65	 ,    124 },
    {1008	, 55	 ,    84  },
    {1024	, 31	 ,    64  },
    {1056	, 17	 ,    66  },
    {1088	, 171	 ,    204 },
    {1120	, 67	 ,    140 },
    {1152	, 35	 ,    72  },
    {1184	, 19	 ,    74  },
    {1216	, 39	 ,    76  },
    {1248	, 19	 ,    78  },
    {1280	, 199	 ,    240 },
    {1312	, 21	 ,    82  },
    {1344	, 211	 ,    252 },
    {1376	, 21	 ,    86  },
    {1408	, 43	 ,    88  },
    {1440	, 149	 ,    60  },
    {1472	, 45	 ,    92  },
    {1504	, 49	 ,    846 },
    {1536	, 71	 ,    48  },
    {1568	, 13	 ,    28  },
    {1600	, 17	 ,    80  },
    {1632	, 25	 ,    102 },
    {1664	, 183	 ,    104 },
    {1696	, 55	 ,    954 },
    {1728	, 127	 ,    96  },
    {1760	, 27	 ,    110 },
    {1792	, 29	 ,    112 },
    {1824	, 29	 ,    114 },
    {1856	, 57	 ,    116 },
    {1888	, 45	 ,    354 },
    {1920	, 31	 ,    120 },
    {1952	, 59	 ,    610 },
    {1984	, 185	 ,    124 },
    {2016	, 113	 ,    420 },
    {2048	, 31	 ,    64  },
    {2112	, 17	 ,    66  },
    {2176	, 171	 ,    136 },
    {2240	, 209	 ,    420 },
    {2304	, 253	 ,    216 },
    {2368	, 367	 ,    444 },
    {2432	, 265	 ,    456 },
    {2496	, 181	 ,    468 },
    {2560	, 39	 ,    80  },
    {2624	, 27	 ,    164 },
    {2688	, 127	 ,    504 },
    {2752	, 143	 ,    172 },
    {2816	, 43	 ,    88  },
    {2880	, 29	 ,    300 },
    {2944	, 45	 ,    92  },
    {3008	, 157	 ,    188 },
    {3072	, 47	 ,    96  },
    {3136	, 13	 ,    28  },
    {3200	, 111	 ,    240 },
    {3264	, 443	 ,    204 },
    {3328	, 51	 ,    104 },
    {3392	, 51	 ,    212 },
    {3456	, 451	 ,    192 },
    {3520	, 257	 ,    220 },
    {3584	, 57	 ,    336 },
    {3648	, 313	 ,    228 },
    {3712	, 271	 ,    232 },
    {3776	, 179	 ,    236 },
    {3840	, 331	 ,    120 },
    {3904	, 363	 ,    244 },
    {3968	, 375	 ,    248 },
    {4032	, 127	 ,    168 },
    {4096	, 31	 ,    64  },
    {4160	, 33	 ,    130 },
    {4224	, 43	 ,    264 },
    {4288	, 33	 ,    134 },
    {4352	, 477	 ,    408 },
    {4416	, 35	 ,    138 },
    {4480	, 233	 ,    280 },
    {4544	, 357	 ,    142 },
    {4608	, 337	 ,    480 },
    {4672	, 37	 ,    146 },
    {4736	, 71	 ,    444 },
    {4800	, 71	 ,    120 },
    {4864	, 37	 ,    152 },
    {4928	, 39	 ,    462 },
    {4992	, 127	 ,    234 },
    {5056	, 39	 ,    158 },
    {5120	, 39	 ,    80  },
    {5184	, 31	 ,    96  },
    {5248	, 113	 ,    902 },
    {5312	, 41	 ,    166 },
    {5376	, 251	 ,    336 },
    {5440	, 43	 ,    170 },
    {5504	, 21	 ,    86  },
    {5568	, 43	 ,    174 },
    {5632	, 45	 ,    176 },
    {5696	, 45	 ,    178 },
    {5760	, 161	 ,    120 },
    {5824	, 89	 ,    182 },
    {5888	, 323	 ,    184 },
    {5952	, 47	 ,    186 },
    {6016	, 23	 ,    94  },
    {6080	, 47	 ,    190 },
    {6144	, 263	 ,    480 } 
};                           

/* Reference Output Data for the test */
#define LTE_DL_OUTPUT_PKT_1_WRD_SIZE  12600

#pragma DATA_SECTION (lte_dl_out_packet_1, ".testData");
static UInt32 lte_dl_out_packet_1[LTE_DL_OUTPUT_PKT_1_WRD_SIZE] = {
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2c02d4, 0x02d402d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4, 0xfd2cfd2c,
  0x02d402d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d402d4, 0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4,
  0x02d402d4, 0xfd2c02d4, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d402d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d4fd2c,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d402d4, 0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c,
  0x02d402d4, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2cfd2c, 0xfd2cfd2c, 0xfd2c02d4,
  0xfd2c02d4, 0x02d4fd2c, 0x02d4fd2c, 0x02d402d4,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0x02d402d4, 0x02d402d4, 0x02d4fd2c, 0xfd2c02d4,
  0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4, 0x02d4fd2c,
  0x02d4fd2c, 0x02d402d4, 0xfd2cfd2c, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2c02d4,
  0x02d402d4, 0xfd2c02d4, 0x02d402d4, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d402d4, 0x02d4fd2c, 0xfd2cfd2c,
  0x02d402d4, 0x02d4fd2c, 0x02d402d4, 0x02d402d4,
  0xfd2cfd2c, 0x02d402d4, 0x02d402d4, 0x02d4fd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4, 0xfd2c02d4,
  0x02d4fd2c, 0x02d4fd2c, 0xfd2c02d4, 0xfd2cfd2c,
  0x02d4fd2c, 0xfd2cfd2c, 0x02d4fd2c, 0x02d402d4,
  0xfd2cfd2c, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0x02d4fd2c, 0xfd2c02d4, 0x02d4fd2c, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0xfd2c02d4, 0x02d402d4,
  0xfd2c02d4, 0xfd2c02d4, 0x02d402d4, 0xfd2c02d4,
  0xfd2c02d4, 0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c,
  0xfd2cfd2c, 0x02d4fd2c, 0xfd2cfd2c, 0xfd2cfd2c};

/** ============================================================================
 *   @n@b read_test_config
 *
 *   @b Description
 *   @n Utility function that reads the input test vector params from a given
 *      file.
 *
 *   @param[in]  
 *   @n fp              Test configuration file handle.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void read_test_config(FILE* fp)
{
	Char            lineBuf[300];
	Char*           ptrBuf;
	Int32           i;
	
    while (fgets (lineBuf, 300, fp) != NULL)
	{
        if ((ptrBuf = strstr(lineBuf, "="))!=NULL)
		{
			ptrBuf ++;
			if (strstr(lineBuf,"TBS")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				tbSize = i;
				continue;
			}
		
			if (strstr(lineBuf,"UeCat")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ueCategory = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"UeTM")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ueTransMode = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"MaxNumHarqProcess")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				maxNumHarqProcess = (UInt8) i;
				continue;

			}

			if (strstr(lineBuf,"Nir")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				Nir = i;
				continue;
			}


			if (strstr(lineBuf,"G")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				numBitsG = i;
				continue;
			}

			if (strstr(lineBuf,"RV")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				rvIdx = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"RNTI")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				rnti = (UInt16) i;
				continue;
			}

			if (strstr(lineBuf,"CodeWord")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				codeWordIdx = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"ns")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				ns = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"CellID")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				cellID = (UInt16) i;
				continue;
			}

			if (strstr(lineBuf,"Modulation")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				modulation = (UInt8) i;
				continue;
			}

			if (strstr(lineBuf,"NumLayers")!=NULL)
			{
				sscanf(ptrBuf,"%d", &i);
				numLayers = (UInt8) i;
				continue;

			}
		}
	}

    return;
}

/** ============================================================================
 *   @n@b compute_cbparams_ref
 *
 *   @b Description
 *   @n Given a small subset of 3GPP Params, this function calculates and maps
 *      them out to actual BCP H/w parameters.
 *
 *   @param[in]  
 *   @n tbSize          Transport block size
 * 
 *   @param[out]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void compute_cbparams_ref (UInt32 tbSize, BcpTest_LteCBParams* pCodeBlkParams)
{
	UInt32          numCodeBlocks;
	UInt32          outputbitsNoFiller;
	UInt32          firstSegmtSize;
	UInt32          secondSegmtSize;
	UInt32          firstSegCount;
	UInt32          secondSegCount;
	Int16           idx;
	UInt8           crcOrder = 24;
	UInt8           delaK;
	UInt16          numFillerBits;
	UInt32          numInputBits = tbSize + 24;  // tb size + 24 bits crc
		
	/* Initialize the code block parameters. */
    memset (pCodeBlkParams, 0, sizeof (BcpTest_LteCBParams));

	/* calculate the number of code blocks to be generated from imput bit stream */
	if( numInputBits <= MAX_CODE_BLOCK_SIZE )
	{
		numCodeBlocks = 1;
		outputbitsNoFiller = numInputBits;
	}
	else
	{ 
      	numCodeBlocks = (UInt32)ceil((double)numInputBits/( MAX_CODE_BLOCK_SIZE - crcOrder));
		outputbitsNoFiller = numInputBits + ( numCodeBlocks * crcOrder) ;
	}
	
	/* calculate number of bits in each block only when numCodeBlocks!=0 */
	if( numCodeBlocks != 0)
	{

		/* K+ = is minimum K in 3GPP TS 36.212 table 5.1.3-3 C * K >= B' */
		for(idx=0; idx < MAX_BLOCK_INDEX; idx++)
		{
			firstSegmtSize = TurboInterTable[idx][0];
			
			if( (numCodeBlocks * firstSegmtSize) >= outputbitsNoFiller)
			{
				firstSegCount = 1;
				break;
			}
		}
		pCodeBlkParams->f1Kp = TurboInterTable[idx][1];
		pCodeBlkParams->f2Kp = TurboInterTable[idx][2];

		if(numCodeBlocks == 1)
		{
			secondSegmtSize	= 0;
			secondSegCount  = 0;
			/* no kminues */
			pCodeBlkParams->f1Km = 0;
			pCodeBlkParams->f2Km = 0;
		}
		else if(numCodeBlocks > 1)
		{
			for(idx = MAX_BLOCK_INDEX - 1; idx >=0; idx--)
			{
				secondSegmtSize = TurboInterTable[idx][0];
				if( secondSegmtSize < firstSegmtSize )
				{
					break;
				}
			}
			pCodeBlkParams->f1Km = TurboInterTable[idx][1];
			pCodeBlkParams->f2Km = TurboInterTable[idx][2];
	
			delaK = firstSegmtSize - secondSegmtSize;
			secondSegCount = ( numCodeBlocks * firstSegmtSize ) - outputbitsNoFiller;
			secondSegCount = (UInt32)floor( secondSegCount / delaK );
	
			firstSegCount =  numCodeBlocks - secondSegCount;

		}
		
		numFillerBits = firstSegCount * firstSegmtSize + secondSegCount * secondSegmtSize - outputbitsNoFiller;
    
		/* Update the code block parameters with values calculated */ 
		pCodeBlkParams->numCodeBks          =	numCodeBlocks;
		pCodeBlkParams->outputbitsNoFiller  =   outputbitsNoFiller;
		pCodeBlkParams->numCodeBksKp        =   firstSegCount;
		pCodeBlkParams->codeBkSizeKp        =   firstSegmtSize;
		pCodeBlkParams->numCodeBksKm        =   secondSegCount;
		pCodeBlkParams->codeBkSizeKm        =   secondSegmtSize;
		pCodeBlkParams->numFillerBits       =   numFillerBits;
	}

    return;
}

/** ============================================================================
 *   @n@b compute_rmGamma_ref
 *
 *   @b Description
 *   @n Computes rate matching parameters for the test. 
 *
 *   @param[in]  
 *   @n numChannelBits        
 * 
 *   @param[in]  
 *   @n mod             Modulation type
 * 
 *   @param[in]  
 *   @n numLayers       Number of layers
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @param[in]  
 *   @n rvIndex         Redundancy Version (RV) Index.
 * 
 *   @param[in]  
 *   @n ratio     
 * 
 *   @param[in]  
 *   @n Nir             
 * 
 *   @param[in]  
 *   @n testFlag        LTE channel type
 * 
 *   @param[out]  
 *   @n pRmParams       Rate matching params thus calculated for the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void compute_rmGamma_ref
(
    UInt32                      numChannelBits,
    UInt8                       mod, 
    UInt8                       numLayers,
    BcpTest_LteCBParams*        pCodeBlkParams, 
    UInt8                       rvIndex,
    float                       ratio, 
    UInt32                      Nir,  
    UInt8                       testFlag, 
    BcpTest_RateMatchParams*    pRmParams
)
{
	UInt16                      Gprime;
    UInt32                      Gamma, E0, E1;   
	UInt32                      G; 
	UInt8                       modulationType , blkMap, numCodeBks;
	UInt16                      numRows;
	UInt32                      vBufferSize;
	
	G               =   numChannelBits;
	modulationType  =   mod;
	blkMap          =   numLayers;
	numCodeBks      =   pCodeBlkParams->numCodeBks;

	Gprime  =   G / (blkMap * modulationType);
    Gamma   =   Gprime % numCodeBks ;      
   
    //E0 = blkMap * modulationType * ((UInt32)floor( Gprime / numCodeBks));   
    //E1 = blkMap * modulationType * ((UInt32)ceil( Gprime / numCodeBks ));       

	E0 = blkMap *modulationType * (Gprime /numCodeBks);
	if (Gprime % numCodeBks==0)
		E1 = 0;
	else
		E1 = blkMap *modulationType * (Gprime /numCodeBks+1);
       
	pRmParams->Gamma   =   Gamma;
	pRmParams->E0      = E0;
	pRmParams->E1      = E1;
	if (testFlag==LTE_PDSCH) //PDSCH
	{
		if(pRmParams->E0 > 64000) //satuarate the CB size
			pRmParams->E0 = 64000;
		if (pRmParams->E1 > 64000)
			pRmParams->E1 = 64000;
	}

	if (pCodeBlkParams->numCodeBksKm)
	{
		numRows = ((pCodeBlkParams->codeBkSizeKm + 4) + 31)/32;
		vBufferSize = numRows * 32 * 3;

		if (testFlag==LTE_PDSCH) //PDSCH
		{
			if (ratio != 0)
				pRmParams->NcbKm = (UInt32) (ratio * numRows * 32.f);
			else //Nir is the input
			{
				if (Nir/numCodeBks < vBufferSize)
					pRmParams->NcbKm =Nir/numCodeBks;
				else
					pRmParams->NcbKm = vBufferSize;
			}
		}
		else //PUSCH_SIC
			pRmParams->NcbKm = vBufferSize;

		pRmParams->rvKmCol = (UInt32) (2 * ceil(pRmParams->NcbKm/(8.f * numRows)) * rvIndex + 2);
		pRmParams->NcbKmCol = (pRmParams->NcbKm - numRows * 32 - 1)/( 2 * numRows);
		pRmParams->NcbKmRow = (pRmParams->NcbKm - numRows * 32 - 1)% (2 * numRows);

		if (pRmParams->rvKmCol > 31)
		{
			pRmParams->rvKm = (pRmParams->rvKmCol - 32)/ 2 + 32;
		}
		else
			pRmParams->rvKm = pRmParams->rvKmCol;
	}
	else
	{
		pRmParams->NcbKm = 0;
		pRmParams->rvKm = 0;
		pRmParams->rvKmCol = 0;
		pRmParams->NcbKmCol = 0;
		pRmParams->NcbKmRow = 0;
	}

	if (pCodeBlkParams->numCodeBksKp)
	{
		numRows = ((pCodeBlkParams->codeBkSizeKp + 4) + 31)/32;
		vBufferSize = numRows * 32 * 3;

		if (testFlag==LTE_PDSCH) //PDSCH
		{
			if (ratio != 0)
				pRmParams->NcbKp = (UInt32) (ratio * numRows * 32.f);
			else
			{
				if (Nir/numCodeBks < vBufferSize)
					pRmParams->NcbKp =Nir/numCodeBks;
				else
					pRmParams->NcbKp = vBufferSize;
			}
		}
		else
			pRmParams->NcbKp = vBufferSize;
	
		pRmParams->rvKpCol = (UInt32) (2 * ceil(pRmParams->NcbKp/(8.f * numRows)) * rvIndex + 2);
		pRmParams->NcbKpCol = (pRmParams->NcbKp - numRows * 32 - 1)/( 2 * numRows);
		pRmParams->NcbKpRow = (pRmParams->NcbKp - numRows * 32 - 1)% (2 * numRows);

		if (pRmParams->rvKpCol > 31)
		{
			pRmParams->rvKp = (pRmParams->rvKpCol - 32)/ 2 + 32;
		}
		else
			pRmParams->rvKp = pRmParams->rvKpCol;
	}
	else
	{
		pRmParams->NcbKp = 0;
		pRmParams->rvKp = 0;
		pRmParams->rvKpCol = 0;
		pRmParams->NcbKpCol = 0;
		pRmParams->NcbKpRow = 0;
	}

    return;
}

/** ============================================================================
 *   @n@b prepare_lte_enchdr_cfg
 *
 *   @b Description
 *   @n Sets up the Encoder engine header for the test. 
 *
 *   @param[out]  
 *   @n pEncHdrCfg      Encoder engine Header configuration thus populated for 
 *                      the test.
 * 
 *   @param[in]  
 *   @n radioStd        Radio standard to use for the test.
 * 
 *   @param[in]  
 *   @n lteChanType     LTE channel type.
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_lte_enchdr_cfg 
(
    Bcp_EncHdrCfg*              pEncHdrCfg, 
    Bcp_RadioStd                radioStd, 
    UInt32                      lteChanType, 
    BcpTest_LteCBParams*        pCodeBlkParams
)
{
    UInt32                      i;

    /* Setup the Encoder header as per inputs passed. */        
    pEncHdrCfg->local_hdr_len       =   3;    

    if (lteChanType == LTE_PDSCH)
    {
        pEncHdrCfg->turbo_conv_sel  =   1;

        if (pCodeBlkParams->numCodeBks == 1)
            pEncHdrCfg->scr_crc_en  =   0;  //Dont add CRC
        else
            pEncHdrCfg->scr_crc_en  =   1;  //Add CRC per FEC block
    }
    else
    {
        pEncHdrCfg->turbo_conv_sel  =   0;
        pEncHdrCfg->scr_crc_en      =   0;  //Dont add CRC
    }

    pEncHdrCfg->code_rate_flag      =   0;

    if (pCodeBlkParams->numCodeBksKm == 0)
    {
        pEncHdrCfg->blockCfg[0].block_size      =   pCodeBlkParams->codeBkSizeKp;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   pCodeBlkParams->numCodeBksKp;

        pEncHdrCfg->blockCfg[1].block_size      =   0;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   0;

        pEncHdrCfg->blockCfg[0].intvpar0        =   pCodeBlkParams->f1Kp;
        pEncHdrCfg->blockCfg[0].intvpar1        =   pCodeBlkParams->f2Kp;

        pEncHdrCfg->blockCfg[1].intvpar0        =   0;
        pEncHdrCfg->blockCfg[1].intvpar1        =   0;
    }
    else
    {
        pEncHdrCfg->blockCfg[0].block_size      =   pCodeBlkParams->codeBkSizeKm;
        pEncHdrCfg->blockCfg[0].num_code_blks   =   pCodeBlkParams->numCodeBksKm;

        pEncHdrCfg->blockCfg[1].block_size      =   pCodeBlkParams->codeBkSizeKp;
        pEncHdrCfg->blockCfg[1].num_code_blks   =   pCodeBlkParams->numCodeBksKp;

        pEncHdrCfg->blockCfg[0].intvpar0        =   pCodeBlkParams->f1Km;
        pEncHdrCfg->blockCfg[0].intvpar1        =   pCodeBlkParams->f2Km;

        pEncHdrCfg->blockCfg[1].intvpar0        =   pCodeBlkParams->f1Kp;
        pEncHdrCfg->blockCfg[1].intvpar1        =   pCodeBlkParams->f2Kp;
    }
    pEncHdrCfg->blockCfg[2].block_size          =   0;
    pEncHdrCfg->blockCfg[2].num_code_blks       =   0;

    pEncHdrCfg->blockCfg[2].intvpar0            =   0;
    pEncHdrCfg->blockCfg[2].intvpar1            =   0;

    for (i = 0; i < 3; i ++)
    {
        pEncHdrCfg->blockCfg[i].intvpar2        =   0;
        pEncHdrCfg->blockCfg[i].intvpar3        =   0;
    }

    return;        
}

/** ============================================================================
 *   @n@b prepare_lte_rmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the LTE Rate modulation header for the test. 
 *
 *   @param[out]  
 *   @n pLteRmHdrCfg    Rate modulation Header configuration thus populated for 
 *                      the test.
 * 
 *   @param[in]  
 *   @n lteChanType     LTE channel type.
 * 
 *   @param[in]  
 *   @n pCodeBlkParams  LTE input parameters calculated for this test.
 * 
 *   @param[in]  
 *   @n pRmParams       Input Rate matching parameters for this test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_lte_rmhdr_cfg 
(
    Bcp_RmHdr_LteCfg*           pLteRmHdrCfg, 
    UInt32                      lteChanType,
    BcpTest_LteCBParams*        pCodeBlkParams, 
    BcpTest_RateMatchParams*    pRmParams
)
{
    /* Setup the LTE RM header as per the inputs specified */        
    pLteRmHdrCfg->local_hdr_len             =   6;        
    
    if (lteChanType == LTE_PDSCH || lteChanType == LTE_PUSCH_SIC)
        pLteRmHdrCfg->channel_type          =   0;        
    else
        pLteRmHdrCfg->channel_type          =   1;        

    if (lteChanType == LTE_PDSCH || lteChanType == LTE_PDCCH)
    {
        pLteRmHdrCfg->input_bit_format      =   0;        
        pLteRmHdrCfg->output_bit_format     =   0;
    }
    else
    {
        pLteRmHdrCfg->input_bit_format      =   1;        
        pLteRmHdrCfg->output_bit_format     =   1;
    }

    pLteRmHdrCfg->num_filler_bits_f         =   pCodeBlkParams->numFillerBits;        

    if (pCodeBlkParams->numCodeBksKm == 0)
    {
        pLteRmHdrCfg->num_code_blocks_c1    =   pCodeBlkParams->numCodeBksKp;        
        pLteRmHdrCfg->block_size_k1         =   pCodeBlkParams->codeBkSizeKp;        
        pLteRmHdrCfg->num_code_blocks_c2    =   0;        
        pLteRmHdrCfg->block_size_k2         =   0;        

        pLteRmHdrCfg->rv_start_column1      =   pRmParams->rvKp;        
        pLteRmHdrCfg->rv_start_column2      =   0;        
        pLteRmHdrCfg->param_ncb1_column     =   pRmParams->NcbKpCol;        
        pLteRmHdrCfg->param_ncb1_row        =   pRmParams->NcbKpRow;        
        pLteRmHdrCfg->param_ncb2_column     =   0;        
        pLteRmHdrCfg->param_ncb2_row        =   0;        
    }
    else
    {
        pLteRmHdrCfg->num_code_blocks_c1    =   pCodeBlkParams->numCodeBksKm;        
        pLteRmHdrCfg->block_size_k1         =   pCodeBlkParams->codeBkSizeKm;        
        pLteRmHdrCfg->num_code_blocks_c2    =   pCodeBlkParams->numCodeBksKp;        

        pLteRmHdrCfg->rv_start_column1      =   pRmParams->rvKm;        
        pLteRmHdrCfg->rv_start_column2      =   pRmParams->rvKp;        
        pLteRmHdrCfg->param_ncb1_column     =   pRmParams->NcbKmCol;        
        pLteRmHdrCfg->param_ncb1_row        =   pRmParams->NcbKmRow;        
        pLteRmHdrCfg->param_ncb2_column     =   pRmParams->NcbKpCol;        
        pLteRmHdrCfg->param_ncb2_row        =   pRmParams->NcbKpRow;        
        pLteRmHdrCfg->block_size_k2         =   pCodeBlkParams->codeBkSizeKp;        
    }
    pLteRmHdrCfg->num_code_blocks_ce1       =   (pCodeBlkParams->numCodeBks - pRmParams->Gamma);        
    pLteRmHdrCfg->block_size_e1             =   pRmParams->E0;        
    pLteRmHdrCfg->num_code_blocks_ce2       =   pRmParams->Gamma;        
    pLteRmHdrCfg->block_size_e2             =   pRmParams->E1;        

    return;        
}

/** ============================================================================
 *   @n@b prepare_modhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Modulation engine header for the test. 
 *
 *   @param[out]  
 *   @n pModHdrCfg      Modulation engine header configuration thus populated for the test.
 * 
 *   @param[in]  
 *   @n radioStd        Radio standard to use.
 * 
 *   @param[in]  
 *   @n Rprime          Number of allocation subcarriers.
 * 
 *   @param[in]  
 *   @n mod             Modulation type.
 * 
 *   @param[in]  
 *   @n operationMode   Modulation mode, i.e., hard/soft.
 * 
 *   @param[in]  
 *   @n rms             
 * 
 *   @param[in]  
 *   @n numOFDMsymPerSubfrm     Number of SC-FDMA symbols per sub-frame
 * 
 *   @param[in]  
 *   @n numCqiSubcs     Number of CQI sub-carriers per sub-frame
 * 
 *   @param[in]  
 *   @n numRiSubcs      Number of RI sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n numAckSubcs     Number of ACK/NACK sub-carriers per sub-frame   
 * 
 *   @param[in]  
 *   @n cinit           Scrambler second shift register initial value
 * 
 *   @param[in]  
 *   @n qFmt            Input LLR Q format
 * 
 *   @return        
 *   @n None
 * =============================================================================
 */
static Void prepare_modhdr_cfg 
(
    Bcp_ModHdrCfg*      pModHdrCfg,
    Bcp_RadioStd        radioStd,
    UInt32              Rprime, 
    UInt8               mod,   
    UInt8               operationMode, 
    UInt16              rms,
    UInt8               numOFDMsymPerSubfrm,
    UInt16              numCqiSubcs,
    UInt16              numRiSubcs,
    UInt16              numAckSubcs,
    UInt32              cinit,
    UInt8               qFmt
)
{
    /* Prepare modulation header as per inputs provided */        
    pModHdrCfg->local_hdr_len   =   4;        
    pModHdrCfg->sh_mod_sel      =   (Bcp_ModulationMode)operationMode;        
    pModHdrCfg->jack_bit        =   0;        
    if (mod == 1)
        pModHdrCfg->mod_type_sel=   Bcp_ModulationType_BPSK;
    else
        pModHdrCfg->mod_type_sel=   (Bcp_ModulationType)(mod/2);

    if (operationMode == 2) // soft modulator
    {
	    pModHdrCfg->uva_val     =	rms;          
    }
    else
    {
        if (mod == 1 || mod == 2) // BPSK
            pModHdrCfg->uva_val =   (rms * 23170 + 0x4000) >> 15;        
        else if (mod == 4) //16 QAM
            pModHdrCfg->uva_val =   (rms * 10362 + 0x4000) >> 15;        
        else if (mod == 6) //64 QAM 
            pModHdrCfg->uva_val =   (rms * 5056 + 0x4000) >> 15;        
        else //256 QAM
            pModHdrCfg->uva_val =   (rms * 2513 + 0x4000) >> 15;
    }

    if (radioStd == Bcp_RadioStd_LTE)
        pModHdrCfg->scr_en      =   1;        
    else
        pModHdrCfg->scr_en      =   0;        

    if (operationMode == 2)
    {
        pModHdrCfg->rmux_ln     =   Rprime;        
        pModHdrCfg->b_table_index=  gind_row_index(Rprime * mod);        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) (numOFDMsymPerSubfrm-9);        
        pModHdrCfg->q_format    =   (Bcp_QFormat) qFmt;        
        pModHdrCfg->cqi_ln      =   numCqiSubcs;        
        pModHdrCfg->ri_ln       =   numRiSubcs;        
        pModHdrCfg->ack_ln      =   numAckSubcs;        
    }
    else
    {
        pModHdrCfg->rmux_ln     =   0;        
        pModHdrCfg->b_table_index=  0;        
        pModHdrCfg->cmux_ln     =   (Bcp_CmuxLength) 0;        
        pModHdrCfg->q_format    =   (Bcp_QFormat) 0;        
        pModHdrCfg->cqi_ln      =   0;        
        pModHdrCfg->ri_ln       =   0;        
        pModHdrCfg->ack_ln      =   0;        
    }
    
    pModHdrCfg->cinit_p2        =   cinit;        
    pModHdrCfg->split_mode_en   =   0;        

    return;
}

/** ============================================================================
 *   @n@b prepare_tmhdr_cfg
 *
 *   @b Description
 *   @n Sets up the Traffic Manager header for the test. 
 *
 *   @param[out]  
 *   @n pTmHdrCfg   TM Header configuration thus populated for the test.
 * 
 *   @return        
 *   @n None.
 * =============================================================================
 */
static Void prepare_tmhdr_cfg (Bcp_TmHdrCfg* pTmHdrCfg)
{
    /* To be able to receive BCP processed packets from BCP on 
     * remote device back on this device without any DSP intervention on
     * remote device, we'll have to send the return path (BCP --> this device via SRIO) 
     * in Tx packet as PS Info. 
     *
     * PS Info of BCP output packet would contain SRIO return path. The SRIO return path
     * information is 8 bytes long.
     */
    pTmHdrCfg->ps_data_size     =   2;
    pTmHdrCfg->info_data_size   =   0;

    return;
}

/** ============================================================================
 *   @n@b add_test_config_data
 *
 *   @b Description
 *   @n Given a data buffer, this API sets up the packet contents for the test.
 *      It adds all the BCP configuration params (header) and the payload to the 
 *      data buffer. On success, returns the number of bytes of configuration 
 *      and payload data added to the data buffer passed. 
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n pDataBuffer Data Buffer handle to which the payload and BCP configuration
 *                  params need to be added.
 * 
 *   @return        Int32
 *   @n >0      -   Number of bytes of data (payload + header configuration) added 
 *                  to the data buffer.
 *
 *   @n -1      -   Error populating the data buffer.
 * =============================================================================
 */
static Int32 add_test_config_data (Bcp_DrvHandle hBcp, UInt8*  pDataBuffer)
{
	FILE*                       pTestCfgFile;
    UInt32                      dataBufferLen, tmpLen;
    BcpTest_LteCBParams         codeBlkParams;
    UInt32                      lteChanType, cInit;
    Bcp_RadioStd                radioStd;
    Bcp_GlobalHdrCfg            bcpGlblHdrCfg;
    Bcp_CrcHdrCfg               crcHdrCfg;
    Bcp_EncHdrCfg               encHdrCfg;
    BcpTest_RateMatchParams     rmParams;
    Bcp_RmHdr_LteCfg            lteRmHdrCfg;
    Bcp_ModHdrCfg               modHdrCfg;
    Bcp_TmHdrCfg                tmHdrCfg;
    Srio_Type9AddrInfo*         ptr_addr9Info;
    Srio_Type9AddrInfo*         ptr_localAddr9Info;
    Srio_SockAddrInfo           from;
    Srio_SockAddrInfo           to;

    /* Get the test configuration from the file. */
    if ((pTestCfgFile = fopen("..\\..\\lte\\pdsch_parameters_DV021.dat","r")) == NULL)
    {
        Bcp_osalLog ("Failed to open test configuration file: ..\..\lte\pdsch_parameters_DV021.dat\n");
        return -1;
    }
    Bcp_osalLog ("Reading test configuration ... \n");
    read_test_config(pTestCfgFile);
    fclose (pTestCfgFile);

    /* Get the code block params */
    compute_cbparams_ref (tbSize, &codeBlkParams);

    /* To begin with, lets test LTE PSDCH channel */
    radioStd    =   Bcp_RadioStd_LTE; 
    lteChanType =   LTE_PDSCH;
    
    /* Start adding BCP Packet headers based on the test configuration we read. */

    /* Initialize our data buffer length running counter */
    dataBufferLen   =   0;
    tmpLen			=	0;

    /* Header 1: Global Header */
    bcpGlblHdrCfg.pkt_type          =   Bcp_PacketType_Normal;
    bcpGlblHdrCfg.flush             =   0;
    bcpGlblHdrCfg.drop              =   0;
    bcpGlblHdrCfg.halt              =   0;
    bcpGlblHdrCfg.radio_standard    =   radioStd;
    bcpGlblHdrCfg.hdr_end_ptr       =   26;  // 2 + 4 + 4 + 7 + 5 + 2 + 2 words PS 
    bcpGlblHdrCfg.flow_id           =   RX_FLOW_ID;
    bcpGlblHdrCfg.destn_tag         =   0xDEAD; 
    if (Bcp_addGlobalHeader (&bcpGlblHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add Global Header to packet \n");
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 2: CRC Header */
    prepare_crchdr_cfg (&crcHdrCfg, radioStd, tbSize, codeBlkParams.numFillerBits, 0, 0, 0, 0, 0, 0, 0, NULL);
    if (Bcp_addCRCHeader (&crcHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add CRC Header to packet \n");
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 3: Encoder Header */
    prepare_lte_enchdr_cfg (&encHdrCfg, radioStd, lteChanType, &codeBlkParams);
    if (Bcp_addEncoderHeader (&encHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add Encoder Header to packet \n");            
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 4: Rate matching header */
    compute_rmGamma_ref (numBitsG,
                         modulation,
                         numLayers, //assumed 1 layer
                         &codeBlkParams,
                         rvIdx,
                         0, //ratio is 0
                         Nir,
                         lteChanType,
                         &rmParams);
    prepare_lte_rmhdr_cfg (&lteRmHdrCfg, lteChanType, &codeBlkParams, &rmParams);
    if (Bcp_addLte_RMHeader (&lteRmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add lte rate modulation header to packet \n");            
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 5: Modulation header */
    cInit = rnti * (1<<14) + codeWordIdx * (1<<13) + ns/2  * (1<<9) + cellID;
    prepare_modhdr_cfg (&modHdrCfg, 
                        radioStd, 
				        0, // number of subcarrier, not used,
				        modulation,
				        0, //uncompressed mode
				        1024,//rms
				        0, //numOFDMsymPerSubfrm, not used
				        0, //numCqiSubcs
				        0, //numRiSubcs
				        0, //numAckSubcs
				        cInit,
				        0); //qformat
    if (Bcp_addModulatorHeader (&modHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add Modulation header to packet \n");            
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Header 6: Traffic Manager header */
    prepare_tmhdr_cfg (&tmHdrCfg);
    if (Bcp_addTMHeader (&tmHdrCfg, pDataBuffer, &tmpLen) < 0)
    {
        Bcp_osalLog ("Failed to add Traffic Manager header to packet \n");            
        return -1;
    }
    pDataBuffer 	+=	tmpLen;
    dataBufferLen	+=	tmpLen;
    tmpLen			=	0;

    /* Add SRIO return path for BCP results in the PS info field of Tx packets.
     * The return path is used by BCP on TN to return BCP output packets back
     * to this device.
     *
     * Return path would be: From BCP --> to this device
     */
    from.type9.tt       = TRUE;                 /* We are using 16 bit identifiers. */
    from.type9.id       = DEVICE_ID_BCP_16BIT;  /* BCP Source Identifier    */
    from.type9.streamId = 1;                    /* BCP Stream Identifier    */
    from.type9.cos      = CSL_chipReadReg (CSL_CHIP_DNUM);  /* Class of Service */
   
    to.type9.tt         = TRUE;                 /* We are using 16 bit identifiers. */
    to.type9.id         = DEVICE_ID1_16BIT;     /* Current device Source Identifier */
    to.type9.streamId   = 0;                    /* Current device Stream Identifier */
    to.type9.cos        = CSL_chipReadReg (CSL_CHIP_DNUM);  /* Class of Service     */
    
    ptr_addr9Info = (Srio_Type9AddrInfo*)&to;
 
    /* Get the address information to which the socket is bound. */
    ptr_localAddr9Info = (Srio_Type9AddrInfo *)&from;

    /* Create the SRIO Protocol Specific Information. */
    CSL_SRIO_SetType9PSInfo((UInt32 *) (pDataBuffer), ptr_addr9Info->id, ptr_localAddr9Info->id, 
                             ptr_addr9Info->cos, 0x1, ptr_addr9Info->tt, ptr_addr9Info->streamId);
    pDataBuffer 	+=	8;
    dataBufferLen	+=	8;
	                             
	/* Add padding to align data on 128 bits */    
    pDataBuffer 	+=	8;
    dataBufferLen	+=	8;

    /* Finally add the data to the packet in the same order as the headers */
    if ((pTestCfgFile = fopen("..\\..\\lte\\pdsch_inforBits_0002.dat","r")) == NULL)
    {
        Bcp_osalLog ("Cannot open data input file: ..\..\lte\pdsch_inforBits_0002.dat\n");
        return -1;
    }
    read_data_from_file (pTestCfgFile, pDataBuffer, &dataBufferLen);
    fclose (pTestCfgFile);
       
    /* Successfully read the test configuration */        
    return dataBufferLen;
}

/** ============================================================================
 *   @n@b my_freeRecvBuffer
 *
 *   @b Description
 *   @n SRIO receive buffer cleanup API 
 *
 *   @param[in]  
 *   @n hSrioBuf    SRIO driver buffer handle
 *
 *   @return        None
 * =============================================================================
 */
Void my_freeRecvBuffer (Srio_DrvBuffer hSrioBuf)
{
   	srio_freeRecvBuffer (NULL, hSrioBuf); 
	return;
}


/** ============================================================================
 *   @n@b run_bcp_client
 *
 *   @b Description
 *   @n Sets up the parameters required to run an LTE Downlink (DL) test case 
 *      using the BCP on remote device. Uses SRIO to send requests and
 *      receive results from remote BCP.
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n hGlblFDQ    Global free descriptor queue handle. The test will allocate all its
 *                  free descriptors from this queue for its run and release them
 *                  back once done to this queue.
 * 
 *   @return
 *   @n None
 *
 * =============================================================================
 */
Void run_bcp_client (Bcp_DrvHandle hBcp, Qmss_QueueHnd hGlblFDQ)
{
    Bcp_TxHandle        hTx = NULL;
    Bcp_RxHandle        hRx = NULL;
    Qmss_QueueHnd       hSrioTxFDQ;
    Qmss_QueueHnd       hSrioRxFDQ;
    UInt32              dataBufferLen;
    Int32               dataBufferLenUsed;
    Cppi_Desc*          pCppiDesc;
    UInt8*              pDataBuffer;
    Srio_SockAddrInfo   to;
    Srio_DrvConfig      srioDrvCfg;
    UInt8               rxFlowId, rxSrcId;
    UInt32              i, rxDataBufferLen, rxPsInfoLen, rxDataTotalLen, testFail = 0;
    UInt16              rxDestnTag;
    UInt8*              pRxDataBuffer;
    UInt8*              pRxPsInfo;
    Cppi_DescTag        cppiTagInfo;
    Void*               hTmp;
    Qmss_Queue          tmpQ;

    ((void)maxNumHarqProcess);
    ((void)ueCategory);
    ((void)ueTransMode);

    /* Setup BCP Rx configuration. 
     *  -   Nothing to setup here since BCP flows are not accessible
     *      on remote device.
     *  -   Just setup transport Rx endpoint configuration instead
     *      to receive all output from BCP onto a Rx queue.
     */

    /* Setup BCP Transport Rx endpoint configuration.
     *
     * The transport we are using here is SRIO. 
     *  -   Setup a Rx FDQ for SRIO on this device 
     *  -   Setup a SRIO Rx flow to receive all output from BCP via SRIO and redirect
     *      them onto a general purpose Rx queue. The application will poll
     *      on this Rx queue for results, and will validate them.
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, 0, &hSrioRxFDQ, NULL) < 0)
    {
        Bcp_osalLog ("Error opening Rx FDQ for SRIO \n");            
        return;
    }
    else
    {
        Bcp_osalLog ("SRIO Rx FDQ %d successfully setup with %d descriptors\n", hSrioRxFDQ, RX_NUM_DESC);
    }
    tmpQ = Qmss_getQueueNumber(hSrioRxFDQ);
    
    /* Initialize the SRIO Driver Configuration. */
    memset ((Void *)&srioDrvCfg, 0, sizeof(Srio_DrvConfig));

    /* Use application managed flow configuration */
    srioDrvCfg.bAppManagedConfig                        =   TRUE;
    srioDrvCfg.u.appManagedCfg.bIsRxFlowCfgValid        =   TRUE;

    /* Configure the Receive Flow */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.flowIdNum          = -1;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_qnum       = RX_Q_NUM; /* Receive BCP output on Rx Q */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_qmgr       = 0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_sop_offset      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_ps_location     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_desc_type       = 0x1; /* Host Descriptor. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_error_handling  = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_psinfo_present  = 0x0; /* No PS Information */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_einfo_present   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_lo     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_hi     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_lo      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_hi      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_lo_sel = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_hi_sel = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_lo_sel  = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_hi_sel  = 0x0;

    /* Disable Receive size thresholds. */
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh0_en = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh1_en = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh2_en = 0x0;

	/* Use the Application Receive Free Queue for picking all descriptors. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq1_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq1_qmgr       = tmpQ.qMgr;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq2_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq2_qmgr       = tmpQ.qMgr;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq3_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq3_qmgr       = tmpQ.qMgr;

    /* Use the Receive Queue for picking the SOP packet also. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz0_qnum   = hSrioRxFDQ;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz0_qmgr   = tmpQ.qMgr;

    /* There are no size thresholds configured. */
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh0    = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh1    = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh2    = 0x0;

    /* The other threshold queues do not need to be configured */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz1_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz1_qmgr   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz2_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz2_qmgr   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz3_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz3_qmgr   = 0x0;

    /* Polling Mode: So dont program the accumulator. */
	srioDrvCfg.u.appManagedCfg.bIsAccumlatorCfgValid        = 0;

    /* Populate the rest of the configuration. */
    srioDrvCfg.u.appManagedCfg.rawRxFreeDrvBuffer           = my_freeRecvBuffer;

    /* No BCP Rx configuration. Setup only BCP Rx endpoint */
    if ((hRx = Bcp_rxOpen (hBcp, NULL, &srioDrvCfg)) == NULL)
    {
        Bcp_osalLog ("BCP Rx Open failed \n");
		goto cleanup_and_return;
    }
    else
    {
        Bcp_osalLog ("SRIO Flow opened to send data to RxQ: %d \n", RX_Q_NUM);
    }

    /* Setup BCP Tx configuration.
     *  -   BCP is not local to the device. Hence cannot configure BCP Tx
     *      queue directly. So no need for setting up BCP Tx configuration.
     *  -   Setup only BCP Tx transport tunnel to be able to send packets to
     *      BCP from this device via SRIO
     */

    /* Setup BCP Tx endpoint configuration 
     * -    Open SRIO Tx queue. We'll use it to send BCP requests to
     *      BCP on TN.
     */
    /* Initialize the SRIO Driver Configuration. */
    memset ((Void *)&srioDrvCfg, 0, sizeof(Srio_DrvConfig));

    /* Enable application manged configuration. Disable Rx flow
     * configuration, since all we need here is SRIO Tx queue to be
     * setup.
     */
    srioDrvCfg.bAppManagedConfig                        =   TRUE;
    srioDrvCfg.u.appManagedCfg.bIsRxFlowCfgValid        =   FALSE;
	srioDrvCfg.u.appManagedCfg.bIsAccumlatorCfgValid    =   FALSE;
    if ((hTx = Bcp_txOpen (hBcp, NULL, &srioDrvCfg)) == NULL)
    {
        Bcp_osalLog ("Error opening BCP Tx object \n");
        return;
    }

    /* Allocate a Tx FDQ for sending BCP requests. */
    if (allocate_fdq (hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, 0, &hSrioTxFDQ, NULL) < 0)
    {
        Bcp_osalLog ("Error opening Tx FDQ \n");            
        return;
    }
    else
    {
        Bcp_osalLog ("SRIO Tx FDQ %d successfully setup with %d descriptors\n", hSrioTxFDQ, TX_NUM_DESC);
        Bcp_osalLog ("\nReady to send packets to BCP over SRIO ... \n");
    }

    /* Build and Send a packet with WCDMA DL parameters for BCP Processing */
    if ((pCppiDesc = (Cppi_Desc*) Qmss_queuePop (hSrioTxFDQ)) == NULL)
    {
        Bcp_osalLog ("Out of Tx FDs! \n");
		goto cleanup_and_return;
    }
    pCppiDesc = (Cppi_Desc *) (QMSS_DESC_PTR (pCppiDesc));    
    Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pDataBuffer, &dataBufferLen);
        
    /* Initialize data buffer we add BCP request data */
	memset (pDataBuffer, 0, dataBufferLen); 

    /* Read test configuration and build BCP request packet */
    if ((dataBufferLenUsed = add_test_config_data (hBcp, pDataBuffer)) <= 0)
    {
        Bcp_osalLog ("Error reading test vectors/populating BCP packet \n");
        goto cleanup_and_return;
    }

    /* Populate the destination information where the BCP request packet must be sent to. */
    to.type9.tt         = TRUE;                 /* We are using 16 bit identifiers.          */
    to.type9.id         = DEVICE_ID_BCP_16BIT;  /* BCP device Identifier    */
    to.type9.streamId   = 1;                    /* BCP device Stream Identifier */
    to.type9.cos        = CSL_chipReadReg (CSL_CHIP_DNUM);  /* Class of Service */

   	Bcp_osalLog ("Sending a packet of len %d to BCP over SRIO ...\n", dataBufferLenUsed);
    Bcp_send (hTx, pCppiDesc, dataBufferLenUsed, &to);
    
    Bcp_osalLog ("Waiting on output from BCP ...\n");

    /* Wait on data to be received from BCP and validate it */
    while (Qmss_getQueueEntryCount (RX_Q_NUM) == 0) { Task_sleep (2); }

    Bcp_osalLog ("Got %d output packet(s) from BCP \n", Qmss_getQueueEntryCount (RX_Q_NUM));

    Bcp_osalLog ("Validating BCP output ... \n");

    i = 0;
    while (Qmss_getQueueEntryCount (RX_Q_NUM) > 0)
    {
        /* Data could arrive scattered across multiple linked descriptors.
         * Collect data from all linked descriptors and validate it.
         */
        rxDataTotalLen  =   0;

        if ((pCppiDesc = Qmss_queuePop (RX_Q_NUM)) != NULL)
        {
            pCppiDesc = (Cppi_Desc *)QMSS_DESC_PTR (pCppiDesc);                

            /* Get the source and destination tag information */
            cppiTagInfo         =   Cppi_getTag (Cppi_DescType_HOST, pCppiDesc);
            rxSrcId             =   cppiTagInfo.srcTagHi;
            rxFlowId            =   cppiTagInfo.srcTagLo;
            rxDestnTag          =   ((cppiTagInfo.destTagHi << 8) | (cppiTagInfo.destTagLo));

            /* Get Data buffer containing the output and its length */
            Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pRxDataBuffer, &rxDataBufferLen);

            /* Get PS data buffer and its length from the descriptor */
            Cppi_getPSData (Cppi_DescType_HOST, (Cppi_PSLoc)0x0, pCppiDesc, &pRxPsInfo, &rxPsInfoLen);
        }
        /* Validate BCP output against expected output */
        if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1, 
                WCDMA_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                pRxDataBuffer, 
                rxDataBufferLen, 
                rxDataTotalLen) != 0)
                testFail ++;

        rxDataTotalLen  +=  rxDataBufferLen;

        /* Check if there are any descriptors linked to this Rx desc */
        while (pCppiDesc)
        {
            /* Save original Rx desc handle. */
            hTmp = pCppiDesc;

            if ((pCppiDesc = Cppi_getNextBD (Cppi_getDescType (pCppiDesc), pCppiDesc)))
            {
                pCppiDesc = (Cppi_Desc *)QMSS_DESC_PTR (pCppiDesc);                

                /* Get the source and destination tag information */
                cppiTagInfo         =   Cppi_getTag (Cppi_DescType_HOST, pCppiDesc);
                rxSrcId             =   cppiTagInfo.srcTagHi;
                rxFlowId            =   cppiTagInfo.srcTagLo;
                rxDestnTag          =   ((cppiTagInfo.destTagHi << 8) | (cppiTagInfo.destTagLo));

                /* Get Data buffer containing the output and its length */
                Cppi_getData (Cppi_DescType_HOST, pCppiDesc, &pRxDataBuffer, &rxDataBufferLen);

                /* Get PS data buffer and its length from the descriptor */
                Cppi_getPSData (Cppi_DescType_HOST, (Cppi_PSLoc)0x0, pCppiDesc, &pRxPsInfo, &rxPsInfoLen);
       
                if (validate_rxdata ((UInt8 *)lte_dl_out_packet_1, 
                                    WCDMA_DL_OUTPUT_PKT_1_WRD_SIZE*4, 
                                    pRxDataBuffer, 
                                    rxDataBufferLen, 
                                    rxDataTotalLen) != 0)
                    testFail ++;                        

                rxDataTotalLen  +=  rxDataBufferLen;
            }
        
            Bcp_rxFreeRecvBuffer (hRx, hTmp);
        }

        Bcp_osalLog ("[Pkt %d]: Total Len: %d DestnTag: 0x%x SrcId: 0x%x FlowId: 0x%x\n", i, rxDataTotalLen, rxDestnTag, rxSrcId, rxFlowId);

        if (rxDataTotalLen != WCDMA_DL_OUTPUT_PKT_1_WRD_SIZE * 4)
        {
            testFail ++;
        }

        /* Increment number of packets received. */
        i ++;
    }

    if (testFail > 0)
    {
        Bcp_osalLog ("LTE DL Test over Remote TN:    FAILED\n");                
        totalNumTestsFail ++;
    }
    else
    {
        Bcp_osalLog ("LTE DL Test over Remote TN:    PASS\n");                
        totalNumTestsPass ++;
    }
    
    Bcp_osalLog ("BCP Remote transfer successful!\n");
cleanup_and_return:
    if (hTx)
        Bcp_txClose (hTx);

    deallocate_fdq (hSrioTxFDQ, hGlblFDQ, TX_NUM_DESC, TX_DATA_BUFFER_SIZE, NULL);

    if (hRx)
        Bcp_rxClose (hRx);            

    deallocate_fdq (hSrioRxFDQ, hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, NULL);

    return;
}

/** ============================================================================
 *   @n@b run_bcp_server
 *
 *   @b Description
 *   @n Sets up the BCP/SRIO on BCP local device to be able to receive and process
 *      BCP requests. All processed packets from BCP (BCP output) are routed back
 *      to a remote device based on return route programmed on the BCP request packet.
 *
 *   @param[in]  
 *   @n hBcp        BCP driver handle
 *
 *   @param[in]  
 *   @n hGlblFDQ    Global free descriptor queue handle. The test will allocate all its
 *                  free descriptors from this queue for its run and release them
 *                  back once done to this queue.
 * 
 *   @return
 *   @n None
 *
 * =============================================================================
 */
    Qmss_QueueHnd       hSrioRxFDQ;
    Bcp_TxCfg           txCfg;
    Bcp_TxHandle        hTx = NULL;
    Bcp_RxCfg           rxCfg;
    Bcp_RxHandle        hRx = NULL;
    Qmss_QueueHnd       hRxFDQ;
    Srio_DrvConfig      srioDrvCfg;
    UInt32              i;
    Qmss_Queue          tmpQ;

    /* Setup BCP Rx configuration:
     *  -   Open Rx queue on to which BCP results are to be forwarded to
     *  -   If running on simulator, we route all BCP output to a general
     *      purpose Rx queue (due to Sim packet type assignment bug) and
     *      then put the correct packet type on the packet and send it out
     *      via SRIO.
     *  -   If running on Si, we route all BCP output directly to SRIO to
     *      be sent out to a remote device.
     *  -   Setup a BCP Rx FDQ, Rx flow to receive data
     *  -   No need to setup BCP transport Rx endpoint configuration since
     *      BCP is local to device and we can receive data from BCP directly.
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, 0, &hRxFDQ, NULL) < 0)
    {
        Bcp_osalLog ("Error opening Rx FDQ \n");            
        return;
    }
    else
    {
        Bcp_osalLog ("BCP Rx FDQ %d successfully setup with %d descriptors\n", hRxFDQ, RX_NUM_DESC);
    }
    tmpQ = Qmss_getQueueNumber(hRxFDQ);
    
    rxCfg.rxQNum                        =   SRIO_TX_QUEUE(0);   // Send BCP output to SRIO Tx queue to be sent out to remote device.
    rxCfg.bUseInterrupts                =   0;                  // Use polling 
    memset (&rxCfg.flowCfg, 0, sizeof(Cppi_RxFlowCfg));
    rxCfg.flowCfg.flowIdNum             =   RX_FLOW_ID;  
    rxCfg.flowCfg.rx_dest_qmgr          =   0;    
    rxCfg.flowCfg.rx_dest_qnum          =   rxCfg.rxQNum;  
    rxCfg.flowCfg.rx_desc_type          =   Cppi_DescType_HOST; 
    rxCfg.flowCfg.rx_ps_location        =   Cppi_PSLoc_PS_IN_DESC;  // SRIO header to be added as PS to be present in desc
    rxCfg.flowCfg.rx_psinfo_present     =   1;              //  Enable PS info. Will be used for sending SRIO header to SRIO
    rxCfg.flowCfg.rx_error_handling     =   0;              //  Drop the packet, do not retry on starvation
    rxCfg.flowCfg.rx_einfo_present      =   0;              //  By default no EPIB info
    
    rxCfg.flowCfg.rx_dest_tag_lo_sel    =   4;              //  Pick dest tag 7:0 bits from the PD dest_tag        
    rxCfg.flowCfg.rx_dest_tag_hi_sel    =   5;              //  Pick the dest tag 15:8 bits from the PD dest_tag
    rxCfg.flowCfg.rx_src_tag_lo_sel     =   2;              //  Pick the src tag 7:0 bits from the PD flow_id 7:0
    rxCfg.flowCfg.rx_src_tag_hi_sel     =   4;              //  Pick the src tag 15:8 bits from the PD src_tag 7:0 

    rxCfg.flowCfg.rx_size_thresh0_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh1_en    =   0;    
    rxCfg.flowCfg.rx_size_thresh2_en    =   0;    

    rxCfg.flowCfg.rx_size_thresh0       =   0x0;
    rxCfg.flowCfg.rx_size_thresh1       =   0x0;
    rxCfg.flowCfg.rx_size_thresh2       =   0x0;

    rxCfg.flowCfg.rx_fdq0_sz0_qmgr      =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq0_sz0_qnum      =   tmpQ.qNum; 
    rxCfg.flowCfg.rx_fdq0_sz1_qnum      =   0x0; 
    rxCfg.flowCfg.rx_fdq0_sz1_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz2_qmgr      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qnum      =   0x0;
    rxCfg.flowCfg.rx_fdq0_sz3_qmgr      =   0x0;

    rxCfg.flowCfg.rx_fdq1_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq1_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq2_qnum          =   tmpQ.qNum;  
    rxCfg.flowCfg.rx_fdq2_qmgr          =   tmpQ.qMgr;
    rxCfg.flowCfg.rx_fdq3_qnum          =   tmpQ.qNum;   
    rxCfg.flowCfg.rx_fdq3_qmgr          =   tmpQ.qMgr;

    /* Setup TM flow entry configuration for BCP */
#ifdef SIMULATOR_SUPPORT
    /* Simulator bug workaround: BCP TM module endian handling not correct. */
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_128;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_128;
#else
    rxCfg.tmFlowCfg.endian_in           =   Bcp_EndianFormat_32;
    rxCfg.tmFlowCfg.endian_out          =   Bcp_EndianFormat_32;
#endif
    rxCfg.tmFlowCfg.format_in           =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.pkt_type            =   30;     // Set Rx packet type to be 30 for SRIO Type 9
    rxCfg.tmFlowCfg.dsp_int_sel         =   0;      // Interrupt core 0
    rxCfg.tmFlowCfg.format_out          =   Bcp_DataFormat_NoChange;
    rxCfg.tmFlowCfg.qfifo_out           =   RX_FLOW_ID;      // Use Rx QFIFO 3
    rxCfg.tmFlowCfg.ps_flags            =   0;      // No PS flags for now

    /* Setup BCP Rx. No transport setup required since BCP is local to device. */
    if ((hRx = Bcp_rxOpen (hBcp, &rxCfg, NULL)) == NULL)
    {
        Bcp_osalLog ("BCP Rx Open failed \n");
		goto cleanup_and_return;
    }
    else
    {
        Bcp_osalLog ("BCP Flow %d opened to send data to RxQ: %d \n", RX_FLOW_ID, rxCfg.rxQNum);
    }

    /* Setup BCP Tx side:
     *  -   Open BCP Tx queue for remote BCP request processing.
     *  -   Setup BCP transport Tx endpoint configuration.
     */
    txCfg.txQNum    =   Bcp_QueueId_0;

    /* Setup BCP Transport Tx endpoint configuration.
     *
     * The transport we are using here is SRIO. 
     *  -   Setup SRIO to redirect all packets it receives to BCP Tx queue we are using.
     *  -   Setup a Rx FDQ for SRIO on this device 
     *  -   Setup a SRIO Rx flow to redirect all traffic received to BCP Tx queue
     */
    if (allocate_fdq (hGlblFDQ, RX_NUM_DESC, TX_DATA_BUFFER_SIZE, 0, &hSrioRxFDQ, NULL) < 0)
    {
        Bcp_osalLog ("Error opening Rx FDQ for SRIO \n");            
        return;
    }
    else
    {
        Bcp_osalLog ("SRIO Rx FDQ %d successfully setup with %d descriptors\n", hSrioRxFDQ, RX_NUM_DESC);
    }
     tmpQ = Qmss_getQueueNumber(hSrioRxFDQ);
     
    /* Initialize the SRIO Driver Configuration. */
    memset ((Void *)&srioDrvCfg, 0, sizeof(Srio_DrvConfig));

    /* Setup the SRIO Application Managed Configuration. */
    srioDrvCfg.bAppManagedConfig                            = TRUE;

    /* Enable flow configuration  */
    srioDrvCfg.u.appManagedCfg.bIsRxFlowCfgValid            = 1;

    /* Configure the Receive Flow */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.flowIdNum          = -1;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_qnum       = BCP_TX_QUEUE(txCfg.txQNum); // Send SRIO packets to BCP
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_qmgr       = 0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_sop_offset      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_ps_location     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_desc_type       = 0x1; /* Host Descriptor. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_error_handling  = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_psinfo_present  = 0x0; /* No PS Information */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_einfo_present   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_lo     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_hi     = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_lo      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_hi      = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_lo_sel = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_dest_tag_hi_sel = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_lo_sel  = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_src_tag_hi_sel  = 0x0;

    /* Disable Receive size thresholds. */
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh0_en = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh1_en = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh2_en = 0x0;

	/* Use the Application Receive Free Queue for picking all descriptors. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq1_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq1_qmgr       = tmpQ.qMgr;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq2_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq2_qmgr       = tmpQ.qMgr;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq3_qnum       = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq3_qmgr       = tmpQ.qMgr;

    /* Use the Receive Queue for picking the SOP packet also. */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz0_qnum   = tmpQ.qNum;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz0_qmgr   = tmpQ.qMgr;

    /* There are no size thresholds configured. */
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh0    = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh1    = 0x0;
    srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_size_thresh2    = 0x0;

    /* The other threshold queues do not need to be configured */
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz1_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz1_qmgr   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz2_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz2_qmgr   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz3_qnum   = 0x0;
	srioDrvCfg.u.appManagedCfg.rxFlowCfg.rx_fdq0_sz3_qmgr   = 0x0;

    /* Polling Mode: So dont program the accumulator. */
	srioDrvCfg.u.appManagedCfg.bIsAccumlatorCfgValid        = 0;

    /* Populate the rest of the configuration. */
    srioDrvCfg.u.appManagedCfg.rawRxFreeDrvBuffer           = my_freeRecvBuffer;

    /* Setup BCP Tx queue. Setup also transport Tx endpoint configuration to forward all packets received on SRIO to BCP Tx queue.*/
    if ((hTx = Bcp_txOpen (hBcp, &txCfg, &srioDrvCfg)) == NULL)
    {
        Bcp_osalLog ("BCP Tx Open failed \n");
        return;
    }
    else
    {
        Bcp_osalLog ("BCP Tx Queue %d succesfully setup. Waiting for BCP requests ...\n", txCfg.txQNum);
    }

    /* Do nothing. Wait forever for BCP requests. */
    while (1)
    {
        Task_sleep (20);
    }
cleanup_and_return:
    if (hRx)
        Bcp_rxClose (hRx);

    deallocate_fdq (hRxFDQ, hGlblFDQ, RX_NUM_DESC, RX_DATA_BUFFER_SIZE, NULL);

    if (hTx)
        Bcp_txClose (hTx);

    deallocate_fdq (hSrioRxFDQ, hGlblFDQ, RX_NUM_DESC, TX_DATA_BUFFER_SIZE, NULL);
	
    return;
}
