/** 
 *   @file  fftc_testcfg_2048.h
 *
 *   @brief  
 *      Header file containing FFTC Test vector details.
 *
 *      The format of the file contents are as follows:
 *          *   Line 1:                         Number of FFT data blocks in this
 *                                              packet configuration.
 *          *   Line 2 onwards (32 entries):    FFT configuration for the data, details
 *                                              below:
 *                  (01)    DFT sample size - each sample is again 4 bytes wide.
 *                          (Fftc_QLocalCfg.controlRegConfig.dftSize)
 *                  (02)    Not used
 *                  (03)    Not used
 *                  (04)    Not used
 *                  (05)    Dynamic Scaling Enable Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bDynamicScaleEnable)
 *                  (06)    Last Radix Stage Output Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingValLast)
 *                  (07)    Radix Stage 0 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[0])
 *                  (08)    Radix Stage 1 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[1])
 *                  (09)    Radix Stage 2 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[2])
 *                  (10)    Radix Stage 3 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[3])
 *                  (11)    Radix Stage 4 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[4])
 *                  (12)    Radix Stage 5 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[5])
 *                  (13)    Radix Stage 6 Scaling value (only for static scaling)
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.radixScalingVal[6])
 *                  (14)    Not used
 *                  (15)    Not used
 *                  (16)    Not used
 *                  (17)    Inverse FFT enable boolean flag
 *                          (Fftc_QLocalCfg.controlRegConfig.bInverseFFTEnable)
 *                  (18)    Emulate DSP 16x16 boolean flag - Not used currently
 *                          (Fftc_QLocalCfg.controlRegConfig.bEmulateDSP16x16)
 *                  (19)    Output Scaling Value
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.outputScaleVal)
 *                  (20)    Input Shifting Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bInputFFTShift)
 *                  (21)    Output Shifting Boolean flag
 *                          (Fftc_QLocalCfg.scalingShiftingRegConfig.bOutputFFTShift)
 *                  (22)    Zero Pad Mode Enable boolean flag
 *                          (Fftc_QLocalCfg.controlRegConfig.bZeroPadEnable)
 *                  (23)    Zero Pad Mode (used only if zero padding enabled) 
 *                          (Fftc_QLocalCfg.controlRegConfig.zeroPadMode)
 *                  (24)    Zero Pad Factor (used only if zero padding enabled) 
 *                          (Fftc_QLocalCfg.controlRegConfig.zeroPadFactor)
 *                  (25)    LTE Frequency Shifting Enable boolean flag
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.bFreqShiftEnable)
 *                  (26)    LTE Frequency Shift value (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftVal)
 *                  (27)    LTE Frequency Shift Table Index (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftIndex)
 *                  (28)    LTE Frequency Shift Multiplication factor (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftMultFactor)
 *                  (29)    LTE Frequency Shift Phase (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftInitPhase)
 *                  (30)    LTE Frequency Shift Direction (used only if LTE freq shifting enabled)
 *                          (Fftc_QLocalCfg.freqShiftRegConfig.freqShiftDirection)
 *                  (31)    Cyclic Prefix Addition Enable Boolean flag 
 *                          (Fftc_QLocalCfg.cyclicPrefixRegConfig.bCyclicPrefixAddEnable)
 *                  (32)    Cyclic Prefix Addition Value (Used only if cyclic prefix addition enabled) 
 *                          (Fftc_QLocalCfg.cyclicPrefixRegConfig.cyclicPrefixAddNum)
 *          *   Block Number
 *          *   FFT input data (sample size * 2 entries)
 *          *   Expected Block Exponent Value
 *          *   Expected Clipping Detect Value
 *          *   Expected Number of clock cycles to be taken for operation          
 *          *   Expected FFT output data - reference data to validate result from FFT engine.
 *              (sample size * 2 entries)
 * 
 *  \par
 *  ============================================================================
 *  @n   (C) Copyright 2009, Texas Instruments, Inc.
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _FFTC_TESTCFG_2048_H_
#define _FFTC_TESTCFG_2048_H_

Int16 inputConfig_2048[] = {
1,
2048,
16,
19,
15,
1,
0,
0,
0,
0,
0,
0,
0,
0,
1,
1,
1,
0,
0,
128,
0,
0,
0,
0,
1,
0,
0,
0,
0,
0,
0,
0,
0,
1,
1892,
-8037,
-1437,
5047,
-10444,
1482,
1239,
6244,
-5296,
-2891,
-1964,
-1698,
-1709,
1081,
-7349,
834,
3980,
-583,
-5206,
5355,
5301,
-1292,
-2261,
-6277,
-4400,
6723,
5008,
-113,
-656,
-3165,
-2156,
1894,
4185,
-1876,
497,
5053,
-4744,
-2414,
5499,
-2607,
-1099,
8434,
176,
2604,
2180,
-3511,
-846,
13279,
269,
1607,
3824,
138,
-4730,
7706,
-8143,
-2962,
2588,
2715,
-1096,
-6641,
-2099,
-3722,
-4184,
1398,
4408,
-2877,
-6842,
4456,
-2452,
819,
2507,
2293,
4698,
-4233,
6036,
-1013,
-3963,
-2696,
-2283,
-3346,
1687,
273,
-5583,
2525,
2335,
-2723,
-6011,
-2499,
-2042,
5281,
6668,
3601,
-686,
9454,
6356,
-3032,
-7869,
-9837,
5249,
-5235,
2936,
3016,
2461,
1968,
-1026,
-179,
-7424,
3676,
-4008,
-6059,
-354,
3080,
-430,
3467,
5384,
4647,
-1622,
-3249,
666,
4340,
-8366,
-5405,
-6553,
-2788,
-1420,
4651,
3049,
-2209,
1504,
236,
5372,
3960,
5174,
6483,
-4298,
-5654,
2203,
-3702,
-6166,
-892,
-3394,
4515,
-4873,
-8652,
1970,
7639,
1073,
-4005,
-893,
3503,
1408,
2999,
-2069,
-2517,
6627,
3314,
-327,
7024,
-17096,
-2076,
-2157,
3530,
5570,
887,
-2875,
5266,
2644,
3246,
2700,
-4108,
-4065,
5152,
6337,
-6742,
390,
2848,
-1076,
-775,
4357,
79,
5740,
-3646,
-1635,
714,
-4329,
-1568,
1031,
-595,
-4834,
5348,
556,
3765,
6409,
5268,
-5471,
-4574,
5561,
-6571,
-2813,
-3365,
-1995,
-4378,
-5320,
-2065,
3197,
48,
8755,
-1012,
100,
-2022,
9251,
-2702,
-1207,
751,
1709,
-2866,
-46,
-4377,
-3667,
7906,
3931,
-4561,
-1649,
5211,
-5253,
-2550,
-748,
-10292,
8112,
2061,
-2139,
1828,
-4858,
-2649,
-3613,
-4659,
7679,
-3285,
-1845,
556,
3478,
-7952,
-107,
-16041,
-5679,
2922,
4160,
759,
-3414,
-2866,
-151,
-1148,
-9879,
6051,
6564,
2873,
-1018,
-2590,
6840,
3451,
-1620,
680,
-2805,
15907,
4456,
1858,
989,
5986,
2764,
3267,
9176,
-4400,
-9974,
-11577,
3740,
1379,
-1138,
-3986,
-305,
3581,
-2131,
-1992,
5232,
7406,
-4049,
1467,
1605,
-3550,
-5723,
-1133,
-616,
10179,
-251,
-346,
-349,
7293,
-2754,
-6571,
-2898,
7514,
-6664,
-2405,
9577,
5548,
1913,
-2642,
1202,
6421,
-778,
-5011,
-2360,
1479,
2644,
-2139,
3788,
-6291,
-1397,
674,
-9,
10301,
3693,
-5625,
3505,
1414,
-1984,
4467,
-73,
616,
-7066,
-8769,
4875,
-8313,
-7029,
-2213,
56,
-2160,
-843,
-657,
1594,
821,
-975,
-12261,
3335,
474,
690,
-1760,
-511,
-3832,
-1284,
9451,
495,
-1274,
-703,
-7031,
240,
-5013,
982,
1522,
-5321,
7704,
-2914,
5309,
4783,
12061,
9080,
-2933,
-1071,
5046,
5853,
2664,
3945,
-185,
622,
5323,
999,
-2930,
4674,
1889,
-1762,
-1990,
-5512,
-2922,
-5611,
-5895,
5516,
4955,
-1917,
8496,
3302,
-2530,
4936,
9345,
2847,
4083,
-7326,
4086,
2522,
5810,
6601,
5974,
-4212,
1589,
923,
4907,
674,
-6688,
5115,
-7747,
4270,
-6154,
-3170,
-1611,
-2939,
4822,
-32,
4878,
-3006,
-2073,
-1601,
1107,
-5117,
-1003,
3534,
-595,
1800,
624,
3634,
-3966,
10432,
1134,
-4706,
8372,
809,
-3312,
-3281,
-609,
5867,
-111,
2871,
-4249,
-1200,
-2528,
11720,
6148,
-497,
1179,
1702,
1850,
5576,
-4700,
-628,
1722,
-1547,
11737,
9817,
1055,
1101,
-4975,
878,
4797,
-6346,
-2178,
-5233,
-1143,
-2381,
-2851,
2459,
-5194,
202,
4089,
1805,
4639,
7537,
4725,
3686,
3192,
-5906,
2375,
4353,
-1800,
1069,
6240,
2903,
10219,
964,
-1457,
4395,
1112,
1303,
4787,
892,
9444,
4394,
2616,
-4310,
1410,
-2879,
-2908,
-557,
3903,
-234,
8061,
-6260,
5372,
-2645,
2710,
-1506,
5433,
-791,
4321,
-4201,
6132,
-96,
1514,
-1306,
-3855,
-218,
2243,
5299,
-755,
-735,
-2347,
-254,
-5191,
5163,
-4148,
-5129,
-7897,
6977,
-771,
-3641,
972,
-692,
-1431,
-6426,
7181,
-1661,
1246,
-499,
-126,
1914,
-2057,
-1496,
4464,
-2438,
4692,
4608,
4768,
4892,
-1076,
4346,
8729,
3271,
-6016,
-3689,
-3271,
420,
6632,
-3600,
983,
-6488,
-8064,
-2635,
2575,
2429,
2105,
7318,
-6942,
811,
-3077,
-3005,
-341,
-8504,
-3662,
-599,
-3217,
1318,
5452,
632,
-922,
2741,
5091,
3017,
-6068,
7772,
2897,
3314,
3884,
3254,
414,
-7420,
559,
7338,
7705,
-8278,
84,
-777,
2689,
-6489,
5027,
-4763,
-1666,
-3693,
-4204,
-6346,
2104,
-3605,
-29,
4708,
262,
-2065,
3279,
1789,
-532,
5378,
397,
4498,
-1652,
-1306,
-284,
-772,
-2356,
323,
-158,
-3796,
-5436,
4984,
3444,
5246,
4784,
-5493,
-382,
-83,
-8239,
-3489,
1285,
7839,
1773,
-2772,
446,
651,
1652,
-1797,
-3299,
2295,
-5140,
1687,
8182,
-9296,
3750,
-3627,
-6418,
1322,
1048,
1362,
1403,
-1055,
688,
78,
2222,
330,
901,
-6436,
-3435,
4655,
-4594,
-3731,
2108,
170,
-13412,
-948,
1651,
3463,
2707,
8898,
-4464,
-2747,
-4362,
9612,
-2563,
-855,
-5191,
901,
-4370,
-3988,
9559,
-5706,
2450,
1691,
-3097,
-317,
-2834,
1388,
-348,
5677,
666,
-4611,
6099,
1674,
4466,
2821,
3751,
-386,
-1010,
-3429,
4976,
-7604,
1085,
-918,
13457,
-74,
2017,
-8628,
-1615,
-663,
4087,
5645,
597,
3278,
4920,
-2987,
-1941,
4641,
1845,
-5736,
5140,
-5576,
2794,
-9463,
-913,
2408,
4447,
-25,
4932,
1318,
-6323,
3114,
66,
-5129,
-6660,
8285,
-5787,
-1608,
12382,
-11935,
1783,
-68,
-4605,
-2923,
-7108,
-2347,
-2002,
3065,
1750,
-4814,
7441,
-2233,
4964,
1740,
6142,
7049,
-1830,
3586,
4300,
-421,
4580,
1050,
-3688,
2204,
-5385,
1412,
-840,
5146,
8659,
7859,
-6232,
4938,
-38,
1419,
-5198,
-924,
5341,
-3720,
7815,
-2982,
3364,
3955,
-1679,
1601,
-1976,
-7162,
4147,
3674,
-713,
8710,
726,
1232,
1921,
-2511,
-3609,
5159,
-3101,
2441,
-5456,
8187,
-2658,
-6399,
-3434,
-1476,
3480,
1347,
16719,
9223,
8430,
5361,
-6162,
1341,
2751,
-1076,
-2903,
890,
4227,
-657,
113,
6826,
-2051,
-825,
-1618,
5378,
-7704,
5711,
7589,
6270,
701,
4900,
-4632,
572,
2968,
-774,
-900,
-6388,
-3444,
-4906,
2296,
7856,
-6158,
7336,
-5999,
-341,
3515,
3380,
-622,
-1447,
553,
2638,
7518,
9368,
6995,
-6033,
-8341,
3613,
-2099,
-3267,
-1573,
3925,
-7953,
4885,
-4424,
5813,
-3908,
742,
-3287,
-10520,
-2191,
-8100,
-6925,
-7320,
6214,
4994,
-2835,
8090,
617,
1268,
7273,
3113,
-5195,
-2932,
-8557,
-6338,
-3362,
-2161,
3132,
-2481,
1658,
-853,
1051,
3689,
-1477,
-2432,
7307,
-4505,
-931,
4363,
-6027,
49,
1570,
2870,
9752,
-1101,
3878,
-7667,
-1212,
-715,
-5371,
525,
-8282,
732,
4104,
-2684,
4526,
2146,
5386,
3705,
-3118,
3562,
8840,
-1227,
-3202,
-862,
222,
-920,
-853,
-2711,
2168,
326,
4103,
6054,
-2889,
1889,
1804,
-1314,
-6544,
-2555,
931,
-3979,
1015,
-5540,
-5822,
1447,
-415,
1391,
-4302,
7573,
2875,
6525,
-321,
2755,
-2517,
-3099,
1717,
1520,
9055,
2429,
2479,
1521,
726,
750,
-2569,
-11770,
-6050,
-4143,
64,
4950,
-9605,
3318,
1167,
2455,
-6865,
90,
5247,
-4042,
5801,
-637,
5695,
-6249,
-8573,
7130,
5361,
-1156,
4993,
-4360,
-4887,
-2138,
-13009,
1053,
322,
1931,
10211,
-5020,
2466,
2233,
-2617,
1112,
2609,
-6193,
6078,
8638,
2674,
7490,
7516,
10389,
7906,
-3001,
-9235,
2362,
2311,
-2995,
1633,
1512,
-2611,
-8457,
2119,
-272,
-2175,
4203,
8236,
3650,
-576,
2244,
-6874,
3831,
-286,
3449,
-144,
-2499,
-6215,
-7329,
2373,
575,
-3215,
-3179,
-8279,
1610,
1361,
5306,
-8984,
-372,
-6147,
-12338,
9543,
-5453,
-4189,
-3323,
-4297,
-76,
7816,
-1795,
-7719,
-10262,
11119,
-581,
1682,
-773,
5503,
-1449,
2369,
-899,
-5447,
-3883,
615,
5072,
-8850,
411,
6658,
-3455,
3764,
-1484,
1861,
3119,
-12,
-5248,
-2502,
-3053,
-7032,
-1685,
12144,
477,
-1395,
-1569,
-698,
5223,
1873,
-1690,
-3626,
-1509,
-4048,
2990,
-2909,
2781,
-5467,
2352,
-6429,
4592,
-5196,
5396,
1880,
-289,
10323,
677,
-4265,
4019,
-5728,
-3918,
5230,
-4666,
2069,
7907,
-6137,
-596,
5728,
-2663,
1803,
4070,
1364,
4497,
3064,
-847,
6873,
-3305,
-617,
-1055,
-2652,
-9070,
697,
-1658,
600,
-757,
-3765,
-7691,
4852,
-3824,
2160,
1276,
-2486,
-3533,
9638,
7747,
4984,
1800,
-5148,
3884,
-1717,
-6702,
-8266,
-2037,
4152,
-3998,
6094,
-573,
-8037,
376,
-3109,
-955,
2606,
-1013,
1173,
-1982,
10979,
6886,
3155,
-269,
-7259,
-2970,
-2475,
-3779,
3466,
1922,
2035,
-4792,
-5092,
-4173,
764,
4250,
3930,
6601,
4974,
1557,
-5149,
-1668,
-5059,
-8702,
-4089,
5178,
-89,
5879,
3409,
9677,
-1492,
-4391,
-3980,
-4599,
3394,
12005,
8119,
4874,
7875,
-1971,
-858,
-2803,
-8282,
-2665,
-5059,
36,
35,
-483,
432,
-3894,
1819,
-5786,
2542,
2409,
-670,
-227,
805,
10,
-556,
-3500,
2187,
6260,
-1747,
9421,
3902,
8257,
3343,
-2654,
-4152,
5730,
-8820,
-9413,
-4672,
-7897,
5182,
326,
-8900,
-6271,
1239,
1888,
-832,
2819,
227,
-394,
-5723,
-1224,
5008,
-3981,
1459,
-6005,
-4836,
-1576,
2277,
3068,
683,
4499,
-3119,
-4542,
1118,
-1508,
2820,
625,
957,
1228,
-4840,
11753,
8051,
1863,
1413,
2579,
69,
127,
-890,
-824,
851,
-706,
-1564,
490,
2322,
3944,
4378,
-268,
4019,
-4697,
3158,
-1942,
10096,
320,
6289,
-2109,
2033,
1485,
-2285,
-2056,
181,
3746,
2,
4218,
-1570,
-1562,
7859,
-5917,
-4496,
-8196,
2383,
-894,
-2794,
7603,
-2142,
-6994,
2644,
5143,
-5134,
-748,
-3329,
1501,
4113,
2710,
11976,
628,
-5159,
3319,
-455,
4769,
-3556,
-3743,
463,
-223,
3299,
1835,
-2130,
6071,
-2992,
-3992,
-3001,
-4163,
4745,
2187,
1796,
5162,
-3089,
-5107,
-2298,
-5788,
-6359,
2084,
-986,
-1212,
-3488,
892,
4694,
1944,
2074,
5139,
8259,
2070,
-4573,
-487,
2113,
2492,
-4469,
-2502,
-4775,
5414,
-3975,
6710,
-4497,
-4638,
-1539,
327,
-6425,
-1775,
3718,
479,
-441,
-6838,
-4718,
-1408,
-3579,
3169,
1404,
8481,
2784,
3062,
-871,
3492,
-8534,
-2792,
2983,
5488,
-8905,
6066,
-5053,
-6374,
-799,
4214,
-9554,
-7531,
-3823,
2906,
-3198,
-278,
3427,
-6552,
4367,
2902,
-8680,
-2249,
3027,
-5257,
3335,
7733,
7473,
11118,
2002,
1634,
-252,
11639,
-1044,
3814,
5046,
-9205,
-7366,
-4747,
-1433,
-1775,
-787,
-3348,
-6935,
313,
6314,
4120,
-1720,
-6073,
-370,
-2231,
-3730,
9540,
-2148,
3952,
6329,
-1049,
4655,
-3205,
-1325,
-2171,
-13443,
6636,
-341,
9547,
5627,
1492,
1804,
1786,
270,
-190,
1296,
3454,
-3675,
-1379,
-5847,
-7670,
-2760,
-7414,
-2064,
-2861,
-2992,
-5273,
2978,
10443,
289,
-3427,
-5222,
784,
5270,
-6582,
4253,
2872,
-2449,
-283,
1002,
1167,
-695,
-5233,
1990,
594,
-8520,
4486,
-1311,
6499,
1494,
5119,
-2963,
-3817,
6270,
4519,
-124,
785,
-11699,
165,
-376,
2599,
-7994,
4367,
9500,
7049,
2732,
805,
-2728,
3478,
-3545,
-2535,
1359,
8241,
3708,
-1076,
-143,
-2851,
2686,
1090,
-3891,
2490,
-2882,
1,
4223,
5568,
-4958,
5376,
-107,
-4184,
2956,
1634,
-10811,
-2206,
-2097,
-9002,
-1891,
3347,
-3139,
-2343,
65,
6240,
-663,
-7812,
2264,
-2980,
877,
5735,
-2600,
-734,
269,
4710,
2671,
-5503,
980,
-1973,
-1442,
1889,
-2617,
-3937,
9192,
4983,
1035,
3104,
-8920,
-2445,
5149,
-46,
-3278,
6222,
6420,
-3694,
1413,
-12283,
1733,
-3590,
-2159,
6119,
445,
2899,
-964,
-8907,
854,
3477,
-159,
1481,
-816,
-2442,
-2175,
9932,
-153,
-1461,
-834,
1284,
4380,
-9923,
-3459,
1277,
1738,
-4387,
-3639,
-2608,
-1891,
-3458,
-5648,
-5744,
-6588,
-1924,
3081,
3014,
2276,
-6014,
-251,
4249,
-5901,
3228,
-3182,
1669,
-4239,
-113,
-6698,
-413,
4977,
-7117,
-1598,
-474,
2793,
-2450,
5032,
-6896,
-988,
-10183,
6851,
-7305,
4597,
8865,
8401,
2522,
-1065,
3836,
983,
-7073,
4484,
437,
-754,
-4378,
-4589,
369,
-1222,
-3159,
2644,
-7290,
-6141,
-160,
3430,
-555,
827,
-4122,
-3316,
-6470,
567,
473,
801,
2802,
5152,
11998,
-1108,
-5,
9500,
-2084,
15795,
4798,
5655,
-8261,
-3200,
-2240,
8221,
9307,
3895,
-8424,
-4803,
4235,
201,
-909,
-1119,
-9001,
-4605,
-2765,
-6084,
-3435,
608,
6821,
2313,
3996,
4398,
-1361,
-2461,
5494,
-7208,
-3410,
12076,
-8541,
-5285,
646,
-4705,
-876,
2741,
949,
-965,
1828,
-1190,
-4284,
1034,
-160,
-4896,
5600,
-3265,
-7035,
9194,
515,
-2587,
4494,
-3303,
3759,
95,
1142,
-824,
-3411,
61,
2207,
2475,
5105,
137,
4656,
1996,
3159,
-4280,
-2001,
4910,
-3628,
-312,
-1784,
6900,
-3592,
12696,
-6234,
3960,
-5573,
6321,
793,
4456,
2861,
-218,
4112,
-4640,
3085,
8926,
-7783,
-2105,
-3187,
-8842,
-1183,
-2677,
-4681,
-2814,
386,
-7792,
10660,
325,
974,
-2729,
-2781,
-3460,
6518,
5748,
-1194,
55,
-10882,
-2717,
-3996,
-4782,
6839,
6300,
3805,
1186,
1836,
-6575,
-916,
4254,
2290,
1828,
1660,
2544,
742,
1661,
2121,
-384,
1332,
-2754,
-2320,
8655,
4326,
1755,
4942,
2619,
-1696,
-1762,
1841,
435,
-3031,
-345,
-3232,
-2284,
-1843,
-4273,
-757,
264,
1728,
-2135,
-6171,
1449,
-1874,
-4566,
3895,
-7295,
4095,
-3281,
-2975,
-998,
-1248,
-4711,
-4621,
4983,
-466,
-1743,
593,
-1526,
2493,
-7374,
2596,
-610,
-2945,
2338,
4787,
-4742,
-3138,
1520,
2196,
3707,
4592,
1718,
-926,
1372,
-854,
1090,
-2388,
8961,
-3090,
2086,
659,
1538,
8080,
5768,
-5497,
335,
3411,
-1003,
-5041,
-7652,
3227,
-4596,
5891,
229,
4907,
-5006,
5897,
-666,
4457,
238,
-2382,
7435,
-6959,
10439,
-2612,
-427,
4698,
-148,
3055,
-2043,
-3171,
-1335,
-2422,
278,
8578,
6905,
3830,
6128,
7319,
-7712,
685,
821,
10403,
1822,
-11996,
-4384,
-5173,
8250,
-6819,
2930,
-2084,
7185,
5635,
-2166,
1112,
-1629,
9242,
-1321,
6991,
6489,
-7641,
2352,
1431,
4170,
-2672,
1264,
5581,
-5098,
6397,
-928,
149,
2065,
-3415,
56,
-5351,
-2262,
3634,
-411,
8451,
-5460,
-3583,
726,
9344,
3400,
3033,
1787,
-377,
-838,
3518,
7825,
416,
1243,
-9692,
548,
-6027,
1728,
5582,
-3687,
-5434,
83,
-1002,
922,
-928,
3948,
12709,
6415,
-15148,
-1780,
4178,
4770,
-7618,
6631,
1344,
5893,
2153,
3205,
2113,
9751,
5911,
5447,
2123,
2817,
-5091,
-2159,
1200,
3882,
-3451,
219,
-6139,
7134,
-2371,
-5871,
982,
-2435,
-4838,
-2219,
3674,
2488,
-2403,
2915,
1301,
-5462,
1085,
594,
-2675,
7462,
-5575,
-1544,
5727,
2815,
6072,
636,
106,
-4515,
-2989,
-3875,
-2218,
612,
-1966,
-5978,
-3212,
127,
2043,
-8698,
2655,
2592,
-2245,
3642,
1824,
-2660,
5127,
5328,
-1266,
1858,
-1233,
9177,
342,
4033,
-3060,
4348,
2226,
4625,
3004,
-4135,
-3104,
2979,
-4079,
1875,
5439,
-10484,
7817,
6050,
7578,
3759,
-642,
-554,
-1302,
-2662,
3111,
5562,
6041,
-474,
4744,
-570,
271,
-2331,
4749,
2652,
5789,
2127,
5145,
-2511,
-3871,
-11042,
5324,
-14,
1806,
-6941,
-2933,
3969,
-5277,
1906,
4674,
5280,
-6071,
-1258,
-1326,
222,
-637,
-3080,
-1195,
-4004,
-2985,
126,
3615,
472,
4446,
2267,
5693,
-3493,
7565,
-3929,
-9824,
3377,
4331,
8113,
65,
1405,
215,
5764,
6399,
960,
-6995,
-2957,
12169,
853,
2583,
-7846,
-7004,
-5499,
-1411,
-307,
7641,
5334,
-967,
1967,
-2128,
-607,
-4563,
5408,
-1315,
3325,
-661,
4850,
1764,
-8,
-8501,
-3251,
753,
-960,
-677,
-3093,
6503,
3185,
4931,
-1875,
2871,
3177,
7209,
-6521,
3170,
1820,
1422,
1603,
-3266,
91,
496,
-6256,
1580,
6420,
-4449,
-5598,
-2604,
-3382,
-4874,
-48,
-2616,
2607,
7311,
-1731,
4803,
-4766,
-2027,
6637,
6183,
-5517,
2572,
2971,
-1713,
-5139,
1674,
1482,
2604,
1994,
7819,
-3286,
3976,
1026,
1326,
429,
1786,
-5774,
1583,
70,
-5645,
-1169,
4006,
2038,
3837,
-4653,
690,
1126,
65,
5899,
2222,
4064,
-1847,
3145,
2968,
-3497,
7091,
-252,
5311,
-995,
-2997,
4348,
2483,
-6006,
623,
3819,
8542,
1681,
4118,
1439,
-3764,
-5539,
-4107,
-4379,
5762,
-6126,
-12489,
2972,
10815,
-2454,
-3237,
-538,
-1299,
-5727,
4639,
-7647,
-4603,
3957,
-4234,
298,
2479,
730,
-4463,
-4313,
-1368,
3644,
5221,
-5738,
-1567,
3482,
-4670,
-267,
163,
1671,
-96,
-8256,
6953,
3829,
-2680,
7620,
4486,
3111,
-5413,
-2863,
-2501,
1436,
-2826,
2545,
4826,
2296,
-6108,
-9390,
4742,
-3316,
-1268,
-3571,
-1961,
3298,
1675,
-9106,
-5283,
10954,
-4545,
5811,
-7233,
4811,
2105,
-3525,
1374,
-55,
2178,
522,
-5269,
2469,
6486,
-11026,
-3930,
7256,
-2761,
103,
2403,
-1172,
6162,
4869,
-4538,
2190,
4091,
1604,
-3305,
3313,
30,
4369,
6791,
-1495,
-3264,
1228,
6525,
3488,
-3894,
-5747,
-45,
3047,
1663,
-4013,
7208,
1998,
5496,
-4240,
-2602,
615,
-6474,
-3058,
7292,
-1664,
-2950,
-3354,
-1825,
13434,
-6449,
-5362,
3028,
2119,
3029,
3515,
2094,
-3199,
-6645,
5725,
-1251,
3281,
1749,
-2640,
9366,
586,
3921,
2026,
4976,
-6489,
-3510,
227,
-1093,
622,
-4574,
498,
-2745,
-537,
-9768,
434,
-12,
6711,
2894,
-6290,
7610,
1458,
-3318,
-4345,
-5891,
-4907,
-7999,
11633,
1884,
-4758,
1868,
-3740,
10062,
-6235,
-357,
465,
18,
6905,
-4365,
-2132,
453,
4987,
-3156,
2452,
-9186,
-4982,
6704,
3175,
1256,
-2969,
-1545,
-8016,
-5756,
-1095,
-3161,
7771,
239,
964,
4873,
-7356,
-1005,
7683,
-10131,
-8998,
859,
-3904,
722,
-4085,
3570,
-3831,
-5659,
-624,
-5104,
-676,
-2319,
4361,
1001,
-2486,
2182,
1567,
-5995,
-2377,
2109,
-5477,
5816,
6066,
804,
9675,
-6139,
1361,
-7312,
-6916,
853,
938,
-6133,
4518,
3419,
9246,
1309,
-6145,
-311,
458,
-5697,
910,
6000,
5539,
25,
622,
-369,
-1053,
-417,
270,
591,
4021,
3758,
-10057,
-18821,
3212,
937,
-1557,
-2360,
4500,
634,
-2832,
-1736,
-8760,
4403,
-4186,
3478,
3748,
6514,
1925,
2771,
8379,
3017,
-4245,
-4898,
1823,
289,
-2168,
2792,
-138,
-11308,
-841,
7040,
-164,
1845,
6581,
-5126,
-1514,
8098,
-4434,
7502,
-86,
-363,
3091,
7706,
4320,
-2897,
3804,
-2439,
-6501,
-3642,
3594,
-6686,
2507,
217,
-2377,
-34,
-1611,
20,
-1946,
1903,
-1275,
-10464,
-207,
-8339,
3785,
-1410,
10782,
-166,
2238,
-659,
-5853,
-10070,
-6463,
-6234,
-565,
3200,
-663,
-9078,
-7881,
4279,
-3148,
910,
-2701,
-1458,
3988,
-2446,
-2014,
-1779,
-3996,
5548,
-178,
4042,
-9088,
-1341,
-700,
946,
2886,
-5887,
-2084,
8544,
-311,
-4139,
3022,
-233,
6698,
4383,
-4335,
4152,
-486,
495,
5561,
-2680,
-1886,
178,
2171,
8337,
7443,
3528,
2868,
-2557,
-4549,
4935,
3200,
2652,
-4980,
3829,
2993,
2384,
537,
-6470,
160,
-4626,
-4216,
7017,
-2163,
-1232,
419,
-8262,
1285,
2127,
3239,
446,
-3849,
-4483,
10267,
-7830,
-8383,
-2035,
1342,
-5486,
8713,
2568,
1500,
-1659,
2296,
-4161,
3371,
3170,
2065,
3052,
2854,
5607,
7955,
-421,
2875,
2210,
2986,
-3536,
14157,
8984,
-1731,
-4290,
-1804,
-56,
-3608,
3540,
3797,
3897,
-5807,
1196,
-4504,
-6269,
6413,
60,
-134,
-6679,
3970,
4055,
1096,
8256,
2175,
-5474,
811,
-5573,
-4822,
5362,
-3132,
10670,
-8817,
4383,
-674,
-1192,
6380,
-5128,
6633,
198,
722,
-1415,
-18,
15669,
-5448,
5208,
-7874,
-1207,
-157,
2304,
-295,
-5347,
-5249,
5154,
4165,
7583,
-679,
-522,
-864,
1412,
-9042,
1721,
-7449,
-2168,
-3854,
-268,
5121,
-1927,
7283,
5718,
-11624,
-6466,
-2168,
3519,
155,
-3399,
5008,
-5647,
3032,
-8654,
-4658,
-2070,
-10571,
-432,
-7240,
-1779,
-5275,
4511,
-1389,
3969,
1772,
5492,
-8283,
-2635,
-2254,
-3032,
8344,
-430,
-3460,
-4987,
-1645,
1465,
-6252,
-1489,
-1874,
-1418,
1408,
-5159,
-479,
-710,
-1240,
-2718,
5655,
952,
-6162,
-192,
967,
-3864,
-856,
-1741,
7299,
517,
6293,
1793,
-5262,
-262,
-3297,
-1211,
-6679,
2769,
-10,
3115,
-575,
-1086,
1508,
4367,
8352,
-5892,
6607,
3049,
-5314,
3609,
-886,
3598,
4570,
-1491,
332,
453,
3364,
9155,
-5042,
-4631,
-2013,
-136,
790,
-2337,
-364,
-2202,
4215,
-2504,
3417,
1890,
-759,
-1810,
-214,
1218,
-2324,
1722,
-260,
2307,
-3288,
-3665,
-3925,
2389,
3637,
-3532,
-1469,
4833,
-2412,
-1331,
4666,
-3159,
3451,
1382,
-735,
1801,
6449,
-817,
9061,
6999,
-5369,
6148,
-7229,
-1606,
-2315,
-129,
-1563,
1717,
-2191,
-4963,
-48,
341,
-8980,
1457,
2115,
-6148,
1579,
2833,
2360,
1794,
-12325,
6898,
1961,
-2210,
-2007,
-2672,
2050,
-20,
3234,
4683,
-8906,
-1763,
-857,
4560,
-4906,
-6874,
3862,
-3355,
13704,
-158,
-644,
-7020,
3927,
2089,
1171,
2190,
6397,
1054,
-3275,
-7865,
-4862,
-5034,
-1379,
1630,
3063,
2711,
-2265,
3292,
-3616,
-6850,
2354,
-6821,
-4583,
-3276,
11077,
-5340,
-8223,
928,
-576,
1530,
87,
-2776,
4695,
1733,
7710,
-153,
-5081,
3976,
-5837,
-7670,
3778,
7035,
-4100,
5056,
-3271,
-5217,
309,
-3408,
6710,
-3651,
4478,
7475,
4491,
5500,
5167,
-1559,
-6740,
775,
4156,
-5683,
117,
-332,
1822,
2785,
5255,
-11197,
-4279,
-7014,
-427,
-754,
-30,
5328,
6153,
2943,
-4998,
-2193,
-3059,
-2233,
4094,
-3559,
-876,
-1637,
891,
-307,
-4009,
7274,
1202,
1229,
-7320,
-10057,
-2027,
-244,
-1096,
9669,
-6418,
-3815,
5078,
4352,
-2254,
3053,
5398,
72,
-596,
7772,
-423,
6193,
-3311,
-67,
163,
844,
4868,
-5728,
13794,
399,
-1826,
2692,
-1486,
-6757,
-1194,
1229,
1954,
-3227,
-5054,
-4717,
-455,
1639,
-484,
-1866,
566,
4972,
-3461,
-410,
1898,
3959,
-2732,
2353,
-2653,
4936,
-4709,
165,
-2250,
1944,
-6467,
-2732,
-2068,
-3268,
-2192,
-1705,
-5449,
4909,
347,
-2170,
-3164,
2058,
-4876,
-8320,
7481,
498,
-4552,
-4849,
-9052,
440,
-2869,
779,
2175,
-839,
7369,
5586,
-3456,
-3146,
317,
1841,
15437,
-1089,
1201,
3287,
783,
-1598,
7291,
306,
-2272,
3747,
5976,
1227,
7200,
1748,
-9163,
-564,
-2513,
-2037,
2138,
-768,
71,
-3637,
-6043,
532,
-5040,
1964,
5226,
-7748,
7330,
5612,
-352,
-670,
14317,
5446,
-4313,
7543,
310,
49,
-510,
1947,
-7902,
-4471,
583,
-5051,
-292,
3807,
-1856,
-4204,
5680,
-4819,
-414,
6711,
-391,
1090,
458,
-3060,
3581,
4243,
2643,
-1192,
-3020,
-2270,
-4539,
6063,
-160,
-3000,
3482,
6109,
88,
-3521,
-1352,
2255,
5491,
2492,
3334,
2567,
2098,
3765,
7058,
-3397,
-862,
-3914,
-2607,
8307,
4366,
-4986,
-7419,
871,
652,
-1639,
-1239,
3724,
4983,
-2791,
-3887,
1428,
-495,
-6876,
-2991,
-205,
-1199,
2362,
-6872,
5447,
5633,
-4052,
2466,
-2859,
-10879,
-105,
-4410,
5467,
4403,
-1484,
12280,
-5307,
-1174,
5936,
2915,
2195,
-3669,
-4241,
-8400,
5855,
-1599,
-4011,
-1048,
-5808,
9090,
9003,
-7317,
-809,
-1477,
-452,
-868,
-1414,
-3544,
-5396,
4661,
-600,
4200,
-2920,
2516,
5951,
-10915,
682,
-1942,
5122,
1667,
-744,
6457,
-4202,
2560,
-7247,
5856,
5058,
1328,
-1068,
-7059,
-10596,
2326,
-5356,
-9997,
-2022,
551,
1369,
-4512,
1304,
-879,
3627,
-5289,
-2006,
763,
3370,
10449,
1475,
-3075,
-39,
8564,
-4958,
-1912,
1278,
-5775,
1425,
-7204,
3104,
5116,
1501,
2534,
-2364,
-2010,
1555,
-4735,
-6946,
-9839,
-2477,
-2814,
2710,
7428,
3276,
1498,
4559,
-1760,
4068,
-766,
7027,
1241,
-8259,
5774,
-2629,
2421,
1457,
236,
6063,
7257,
3305,
-3638,
9414,
-13559,
2973,
2325,
-2128,
-5904,
-3081,
1055,
-1151,
5079,
-2344,
5610,
-2288,
9881,
-603,
4540,
-4298,
3531,
7068,
408,
2422,
542,
-2323,
4855,
378,
3019,
1270,
6127,
1962,
3424,
6506,
-4662,
-3687,
-5115,
-5242,
288,
903,
-5898,
2505,
470,
-1115,
-4057,
2700,
-1433,
-2932,
3399,
-2725,
-7893,
-1486,
-1762,
-515,
1482,
2173,
-3264,
7035,
-5235,
-131,
-12314,
1839,
3071,
14747,
-2227,
-1255,
1934,
-4874,
-1798,
-6136,
2277,
7299,
-2026,
-910,
3515,
-344,
-14686,
-1393,
-12232,
-5373,
6219,
6430,
3915,
-2447,
-1707,
-1748,
-8366,
5745,
-376,
1005,
-598,
864,
1192,
-270,
8603,
88,
3318,
-2708,
-3384,
-9051,
4138,
-4149,
1689,
-2326,
-6738,
2209,
1102,
-16426,
1171,
-6346,
67,
-2805,
-1224,
-2400,
1536,
166,
-2597,
1364,
5740,
-83,
4,
-7839,
-642,
-6519,
4264,
4885,
-674,
-6569,
1590,
2005,
3271,
-618,
-8464,
-8822,
10341,
-971,
9939,
-4268,
3787,
-1865,
-6606,
-686,
707,
-1263,
-15,
-10528,
2782,
554,
-3440,
8416,
1845,
1466,
-1075,
-1592,
9919,
6154,
2890,
-3253,
-1639,
3867,
5321,
-6848,
7431,
-5271,
152,
-10160,
-3307,
5969,
-197,
-10254,
4765,
-5251,
336,
-9029,
-5031,
8014,
913,
4644,
6425,
-7396,
7112,
2935,
3302,
-1736,
281,
1793,
5721,
-8790,
-183,
5482,
-5345,
-7537,
-502,
-343,
-2799,
-2379,
-4325,
-1899,
4039,
3745,
925,
-1775,
1918,
-1552,
-169,
-2445,
6294,
-4083,
5359,
-4168,
-1004,
-787,
-1587,
-2040,
-3009,
2990,
-2440,
-1704,
-4434,
-4212,
965,
6687,
3056,
-883,
4062,
7270,
-4570,
2357,
-1465,
374,
-2205,
-8873,
-5329,
-2267,
183,
9301,
-6477,
4374,
-11437,
6083,
370,
4860,
-1775,
530,
-5746,
-6127,
10823,
-3762,
-10283,
-7473,
-3750,
-284,
-4910,
2217,
-281,
-3556,
-2064,
3877,
-12330,
3578,
2448,
954,
2520,
-647,
-3260,
132,
-6627,
-2313,
2608,
-5094,
-3574,
5326,
2997,
-2841,
1584,
244,
2714,
-531,
3176,
6605,
-1368,
-1254,
316,
-2676,
-10658,
348,
5468,
-7868,
7444,
-1602,
-616,
-5693,
-2789,
-4001,
6349,
-5166,
-3637,
-287,
-2754,
-831,
3946,
194,
5351,
-4552,
-7298,
-1647,
3668,
-1552,
-2410,
-527,
2126,
-7941,
-7873,
-1392,
-5447,
2820,
7358,
-2591,
-6047,
-2474,
-2703,
-1008,
2567,
-3439,
4885,
906,
4702,
-2531,
285,
4616,
-1644,
5736,
-1833,
7874,
530,
-5988,
911,
160,
-3661,
-2469,
1104,
1641,
-846,
9387,
394,
126,
4021,
8319,
-5214,
1492,
10172,
3145,
11966,
3812,
-3957,
-505,
-10253,
-2361,
-2038,
-1714,
-2673,
-1703,
-1991,
-3372,
-7984,
9129,
804,
-7136,
-1453,
8614,
6587,
-4523,
3759,
-1442,
-9579,
-2797,
-2710,
1667,
257,
-637,
-6949,
-6825,
-3293,
3606,
-1058,
494,
5714,
-3743,
-957,
4395,
-2794,
-3696,
5237,
5348,
-5225,
-6913,
-5025,
5918,
-447,
2818,
5573,
5023,
-2471,
555,
3769,
-6192,
6533,
-3332,
52,
-7637,
2049,
2179,
4927,
1355,
4027,
-365,
-3907,
-1924,
-1752,
-833,
3361,
-1834,
-2766,
-2980,
-2273,
-7737,
6481,
3804,
-284,
-4258,
1004,
-1857,
-1569,
-519,
272,
-2902,
3350,
184,
5312,
2422,
-406,
3149,
4704,
2934,
5175,
-100,
-7347,
1830,
4426,
1888,
-1696,
-3093,
682,
-999,
3911,
8454,
-1945,
-8665,
-2548,
-6524,
-719,
987,
3877,
4287,
761,
4030,
5063,
-223,
2025,
4508,
-130,
5489,
-1932,
-1648,
6495,
3045,
-4649,
-4780,
-2568,
6573,
-4615,
7670,
-3198,
-2893,
2571,
613,
2155,
-1721,
-3836,
-323,
270,
3387,
-513,
-1774,
-10699,
7730,
1802,
2627,
5374,
73,
-1319,
-9383,
-5130,
-3222,
-1176,
6506,
6527,
-2737,
1319,
-462,
-382,
-7817,
-589,
-4113,
562,
5325,
-597,
-4046,
-1472,
1046,
-2228,
-3886,
-4674,
-2701,
-3630,
-12290,
5896,
7099,
3488,
3008,
-5561,
1326,
8049,
7291,
4475,
4217,
-11529,
-2865,
6374,
6313,
-5461,
-3177,
1096,
-7664,
-2180,
-14255,
-1246,
1821,
2465,
2707,
-6742,
-7106,
5024,
6974,
4229,
2768,
-1759,
1013,
-13652,
3632,
-2277,
-7963,
2242,
-6874,
3504,
6180,
-316,
-7829,
-8386,
-2619,
3669,
4347,
254,
3690,
1526,
3540,
-43,
-9848,
-7029,
4572,
3600,
-37,
-5126,
-1497,
-2088,
-415,
-4035,
-6537,
4224,
2866,
2498,
1054,
-7674,
1348,
-2913,
3116,
-1216,
-1301,
-2310,
3291,
-10672,
3281,
4444,
-80,
580,
3447,
900,
3818,
3991,
-7017,
7724,
2516,
-1276,
1396,
1630,
-4359,
2526,
-2702,
-6286,
4587,
3580,
-3502,
3642,
-1236,
-7285,
6330,
-187,
9926,
-6330,
8334,
-7391,
-7648,
2623,
1540,
6771,
1809,
619,
6762,
2303,
-1925,
-2770,
1392,
-949,
-4729,
4833,
-792,
9445,
-4760,
2690,
-10594,
573,
3777,
5820,
8561,
-385,
-8828,
3958,
-4153,
-772,
2228,
-4254,
-3372,
-6464,
9733,
-11497,
-595,
-919,
-4349,
-6125,
-2951,
-4531,
-3884,
10493,
-5712,
-1117,
2110,
-868,
1996,
4594,
22,
7313,
310,
-11724,
4900,
404,
-306,
6936,
5199,
-3938,
2075,
2201,
-4560,
-2775,
-5229,
6559,
-3253,
-3168,
-6744,
2372,
2212,
-4830,
-1326,
4316,
-6795,
1848,
4021,
-175,
4985,
-4317,
1007,
6981,
-1006,
-4763,
-748,
-4252,
1861,
22,
-6894,
689,
-4036,
72,
1304,
2881,
-3980,
-614,
390,
1216,
-3583,
-1964,
3407,
-840,
1644,
-3451,
-1631,
-9238,
-2747,
1599,
2427,
-256,
4573,
-4915,
3837,
8139,
3399,
808,
-1841,
-1529,
805,
1277,
808,
-2363,
-11094,
4955,
-4773,
5171,
-2153,
-3097,
-2700,
-11005,
8476,
-1969,
-10862,
2575,
-8383,
-220,
-2323,
-1538,
-5087,
2871,
2265,
-2003,
10531,
2141,
-1966,
2823,
4817,
2990,
-5351,
-1598,
-1907,
-1618,
8192,
4810,
5513,
-7902,
5010,
-3108,
-804,
1306,
-762,
-1059,
3954,
1746,
7465,
-8517,
2257,
585,
-1105,
-2084,
735,
-1182,
3044,
1656,
-1496,
-1099,
765,
5894,
-2404,
-1291,
3774,
2885,
-236,
5674,
-4152,
-1571,
1751,
-2351,
4338,
1257,
-40,
3210,
8751,
-231,
-904,
2567,
2889,
186,
-7633,
-1419,
4773,
-1253,
2087,
-7086,
2253,
-1142,
-2605,
1214,
2189,
1464,
5,
0,
1555,
10428,
6375,
-361,
2676,
5904,
8551,
-6622,
-5117,
6933,
-1737,
-6673,
1224,
-770,
-13618,
4316,
6926,
3857,
3610,
-8297,
-9353,
1340,
-2353,
-583,
-621,
4313,
-6561,
4849,
12062,
-4778,
4831,
-10563,
2704,
14240,
2811,
-1970,
4240,
-2644,
-2214,
2652,
5197,
11561,
398,
2943,
981,
-15418,
2473,
8575,
-5257,
4595,
2898,
8378,
1043,
739,
611,
-2592,
1398,
8411,
-4839,
-2466,
-1653,
18070,
-6544,
13450,
1948,
-3132,
12090,
-3535,
3031,
-496,
-8812,
-4025,
573,
-1901,
-6914,
-11765,
-2743,
-4725,
11964,
1037,
916,
-6256,
-5819,
543,
-1701,
10800,
2349,
3278,
8301,
-2107,
-7910,
4159,
-3377,
-6460,
-4649,
-622,
7789,
-3144,
-6313,
10317,
-3291,
7026,
-3687,
1188,
-10885,
4633,
752,
4500,
-5558,
9991,
-4095,
-11324,
8275,
5261,
2179,
-4329,
-4527,
5501,
-14993,
10730,
-953,
3206,
6151,
2664,
-5650,
4453,
3276,
2924,
3612,
3450,
-1939,
10064,
1286,
-9449,
-5476,
-1700,
-8083,
-5589,
1085,
2851,
-739,
1877,
-2533,
-523,
1308,
-5165,
-10825,
2796,
-1550,
4006,
-1128,
-12653,
-3707,
-3661,
4327,
-2898,
-19743,
2541,
-5786,
11443,
3463,
4028,
225,
-1408,
9305,
-765,
-7918,
-7123,
4280,
-3711,
-15508,
10769,
-3529,
11160,
-15314,
-3473,
7006,
-3524,
4712,
-10927,
5468,
-2562,
5283,
3083,
669,
7676,
-1000,
-6285,
4616,
-7653,
1325,
-1733,
333,
-10711,
-9380,
-9613,
509,
6254,
4332,
712,
3815,
8991,
-3064,
116,
-11460,
8061,
-1035,
6399,
1078,
-369,
6028,
-13264,
-585,
-4380,
5058,
-8219,
-8379,
4192,
4444,
10800,
-19406,
-18169,
13556,
919,
-2058,
-10031,
-4800,
1551,
-6468,
9571,
-6384,
9857,
-8223,
10180,
-428,
-1679,
3018,
-11125,
13038,
11219,
3658,
-6479,
-8199,
7313,
413,
-5561,
2557,
-1991,
-333,
-4653,
-6336,
10755,
12446,
4041,
9145,
-11425,
1965,
-5089,
349,
3265,
4947,
8120,
3158,
1668,
845,
1446,
4182,
9817,
3203,
7365,
-1157,
-2331,
-8474,
-12125,
-7812,
-1890,
-7823,
3405,
1129,
4182,
-9988,
14479,
-3990,
3824,
7168,
-10022,
-14055,
1975,
10020,
-4623,
2757,
-4882,
4015,
8229,
848,
5602,
-16294,
-177,
-1959,
-1932,
-7135,
525,
-7850,
-9020,
-9229,
2582,
-14,
-5466,
8729,
12212,
-2109,
9849,
-4267,
-575,
2402,
13966,
-914,
3971,
1693,
5245,
-477,
3017,
-8695,
-7071,
10002,
-7064,
768,
8676,
-1294,
2102,
1892,
5365,
-5783,
8556,
-5607,
3439,
7989,
-3080,
-2785,
-3842,
-5995,
5460,
-1982,
-2453,
-8789,
-5061,
11995,
663,
-10973,
7220,
5817,
-10285,
-1824,
11250,
-11210,
-7413,
-5695,
2933,
3739,
-2597,
6444,
-8280,
6962,
-18649,
-2112,
-8784,
5180,
2366,
5680,
-9980,
-9520,
-593,
11665,
234,
1323,
-9194,
8046,
509,
1904,
-4594,
8092,
8432,
7986,
-3257,
-4320,
-11041,
1864,
462,
1158,
-7949,
-8976,
5474,
-7306,
11788,
25386,
9531,
-1055,
-2353,
-888,
11570,
-845,
-2911,
-4159,
-2257,
-7260,
-7249,
630,
-448,
968,
-1667,
7700,
5591,
2228,
9898,
2197,
-6349,
-3353,
-5339,
-2839,
-6148,
-6352,
454,
7381,
4623,
1188,
-5656,
-2865,
6373,
1669,
116,
3155,
-11682,
-1590,
-5241,
-1615,
7126,
6975,
-10804,
-11135,
3400,
1832,
6713,
-3881,
-3354,
-11948,
-3958,
-375,
-7547,
1519,
-8031,
-6890,
-2305,
-3288,
-10003,
4944,
-1141,
-14108,
10071,
-4291,
25,
-8801,
4422,
-13620,
-291,
-9777,
-6734,
10150,
6641,
4340,
166,
269,
-7340,
-9534,
3309,
-6610,
4944,
4260,
3047,
4129,
-14341,
10303,
-6350,
1339,
-5531,
-4334,
-2536,
1890,
-12478,
2598,
-10342,
-3160,
3800,
1119,
-4977,
-201,
-2530,
-3655,
2451,
-2918,
-1012,
7368,
14951,
-2134,
3948,
-11407,
-2167,
810,
-10744,
335,
7634,
11862,
-527,
-103,
-9704,
173,
-5934,
8316,
-2528,
8691,
-9202,
4261,
-8210,
3224,
7985,
11340,
-2378,
2203,
-5019,
5198,
-1920,
1138,
1961,
-1842,
629,
3002,
-1435,
9898,
15008,
-3121,
2321,
-4669,
3475,
-8773,
-6468,
-3869,
291,
5066,
1447,
3640,
-299,
2182,
1330,
3521,
-5086,
1228,
-1634,
4826,
-8411,
-7844,
-1875,
1599,
-7826,
1753,
-11124,
-11325,
10796,
7791,
2347,
-377,
-11096,
12944,
7244,
-1357,
-4356,
199,
169,
8200,
7145,
6572,
-4589,
-5417,
-5441,
-692,
8901,
-618,
2570,
-7806,
17152,
-1762,
-11645,
2529,
-3152,
2333,
-6997,
-5113,
-7979,
-4660,
13714,
-5202,
3925,
-5642,
-4744,
-1454,
2126,
-2425,
-6190,
6121,
2738,
11715,
-1285,
8483,
6316,
-2920,
130,
9350,
3081,
925,
-11776,
6532,
-4728,
-1855,
-6408,
-6519,
-3531,
-11637,
-4092,
-5349,
1225,
-2113,
-2286,
9404,
-5173,
8606,
-5691,
405,
3850,
-9888,
11808,
452,
-1555,
7194,
-896,
-6470,
8836,
-4726,
1930,
-3147,
665,
-1928,
-6565,
-142,
1921,
11165,
-2515,
10261,
-3941,
-121,
1966,
-13039,
1863,
11532,
10857,
7814,
552,
-3235,
2503,
-1896,
5630,
3864,
5887,
1222,
7024,
-4128,
7344,
3285,
-7088,
-8496,
-12831,
-16717,
4381,
-8505,
4018,
587,
2416,
-3563,
-1438,
5346,
-1230,
-6216,
-5758,
-6690,
8825,
2184,
1633,
5531,
5601,
7589,
-4165,
-4625,
2695,
-6806,
-6529,
-13608,
-11900,
9109,
-859,
-694,
15278,
1800,
1170,
3978,
-1954,
-1055,
10095,
2500,
6093,
-6743,
-14151,
8085,
4427,
2490,
-5018,
2679,
4328,
-3904,
10229,
4601,
-9025,
-518,
-7667,
3660,
-14961,
8898,
1230,
4214,
-2074,
-3135,
4520,
5534,
532,
5812,
2608,
9010,
3486,
3821,
-4400,
8406,
12079,
1576,
-4360,
-9328,
442,
408,
-1027,
-3079,
2424,
1696,
6664,
-10194,
-3928,
-4757,
-4438,
3359,
16958,
-5015,
-7946,
-4470,
-8730,
-6201,
4527,
3666,
1108,
3320,
9027,
7668,
892,
-7251,
3035,
4156,
-12252,
7705,
-6637,
-3676,
12283,
3506,
142,
-3905,
-2482,
2216,
3437,
13937,
-5897,
4887,
-8167,
1621,
-9265,
14515,
-10202,
5993,
14243,
-3728,
1172,
-10079,
-2481,
-4792,
-1514,
-347,
708,
-2902,
4336,
-3792,
8472,
8714,
1228,
4226,
-2190,
-7612,
15224,
5485,
14487,
277,
-10744,
3907,
-3359,
2643,
3506,
-166,
2367,
3072,
11230,
-14762,
136,
6522,
540,
6228,
2803,
5859,
-5176,
3883,
6530,
5882,
-5332,
-2364,
-1865,
1288,
-2231,
4582,
6199,
4572,
-106,
3014,
326,
-3686,
-1445,
-11115,
5932,
-1402,
2841,
6476,
2161,
-6516,
-4828,
9865,
-635,
7282,
-4746,
11081,
2824,
-3356,
10607,
-3347,
-3460,
4046,
-18323,
3283,
-3695,
-13409,
9707,
-179,
-2951,
14552,
-551,
1468,
-2735,
2608,
3380,
-10605,
-2713,
4185,
-2904,
7248,
12260,
-4012,
-3791,
-6335,
12709,
-1121,
10031,
4865,
8025,
2416,
-11796,
1607,
868,
14449,
-3435,
-9995,
9114,
-7120,
-1154,
-9304,
-507,
-4926,
5504,
13027,
-3656,
5439,
1199,
804,
3364,
2712,
3919,
2405,
3927,
1762,
-3283,
-1478,
-1408,
9739,
7678,
9335,
-6394,
4610,
-9915,
-5130,
-6821,
-17223,
-5453,
-782,
-297,
1461,
-1067,
3603,
13275,
-1067,
2265,
-1874,
-6265,
1287,
-11118,
1311,
-1546,
-6104,
5658,
9356,
431,
10490,
-6028,
2986,
-1327,
-10909,
-5848,
1912,
-2705,
4122,
8724,
13744,
4373,
-5182,
-6100,
-3674,
-2507,
2284,
7223,
8411,
2584,
-9750,
3448,
3400,
-5124,
-4538,
-3400,
2427,
6198,
-8300,
-350,
-65,
321,
-1343,
-2794,
-5702,
-11267,
-6917,
-6312,
-1803,
1289,
-8381,
-2554,
-4114,
-3497,
-2913,
2282,
-8605,
7518,
-7696,
-10932,
15272,
13835,
-2745,
-17888,
4755,
-4675,
-3592,
6105,
2447,
6680,
844,
8018,
-3962,
-8538,
-5265,
-15288,
-1251,
-258,
-8731,
-4628,
-3980,
-6075,
3085,
2982,
-10321,
-3864,
12191,
-10984,
215,
7228,
-12899,
1681,
4399,
-10196,
3643,
6510,
-808,
872,
7478,
4562,
-4248,
-6340,
5178,
12793,
-11109,
3597,
-9125,
4782,
-9343,
1763,
800,
2832,
-2592,
4290,
-8343,
1840,
6054,
-1040,
8443,
-6508,
-9528,
-8957,
2978,
-8081,
6754,
3667,
-5771,
-930,
-5187,
-2966,
-8846,
-2857,
-128,
-9553,
-7524,
-1631,
226,
3714,
9782,
5438,
30,
-7783,
-3179,
-2300,
-2398,
3461,
12750,
-9770,
-3795,
-8410,
1089,
1468,
5139,
-1732,
-3091,
3301,
-4800,
-3380,
-2324,
7817,
1634,
10977,
-1005,
-58,
5386,
8080,
1616,
4492,
-7012,
-864,
5684,
6960,
-8753,
6890,
-921,
-12061,
9877,
4545,
462,
7086,
2221,
-7014,
1362,
-9094,
1590,
5101,
2853,
-5079,
-4804,
-4981,
-4005,
13812,
-5971,
-1557,
4227,
-2663,
-8292,
-5848,
-4384,
12023,
-2515,
-3927,
667,
1901,
5662,
7830,
1828,
2101,
-1846,
-2673,
3192,
-1659,
-1811,
-5091,
1743,
12766,
-380,
-13363,
-5828,
4159,
2630,
12685,
7202,
4206,
7194,
4499,
-3411,
4865,
5858,
1705,
7055,
-4560,
-12706,
-11372,
-2327,
-14427,
433,
-4599,
9560,
523,
5559,
3493,
9088,
5076,
-15691,
892,
-7373,
-2003,
8066,
-10489,
10300,
5530,
15049,
1976,
7305,
-9596,
1750,
-8695,
2209,
-737,
2677,
5083,
-1877,
2467,
-13600,
1848,
-5542,
-6053,
1287,
9260,
4420,
-723,
-177,
-831,
-4259,
-61,
17332,
5961,
2576,
4965,
4172,
714,
1963,
7599,
-15206,
-5713,
5050,
6129,
-12987,
17211,
6934,
-3443,
6746,
-1386,
-3822,
13451,
7691,
-6433,
-1963,
-1780,
-6107,
-1613,
1047,
2996,
3120,
-7543,
2643,
4330,
-5623,
-1695,
419,
-1839,
-771,
-10273,
-5964,
-7369,
5271,
8555,
3534,
3526,
-17072,
3789,
3413,
-3131,
7309,
-7737,
-991,
-725,
11111,
1581,
4578,
-3365,
-4969,
242,
4757,
6,
-2171,
6816,
-5979,
10992,
10343,
1338,
-5592,
4905,
-7595,
-1291,
-4930,
-3037,
-1287,
3144,
-3644,
-4063,
-673,
12807,
-7136,
-11751,
1417,
1339,
-2435,
-2193,
-9428,
4211,
-1916,
2103,
-11180,
6141,
-352,
1563,
1141,
4279,
5383,
3085,
-3201,
-4700,
7188,
3570,
-5522,
-2930,
8448,
-7039,
-3855,
-14989,
2073,
457,
7421,
6697,
-3962,
5463,
-664,
-1391,
-14398,
8239,
2372,
5760,
12086,
-1199,
544,
-11736,
-3593,
10195,
16405,
8344,
6953,
-7487,
-170,
-1437,
13819,
-3281,
-970,
7467,
1324,
2078,
6796,
-1974,
3550,
-178,
2534,
-4169,
-7121,
-6096,
3833,
-2310,
13553,
-12617,
-717,
2759,
-3385,
216,
-673,
5061,
-2284,
-6075,
-8628,
9176,
-5882,
-6965,
-142,
10292,
14782,
2389,
5838,
3341,
-1099,
2093,
1687,
-4287,
-6180,
7498,
4307,
13198,
335,
2872,
-6993,
2934,
-8985,
11100,
8240,
14000,
-11055,
1556,
-3919,
-7470,
-3209,
9382,
-12537,
275,
5453,
2549,
263,
6791,
-9326,
-5699,
6962,
12533,
-9333,
-2299,
-9228,
5353,
-3467,
-2507,
-3797,
-92,
12071,
-1854,
-6785,
667,
15495,
8431,
8009,
1922,
-6936,
-563,
2646,
-5043,
14027,
5477,
5484,
-965,
-6305,
-2262,
-2593,
5951,
-10394,
-9560,
-5084,
9580,
3537,
-2241,
1700,
-6632,
4785,
7619,
-12097,
2453,
4632,
1093,
-529,
6567,
-6020,
6467,
3946,
-1210,
15129,
6016,
-9549,
3788,
3508,
2730,
-9101,
13952,
-3180,
2606,
2902,
7789,
-7474,
-3138,
13546,
-9915,
-7739,
-14861,
-4774,
-5297,
2017,
874,
1884,
-2204,
-1379,
-1766,
-3228,
5485,
2793,
5889,
-779,
7240,
2697,
-8793,
2023,
1818,
2356,
-2593,
-2736,
-1003,
278,
13529,
7409,
-4657,
-7633,
-3250,
-11182,
3003,
-4998,
-7640,
32,
-7486,
5804,
4182,
10840,
-1382,
2255,
-6952,
-4656,
4853,
-1253,
-3701,
-2018,
-9623,
-16455,
-3787,
-2008,
637,
4272,
12698,
-10153,
-15223,
-3233,
1992,
6833,
-10958,
2015,
-4218,
-1025,
8857,
13330,
3502,
286,
-1610,
2155,
-1563,
3688,
9850,
4411,
4361,
-4097,
-11546,
-5184,
4597,
-6619,
3412,
2877,
7279,
9605,
-4878,
-1961,
9251,
5231,
5857,
2976,
-7158,
-11231,
-3505,
1481,
-8613,
-4943,
3133,
9797,
4452,
7956,
2091,
-11808,
1811,
-9621,
-832,
-4199,
-7247,
6076,
-4230,
-8649,
-6411,
-10031,
5587,
-9135,
1301,
3799,
10467,
-7509,
-6280,
-2733,
-3419,
-1030,
-451,
10339,
1003,
-842,
-6032,
8747,
-548,
-10183,
9522,
-2856,
-1020,
-5721,
-4138,
-1632,
3475,
-8340,
-5019,
-1363,
-3207,
-4253,
-337,
-2589,
-264,
9,
3717,
12417,
4639,
-1938,
4959,
-9172,
-7967,
-9237,
6835,
12846,
-11343,
-199,
8934,
-7365,
-3509,
-7816,
2993,
-14528,
2137,
2369,
-16728,
1923,
-525,
7031,
9308,
11713,
1395,
9956,
-1780,
2396,
190,
10098,
9416,
5002,
-2293,
-11050,
12297,
10935,
63,
9413,
-6618,
-3246,
-13996,
-9374,
2136,
637,
-1022,
2354,
-2725,
-2821,
-8280,
-3027,
776,
1426,
7859,
5749,
5096,
-3908,
-285,
5357,
3298,
4908,
-12551,
2482,
3621,
-1428,
-2232,
11670,
-1098,
844,
5701,
1327,
-260,
-32,
-6663,
-1180,
-4905,
6773,
7310,
-2079,
-5581,
-1610,
6079,
-14717,
1408,
-9385,
-9162,
10090,
5669,
5293,
4796,
8691,
6017,
-5218,
3762,
6222,
-5450,
2717,
12287,
2839,
5695,
-2497,
-7225,
744,
2924,
9784,
-13450,
-4257,
8291,
2303,
2131,
20891,
9000,
-7841,
472,
882,
1466,
3407,
-1751,
-1192,
-4152,
1107,
13138,
79,
-5757,
-2053,
9268,
1616,
-396,
4317,
7252,
2865,
6015,
-5176,
8756,
-10566,
-2467,
4483,
7346,
-5286,
-10134,
-9796,
-3892,
1769,
9985,
-245,
-4212,
-6393,
-4376,
-7335,
-3871,
-6018,
-1663,
-3139,
2886,
-1557,
6880,
7116,
-4080,
-3906,
3785,
-2275,
-8496,
7391,
-814,
-7651,
9410,
-5763,
7013,
-2134,
-10536,
6477,
10619,
6562,
79,
875,
2554,
-5488,
-2109,
-6011,
6667,
-183,
-1073,
-7205,
-640,
-4322,
-1112,
-3466,
7116,
-3553,
3382,
-4552,
3227,
-10140,
2624,
15905,
9061,
3770,
6249,
2584,
4878,
-7882,
2331,
6098,
-3391,
1786,
7847,
-1837,
3968,
-5271,
-13826,
2082,
-6449,
785,
-2307,
-11597,
8967,
-2945,
2883,
3142,
-13509,
13611,
4471,
-8213,
3744,
-1668,
-976,
7909,
2645,
4446,
-4129,
-4124,
-8920,
9552,
-5585,
618,
-7906,
8515,
9876,
-2101,
5848,
7037,
-7745,
7735,
-967,
-929,
-392,
7364,
-10052,
-1556,
5705,
-4981,
-822,
-6390,
-9426,
-5030,
1834,
-8339,
-7317,
-4658,
10786,
-1108,
10403,
2847,
-153,
-5489,
7969,
9028,
-4528,
309,
-13261,
6025,
2634,
8931,
4207,
-4948,
-4245,
399,
3260,
1173,
12351,
-5407,
7862,
4955,
-4225,
7782,
-1245,
-5711,
-9298,
3843,
4553,
-6814,
17175,
6926,
8417,
-1607,
1198,
5644,
2977,
-3100,
-10140,
-5335,
-3260,
-11098,
4061,
-2283,
6468,
2199,
-6119,
11110,
-1910,
9539,
-1807,
3128,
-10106,
-3966,
-6132,
-9234,
-8374,
-1568,
13145,
-13514,
-3956,
4100,
4514,
3537,
5272,
-1570,
-5136,
7901,
236,
-146,
6739,
-1403,
1123,
-9040,
759,
1149,
1862,
2095,
-11182,
-3244,
-787,
7905,
-12807,
-3429,
-18526,
1849,
5592,
-629,
-4968,
463,
8882,
-823,
4528,
-999,
-2482,
7451,
2280,
-4028,
-7185,
1929,
-5638,
3232,
4513,
15791,
-6381,
1038,
-2378,
2825,
-5763,
-2182,
1552,
-2498,
1766,
6676,
599,
-4694,
1352,
405,
-2745,
-2471,
5292,
-3597,
-15264,
-2568,
15853,
3897,
-3780,
-10666,
7819,
-4978,
-7382,
-549,
6214,
-5031,
-2693,
-836,
15167,
4358,
-1873,
-9690,
-1676,
-302,
1431,
-4376,
-4044,
20758,
-828,
1287,
1311,
-9866,
-3369,
-3728,
-180,
4497,
-8847,
484,
-904,
-1128,
-5444,
-3685,
9302,
6694,
13162,
2041,
-2995,
4568,
7373,
-1834,
5404,
-7513,
4933,
-3549,
5037,
-2237,
-1808,
2153,
-4846,
-4414,
10155,
-9083,
3050,
-7985,
-989,
-5044,
1562,
5263,
6936,
-253,
-2490,
252,
-845,
-1866,
-1075,
8198,
7365,
7406,
5969,
-4633,
10300,
-1894,
4061,
-7314,
1836,
12385,
4710,
-3882,
-8753,
-11370,
-3368,
-3203,
-1290,
-4178,
-12056,
3343,
2881,
-9874,
-6103,
-375,
8590,
2857,
4377,
12692,
-8988,
-6986,
21510,
-274,
5494,
-482,
-2969,
-1048,
11241,
4983,
-6674,
2683,
-755,
7504,
5926,
2777,
-2527,
-7583,
-3705,
6978,
2875,
-7483,
-4148,
11596,
9499,
-6524,
-1818,
-476,
3901,
16478,
-5433,
-6124,
1403,
-15664,
-4286,
-1626,
12591,
-5587,
1717,
-4395,
-6748,
15785,
234,
4759,
8265,
871,
-12476,
-7586,
-729,
8101,
-441,
-2167,
10540,
-2029,
-10456,
-5986,
6949,
-3165,
2042,
3434,
-1697,
-2534,
2258,
658,
-5320,
7744,
1701,
487,
-7289,
1631,
-2024,
-7171,
-4313,
10683,
567,
2642,
-5870,
-2110,
-1507,
-705,
12075,
-12835,
4131,
-4123,
3586,
10406,
-5931,
2878,
-4008,
4407,
-405,
567,
-3094,
2431,
2430,
8144,
2631,
7285,
-2721,
-12112,
5381,
-9219,
43,
-6914,
-3835,
-2064,
-1371,
-341,
-2398,
10213,
7156,
-2536,
-2744,
-8992,
4057,
-2201,
838,
-2194,
-8942,
1884,
6708,
-530,
-1428,
6784,
-1425,
-5380,
2136,
-6092,
4773,
5842,
-2939,
3253,
8635,
-4019,
2736,
7895,
-1786,
6910,
-74,
1363,
-3443,
4659,
-2455,
-3988,
4913,
7453,
-3535,
2107,
-1485,
-2929,
3441,
10607,
14289,
-4479,
-10130,
3587,
-929,
7235,
9368,
7305,
9156,
-7973,
-4048,
7381,
-3206,
-1680,
12649,
6971,
-5296,
-1565,
3737,
-1342,
-8258,
-1687,
-132,
3481,
10723,
5427,
-726,
-3985,
385,
7465,
823,
-4607,
5667,
7810,
-4781,
-687,
-1839,
5573,
2438,
-4854,
2366,
3775,
-7876,
6011,
-6986,
6994,
2686,
-6633,
-923,
5711,
3771,
448,
-8545,
-4558,
9154,
-3418,
-8595,
-4527,
-5400,
2726,
-7384,
4238,
6066,
5799,
846,
-9542,
7378,
5744,
-9028,
-2162,
-56,
3396,
1720,
13510,
-9229,
-13126,
-1114,
4497,
21285,
990,
2039,
-2450,
2662,
-5566,
448,
-6413,
-1525,
3666,
-6022,
-6305,
-5657,
2524,
1488,
7377,
3556,
-6017,
-214,
1910,
-15653,
12328,
1940,
3467,
594,
6585,
5119,
-7057,
-2064,
6523,
-6469,
-5158,
-1785,
-3537,
-3199,
2893,
-2596,
-6090,
3766,
-786,
-5982,
-1531,
-3147,
7298,
5645,
4066,
627,
3258,
-8056,
1076,
-4854,
-5568,
-5469,
-4440,
-12168,
1,
-9768,
2970,
-457,
-7627,
8713,
6273,
13271,
-6072,
-4699,
5650,
-4694,
7661,
2416,
5467,
-759,
2071,
16067,
-2767,
-469,
-2197,
-1498,
-5977,
-2208,
-1465,
-4267,
-7340,
363,
7378,
1238,
4014,
4226,
392,
-4264,
9594,
7387,
-6002,
4284,
5113,
2763,
4848,
1426,
-5485,
-1169,
4034,
9241,
-736,
2327,
450,
1613,
-469,
-4318,
-693,
369,
4172,
1293,
1018,
-10695,
846,
-7202,
4151,
1241,
-101,
-2938,
-7806,
-4911,
-409,
7346,
1349,
-5197,
-1207,
-8469,
6426,
2732,
-3600,
2025,
2564,
2784,
20298,
14812,
-4776,
-30,
761,
3021,
4767,
2036,
1352,
3019,
1093,
-9595,
-7267,
10156,
2085,
-6494,
-8921,
1044,
9180,
-11412,
1918,
805,
6801,
14711,
-1423,
-1058,
-1215,
7189,
2067,
6345,
7949,
5497,
-435,
1731,
-10885,
-5180,
4980,
4961,
-3648,
10701,
204,
-13640,
-12104,
6438,
-2896,
-7274,
11515,
-44,
3462,
3105,
4583,
2178,
2842,
-1776,
-4205,
-768,
-4074,
3540,
1962,
-284,
-2029,
4361,
2419,
1523,
2419,
-3760,
-2266,
-2987,
-4286,
696,
-2622,
4620,
664,
-218,
-4865,
-3780,
393,
355,
11520,
14053,
1635,
5911,
-1500,
-7743,
2715,
2588,
6306,
11283,
-3272,
-5391,
-2518,
-619,
4429,
-5096,
-3841,
6074,
-4811,
-9292,
-16609,
-5970,
5584,
-686,
-4149,
-5197,
1750,
2667,
-6889,
2881,
-142,
201,
9838,
-3193,
-1429,
-7459,
5849,
-702,
-8145,
-9241,
9941,
4835,
-12419,
-6226,
4394,
266,
-6004,
401,
414,
-669,
-5210,
-733,
-4149,
4923,
-6395,
5397,
4387,
-52,
10840,
9417,
-8309,
-5217,
9210,
-5301,
482,
10804,
4361,
-9720,
10623,
316,
-3095,
-11763,
-3841,
9619,
-1355,
-498,
6699,
211,
6395,
-661,
4216,
-10020,
-3304,
5594,
-6772,
-7818,
5572,
6734,
-5853,
-4636,
13659,
3094,
-59,
3421,
-12624,
1949,
5448,
7151,
7300,
2479,
-3640,
10562,
9952,
5004,
-5779,
-2990,
5242,
203,
2746,
6207,
10837,
2982,
-10853,
3805,
9127,
-9952,
-2127,
4193,
770,
2537,
-5390,
5017,
-5152,
8736,
-15881,
-4189,
11804,
14246,
-4197,
-1041,
3539,
13350,
1519,
4288,
342,
8001,
-2124,
4435,
-4304,
8493,
-267,
-10793,
-10191,
-3047,
4430,
10170,
1331,
-12108,
-1804,
-8651,
-10364,
5474,
-3353,
-3115,
-4780,
-2577,
-290,
-4547,
761,
-5502,
-1210,
5010,
667,
-1600,
-637,
4424,
768,
1003,
2857,
5415,
-2632,
-6922,
-3456,
-7100,
-2139,
4133,
15078,
-4687,
4362,
4864,
-6816,
11003,
-1533,
-2637,
5071,
8513,
-621,
-1924,
2655,
-9088,
566,
-8988,
6881,
10271,
16560,
4279,
-8879,
6932,
2319,
-1494,
-6952,
7373,
-12800,
-6639,
-1720,
5708,
-2157,
6433,
-8033,
-573,
-5682,
11241,
4008,
3386,
-1140,
7927,
1798,
1184,
-2877,
4793,
8526,
3383,
-2044,
6787,
-2744,
-9217,
-889,
3047,
223,
9470,
17221,
-22,
7005,
-8820,
1923,
3764,
-616,
5557,
4921,
2483,
3633,
-41,
4677,
-10777,
4958,
4221,
11252,
8371,
4606,
-3817,
-2195,
-547,
1489,
-10494,
1596,
10294,
-162,
8702,
5607,
-3667,
-7806,
2539,
4745,
9852,
901,
950,
5294,
-7704,
-8199,
11394,
-1591,
5098,
5449,
5231,
-3734,
5358,
9884,
-1652,
-3956,
-8562,
-9351,
1269,
3327,
-4406,
-14228,
2416,
5061,
16245,
9300,
5104,
4519,
-2183,
-1471,
2570,
4394,
-3668,
1595,
1139,
-2187,
-3074,
6722,
-14008,
9051,
2428,
-3525,
-10317,
-6726,
-1568,
10189,
1650,
3901,
-7617,
14803,
2798,
-7284,
-6761,
9796,
-3873,
-9522,
-2932,
-2333,
6097,
-2065,
3142,
-5325,
3404,
-3078,
-4486,
1204,
2879,
1762,
-2243,
3904,
-1264,
789,
-3855,
3151,
3147,
8094,
-8970,
-656,
3526,
3128,
10631,
-690,
-5127,
15577,
-4565,
5423,
3068,
-4093,
-7512,
2501,
-4722,
-11411,
-2694,
-14599,
-14557,
3331,
-8033,
-12946,
5514,
-517,
944,
-9162,
14133,
7184,
-11832,
-2114,
-2485,
-4784,
1796,
1384,
1496,
6047,
-1793,
-12236,
2986,
4705,
4980,
-9,
-7291,
-4845,
-1322,
-1630,
-11944,
-5573,
-9091,
3617,
2142,
6221,
2974,
-7444,
-657,
-1731,
-4148,
4989,
-4530,
-1702,
3137,
-148,
1650,
4207,
790,
1894,
896,
-938,
12118,
-5958,
5228,
-7213,
212,
-190,
12035,
-3264,
-12743,
-1369,
11300,
-8899,
-14542,
-7365,
78,
-509,
-2183,
11894,
-4624,
7481,
2472,
-1089,
-9430,
-2797,
982,
-8528,
-877,
1107,
3531,
5282,
-7857,
14377,
3359,
12776,
11756,
2692,
-5257,
-1487,
-7074,
-4323,
1907,
10974,
-3143,
-246,
3169,
-1354,
-2054,
-1102,
-2373,
-4554,
5926,
-3358,
6027,
-654,
-2777,
7134,
4233,
-8790,
-3995,
-12075,
1435,
3449,
-5924,
-5869,
-377,
6010,
-7985,
-204,
4025,
-2106,
-8605,
-99,
7605,
-6579,
1835,
6029,
18427,
-4233,
9246,
-730,
62,
1955,
-1923,
-3782,
3207,
634,
-3871,
-12190,
-1898,
1236,
-1292,
-1706,
-6817,
499,
-8309,
-808,
3007,
-3815,
-3429,
13162,
-2409,
9205,
3909,
-6243,
5100,
5274,
3204,
-730,
-7506,
8725,
10761,
1959,
1602,
8336,
3350,
-4209,
13282,
234,
-5390,
2969,
-359,
2697,
3242,
-5921,
-4323,
13957,
-3991,
-3897,
8678,
5836,
7556,
-1236,
-4751,
5644,
-2308,
2925,
7611,
-9540,
4083,
2630,
-2026,
4154,
-8043,
1189,
1861,
-1546,
-2933,
1833,
-460,
-7402,
6978,
6811,
-1710,
8167,
7015,
-2774,
-2609,
1325,
-397,
-1087,
1889,
2812,
-9336,
3405,
-12541,
1396,
4108,
-8091,
-1656,
1789,
8686,
11508,
8851,
-2157,
-13057,
-2655,
-1165,
-1423,
6335,
-9700,
-7372,
5847,
-8157,
-4941,
-13144,
6532,
3845,
-10249,
5789,
2482,
12242,
12153,
-3416,
2566,
352,
3508,
-1099,
-704,
3547,
-6474,
5180,
6236,
-14403,
-6995,
-7013,
780,
1689,
-2403,
1073,
-584,
-6763,
5478,
-8140,
4,
8056,
5718,
-6154,
10299,
2851,
-7889,
11033,
-1119,
-9962,
-399,
-13529,
9212,
-12644,
15602,
198,
-12446,
-1635,
-1107,
-4658,
-1144,
1301,
219,
-4072,
6855,
-1468,
-8727,
-3048,
-128,
10136,
-2060,
-3928,
2523,
-6870,
13952,
4494,
1251,
-12634,
-3582,
-10766,
-615,
4144,
1952,
5509,
1858,
-13560,
9803,
2824,
3972,
896,
-2305,
-10819,
3071,
5075,
1623,
-8096,
-6302,
-8170,
-2307,
3967,
3771,
-6250,
-2329,
1095,
6245,
-3610,
961,
-15451,
-9612,
-6881,
-490,
-5151,
2581,
15564,
-15980,
9656,
4139,
9614,
-9298,
-310,
2703,
-2793,
16687,
-3084,
4352,
1,
-456,
632,
-8817,
6987,
1384,
-19179,
-3425,
3357,
-62,
4910,
3587,
3213,
-6626,
-451,
12997,
-5635,
2759,
1421,
-3746,
3242,
-1266,
3363,
6481,
4853,
-15195,
-999,
4529,
-329,
7532,
-2572,
-3227,
-5177,
10608,
3950,
9493,
2071,
5719,
-1030,
-1029,
10698,
-5117,
1927,
7243,
-10897,
8783,
-6355,
-5925,
9993,
-186,
-22104,
6724,
-524,
-6131,
404,
1773,
-94,
-630,
-14968,
4823,
15030,
-1071,
852,
3662,
10635,
5919,
3118,
5272,
5239,
630,
567,
-2525,
2753,
-8751,
6728,
-8242,
11772,
11013,
7472,
4749,
162,
-3704,
2320,
822,
-4613,
3986,
2404,
2683,
-181,
-633,
9586,
4386,
10753,
-4330,
724,
-10111,
2932,
-8845,
5922,
-5811,
-736,
-6150,
7524,
-11931,
-3688,
7519,
2390,
9389,
-19431,
-644,
2597,
-8825,
-4027,
393,
-8357,
-3698,
-4151,
8195,
17394,
-20338,
15577,
-7494,
-347,
5281,
4232,
2217,
-3674,
5181,
12337,
15372,
-8856,
4657,
-7440,
2120,
-11527,
-2626,
525,
5153,
5328,
-6481,
-809,
-2320,
8285,
9859,
1477,
2373,
-2576,
-3389,
-16051,
6554,
5610,
-8482,
-8025,
5309,
9250,
-2246,
8566,
9302,
-4024,
-8490,
966,
-10999,
-3518,
-8876,
4035,
7546,
3891,
14970,
8029,
-4309,
6598,
-8924,
6944,
12318,
3915,
621,
8928,
5327,
4894,
5686,
5063,
6586,
-1135,
-6252,
6282,
2083,
3373,
-14991,
11802,
-7850,
7110,
-6907,
9653,
-5064,
-9023,
-14653,
-1702,
-5057,
6409,
-2798,
-643,
-2230,
10582,
-1577,
-11991,
-7907,
-133,
-16743,
93,
2555,
-2246,
8124,
-1198,
-9801,
4857,
-9675,
-1655,
-12444,
-4537,
1707,
1431,
7293,
-4975,
-85,
2132,
2039,
3228,
-1733,
3363,
-4813,
2384,
5090,
1825,
2160,
-7096,
-14307,
7723,
-12605,
-79,
-1134,
-2751,
-8553,
6276,
-3866,
7087,
2050,
1434,
4479,
-659,
-7781,
5097,
-14051,
-6595,
-4122,
-5251,
-2072,
-11566,
-7777,
1444,
677,
6988,
-1663,
4251,
6100,
-2800,
-2610,
-7898,
-2354,
8489,
-3589,
-1644,
1899,
2942,
6314,
2506,
8412,
-7662,
-4614,
5520,
-2734,
294,
2234,
922,
-2094,
5041,
-7251,
-1122,
3332,
1185,
-9392,
104,
-9574,
2657,
4047,
8959,
-1277,
-1554,
5614,
976,
-7120,
2436,
-5090,
3238,
-7223,
-10862,
-3066,
-3536,
-19660,
3894,
6777,
-4538,
-145,
5093,
-5722,
3409,
6260,
4286,
-6390,
-5861,
-1991,
-7684,
6266,
-5767,
4852,
2452,
-9974,
-11661,
-3684,
2800,
4857,
2038,
2846,
1396,
-6888,
-8590,
-1055,
-3108,
-11156,
-9624,
-2240,
-1545,
6598,
-383,
1705,
7181,
1145,
-6010,
2429,
-4716,
1535,
16436,
3,
9043,
3475,
4942,
-5664,
-2569,
-2534,
946,
635,
-4420,
-7447,
-8047,
-14510,
1668,
-3400,
4777,
-4004,
3641,
-794,
-6444,
7352,
-403,
6329,
10298,
604,
-4004,
9506,
-6761,
-641,
-5259,
-7934,
-1447,
-4387,
3223,
-791,
982,
4912,
6649,
6265,
-1449,
-840,
-6565,
-748,
-2195,
9668,
6384,
8552,
-2965,
9,
2429,
6547,
-481,
3035,
2247,
386,
-157,
-10068,
6632,
-7323,
9155,
2120,
5659,
3137,
2856,
-4466,
6500,
6625,
9807,
6248,
1023,
-4173,
-653,
-14810,
4042,
-5552,
7488,
-3810,
445,
-301,
5716,
-4539,
-9132,
-3521,
10338,
6052,
-10351,
1531,
-12308,
5372,
-5830,
4789,
8120,
6763,
-10011,
8358,
-4972,
-8755,
-5487,
6326,
-1739,
11344,
-6155,
1565,
4835,
3359,
-6357,
4032,
-918,
-3476,
-7107,
-12029,
3585,
-6613,
2862,
-8054,
4735,
-12206,
677,
-1119,
2999,
9602,
-5573,
34,
-382,
1008,
7037,
5333,
513,
6194,
13978,
4830,
-3008,
-7107,
1599,
3386,
-1527,
-5159,
-4753,
-7072,
10672,
-1696,
48,
-7341,
4553,
-5274,
5322,
-1743,
-6056,
-13645,
-8781,
-8159,
12870,
17184,
-12945,
-6348,
-1766,
-2086,
6314,
13847,
-346,
-7808,
-4344,
-8921,
-658,
-4556,
1807,
10281,
-9781,
-4060,
-4203,
7291,
2438,
-3082,
2071,
-4803,
1068,
7525,
-17775,
8960,
952,
10503,
7886,
620,
-5818,
1041,
5641,
-2022,
-4976,
-1797,
1702,
-240,
-8592,
840,
-1515,
2791,
-11508,
-6165,
-3952,
-660,
-3338,
-6925,
-8188,
228,
2829,
-4667,
4012,
1298,
-9413,
-2206,
10821,
-9277,
1469,
-2761,
336,
-5235,
-3404,
-17208,
3586,
4307,
-5296,
-1896,
-3112,
-3783,
-718,
2300,
3026,
5914,
-1986,
13324,
1477,
-9033,
-2265,
1346,
3059,
9794,
9084,
12197,
148,
-1366,
-10431,
-4112,
-12,
8438,
9378,
-2959,
-7563,
6352,
-2084,
-4387,
-7203,
-11164,
-9332,
10417,
-2068,
-4219,
10608,
-4725,
20811,
-6633,
6228,
12667,
10020,
-4493,
371,
687,
7560,
-10468,
21286,
-18858,
-1415,
6144,
-201,
7580,
-7599,
6504,
-15227,
86,
14052,
-8876,
6273,
-5444,
5676,
-1452,
6538,
10738,
-5868,
-1178,
-1529,
-4704,
-8111,
1430,
-8827,
-2386,
-8669,
-7297,
6904,
-10337,
5220,
4766,
-16079,
1456,
7053,
4782,
1069,
6695,
-7852,
-1225,
3216,
5578,
4619,
8426,
-11943,
3895,
-3413,
-2654,
-6527,
3833,
10232,
-8340,
-7937,
-9545,
3649,
627,
1003,
8503,
-2625,
-4753,
4270,
-411,
-6382,
-5390,
-1895,
2071,
2355,
-3919,
-27,
-3367,
5100,
21939,
-8193,
4416,
-7849,
1815,
3319,
-2238,
10283,
2141,
4536,
-1828,
-8440,
-9254,
-3738,
-5449,
-2977,
-12423,
2940,
-6921,
1909,
-3638,
-3719,
6619,
400,
-1373,
9868,
-990,
6916,
-458,
5002,
-4955,
1011,
7585,
456,
-5899,
4952,
-225,
-1103,
-1713,
10229,
1826,
-7769,
-4989,
-3852,
-7619,
690,
-4966,
-4527,
3959,
5455,
7279,
4487,
4017,
8328,
-3029,
3962,
-307,
13327,
-5587,
1625,
-2805,
-1887,
5321,
-1200,
-21932,
16008,
-13776,
1471,
-6771,
7616,
1038,
-3380,
-3421,
-8757,
-777,
9122,
-10886,
-4676,
-5411,
-4935,
-12786,
4705,
-10788,
-2711,
10679,
1197,
-3156,
-2007,
2402,
4559,
-5543,
6010,
-2906,
-6892,
-10481,
1771,
2621,
-3466,
-8428,
-2872,
-3177,
-5,
2405,
-5979,
-5007,
-12458,
-7086,
951,
3388,
-2247,
6819,
-317,
-1658,
-13897,
1159,
-9208,
4176,
-6150,
1890,
802,
5951,
-6360,
2191,
1983,
2821,
-1658,
5488,
-584,
26,
-5183,
1108,
1454,
-1054,
3231,
2675,
2645,
3532,
-4014,
6381,
8822,
-1264,
-2192,
-1457,
3957,
1233,
-3755,
-6535,
3569,
9655,
2,
2703,
-4096,
-1946,
4980,
11388,
3141,
-9099,
-3397,
-1654,
-1470,
10774,
1241,
-1122,
-2324,
5074,
-11009,
4842,
5235,
211,
-5333,
-7702,
-731,
-8272,
633,
-7318,
-2811,
5001,
-48,
-2488,
-10522,
-1788,
-6087,
526,
-5491,
2847,
-9412,
-3684,
-1163,
5072,
-1170,
12844,
27,
-4141,
532,
3206,
-1687,
9173,
-24,
7781,
-3615,
-15177,
4804,
1251,
-3745,
-3440,
6402,
561,
-6718,
-5339,
-6723,
1292,
-6395,
1756,
2002,
164,
1279,
-2936,
-2376,
4153,
-2899,
6892,
-3825,
2403,
-3057,
2079,
-617,
2814,
-3490,
3071,
3648,
-325,
3138,
603,
-2339,
4206,
277,
-600,
-1330,
6104,
1260,
1242,
7270,
-1196,
-10662,
8255,
3283,
-3805,
-5985,
7270,
2567,
2121,
5070,
-1164,
4026,
-294,
10620,
-1509,
-123,
-8047,
-2303,
-6121,
4921,
-1616,
-1494,
3750,
-1550,
-2522,
1907,
-2294,
-1141,
5049,
915,
-3252,
5892,
1365,
-3094,
-7288,
8902,
438,
2442,
-2347,
4774,
-5140,
-5312,
-9877,
-1955,
7079,
3037,
6180,
-903,
3996,
7298,
1624,
-258,
272,
-5902,
-5317,
2918,
-7515,
-1330,
13159,
-8800,
5081,
-2146,
2744,
7997,
-5991,
-7348,
-3498,
963,
9481,
-4596,
-299,
8437,
-1584,
-2848,
-5081,
-4945,
3788,
2438,
6660,
-12429,
-1208,
-5580,
5033,
1350,
-54,
5329,
3120,
-4713,
-1381,
-2795,
-7393,
12686,
-1871,
-6530,
1197,
2989,
4262,
4691,
5182,
-2280,
-6728,
2109,
-3746,
7808,
3067,
1398,
531,
-3628,
-1664,
-3038,
-7895,
4597,
1497,
4193,
4915,
-3543,
-3497,
-9369,
-3749,
121,
3711,
-3436,
-5744,
-2439,
3733,
-1785,
6414,
-1535,
3057,
-1203,
6869,
5204,
-3647,
-3760,
-1196,
3338,
-1079,
-8053,
-6186,
4485,
-8979,
9301,
-1081,
9793,
8272,
886,
-3848,
1966,
2597,
-6335,
-3579,
-451,
428,
3987,
7522,
-2666,
-2735,
602,
-8011,
513,
-7854,
-832,
3970,
-1882,
7125,
-330,
-1560,
-4981,
12742,
-5700,
8111,
-7042,
-2778,
-1449,
3340,
-818,
-5558,
2173,
5016,
597,
8509,
-2487,
-2027,
581,
850,
2490,
-3694,
-653,
1013,
1459,
3095,
-854,
275,
-9134,
-1283,
-3851,
-1563,
-1333,
4525,
585,
4476,
-9504,
4012,
5085,
7300,
1593,
-1658,
-5362,
-1173,
-7640,
-1384,
-1333,
6628,
7581,
-8768,
-5114,
3046,
1787,
3952,
-8178,
-3632,
-1995,
761,
105,
2530,
-4495,
4306,
-2897,
673,
-6221,
2260,
-1109,
-7720,
3382,
791,
-4165,
3375,
7475,
-7039,
-4503,
-162,
-229,
-1900,
-4958,
-5557,
-1577,
471,
3877,
-1185,
-4768,
2649,
7019,
162,
7529,
-1126,
-3478,
2388,
-3117,
-4639,
-731,
3140,
3019,
1969,
3914,
6346,
-1760,
13842,
-7623,
4706,
-4591,
-1664,
6017,
552,
-2629,
838,
-8737,
4335,
-3832,
-783,
2269,
4416,
873,
-4655,
-1606,
963,
-81,
-7286,
3687,
-8063,
5557,
8594,
-5453,
-2943,
1555,
4447,
-3434,
-5739,
-5760,
4280,
-2956,
2970,
-5591,
6612,
963,
5961,
-2770,
-310,
-1553,
-9133,
-2140,
2117,
-6282,
8516,
-2004,
-4161,
-921,
6473,
-3610,
187,
4697,
-5855,
-8148,
5273,
-3800,
-1134,
709,
2420,
11981,
2351,
2134,
2801,
-932,
5461,
-1480,
-9068,
-3709,
6175,
1233,
-2235,
-1578,
-13167,
5219,
3109,
7274,
585,
-3971,
1470,
1440,
-393,
74,
-3822,
3604,
-465,
10004,
4706,
-180,
6738,
1267,
8805,
-8133,
-8212,
-2407,
-1033,
1667,
-2088,
-555,
-7814,
-2200,
2518,
-326,
-4611,
407,
-1416,
1539,
5201,
1352,
4836,
1250,
1136,
3281,
5360,
-3009,
-2484,
790,
1231,
1758,
-8751,
3989,
1658,
1398,
7420,
2010,
7237,
-5389,
-1958,
4176,
-9527,
-3586,
-10313,
-5935,
235,
2623,
-923,
869,
98,
3771,
-5328,
-1713,
3348,
-3548,
-3018,
3046,
9978,
1387,
11129,
1007,
-7975,
11288,
1159,
-2314,
3162,
-1978,
1230,
-1575,
-3749,
-2887,
-787,
-1797,
1142,
-2518,
1398,
315,
-6620,
-362,
1006,
-8437,
-903,
-1354,
2824,
1629,
-1410,
3225,
648,
-3022,
2931,
4171,
-1247,
-1713,
5506,
3705,
-66,
2369,
-3853,
7940,
6704,
-2312,
-480,
-816,
5598,
1688,
2567,
2069,
-8906,
4891,
-1904,
6211,
1384,
11086,
3970,
-2011,
-1843,
-4773,
1685,
612,
-9398,
-3810,
228,
-2146,
8514,
-5329,
-1465,
1684,
2108,
2229,
5765,
3322,
5448,
-3728,
-963,
-6765,
-12262,
2748,
4711,
-7579,
-1171,
-2007,
12630,
1335,
4096,
2295,
-2900,
1241,
-3571,
-350,
1168,
-2024,
6110,
-3067,
2777,
8562,
-734,
1187,
-4818,
-238,
2884,
-412,
-1890,
13820,
466,
-3580,
6750,
8546,
-1693,
-3885,
-4371,
-1919,
5230,
5271,
-4318,
-3888,
-1251,
-4258,
9594,
-49,
-1246,
2724,
4378,
3768,
-1033,
-9860,
-5411,
-2128,
3443,
6268,
5530,
739,
1084,
6106,
2877,
-1607,
5333,
1420,
-1718,
10493,
11976,
-1722,
5480,
-8448,
2543,
-2108,
2793,
-3326,
98,
-1476,
-1842,
-6354,
-2677,
-2141,
212,
2992,
-937,
-654,
-1418,
5838,
6619,
-2133,
-3689,
-10769,
-6377,
-1777,
-6086,
-3323,
145,
8250,
3277,
-2841,
-2162,
2540,
2393,
5614,
-7386,
1536,
5091,
-3364,
-1040,
-7363,
-4153,
255,
759,
-2585,
-1843,
-3889,
1053,
2835,
4772,
4678,
-2106,
-854,
-3547,
-1571,
4637,
5352,
-4339,
-947,
-1124,
3950,
3826,
4922,
3070,
-4135,
-3813,
2894,
-4322,
-7102,
-328,
-8784,
2732,
-1208,
-2898,
-4001,
-1580,
1615,
3852,
3526,
2101,
-116,
1764,
-7154,
-11343,
2731,
2436,
1297,
1881,
-1631,
-521,
979,
6076,
5014,
6912,
-5761,
-5556,
-4502,
5706,
7354,
444,
-715,
6206,
-6179,
-11898,
-2246,
8544,
-1382,
-48,
-3276,
2380,
1156,
1866,
-3800,
8242,
5842,
-3329,
269,
-6557,
-7489,
-3140,
-2236,
-1183,
9885,
3240,
-396,
-105,
-4406,
-959,
411,
-5201,
2376,
1606,
-526,
-2483,
2879,
-371,
3610,
4456,
3204,
-5288,
-4767,
-2325,
-10818,
37,
-9043,
-2759,
-5206,
1152,
-9203,
-1131,
3797,
6366,
-5683,
4400,
2511,
2328,
4427,
-5100,
-3656,
-821,
-4865,
-7568,
3673,
7078,
1599,
718,
2340,
-331,
11383,
-5579,
-548,
2297,
-5315,
-3397,
-355,
-2514,
1171,
-4165,
-3087,
1776,
3565,
-6119,
11721,
-3572,
-8531,
-10741,
-3208,
-5436,
-6637,
-91,
477,
-2214,
-6245,
3565,
-76,
-1512,
5709,
1280,
-3969,
-2321,
-1965,
-352,
-1034,
12509,
6540,
3287,
6904,
-1487,
6043,
-5305,
2669,
1376,
-6905,
3333,
-6780,
-4863,
-1690,
-3045,
-11689,
684,
-3332,
-3202,
-4053,
-7073,
-1961,
-4849,
2867,
-2727,
-1558,
3890,
-3767,
7514,
1622,
-4233,
4474,
1280,
-2863,
5141,
-612,
6431,
1698,
-1551,
5567,
-2797,
-4916,
6319,
-6527,
-218,
7586,
4118,
-4802,
-153,
1615,
6623,
1436,
7872,
-324,
-9993,
-213,
-2767,
-1649,
441,
5920,
2501,
1621,
851,
-1889,
-768,
1723,
-6137,
10157,
-3194,
-3231,
-2050,
6281,
4926,
-2440,
2808,
-6402,
-5214,
5294,
-2479,
-2838,
-9379,
-12995,
-4690,
-4696,
4918,
5426,
-402,
5302,
-1326,
8430,
6559,
200,
-4695,
1023,
-4084,
806,
432,
-2336,
-2507,
-4649,
-6978,
-1062,
-8216,
1267,
-2158,
-820,
-4149,
6471,
2202,
585,
5889,
2192,
7538,
-2799,
623,
-1529,
-4789,
402,
-2813,
-848,
3462,
-3838,
3896,
1736,
-1060,
-7500,
43,
-2825,
3749,
1487,
3152,
1842,
-514,
527,
7059,
7087,
8423,
15816,
13069,
-804,
-3592,
6240,
-2738,
-7930,
-5101,
-10681,
7248,
5403,
-411,
750,
2023,
5094,
3523,
4354,
4470,
8243,
345,
4644,
5097,
4601,
-7033,
1357,
3984,
-4890,
-539,
2208,
-4821,
-5422,
-3710,
7930,
-5275,
-526,
-5076,
-474,
-1907,
-1237,
-839,
-9170,
71,
-1280,
-4640,
-2006,
455,
3861,
-1545,
4033,
1973,
5698,
1005,
-1731,
-5393,
-270,
1855,
4179,
3209,
4507,
691,
7558,
4547,
2140,
-7933,
5659,
4118,
-524,
1986,
4646,
-2068,
-6576,
-1336,
5582,
-3260,
-1403,
-4011,
-3739,
5902,
392,
-273,
-2484,
1797,
3427,
-1971,
-4533,
494,
-3945,
-3290,
5539,
-2984,
-8989,
-1285,
1742,
7638,
5619,
3105,
-2352,
7368,
1361,
-2824,
-5677,
633,
-2234,
2666,
593,
1081,
-6700,
1901,
1428,
934,
-1765,
5973,
2080,
6782,
-2652,
-1828,
5142,
2320,
540,
2420,
4879,
-5786,
2071,
-1522,
881,
160,
1716,
2517,
4735,
5938,
2248,
-1424,
-1780,
138,
-1323,
-239,
-3492,
-9264,
6644,
8617,
1766,
-2055,
-2602,
-3593,
-5508,
-7150,
4624,
4090,
-1253,
-3557,
-4042,
-3926,
-7095,
6058,
2496,
-5769,
691,
-5644,
5844,
3635,
-2888,
-4350,
-3131,
5114,
2121,
-1848,
-8586,
-4583,
1377,
-4946,
7106,
1250,
850,
-1811,
837,
1767,
1703,
-1619,
5681,
3262,
-10424,
7342,
189,
8016,
5070,
-5579,
-4783,
1963,
-5593,
8478,
-7089,
1981,
-3174,
-3568,
-13215,
6194,
-1500,
2083,
4753,
-2999,
-4578,
-2804,
9446,
2631,
4206,
-7366,
3165,
-4373,
-7338,
10039,
-3033,
-71,
1897,
5514,
-5082,
6501,
8123,
-1159,
580,
-2201,
601,
-6272,
-731,
8826,
1796,
4129,
6956,
2515,
7976,
9,
-6025,
6082,
-4354,
-3747,
-439,
-3138,
-110,
-448,
3028,
-4027,
6092,
-1919,
6034,
11052,
226,
-99,
-6914,
6232,
5140,
-1967,
5654,
-1756,
2433,
3921,
1460,
1631,
-2348,
1606,
-2073,
-77,
2387,
-58,
2149,
1149,
-3095,
5202,
-7561,
-1639,
-4668,
-9654,
-1886,
378,
-1524,
-4645,
-401,
-3792,
-56,
-4894,
9364,
7912,
-2609,
11587,
-4146,
189,
341,
2985,
3163,
2217,
-7901,
-7522,
2342,
1229,
-910,
-669,
-4544,
-4560,
3052,
563,
2287,
6233,
4929,
3886,
-796,
-625,
-12282,
2799,
-7851,
-5826,
338,
1410,
7338,
-1285,
-2714,
-7574,
-3861,
7700,
251,
-5122,
-2923,
4203,
4323,
-1994,
-9247,
4913,
-4533,
-2754,
-1183,
-1470,
-902,
7310,
-4444,
-5584,
-2847,
-10762,
-3601,
-3921,
7016,
-1332,
-7361,
-1229,
2108,
-3250,
-4833,
-3743,
-2454,
976,
-4376,
5037,
1671,
-2369,
-1596,
-3091,
1915,
6941,
7365,
8144,
7075,
-6982,
855,
769,
-3873,
-1059,
2458,
514,
-2280,
12524,
3128,
-1107,
-3590,
3739,
-131,
5855,
3325,
-4591,
-172,
-7230,
2867,
-32,
-4668,
3216,
-1349,
-450,
3575,
-38,
39,
3879,
-6334,
1455,
-3636,
-2270,
3569,
8037,
2745,
504,
1119,
-373,
6907,
-3743,
6671,
5552,
-3947,
-3939,
-3668,
-1098,
-10814,
-1633,
-6506,
8607,
-171,
1670,
959,
1542,
-1573,
-9919,
5070,
4284,
3833,
6181,
4418,
8578,
-2087,
-5663,
-355,
2906,
2798,
617,
-1338,
6075,
-1645,
-2375,
-2027,
791,
88,
1382,
-3017,
-4834,
7046,
4228,
2194,
-3026,
784,
3523,
-14270,
-5402,
799,
-1239,
-4827,
540,
-428,
1417,
1428,
723,
-2773,
-344,
4008,
-437,
-163,
1962,
-6401,
1897,
-3965,
-4465,
-428,
6813,
1592,
-1419,
-3124,
-5020,
2698,
6419,
5451,
4141,
-565,
-8399,
-1591,
8038,
-3781,
-2971,
1527,
5627,
7172,
1835,
-238,
5358,
-3578,
4347,
666,
-3468,
9589,
-5215,
-3498,
-1759,
8080,
-818,
-3813,
3598,
299,
3981,
-3160,
7678,
-7368,
759,
-7005,
3820,
-1095,
-10611,
-4904,
3969,
-2111,
-2899,
8874,
-8606,
-6627,
-2235,
6454,
-895,
10171,
5299,
-3295,
1499,
1586,
6125,
1548,
-4795,
-283,
-3497,
-1851,
228,
-3119,
-1163,
-2593,
-1699,
5459,
7778,
1192,
-2815,
2625,
3528,
1409,
-3007,
-1187,
3294,
-6417,
-1365,
-166,
-4683,
308,
3268,
10,
7334,
1600,
-2016,
-234,
293,
6597,
2610,
5676,
3338,
3798,
-1761,
-3862,
5454,
-2813,
2593,
-700,
-1075,
1914,
-1307,
2757,
-6716,
780,
-8083,
3394,
-262,
-35,
-2309,
-1006,
-1607,
1073,
2253,
1200,
-783,
-1090,
-8268,
-5339,
3864,
-9799,
-1150,
-1140,
2756,
-3461,
-2261,
-4437,
-2374,
-3223,
-3241,
-338,
6655,
-633,
-3265,
283,
2843,
12,
-8147,
3996,
-1551,
4603,
3063,
1491,
170,
106,
7430,
-44,
-832,
-2342,
-13193,
1933,
658,
4300,
-647,
339,
1087,
-3201,
6058,
-2003,
-1756,
2161,
-666,
-2970,
3026,
-4495,
301,
8401,
3317,
9000,
-5069,
4978,
-1511,
6771,
-3884,
8767,
-4735,
-1051,
398,
-1030,
10554,
2594,
-184,
8471,
1643,
-9328,
5389,
-8505,
-10271,
-8397,
1137,
2932,
4991,
6909,
2126,
22,
-2002,
-3607,
-3744,
789,
-353,
-1916,
3993,
-9740,
-1144,
1170,
-7276,
-6979,
-1312,
-10369,
2827,
-5558,
-6341,
625,
-6765,
-665,
-6194,
-7745,
-3565,
36,
-407,
4318,
3008,
3931,
-959,
-2260,
3328,
2915,
-5245,
198,
-1372,
-12354,
4769,
11979,
4029,
4671,
-2320,
-9591,
4541,
2352,
-757,
-132,
2398,
9969,
1251,
2402,
-9223,
3586,
-341,
929,
-295,
62,
-4804,
725,
-3432,
-1907,
-1440,
10061,
9110,
-1271,
2957,
3578,
8330,
3939,
6215,
-9012,
-29,
-3576,
472,
-3075,
-4721,
-11739,
2721,
-4842,
4712,
6065,
4061,
2330,
5361,
4473,
-7134,
-9655,
477,
4900,
-3434,
-4326,
4530,
-7759,
-8023,
3866,
3740,
-1532,
5852,
-800,
1550,
-7951,
-4697,
1895,
-1176,
1885,
-7495,
1050,
-2184,
-12422,
1402,
-1917,
-1417,
5240,
-7496,
1335,
-5699,
2199,
1175,
1666,
2991,
71,
2752,
2969,
3084,
2712,
-4300,
2294,
1275,
957,
7408,
-4081,
1191,
-5318,
-1318,
-5857,
-4019,
-8761,
-5185,
-381,
-4355,
8147,
-8822,
-2427,
-4555,
-4985,
-1029,
-5330,
-632,
1879,
6852,
391,
-4237,
129,
-2523,
-6770,
-1004,
-2823,
10728,
-3895,
-3955,
3257,
-217,
-2745,
-2631,
-8148,
3650,
4425,
4464,
961,
4814,
-573,
559,
-2686,
-4523,
231,
1481,
7313,
1739,
2653,
-2191,
7390,
-880,
2085,
-15061,
-3299,
-10195,
-1487,
2024,
-2849,
8256,
-2677,
1887,
2693,
-3480,
1962,
-8060,
291,
1335,
3804,
1624,
9404,
-7307,
2872,
-1867,
1735,
382,
8481,
-3561,
-6179,
-3908,
-7060,
-1284,
4205,
2973,
-4330,
4271,
11200,
1019,
1204,
-2689,
5974,
1454,
3652,
-1538,
1262,
2205,
-3840,
3743,
-10903,
-3813,
-1245,
1524,
-5265,
-2651,
1504,
-14,
-330,
-1806,
-4840,
9120,
8561,
-8681,
250,
6934,
-8961,
-2170,
141,
-5322,
2478,
1627,
-6198,
-4299,
-5461,
3359,
-3820,
13757,
-1379,
-9653,
-1312,
-950,
1885,
-7540,
4368,
1721,
5431,
-3531,
-634,
-2825,
5542,
-1809,
-3109,
6432,
-3658,
-758,
3420,
-8493,
-1849,
779,
-3747,
-2627,
1630,
81,
-7334,
4961,
-2209,
3220,
-5447,
-8884,
-978,
-935,
-32,
-5995,
-3313,
-959,
-5505,
-541,
-8682,
-3834,
-7657,
3294,
-3109,
-9570,
5447,
-7875,
-1115,
345,
5956,
2588,
1931,
5547,
6300,
5154,
398,
-4893,
742,
1792,
3212,
4143,
5903,
1551,
6311,
1703,
1360,
3741,
-785,
-6265,
-9307,
857,
-2235,
8075,
3548,
3074,
3192,
-5954,
-2682,
-759,
3827,
5111,
4069,
5473,
-1129,
6095,
442,
167,
189,
-9234,
4528,
8943,
1753,
5513,
5489,
8072,
-979,
-878,
5016,
-5409,
7209,
-2247,
803,
496,
3604,
-2203,
1257,
-1666,
-1828,
6568,
-123,
-6826,
-1405,
-7706,
5199,
2681,
-4101,
-1927,
1959,
-4316,
-2953,
-3951,
1287,
-8225,
-10184,
-6260,
3515,
2208,
1509,
6964,
7897,
-4553,
-4341,
-4986,
4956,
-958,
-961,
-1084,
6528,
-14045,
-4599,
-3878,
-8137,
-8647,
-9378,
-2152,
-5350,
-1302,
-306,
-1383,
-7616,
-4381,
-1336,
-246,
6609,
3483,
7614,
10277,
1155,
-3337,
12720,
8308,
-5625,
2703,
-3173,
6892,
-6822,
-3376,
4244,
448,
5414,
-3956,
516,
-7387,
2371,
-1025,
4281,
2039,
-3165,
2431,
-6465,
1055,
-5670,
-5582,
-4838,
331,
3903,
-8571,
-3704,
605,
11143,
-2790,
-8789,
-80,
934,
2407,
3590,
1927,
-4466,
4804,
7488,
209,
-2278,
-3408,
4699,
-6803,
-277,
-4783,
2867,
1439,
3948,
-6032,
-1111,
2351,
1182,
-2023,
2760,
-6674,
-5438,
9045,
3455,
5650,
4174,
-946,
-3361,
629,
-6040,
-2163,
896,
-647,
-1420,
5504,
-7910,
4167,
-4085,
4002,
1127,
-1502,
8333,
1443,
1934,
-3153,
2144,
-11292,
-1737,
-4378,
2293,
-3448,
304,
4018,
-5688,
3496,
-2874,
-2032,
6169,
516,
-1469,
-8791,
955,
433,
1866,
-716,
4927,
490,
9918,
-3065,
-7693,
799,
5259,
-709,
3445,
3573,
4781,
-119,
1849,
-1093,
-4311,
-2490,
8092,
-243,
1656,
-10609,
3103,
1015,
3920,
377,
-5883,
3985,
489,
-3536,
-943,
-45,
1709,
-6529,
-665,
1650,
-2608,
-2212,
82,
-1884,
-5853,
1059,
600,
-7414,
-12013,
2264,
4619,
2833,
-1312,
-1983,
10692,
-7583,
189,
-5226,
1790,
1724,
-8279,
-8546,
-2804,
1157,
-2875,
-9841,
2733,
-3477,
5062,
4282,
-8349,
-10273,
-2347,
10235,
-1574,
-949,
4472,
-914,
2106,
2263,
1355,
-381,
-5529,
-9103,
151,
4935,
6114,
7772,
-1902,
8776,
-1364,
8629,
2928,
2078,
-3395,
-3244,
-202,
8427,
-1510,
1899,
-7020,
-4265,
9443,
-592,
6837,
1550,
-4418,
2203,
-5001,
6348,
2508,
2597,
-1494,
1573,
-521,
-2099,
-2577,
2553,
-1181,
-253,
4385,
4791,
-5318,
-2897,
3730,
-2638,
4679,
2681,
1678,
-4644,
10635,
1735,
3021,
-4735,
-818,
-3743,
-929,
-2317,
2676,
-2204,
3708,
10663,
-4575,
7952,
-6800,
-1208,
470,
-1803,
-3784,
3164,
3021,
5686,
422,
-8034,
2238,
11330,
-7756,
8575,
3445,
4928,
5661,
6362,
-2170,
-3321,
4781,
-6490,
-2687,
12520,
8129,
125,
-636,
-112,
656,
3332,
123,
-5108,
-5816,
2403,
-3826,
981,
-9508,
3696,
-1358,
-1309,
2758,
980,
2778,
4927,
-233,
-98,
10337,
-1953,
-1788,
-3157,
2225,
-411,
-6707,
-7048,
2282,
-2148,
928,
1418,
-100,
-3851,
9239,
-1200,
-4394,
-6971,
-6178,
-1702,
7199,
-2164,
2916,
-4737,
-10021,
2779,
6819,
-1397,
-2942,
10647,
-13713,
-2899,
9426,
792,
1867,
528,
866,
-4550,
-5707,
6287,
5559,
3139,
552,
1899,
-1157,
8627,
-1067,
-1239,
2559,
1003,
-5570,
1180,
-6497,
11486,
2125,
7428,
-417,
2102,
5012,
5065,
-3267,
1956,
2730,
1851,
7799,
-4097,
-4524,
-12766,
-4702,
-2960,
5954,
-3482,
-2922,
4883,
3585,
7741,
4038,
-2772,
-6400,
2561,
9848,
-1339,
1074,
5384,
-12660,
-207,
697,
1049,
-2934,
224,
-4970,
-6322,
-8301,
1102,
9632,
4295,
-2320,
2018,
-1423,
779,
2840,
2140,
-2612,
-11217,
-2021,
-1626,
-4933,
21,
-1251,
-2632,
4605,
-1656,
-1471,
-172,
5112,
-2329,
3529,
393,
8896,
2655,
-3643,
-535,
-3732,
-8931,
8777,
636,
-1569,
-6200,
157,
-2987,
-3874,
-1644,
3151,
2349,
-5125,
3949,
3208,
-3921,
-1301,
6734,
9733,
9094,
-788,
9859,
5741,
1404,
-7064,
1719,
3444,
1230,
-1156,
-783,
1923,
-1059,
15580,
-2189,
-2093,
6647,
-3934,
-4682,
2798,
1943,
-1084,
6158,
2306,
-1500,
3551,
6358,
3562,
2862,
4974,
5918,
5770,
-1122,
-784,
1610,
10708,
5275,
9274,
-362,
-7383,
-5429,
-1235,
7364,
-2507,
2307,
-6166,
546,
9797,
-8996,
-4793,
-3220,
1733,
10657,
-5566,
1610,
-85,
-6435,
3123,
-5411,
88,
8097,
4957,
-1384,
-8729,
-3911,
-6260,
10587,
-74,
4928,
8985,
-91,
3477,
1665,
-3909,
6405,
413,
-3849,
8889,
1114,
-992,
-9856,
4877,
4344,
-744,
-174,
7,
1917,
-1443,
5117,
-3756,
2906,
1141,
4608,
-2707,
-1393,
-370,
11332,
4122,
5470,
147,
615,
-363,
-3304,
-394,
-3995,
-4904,
-2953,
3183,
2241,
-5641,
1409,
-9405,
5765,
-5639,
-5109,
2676,
665,
5334,
5921,
-6384,
-2724,
2935,
2037,
290,
-6303,
-5047,
-133,
-11227,
2351,
7955,
-1361,
531,
-5891,
-2080,
-7795,
4889,
957,
7805,
-4852,
4863,
4640,
-2416,
496,
-7110,
-2623,
-3962,
7448,
-6651,
-5627,
-1565,
3833,
-4,
-5575,
-1661,
2416,
-13190,
11112,
-1902,
9450,
-6511,
4276,
-3444,
443,
-414,
6554,
-1869,
177,
-1417,
2304,
1529,
3256,
-3171,
2519,
8133,
473,
5817,
-1878,
12,
-5381,
-181,
-2920,
-5152,
-6112,
2531,
-2370,
1293,
-3714,
-4179,
1999,
1137,
-834,
-1819,
-5633,
4752,
58,
8247,
-3579,
-9486,
-5254,
320,
5104,
1046,
-7996,
-1418,
-3593,
886,
1246,
-1858,
-694,
6652,
2408,
-1464,
-2033,
5958,
-3570,
-1903,
-1590,
1728,
7829,
-771,
-2525,
-4924,
-2341,
4399,
-8497,
-5766,
-1353,
-9291,
-2927,
4985,
3238,
728,
-4314,
-343,
-4759,
4905,
7971,
-2345,
3934,
4741,
10753,
432,
480,
-5106,
7011,
2227,
7310,
-1705,
2035,
-4157,
-7067,
-9261,
-5273,
7528,
983,
-489,
-5379,
5222,
-3397,
-7919,
-5105,
1673,
1451,
8510,
2905,
-7486,
959,
-510,
3210,
9254,
-4353,
-651,
4245,
-420,
4030,
-141,
-11202,
-7134,
6490,
-3004,
6519,
-9132,
2587,
5976,
3519,
-4938,
-474,
-4158,
-5144,
12822,
-8191,
2593,
1500,
-2682,
2200,
-9843,
-2335,
8249,
2874,
-2353,
5029,
4471,
-665,
-6988,
10080,
-2735,
4318,
-5332,
2849,
-1084,
9517,
-2749,
372,
179,
343,
11324,
-749,
-2097,
2887,
348,
8022,
-9021,
-394,
5554,
-2088,
7847,
-4890,
-1421,
-1703,
-1236,
4201,
171,
4244,
73,
528,
-175,
4189,
-3104,
-2466,
3278,
-1815,
-1315,
6245,
-6056,
5919,
335,
626,
2516,
-3590,
-5746,
-1296,
-3198,
-316,
10583,
3021,
1705,
-40,
3369,
-3182,
-3215,
-101,
7885,
-2186,
-9499,
-2687,
-369,
7955,
-791,
-715,
-2385,
-1807,
8256,
-1115,
9599,
-3645,
-1282,
-4284,
3926,
3633,
6294,
841,
-3662,
5387,
-3960,
-4551,
-2928,
4197,
1888,
1154,
-4448,
2654,
373,
-422,
-2339,
3922,
4883,
-1498,
-3058,
1956,
-865,
3232,
960,
-6696,
-1865,
-5415,
4614,
3330,
-574,
-1590,
593,
-2411,
-3645,
-1649,
1160,
-5149,
353,
-7339,
3392,
1855,
-3755,
-933,
-5232,
6741,
-1967,
2560,
-3968,
1452,
-3421,
1558,
14653,
6035,
2690,
-356,
-5210,
3745,
7172,
-1629,
-4216,
704,
-1473,
8752,
3003,
356,
-1568,
-666,
-4611,
2571,
-2690,
70,
-2369,
8267,
-8646,
6049,
3143,
-6235,
-4266,
-1740,
-3134,
777,
507,
8943,
1502,
-2129,
-1270,
-9704,
4282,
-6408,
-670,
2669,
3227,
-2263,
-4924,
-5397,
1523,
-6703,
7329,
-7733,
-5355,
2783,
-2296,
-3822,
174,
-4269,
3074,
-2777,
3363,
-8255,
3741,
215,
6645,
-817,
-1822,
129,
1619,
-8566,
8298,
-6272,
4474,
3286,
-279,
-1982,
-1573,
3515,
-12565,
-3060,
-3847,
-4091,
-8276,
2606,
2730,
-5636,
-1858,
4891,
9404,
2842,
5909,
820,
-6002,
-2535,
2836,
5136,
3236,
1283,
4150,
4497,
-5988,
-2589,
-4627,
-1285,
-793,
3249,
-6981,
-1186,
3382,
-1349,
-1068,
811,
-4718,
329,
-5726,
477,
3292,
-3107,
6264,
3353,
-2930,
-945,
-6299,
-5178,
923,
2650,
-2132,
4,
-7035,
-423,
2936,
2890,
4726,
-2115,
7452,
6015,
3304,
1341,
-2999,
4082,
4775,
6219,
774,
3342,
3543,
1897,
-690,
-3631,
5871,
7601,
-3408,
-1492,
-6571,
-1885,
4026,
-983,
61,
42,
-4644,
-517,
3573,
-2167,
-3029,
-2167,
3688,
10065,
-3047,
3817,
1954,
-4955,
7537,
-3937,
2882,
-2361,
1825,
529,
124,
750,
-5327,
-2058,
-1913,
3949,
4347,
-5123,
3840,
-248,
-3805,
-2373,
2950,
3367,
9424,
9818,
-4304,
-3879,
-653,
1280,
-169,
-2945,
-7140,
-8667,
1758,
6984,
1704,
-7995,
695,
5644,
-1145,
-7757,
-1581,
-1373,
-1931,
1963,
1411,
2665,
5154,
-5153,
433,
-930,
2565,
-4530,
4133,
-764,
-2234,
2385,
-3620,
-43,
-5701,
4430,
-7929,
714,
6573,
7432,
578,
4773,
846,
-8427,
5028,
-2727,
9357,
-5414,
-1112,
6759,
2037,
-9770,
1870,
4222,
-1991,
7147,
-2730,
1585,
-626,
2946,
-4125,
-115,
-521,
2323,
-129,
-6800,
-11283,
1641,
43,
-399,
939,
7788,
16527,
-2232,
7010,
2274,
6056,
1797,
4704,
-1665,
-424,
7369,
-2425,
-3713,
-3909,
-6482,
-8173,
-2468,
-8019,
2332,
1532,
-9243,
-210,
1857,
-4633,
2198,
1316,
-4942,
-764,
1745,
-480,
-1622,
134,
4347,
-2613,
1777,
6262,
-581,
3614,
1438,
691,
4998,
-1461,
-1649,
-1261,
-1386,
4322,
-2319,
1130,
-1878,
-8376,
3261,
2370,
17486,
-5987,
1978,
3686,
-2713,
350,
-1079,
2017,
-3408,
-3365,
-9101,
6670,
-3190,
2040,
-3845,
1629,
-497,
-8426,
-965,
1329,
6732,
-610,
68,
-1865,
1660,
-3012,
5033,
-927,
-817,
7567,
-6130,
-1623,
-8828,
-4296,
-468,
-5800,
-1264,
-1309,
1180,
-4062,
-5034,
4914,
-1964,
-1976,
5706,
-4278,
-5840,
-5193,
4571,
2177,
-4341,
-1089,
-1875,
4871,
6520,
1556,
-1887,
-3761,
2537,
-409,
2164,
566,
-3744,
-2978,
-6154,
367,
-999,
4920,
3148,
-5181,
-3444,
1788,
1710,
3392,
-3915,
-1527,
5054,
-7222,
3212,
-4362,
4411,
681,
2512,
1407,
1357,
1411,
5384,
7719,
-8400,
2194,
430,
706,
-3608,
8867,
1321,
-4486,
13079,
9970,
5980,
-1809,
1170,
2386,
-4082,
2599,
2818,
4327,
-2084,
-1749,
273,
-5141,
-10263,
8156,
2786,
-1492,
-20,
2540,
7141,
6690,
-5207,
4302,
-2935,
1899,
-7060,
-3543,
2683,
1598,
-101,
922,
1737,
-337,
-10257,
2590,
1984,
-1491,
4873,
-9509,
-6573,
-42,
-780,
2386,
6473,
2470,
2757,
-5213,
2964,
1438,
-2910,
6814,
2711,
-6955,
-6463,
3816,
-5408,
3452,
-393,
1596,
2610,
-467,
-5810,
-6488,
4534,
684,
-6056,
1606,
-2187,
2238,
2923,
1285,
-4119,
8068,
-6767,
-1327,
-421,
-1985,
5968,
6629,
1117,
-5663,
1061,
2180,
4874,
1534,
98,
743,
-12195,
-6141,
-3644,
6891,
3284,
516,
3442,
-5000,
4750,
5192,
6259,
662,
-4072,
-2408,
-6320,
-3825,
-926,
-1168,
2252,
-5935,
-864,
-2925,
12481,
-3294,
843,
7461,
-8841,
-7273,
-4344,
1405,
-11545,
1190,
5394,
2770,
-5332,
-6772,
-4187,
811,
-1149,
1305,
2029,
246,
3519,
1939,
3309,
3742,
-3250,
6171,
3760,
833,
3305,
4429,
-1653,
4117,
-1562,
1918,
-3881,
-7437,
-1097,
-3537,
-2153,
-580,
361,
5799,
2894,
2879,
3046,
-1118,
-1424,
1810,
5846,
-6988,
8778,
-3973,
-3059,
3986,
483,
-856,
1943,
-2600,
4687,
-4290,
-5479,
2811,
4716,
6060,
537,
222,
-3856,
-6574,
6601,
-7382,
2241,
11665,
-3183,
-2017,
-2593,
457,
-395,
-917,
1939,
-5379,
-10469,
-1092,
7448,
1719,
4750,
-3554,
267,
-2510,
-6193,
-2076,
4294,
-3684,
-4181,
-11635,
-6163,
3350,
5431,
5397,
-2921,
3832,
2164,
5771,
-2598,
1396,
-4026,
1266,
-7306,
-763,
-9753,
-9157,
579,
-3798,
2433,
855,
-8964,
8944,
110,
-9456,
-199,
-5342,
6186,
-12577,
-6849,
10237,
870,
2749,
-3138,
-2904,
235,
-3901,
628,
-1595,
5977,
2879,
4226,
-1289,
-12509,
5003,
734,
-1871,
-5836,
-3634,
13572,
9492,
856,
-992,
-2408,
-6299,
4810,
-876,
-4884,
-1501,
-404,
-3133,
-5247,
10273,
637,
526,
-497,
-4314,
-2636,
2342,
-2517,
1911,
-2495,
-2350,
6282,
-3938,
13246,
4377,
1465,
-11184,
4587,
262,
1794,
-3158,
-419,
-112,
-529,
-2571,
-5337,
8288,
5428,
42,
-314,
7599,
-1339,
1809,
-9246,
5121,
8744,
-156,
10203,
1241,
6604,
1700,
971,
2245,
539,
3400,
9546,
2862,
279,
1468,
5562,
-1084,
-4029,
-5213,
1228,
-266,
5636,
5732,
-12,
2479,
-2540,
659,
-1644,
-4835,
6982,
462,
-709,
3571,
2250,
4821,
-1054,
-606,
2103,
1832,
-5652,
-253,
-10123,
101,
-4024,
-7506,
-2607,
-4117,
-8026,
-1961,
-5561,
2342,
-352,
-2351,
2146,
-2751,
3055,
1716,
-4729,
294,
633,
-3784,
456,
-866,
-3500,
149,
1513,
2850,
3290,
5082,
454,
697,
636,
2038,
2843,
-3729,
5830,
-4991,
1849,
2844,
-2736,
2437,
-950,
5360,
5892,
-64,
7576,
-2875,
-2218,
8922,
-623,
-6136,
-830,
882,
7591,
-501,
7572,
-7666,
-7258,
-3717,
61,
-4705,
-2954,
-1352,
2658,
7947,
4134,
7993,
-3619,
4152,
-5783,
2488,
1110,
269,
143,
4061,
-6418,
-4637,
3950,
-1284,
58,
-2231,
2915,
-2240,
-1834,
-4382,
967,
2760,
-1641,
4161,
-2847,
-7794,
2705,
3743,
955,
-3302,
7471,
4575,
1958,
6484,
-6735,
1287,
11507,
489,
-4081,
381,
2716,
3957,
-7895,
7855,
2672,
-4887,
-11242,
5380,
886,
-4354,
2071,
6296,
676,
-1292,
-8329,
-5789,
6633,
-2497,
813,
1369,
385,
2475,
2171,
-2410,
-2309,
-7333,
4078,
4234,
1229,
-1582,
1606,
3868,
77,
-1618,
-7701,
3705,
-2066,
691,
2225,
-3471,
-9471,
5635,
-3550,
-4690,
-11403,
7178,
3461,
213,
942,
5784,
6008,
1289,
191,
-1324,
708,
338,
5956,
-7299,
-2653,
-3159,
-774,
-1203,
-2772,
1444,
4510,
1653,
577,
-2308,
-2200,
-3519,
-4390,
-1723,
-1811,
3270,
8327,
3661,
-1311,
-1658,
8006,
13187,
-963,
-3229,
107,
7,
2878,
-3608,
928,
-1816,
5142,
-1439,
2646,
-730,
-7717,
231,
2611,
8014,
-1378,
1664,
4698,
-6183,
-4093,
-3308,
4397,
-1119,
-6001,
3819,
873,
-8362,
-4002,
-5319,
4715,
-817,
-3943,
4437,
5014,
1273,
7298,
2077,
-1324,
5729,
80,
-5839,
-10625,
-790,
-3593,
-5087,
5987,
-62,
3513,
-7700,
-7566,
4648,
-8119,
-697,
-7684,
-2222,
3745,
-2597,
-4996,
-686,
7317,
-3757,
5457,
-277,
-6559,
-1567,
1224,
2308,
-8574,
5009,
8597,
-2369,
-5775,
9177,
9355,
2047,
2418,
-14630,
1306,
-4428,
3576,
-6127,
7780,
2512,
2139,
-1620,
7814,
2960,
9138,
469,
-5286,
-4529,
5576,
309,
-1555,
1347,
-3810,
3148,
1051,
2290,
7177,
2494,
999,
1236,
2094,
7568,
-4813,
-9674,
1427,
-986,
6453,
-6006,
-7749,
-1908,
-6327,
-3111,
-4697,
5070,
592,
4548,
-1985,
-683,
6800,
-2070,
-85,
9425,
-1146,
6095,
3973,
339,
3952,
-6571,
-2149,
6423,
315,
538,
-3305,
1940,
3186,
-803,
-6103,
4185,
-6007,
-497,
4687,
-3611,
2781,
4688,
494,
-4822,
-628,
-7143,
-538,
-114,
323,
3292,
2167,
-1793,
-1995,
-6898,
2535,
2677,
1824,
5156,
-5880,
695,
732,
-3049,
-5370,
9485,
1595,
2381,
454,
-4968,
3870,
3670,
1638,
1104,
2243,
-7959,
9671,
4111,
-5877,
3220,
-829,
-6418,
-3524,
-3844,
5880,
-4752,
-350,
10136,
5835,
-914,
4696,
278,
-317,
-4449,
-1092,
-2937,
3169,
4337,
-2673,
-6018,
-4351,
7883,
-5205,
5262,
1155,
6600,
626,
2506,
-9452,
9737,
4517,
-8063,
608,
2597,
-1545,
-8354,
-2990,
-1675,
-5264,
3415,
5328,
-3577,
8702,
-7412,
3546,
-1200,
-3596,
-2052,
1410,
-11445,
4341,
-6596,
570,
-3142,
4908,
4883,
-17,
-2330,
3696,
-8525,
-2553,
4526,
5,
0,
1555,
-4084,
-2323,
-1508,
-2949,
5480,
-9239,
-1332,
-8675,
-2736,
-1520,
5263,
-3430,
-102,
-907,
-12903,
-2970,
-11502,
-7586,
-5765,
4202,
676,
-2699,
-4957,
13088,
11353,
9153,
-6294,
-9636,
80,
8883,
2925,
2858,
1324,
4620,
-197,
18575,
-3518,
-493,
-4752,
2014,
-7941,
11106,
-2415,
3296,
10605,
16168,
2387,
2265,
7336,
4613,
421,
4256,
-3840,
13205,
-5903,
7508,
-5611,
-4380,
5732,
-24150,
473,
7230,
-11960,
-8531,
5262,
5488,
-2692,
4601,
-7317,
-4161,
-2736,
-9219,
-9556,
8159,
-7070,
-11150,
-20,
-4494,
438,
3306,
-7284,
-8769,
5140,
-6772,
-2857,
-5089,
-3874,
-31,
1454,
9022,
-1331,
-7985,
3425,
1162,
3159,
-9196,
3023,
-842,
8354,
-7956,
-1538,
-7321,
6104,
1263,
2722,
8889,
-10852,
-15415,
-7735,
-4524,
-2691,
3335,
-8854,
-2041,
5986,
1440,
2053,
-9123,
2077,
-17234,
-3382,
9863,
-7004,
-942,
-4399,
997,
-8692,
10722,
-6371,
-11370,
-9687,
2494,
-4538,
6247,
-11029,
-4881,
5377,
-9175,
5586,
9227,
-1619,
8686,
1657,
-18473,
1663,
6897,
1608,
-4700,
5488,
4876,
-1156,
1848,
-6209,
6242,
10298,
-3076,
-497,
5753,
-5942,
-6433,
5259,
4719,
8609,
-3298,
6421,
2583,
7340,
5885,
-6443,
-10309,
-9475,
-6564,
-6719,
7800,
3763,
6345,
-3459,
12256,
6939,
-12297,
-7280,
2971,
4452,
-5566,
-5647,
-6121,
1020,
2774,
-2049,
-12393,
-9125,
12838,
-2522,
-2612,
8360,
-6483,
-2692,
-4754,
15719,
872,
3970,
2342,
4287,
6165,
5613,
3255,
-8199,
2033,
-6526,
7319,
3003,
-10873,
5356,
2554,
-12302,
-5939,
-7264,
155,
3663,
6913,
15413,
7199,
-791,
6205,
-4754,
-761,
5395,
-1412,
526,
-955,
2783,
-1271,
14428,
2805,
-5961,
-2542,
3690,
-2631,
2710,
2895,
-2508,
869,
4609,
7466,
-290,
1446,
-20498,
-3428,
4553,
812,
5842,
4394,
-1450,
10292,
7602,
10181,
933,
-3578,
-383,
-1649,
5363,
1242,
-441,
6901,
6994,
-2329,
8018,
9278,
-1980,
4897,
7410,
10871,
-6839,
-6628,
-5043,
-7262,
-12068,
1875,
-4788,
-7712,
5960,
-5541,
9640,
10474,
1625,
-9198,
-12464,
-5019,
-7165,
2677,
902,
6304,
-7003,
-8671,
-2544,
4722,
-2472,
-3086,
-1082,
-15786,
-7553,
3596,
1878,
-8325,
2352,
4469,
-6811,
7048,
-3737,
711,
-9476,
488,
-3108,
-1364,
-5447,
7182,
8362,
-9044,
-3326,
2465,
1137,
-18747,
-3772,
-8484,
1403,
1365,
-11509,
11256,
-7356,
-4019,
-885,
-9983,
7377,
9401,
-7668,
978,
57,
-12342,
8069,
486,
1675,
-9333,
-10166,
-244,
11526,
14615,
1397,
-3778,
3935,
5965,
-9074,
-9441,
-4412,
-2685,
-9963,
2213,
12268,
-674,
3480,
-7022,
2830,
5618,
-5819,
-7736,
1381,
1344,
-8115,
3540,
15260,
-8914,
10943,
-794,
-9770,
1991,
918,
-5671,
-6409,
7950,
11317,
-4712,
-11587,
-12227,
1599,
1895,
3246,
-2431,
-1053,
9499,
12560,
-11092,
1384,
11595,
-10099,
667,
-4358,
-2542,
-1576,
-7319,
-6969,
397,
8045,
8178,
-9104,
-1953,
3024,
-7717,
4995,
1670,
-11456,
-3519,
4767,
2087,
-1130,
-214,
-10970,
11086,
7956,
4809,
-1553,
3335,
2136,
-4713,
-849,
5233,
1963,
9728,
-5691,
-6884,
5517,
-1004,
6881,
3361,
-14103,
-6639,
10865,
5876,
14259,
-305,
3466,
-3825,
-5515,
-15301,
2291,
-7972,
2579,
-7303,
9708,
-5266,
-848,
-841,
21388,
3182,
10400,
6430,
-7028,
-1575,
-13996,
1546,
9083,
-14472,
-973,
8970,
6437,
8006,
2205,
-3603,
4708,
-14169,
-17501,
1029,
8237,
-1027,
5350,
8112,
1337,
9365,
-3177,
5582,
7228,
7632,
-6880,
-2917,
4399,
-7513,
-6849,
2754,
3150,
-4715,
7769,
-3479,
-9854,
1682,
-2371,
3300,
-1316,
8093,
-4640,
-12904,
-5862,
-2883,
-5483,
-1868,
-11364,
7864,
5990,
-6310,
-7799,
-244,
1049,
-12944,
16,
6332,
-5272,
4646,
-3263,
-7699,
2617,
-2857,
7373,
-6256,
15152,
5857,
7047,
-7083,
7058,
5858,
10739,
-7052,
-3899,
7140,
-4769,
4638,
-6331,
1852,
2683,
2917,
-702,
82,
4911,
-429,
-11289,
10518,
1664,
-2772,
835,
11289,
41,
26,
6131,
-7313,
-1216,
11977,
-2500,
3568,
7597,
12371,
626,
4146,
1934,
694,
12465,
16264,
-2792,
2012,
3064,
6191,
357,
-2262,
2302,
3684,
-13173,
-9236,
74,
-5278,
-5586,
3129,
-13022,
7527,
599,
2116,
4884,
-6719,
2134,
12100,
-8130,
-8043,
-3969,
2252,
-610,
-2759,
-3075,
6835,
-5785,
7161,
-5809,
-635,
-4807,
-1369,
2593,
646,
-3108,
-2482,
-1412,
5454,
16210,
1180,
-1500,
4255,
1552,
-3660,
5703,
-1219,
2357,
-2419,
6345,
164,
-1629,
-2029,
1741,
1501,
3232,
-2873,
1134,
1539,
4335,
-3823,
-4810,
9320,
-1130,
-591,
-7646,
-7115,
6876,
331,
-9799,
9205,
-1534,
-6644,
-7197,
-5441,
2844,
5178,
-5790,
-3728,
-6264,
-4470,
3995,
7274,
2341,
-1845,
-5084,
-5543,
-11325,
-3610,
4878,
-9621,
-15695,
-2245,
3833,
10347,
8443,
-8048,
-6243,
-1007,
3644,
-11396,
-6917,
-5568,
-10373,
100,
-3963,
4068,
7192,
851,
8580,
1303,
7278,
-7970,
895,
-1399,
10937,
5411,
-3439,
-273,
7229,
-315,
4742,
-1262,
-793,
-8162,
-7231,
-5956,
1394,
-1569,
1002,
4761,
-1205,
-8840,
-5236,
7788,
-11217,
5330,
-7454,
-1381,
10458,
-2731,
-6454,
-8354,
-8519,
12070,
-426,
2865,
4208,
-3889,
-7458,
3005,
-3280,
2792,
-2407,
-483,
-15390,
5613,
-3145,
1771,
11359,
-1525,
1070,
-6706,
4130,
540,
377,
-14655,
-821,
5168,
-9521,
5208,
3966,
1964,
6860,
6896,
6863,
-633,
-13661,
17309,
8022,
-11677,
-15705,
-5123,
-1128,
-3000,
-12426,
6418,
2995,
5537,
4985,
3435,
-6761,
-5004,
10562,
2757,
-5746,
-1433,
-9779,
-5998,
6975,
-1143,
2385,
1945,
-12967,
6911,
-10955,
-7886,
7171,
5235,
4911,
2678,
-11930,
2687,
6441,
-1238,
-1065,
-954,
-4067,
8381,
3131,
2416,
-3866,
-1635,
1146,
-7195,
2790,
11463,
5239,
-888,
3463,
5687,
143,
844,
-8925,
7986,
7894,
4571,
1417,
-13326,
-1394,
3845,
10416,
10527,
5688,
-4019,
-591,
12104,
-609,
7206,
312,
-2547,
8699,
-8389,
2184,
-649,
-6499,
-800,
8382,
5401,
765,
-9262,
9669,
1461,
-5230,
4486,
-1921,
-8747,
-4206,
4045,
3576,
-1416,
1845,
-6730,
-1554,
-893,
-4094,
1395,
5363,
-997,
-10204,
-14565,
6010,
9107,
-1264,
927,
8039,
9725,
-1456,
4268,
-15059,
-2939,
13203,
4592,
6919,
1622,
11589,
-3501,
-11663,
-7925,
11403,
-362,
1066,
-8850,
8393,
7175,
6309,
10056,
8307,
-2571,
3913,
3167,
-13474,
1926,
2616,
-4845,
3685,
-2401,
3908,
-4326,
7069,
1280,
-1847,
4024,
13653,
2471,
-4497,
-10186,
2969,
7,
-4206,
-4127,
6421,
1483,
-765,
1647,
-13641,
3284,
-6365,
2699,
-12225,
10491,
263,
-6939,
490,
-2592,
726,
2771,
-1277,
2375,
11052,
8090,
-361,
-1331,
9022,
6564,
1714,
-1293,
-9193,
-5464,
1613,
-367,
-858,
5794,
-5747,
12953,
1623,
-1218,
-4625,
6672,
1649,
946,
4329,
-2030,
1301,
-455,
9121,
-8028,
-11173,
-13357,
5437,
-3161,
3867,
-2917,
-374,
-2137,
-8944,
8603,
-12438,
2245,
-5584,
14861,
20915,
-6031,
-3843,
-1241,
-3855,
-3220,
9139,
923,
-1500,
-8394,
-12246,
2466,
10132,
-749,
7016,
-8916,
-1822,
15421,
6059,
-560,
-985,
9985,
6396,
-5809,
5774,
-3621,
-6495,
8289,
13813,
4387,
4301,
-2900,
11954,
168,
-1356,
-8145,
-469,
-842,
-4595,
2104,
3925,
9311,
9709,
4613,
-7678,
1830,
-4948,
8023,
-9914,
-1526,
-6764,
-3191,
10021,
-4134,
6438,
1314,
-6431,
-6,
1255,
-9360,
-4505,
8595,
8541,
9740,
-4350,
4581,
220,
-3456,
10526,
4939,
-8356,
8,
-7894,
-5729,
4618,
3873,
-1435,
3111,
-6555,
16,
1834,
205,
-2327,
3728,
-6190,
-418,
-19641,
1569,
-12190,
4384,
-3152,
1347,
6393,
11369,
6407,
-8115,
2484,
-13417,
-2166,
9194,
2908,
-363,
-7230,
2319,
-857,
-9854,
664,
4591,
-7584,
-8833,
-3736,
-13026,
-4496,
-3690,
-316,
-3937,
-6230,
2904,
-5078,
-1131,
5945,
2342,
-8639,
-780,
8205,
13212,
10791,
8535,
836,
1881,
-3776,
1032,
-1780,
-3674,
11773,
-2348,
-5100,
3996,
11086,
-14233,
3883,
-3213,
5762,
4208,
-14582,
-1490,
-7587,
-1552,
-3709,
-2647,
5905,
7591,
172,
14151,
4916,
-1215,
-9345,
-7265,
-5545,
-8079,
-1599,
-16223,
5981,
2189,
-5864,
-1802,
1538,
-5114,
2042,
-8051,
-5520,
7061,
-9075,
-1576,
11700,
-4488,
6146,
10250,
-2551,
-1054,
-11654,
-2476,
-3218,
-1042,
7845,
3854,
3269,
9576,
11894,
4090,
2593,
6121,
2769,
-8780,
5813,
6955,
5710,
-13262,
10498,
-974,
8567,
4842,
2358,
14439,
-5012,
7059,
6823,
2860,
7235,
3474,
9410,
8121,
4918,
3424,
-6538,
-3812,
-1513,
-2411,
-3033,
2204,
-3086,
667,
4308,
-2180,
-6433,
3893,
-9434,
1416,
5819,
464,
11836,
151,
-380,
3826,
-1436,
4912,
232,
3015,
-456,
7882,
-4310,
5407,
-4533,
4218,
5834,
-3240,
9748,
340,
-10098,
1771,
1852,
-173,
-7738,
-13300,
-5909,
16,
-6711,
-1921,
-4672,
7074,
4488,
2650,
-854,
7374,
15837,
-4307,
-4269,
-10033,
8401,
-12685,
-3783,
4713,
5989,
11277,
-865,
5653,
-677,
2947,
-127,
-6423,
7336,
-14113,
-13868,
-3547,
9975,
4378,
4061,
475,
5598,
-6302,
-11833,
6538,
11708,
-9050,
683,
-1718,
-2289,
-5201,
-1958,
4949,
-8326,
4113,
9771,
-3753,
-1957,
7163,
2094,
3707,
1638,
10267,
204,
-472,
-377,
970,
-6792,
-12918,
4706,
-1078,
-10139,
7032,
4599,
-8419,
-1964,
-3823,
-2842,
5603,
3264,
6225,
2690,
-2260,
3978,
1688,
9782,
-4439,
-2798,
3064,
10750,
-2414,
3997,
1095,
6887,
-9693,
4227,
4028,
3996,
-1733,
3923,
10103,
697,
-7732,
-8439,
623,
3743,
-821,
-3204,
-1999,
1938,
-16410,
-9997,
12228,
-11741,
-3032,
-4963,
539,
8199,
-2203,
9376,
-1894,
4337,
-600,
866,
-533,
917,
-2349,
-9454,
-1349,
6462,
30,
17650,
1820,
2617,
-5205,
-10397,
6182,
889,
-5732,
998,
6699,
-15837,
-4929,
-3403,
-6171,
6198,
-3812,
-5118,
-3810,
-7575,
3987,
-2674,
5173,
-699,
-1297,
-12986,
-1083,
854,
-4636,
3106,
-4590,
-4061,
6922,
2308,
1847,
-7753,
1582,
-8780,
-930,
6890,
-9438,
779,
-12394,
2410,
-3276,
3267,
-962,
6535,
-8355,
-736,
5228,
-2140,
-854,
-6654,
13954,
-1322,
11974,
-6922,
-11061,
4579,
-1023,
-3402,
1605,
-1441,
-7980,
5913,
1231,
1997,
10429,
659,
3611,
5977,
3592,
2904,
-2006,
-1479,
-2240,
2272,
-3887,
724,
-9226,
806,
5428,
4375,
10185,
4658,
13826,
-1955,
17449,
-7893,
5275,
2145,
-967,
5788,
-4523,
-16629,
442,
1927,
1121,
1116,
5820,
-10166,
2145,
-1462,
3358,
6957,
8614,
2564,
5911,
14695,
759,
-6394,
-42,
8844,
5338,
5245,
8131,
5028,
6154,
96,
-4576,
15606,
-1104,
990,
3281,
-1768,
3186,
-2651,
663,
5010,
9392,
-571,
-2782,
-5722,
10943,
6257,
1627,
-3813,
-4394,
2022,
-526,
761,
-2822,
-12896,
3200,
-117,
7391,
6302,
-3465,
-4193,
15704,
-4519,
8623,
11060,
4739,
-3576,
-6582,
9056,
213,
-88,
-1293,
-3142,
-6961,
-7367,
3345,
57,
2278,
4785,
4323,
78,
-4771,
-2339,
9655,
15581,
2868,
276,
2537,
-8853,
-5818,
-363,
-480,
4072,
19,
-1256,
9133,
2460,
7525,
11174,
-4789,
-3324,
-6715,
-3708,
5102,
3748,
1207,
5194,
2058,
4386,
1126,
14258,
8860,
12142,
-8089,
3399,
4198,
-7051,
-189,
8793,
-6862,
6741,
-7392,
15571,
-647,
-1614,
3563,
-2864,
-3553,
8301,
8638,
1578,
3084,
5665,
-4048,
4163,
1404,
-2311,
-2263,
-3765,
-2880,
-7267,
-684,
1005,
3353,
-2135,
1112,
2060,
12525,
9558,
8871,
7947,
-11021,
-8446,
-1679,
-853,
3214,
-8830,
-428,
16838,
6214,
-3644,
4012,
3631,
-4440,
2094,
-11179,
10369,
4262,
-1315,
-3950,
7973,
6885,
6993,
3474,
1138,
-914,
-4025,
-3679,
-6856,
7449,
10132,
4911,
2188,
7534,
3093,
-1331,
9317,
362,
-10538,
-2045,
9330,
-5027,
-663,
-1176,
1426,
2608,
-1765,
1608,
-11802,
-15302,
-13208,
10361,
7722,
-5410,
-7319,
-9551,
-2923,
789,
12707,
1643,
-19522,
-6478,
40,
2676,
-1966,
6247,
7519,
2719,
9239,
-1580,
16461,
561,
-2369,
-8655,
7628,
3985,
9236,
-8702,
7172,
6554,
-12196,
-1474,
-18,
-763,
11371,
7884,
-5191,
6629,
2806,
8028,
6187,
-362,
-6968,
3148,
7688,
4442,
-4156,
240,
2037,
-1627,
-2200,
6446,
3609,
-3581,
-6387,
2799,
-5224,
1230,
8014,
-3170,
2882,
-14,
7026,
-6548,
9782,
-271,
7853,
792,
-11943,
-3573,
-2133,
-8219,
859,
982,
3067,
9580,
11349,
8809,
-4423,
125,
-6908,
6459,
-225,
-10723,
1635,
7919,
-13562,
-4902,
11333,
-5156,
856,
-6245,
-1261,
270,
-6846,
2768,
342,
1259,
-19448,
1312,
2286,
-1530,
10058,
-7175,
1094,
1836,
3862,
-3708,
2576,
-3664,
10324,
6638,
941,
-2876,
-5089,
9631,
1876,
-6186,
-4375,
-18885,
2923,
3141,
-3745,
2316,
-9240,
-2113,
-1310,
4949,
-7619,
-10969,
8457,
-18763,
2868,
3931,
-3359,
10575,
-8728,
3096,
5367,
-14885,
-9335,
-2249,
-10121,
-8200,
7705,
-11654,
985,
-4958,
-3477,
6727,
-6529,
-1470,
2052,
-6562,
5968,
-13211,
938,
7551,
3133,
7542,
-17332,
-8324,
7096,
5247,
-8660,
6204,
3548,
2556,
-4582,
1096,
-1141,
8288,
-5226,
2167,
2169,
-300,
-3307,
1745,
595,
-11667,
-13126,
-10578,
-4440,
9741,
4915,
1534,
4736,
-3357,
2190,
394,
-8572,
17503,
7187,
-6241,
55,
4854,
-4259,
19150,
10194,
-5723,
14025,
665,
1860,
196,
-2763,
-9100,
11267,
5988,
6613,
-14320,
-330,
-3450,
-10268,
-7713,
1990,
-3378,
651,
-1753,
8237,
3644,
-3878,
-15461,
-100,
13521,
4430,
-4129,
-3971,
-4857,
-13259,
9775,
-7647,
3372,
-3690,
252,
3300,
-260,
-5874,
-3110,
-954,
-1310,
-2794,
-9853,
568,
-5764,
1356,
-6695,
-1812,
-4965,
-1539,
-11070,
5360,
4273,
7925,
2011,
-9846,
-4777,
-2278,
-5268,
-1277,
4238,
4186,
-4881,
4659,
-15123,
3097,
-1066,
-1748,
8910,
4108,
7604,
4259,
-6902,
5902,
19,
-798,
5121,
1699,
802,
6811,
14587,
-8411,
-4571,
-535,
5982,
2914,
-429,
-6457,
2192,
-9336,
1304,
4819,
7211,
-3193,
7013,
2251,
5817,
-3592,
11104,
15549,
4073,
-1983,
-3737,
5390,
1877,
2048,
12436,
7658,
2949,
10915,
847,
-8981,
-606,
-2669,
4359,
-1914,
-5003,
11221,
-7727,
7529,
-444,
-6865,
221,
-1229,
-2793,
20903,
-3977,
2854,
-11117,
7390,
311,
-3318,
-9927,
-6396,
-3466,
3687,
4959,
-291,
8754,
-4814,
4456,
1232,
-4602,
-6989,
5172,
-12165,
13101,
-10479,
-176,
-14036,
6234,
-8445,
-3253,
2474,
-2277,
5589,
-6721,
-1349,
-3446,
7635,
40,
-10524,
-1180,
-6137,
8373,
-14782,
8249,
-1103,
505,
406,
-5605,
-1889,
-7104,
-12846,
8710,
425,
3274,
-3124,
-3300,
312,
-8409,
5929,
7338,
5461,
4778,
-3045,
7704,
-7324,
-1853,
-2954,
-2816,
-7334,
916,
-5540,
-3360,
-303,
-13064,
631,
11954,
3435,
12502,
-7891,
-3799,
-2394,
1319,
-3073,
-8324,
-7327,
-281,
2390,
-1381,
-6354,
-7142,
-341,
-8009,
-3375,
-7014,
-2150,
-7539,
-4089,
6644,
-1079,
-5461,
9346,
1459,
-3256,
-8859,
6175,
-1192,
-8455,
-2727,
-3638,
4292,
2624,
-7961,
12132,
1466,
2670,
-5273,
-939,
14841,
1528,
-6919,
13648,
-3049,
5203,
-7013,
-3946,
-1864,
1954,
-9067,
130,
-6834,
289,
-604,
-6831,
-2412,
5497,
5792,
166,
5668,
-12767,
8811,
3997,
2391,
-2132,
-2067,
4847,
-4258,
-6778,
-1415,
-8163,
1536,
-5911,
10057,
-7274,
-4054,
1107,
12544,
709,
1063,
244,
639,
-4399,
-7193,
-6564,
-8811,
1519,
-4049,
-8507,
-7436,
-7841,
-10836,
-7138,
-2831,
6831,
8095,
-6337,
-13838,
-4976,
532,
10866,
416,
-2702,
-4351,
-4425,
-3236,
-1919,
8494,
-4452,
9697,
2704,
-4065,
-15855,
7247,
-4688,
-3213,
-4309,
5146,
-3495,
651,
-13839,
-2840,
-1210,
-7863,
-1328,
4090,
-6051,
-11375,
2079,
5649,
-5914,
-8525,
-6450,
-7962,
-10426,
-5339,
4359,
-2563,
-2176,
11816,
2340,
-8097,
5086,
11490,
6443,
-5509,
-11597,
-6320,
-12371,
-2817,
8872,
-3122,
-1571,
-9021,
4048,
-3989,
10974,
-754,
3396,
-9983,
-6094,
-809,
8051,
-4829,
3326,
3526,
-5191,
2476,
-4467,
5411,
-8248,
-2274,
4642,
-12173,
-672,
5904,
-4066,
-12065,
-9967,
7952,
1955,
11557,
4920,
1114,
-2714,
5497,
3456,
7,
-15948,
-189,
10009,
-2091,
-3038,
-5103,
-6071,
-14140,
6850,
-18942,
7039,
-503,
5539,
5156,
2065,
-4609,
6514,
1020,
5391,
-8821,
-5214,
2885,
-5890,
2842,
5897,
4862,
3767,
1801,
-2100,
-3222,
-8483,
-11599,
1874,
1280,
6595,
1342,
3897,
-4245,
-9175,
-3288,
474,
-11585,
-1228,
18661,
-1932,
-168,
-5374,
-1627,
-13726,
2505,
-1427,
5500,
8011,
7413,
-5261,
-15601,
1457,
1419,
-5337,
3157,
-9692,
3326,
2295,
-6998,
-3752,
13559,
-8006,
5396,
1671,
-13140,
-2481,
-3208,
-8032,
-11239,
3917,
-3741,
-2237,
-5140,
8230,
2152,
3417,
-10085,
11183,
4163,
-5730,
108,
3845,
4726,
5069,
-6944,
1980,
-8757,
2653,
-4871,
-1379,
7508,
2047,
-3777,
-5647,
2655,
-3031,
-6349,
5074,
12764,
2624,
-905,
16427,
638,
4448,
-7651,
-11992,
-3648,
15684,
-10913,
-57,
-1392,
-4655,
5539,
-7651,
-6534,
-8242,
215,
2033,
4701,
-3135,
9355,
6923,
196,
1805,
7971,
-8401,
2820,
-720,
-1445,
12499,
9546,
-1270,
5477,
-3071,
6089,
6636,
-13548,
-1490,
296,
-5382,
-10117,
-1029,
2935,
-1144,
2331,
2908,
5142,
-13042,
2332,
4303,
-3924,
-644,
-395,
5706,
-7382,
3371,
10612,
1321,
5174,
8345,
-8229,
-5439,
-2856,
916,
-3004,
1139,
-4572,
-7125,
-10292,
433,
5305,
4761,
-7102,
-15560,
9409,
-12442,
8964,
925,
5123,
4199,
8961,
6415,
-2705,
-3328,
454,
-4256,
3483,
3579,
14151,
-282,
17624,
-3582,
-4300,
-3736,
2359,
-7002,
8275,
3872,
4068,
3078,
-2321,
2228,
10119,
2001,
-5780,
-12309,
1212,
-1822,
6163,
7945,
4425,
-7812,
-6625,
-1509,
9383,
-2901,
-565,
5686,
811,
2262,
-2155,
-5961,
-640,
9079,
-3278,
-2335,
-6103,
-12241,
-2406,
4924,
-5636,
-2175,
7780,
-1712,
-6228,
-2761,
4575,
4758,
-2445,
-1558,
10264,
-2368,
17036,
578,
1761,
-3815,
7530,
-2587,
12663,
5900,
-16663,
6279,
-6677,
711,
8258,
-3397,
14667,
7673,
-7823,
-16552,
-6211,
-4734,
2279,
-1903,
4930,
10046,
8707,
2683,
-1315,
7957,
-11031,
-2613,
-1754,
-6272,
-2944,
-9664,
-5921,
198,
3042,
7721,
10584,
4155,
8247,
281,
-2555,
-13373,
521,
2809,
-177,
8813,
3807,
239,
-705,
7815,
-5962,
14234,
268,
2366,
-2518,
-91,
-972,
-11256,
5760,
179,
-12745,
-2056,
-4890,
-5936,
-6511,
-2198,
7895,
-2550,
-3569,
-8844,
-5127,
6410,
1871,
363,
-11455,
-4466,
3641,
-16210,
3808,
-12,
-3777,
3615,
3432,
-4134,
5134,
1113,
7600,
-334,
-14186,
1206,
-4772,
2982,
4852,
5201,
-1077,
-1858,
-7521,
8474,
525,
4886,
8017,
2909,
-675,
2884,
-5448,
-5440,
-7484,
-8558,
-10432,
-1891,
4530,
9898,
-547,
2481,
-663,
-7071,
-3012,
-9346,
462,
1201,
-7027,
-3023,
7360,
6537,
-6633,
7753,
-2096,
4392,
6652,
7789,
-1233,
-8085,
-2011,
-1888,
5892,
12268,
3964,
8990,
12394,
-11119,
9759,
-1716,
-1322,
84,
798,
11419,
7320,
11036,
-5473,
13394,
-3267,
256,
-1116,
3883,
-1665,
1885,
6646,
477,
-4246,
-2498,
15097,
-5609,
-5762,
-10020,
-3473,
-418,
12034,
10646,
-13979,
-2832,
-3105,
5638,
-679,
3507,
5504,
-8366,
-2073,
6808,
-1300,
70,
3954,
14535,
-16961,
-5444,
3746,
5589,
2962,
-104,
8588,
-15731,
1561,
12162,
9997,
-1463,
2297,
-13226,
-6533,
-14595,
-3743,
3990,
1256,
-4448,
-2244,
2984,
-1077,
4230,
-9363,
-10545,
-9539,
-9146,
15791,
4184,
408,
-8024,
3196,
11554,
7515,
-3483,
-5137,
-8179,
-3159,
-9437,
1090,
234,
1053,
7162,
3205,
10802,
-9554,
7566,
4142,
-3362,
-6981,
-6131,
1332,
2228,
10606,
-2981,
-617,
-7626,
2266,
2349,
-7943,
-1827,
551,
-5605,
-4163,
-6516,
-5032,
-7003,
-477,
12391,
5113,
3050,
8277,
1107,
1638,
-4916,
-9206,
888,
-2281,
-4771,
-4180,
-9724,
4190,
-660,
-4316,
9533,
-11893,
-9597,
2408,
-366,
-2629,
3310,
274,
1386,
-12610,
7812,
13542,
-3483,
4196,
14994,
4828,
5943,
5386,
-2968,
6665,
6288,
-3270,
2013,
5902,
6608,
886,
-11641,
-4879,
5928,
-5857,
9929,
-4344,
2303,
5223,
-2043,
7026,
-612,
6714,
3286,
7215,
371,
-9789,
-13631,
2834,
9197,
6515,
621,
-2132,
4587,
-5598,
-6680,
-2433,
-20160,
-8796,
-2167,
-42,
3234,
-11292,
1332,
-14288,
-7776,
809,
3935,
2871,
-554,
1921,
-2041,
-93,
15651,
633,
3197,
-3472,
7844,
3506,
1159,
-12383,
-1529,
8277,
-2797,
-5273,
3067,
-7537,
-1794,
1210,
-13563,
-9071,
6402,
-3132,
1974,
-5560,
-8890,
-5897,
3362,
884,
-10942,
-3467,
-12206,
5655,
-765,
-13618,
5096,
3122,
3317,
-3103,
-10226,
1178,
-12135,
-10091,
-7609,
5625,
2073,
-2242,
14526,
-9396,
44,
5535,
-4550,
-1053,
-1990,
5353,
-6944,
-5421,
1225,
-3536,
-7343,
-2636,
15314,
-8954,
18067,
-933,
4921,
-804,
-3022,
4107,
939,
-651,
-4646,
-117,
2062,
12669,
-162,
6119,
-4635,
-13741,
-3970,
-7875,
8197,
-4789,
7213,
-1417,
-1576,
3552,
4694,
1151,
1881,
3598,
5663,
4635,
3935,
1088,
-5267,
3288,
758,
918,
3144,
-6934,
19153,
-5314,
1305,
-7945,
-9971,
16120,
-4647,
-3312,
-9917,
-4375,
4316,
-1911,
-4462,
12568,
-8814,
3096,
-3964,
-9079,
1971,
-2227,
-7970,
-1677,
2640,
-7916,
-497,
5961,
4496,
-1240,
7317,
4509,
583,
-4229,
-2284,
-2709,
-3841,
6580,
1602,
1022,
-3322,
-9643,
-2187,
-5238,
-5161,
4840,
5262,
-2952,
-509,
4040,
-4291,
2321,
4048,
-10642,
-2006,
1219,
-10069,
-5634,
-724,
1087,
9179,
-1604,
11569,
-1766,
1220,
-7590,
-1320,
-657,
5487,
2649,
17718,
6600,
-4514,
-4431,
-2627,
-5387,
-6094,
11709,
3353,
-12684,
6199,
-6047,
-18147,
6730,
-595,
4369,
464,
-9797,
2074,
14421,
1610,
-1273,
-5101,
-62,
6487,
3972,
1454,
-12594,
1825,
2411,
-12879,
-4023,
8522,
2305,
-1688,
-2780,
3195,
5614,
5729,
6618,
1230,
3168,
-4537,
2637,
4512,
-8258,
4437,
6706,
6080,
-4201,
-3203,
-7523,
8217,
-5716,
3077,
-3212,
-13597,
383,
6516,
-2960,
-3795,
1763,
885,
5173,
1262,
8493,
-1434,
-12988,
-7539,
-10440,
13974,
3567,
-477,
-3494,
-10023,
7997,
11,
-6675,
351,
-6007,
5397,
8487,
16564,
-230,
3874,
900,
-320,
4426,
8674,
13693,
-2657,
10579,
-8767,
-2745,
-2000,
12957,
4996,
-623,
5549,
1222,
10024,
10960,
9307,
3890,
8395,
7171,
405,
-2540,
686,
1532,
5033,
5123,
3558,
-2619,
-11483,
-199,
-660,
1234,
-4112,
1783,
3169,
-2862,
1662,
176,
-2925,
-7478,
-2279,
-8232,
-2903,
-1709,
-576,
-3086,
492,
8147,
947,
381,
4497,
-270,
-967,
11093,
9827,
-13167,
1922,
1128,
-11280,
-7458,
118,
-4802,
-4219,
12373,
6618,
1664,
-1606,
8322,
-1542,
-5776,
-3618,
8405,
2845,
-1311,
4025,
-7114,
-8008,
-9591,
663,
-3939,
8188,
-1151,
-5543,
-758,
-12650,
-4225,
-12233,
-1179,
6088,
-2478,
-4055,
-7936,
-6095,
-9901,
-3158,
-1557,
-5193,
11882,
-12221,
-8977,
-7216,
6462,
2481,
-2364,
-3352,
-9830,
4310,
614,
7479,
568,
3082,
3821,
-5928,
-281,
6014,
273,
617,
-4045,
12874,
-195,
-14426,
-8199,
-7810,
3799,
-9605,
13314,
8311,
10692,
-2895,
-13590,
14930,
-12512,
-7473,
1435,
-5643,
6872,
-2161,
3423,
-4604,
-864,
10835,
1993,
12611,
2443,
3030,
18959,
880,
13715,
5601,
-1219,
87,
4845,
-3902,
704,
795,
-841,
6249,
5405,
-1006,
10392,
-2593,
-4173,
-4815,
4742,
-6985,
-675,
4914,
-2246,
4136,
5256,
8213,
6629,
-10035,
5888,
-7924,
-2054,
-21383,
-6953,
-17727,
-6265,
566,
-4136,
1904,
1547,
-7867,
1098,
-6537,
8329,
-3690,
10717,
3306,
-4215,
1209,
2630,
-7656,
2526,
8266,
4434,
-15199,
-4310,
2759,
6031,
-3419,
8912,
9062,
-2162,
-4595,
-3780,
8155,
-9290,
-11788,
7217,
16633,
2786,
-14341,
-9775,
-2364,
2755,
-3561,
2832,
-2369,
-1286,
1109,
3870,
5129,
-4041,
-7598,
1944,
6259,
-8402,
10316,
14975,
372,
2930,
176,
-10874,
-5662,
10478,
12015,
3242,
8272,
13035,
1438,
8301,
-3455,
-6580,
-4455,
-14335,
-3484,
579,
10886,
-7824,
10282,
8827,
2329,
-660,
-13153,
-3422,
3343,
-2614,
-1221,
-2975,
11305,
-2157,
-380,
-1362,
-1506,
-7629,
5423,
-90,
-4742,
11244,
-3920,
-14101,
-6296,
355,
15096,
-11016,
-1722,
3683,
2794,
7123,
4258,
-3069,
-13489,
-1380,
-15490,
6941,
-4298,
-3939,
-7719,
-2683,
10480,
3408,
-2535,
-11487,
-2247,
-10645,
30,
-10601,
-6330,
-8094,
-10450,
2494,
4845,
-3807,
-654,
4944,
13633,
-805,
4918,
-2336,
-930,
2556,
-8744,
-691,
-8129,
282,
6756,
-11669,
1788,
225,
-594,
-10438,
14507,
-2516,
1759,
617,
3923,
8446,
10773,
10867,
-5853,
11326,
-13128,
1297,
9289,
125,
-1706,
-8408,
-2278,
-537,
-1864,
-7775,
1116,
9480,
698,
-4594,
-1310,
3223,
-610,
-4067,
-10917,
-6827,
-651,
7485,
768,
10211,
-8394,
-11412,
1992,
1323,
2279,
5033,
-1611,
3097,
5169,
5281,
-31,
-2085,
4269,
5724,
-5224,
-2168,
3294,
-6465,
2500,
743,
1534,
3990,
9107,
4414,
316,
-9655,
-10488,
-5748,
1706,
-75,
-4738,
9489,
-7289,
-961,
9319,
5312,
5158,
-13707,
1061,
4449,
-5490,
-9442,
261,
-3487,
664,
-5467,
1063,
-1424,
-1982,
277,
-8855,
2930,
5629,
16376,
3331,
-4846,
15416,
8725,
1880,
8623,
8095,
-5716,
4592,
8085,
-2592,
4715,
5426,
2006,
-7154,
-7381,
-17796,
-9695,
5938,
558,
4366,
4975,
11413,
-5081,
-845,
-3012,
7863,
-15897,
2352,
-8340,
2343,
-7350,
-7598,
-2396,
6913,
6178,
2430,
-770,
-1184,
-8427,
2617,
-6541,
5484,
3251,
-3694,
7686,
-2998,
5047,
-339,
-9444,
249,
-14657,
10150,
-3711,
4368,
-4726,
-11840,
-4767,
-9082,
2443,
-7917,
-3635,
-6653,
-734,
-4341,
3701,
-5081,
3651,
-6529,
-4325,
4504,
9058,
-3498,
5746,
2812,
-4712,
-22,
4654,
7204,
5847,
1950,
3781,
6010,
-11101,
10089,
-405,
2035,
-7296,
-8097,
11263,
7452,
-1864,
9869,
-2016,
-5074,
-1800,
-12330,
-2506,
-5405,
4536,
545,
-9444,
-8739,
4450,
10588,
2217,
-4963,
-3065,
4826,
-535,
-1363,
8858,
325,
2924,
8971,
-1855,
4104,
4633,
9830,
13041,
-4280,
-611,
5561,
6700,
-190,
5416,
-4294,
-5945,
8938,
-4239,
4897,
6083,
2855,
2126,
5944,
4324,
7852,
4702,
139,
3921,
-8162,
1109,
9782,
-3477,
3378,
1567,
10608,
11536,
-2993,
4199,
11974,
606,
-1426,
8390,
5359,
-3634,
1099,
-8214,
3808,
-3944,
-2691,
-1923,
-5243,
-2683,
9800,
-16316,
-2937,
-1081,
-9908,
103,
-1642,
-7314,
-7851,
60,
-2144,
5301,
-728,
-9321,
-6197,
-9338,
940,
-5943,
-2332,
4090,
4247,
11248,
-238,
7130,
2094,
-5557,
-9136,
9622,
-2869,
-2267,
4149,
-2337,
-7423,
4148,
5358,
3560,
4019,
-10454,
2151,
1648,
-17836,
2009,
7278,
1050,
-9350,
-1446,
5950,
-13631,
-6218,
1091,
-6681,
-7861,
-5257,
-8952,
-9881,
-1154,
11473,
4725,
6834,
13475,
6171,
-8806,
-1793,
11341,
955,
2315,
-6518,
3550,
-4542,
3466,
-2883,
14939,
-10190,
6498,
-5187,
6373,
5771,
11143,
-10419,
6625,
-11681,
-4398,
-513,
1578,
-2908,
-735,
3840,
2747,
-7899,
-8738,
-6676,
1796,
2926,
1557,
6649,
-2684,
7197,
-7200,
2903,
-2418,
2341,
3686,
5521,
-6449,
-3696,
7071,
-15074,
-7772,
1835,
-1422,
3944,
4767,
6372,
-15221,
7690,
-1381,
-1762,
-11901,
-4802,
10738,
-6646,
3494,
188,
-5099,
-7985,
-3908,
2805,
934,
-13811,
5765,
-4850,
978,
1640,
5195,
-13791,
-10237,
7071,
-2161,
-1421,
-1574,
721,
5103,
12078,
6476,
-1685,
-2545,
-13102,
464,
941,
-1201,
4680,
-2978,
5327,
6842,
1119,
2822,
-4647,
11305,
-3677,
-4543,
6173,
-4554,
8001,
-5663,
-46,
-5719,
1373,
-2387,
-2284,
13088,
-2920,
-16442,
3450,
-6599,
-4777,
-20,
-907,
-13120,
-5999,
-1477,
-407,
7927,
-2486,
-2135,
-11290,
-3037,
-6577,
7060,
-292,
-6795,
-1051,
-930,
-6302,
126,
4634,
-353,
-4967,
10850,
-3389,
8453,
12168,
3785,
534,
-2758,
2554,
3185,
5818,
-4525,
10475,
7522,
4623,
-6227,
9121,
2647,
3879,
10497,
5238,
742,
-68,
-8207,
768,
-1465,
-1355,
-8121,
5719,
466,
3486,
-9495,
6090,
-2479,
-98,
3932,
3111,
13081,
1786,
3787,
-12562,
8031,
363,
3583,
-5472,
-7328,
9392,
-14715,
-3404,
6198,
8243,
962,
10604,
3208,
-3877,
4577,
7525,
6498,
14772,
518,
2062,
4418,
1866,
-6611,
-1992,
3801,
4832,
-4792,
3189,
-1516,
-4179,
614,
2366,
-7306,
-5478,
736,
9604,
2746,
-5972,
-5428,
3526,
4489,
9017,
12994,
946,
9386,
6885,
7782,
11681,
4555,
1325,
1160,
7474,
7642,
3076,
4247,
-1822,
-1774,
-2171,
7729,
-3830,
-1884,
10552,
-865,
-1696,
-8425,
-4503,
6200,
-7784,
-4786,
-4223,
11007,
-2439,
-234,
6916,
8007,
16791,
4552,
-20534,
-8723,
1073,
1485,
-4759,
-1856,
6746,
-181,
16131,
2304,
-1034,
-782,
-16895,
10992,
5139,
-9490,
7230,
-4387,
8253,
-5799,
-2272,
7735,
-9714,
1033,
-3626,
-8595,
835,
10822,
-3235,
4318,
-7395,
-5548,
-5932,
4337,
-6505,
8836,
-1731,
8735,
-1886,
4822,
2791,
3906,
-1533,
-1367,
7947,
2533,
4589,
6256,
12193,
-6188,
-2671,
5603,
2033,
11705,
-741,
-7813,
-1402,
15809,
-4280,
-1437,
-4649,
2163,
-942,
9220,
2714,
-13466,
-2433,
-12518,
-1064,
-10854,
10198,
-9322,
17930,
6033,
-5402,
8143,
397,
1675,
-9038,
-3152,
3865,
341,
-345,
-4876,
-7898,
5024,
-893,
-2835,
-11588,
490,
9958,
1385,
508,
-1436,
8973,
8998,
-5722,
1168,
-9115,
11171,
2424,
-5363,
992,
2555,
-786,
-3292,
2586,
-2747,
-4866,
6598,
-8491,
7758,
2523,
-1977,
-8350,
-2078,
-3280,
-1032,
5698,
-1954,
303,
1111,
-3682,
255,
597,
5295,
-293,
-7498,
4361,
6306,
6952,
-9509,
6006,
8363,
5464,
-12820,
6205,
13099,
-2497,
2407,
-9539,
1282,
3654,
15579,
-1068,
-13195,
-2790,
-7387,
-4941,
-7558,
-6132,
2436,
3259,
-6848,
-2494,
5513,
2290,
-7711,
2484,
4190,
3,
3818,
-3283,
-2493,
9994,
-2718,
4719,
3652,
1272,
-2757,
-2367,
-1200,
1139,
-7496,
-2552,
241,
2392,
2766,
2372,
982,
-1343,
5570,
10101,
-3370,
1138,
1861,
-6405,
-5280,
1939,
754,
-4103,
-5399,
197,
-4421,
963,
-465,
-4239,
-5399,
886,
3595,
-4977,
-7539,
2093,
921,
-2895,
1140,
-4886,
2002,
-5994,
-1800,
-10854,
2448,
-3347,
5637,
-2603,
2350,
-7342,
2128,
-1595,
3177,
10553,
292,
-2223,
-3860,
-2896,
10480,
3122,
5407,
3800,
31,
-1500,
1122,
980,
-3396,
790,
1280,
-752,
-5057,
-1812,
6386,
-2326,
-8228,
709,
4393,
2272,
-391,
-8633,
-19,
-473,
5163,
8079,
-4552,
-3197,
7453,
-4588,
5946,
8417,
-6854,
-4430,
-8774,
1152,
1381,
34,
-4885,
13373,
8152,
6482,
-6697,
3502,
-4765,
6083,
497,
10886,
2119,
3086,
-107,
5172,
-3816,
-8257,
-5087,
4144,
3046,
-2209,
6104,
-5301,
-1494,
-1777,
4833,
632,
-3304,
9248,
-212,
2832,
2240,
-1469,
-6905,
-894,
-6367,
-4892,
2996,
-9414,
424,
6082,
4671,
-2401,
-1449,
-7163,
-5314,
-5196,
975,
1207,
-198,
-1629,
-14048,
9220,
-1154,
658,
1046,
4176,
-3039,
-2318,
-7899,
-5640,
662,
-46,
2952,
-1644,
-5468,
1149,
2900,
-7752,
-3183,
3126,
1700,
-4799,
-2978,
6060,
-845,
-5743,
-9600,
-9636,
1059,
8381,
651,
-6825,
7459,
1050,
-6958,
4281,
7327,
-3157,
-2583,
-5461,
-5080,
-2383,
-1690,
4005,
8335,
1225,
2939,
2645,
-598,
-589,
-1489,
2731,
6051,
-1433,
2536,
-298,
2515,
10775,
-1808,
-3099,
-1150,
-4863,
-5555,
-288,
1510,
1782,
-1322,
-4576,
-3973,
603,
-6889,
-2240,
42,
-3385,
-3757,
-3688,
2316,
-8136,
12248,
-2739,
-288,
-6257,
5632,
4752,
617,
-3067,
2654,
5481,
7001,
137,
2200,
-6291,
6616,
1054,
-4745,
1079,
-3235,
-6436,
892,
-191,
-3534,
-5884,
1421,
5733,
2143,
-6491,
6082,
6977,
1287,
-4763,
-8251,
-200,
99,
-1867,
4681,
2119,
-786,
339,
-8176,
6990,
-2065,
-1368,
-6668,
-1613,
7960,
4350,
-543,
-5684,
3599,
981,
-1312,
1399,
1872,
965,
8047,
-3469,
16404,
1214,
13128,
-2006,
-5716,
-9561,
-2143,
3503,
363,
5597,
-5947,
3004,
4599,
-390,
7317,
4772,
1818,
6363,
744,
7672,
-580,
588,
11522,
1441,
-10915,
-1788,
1516,
6538,
-2386,
6452,
282,
-1606,
-2162,
3653,
2170,
4142,
-1836,
685,
760,
-2046,
2321,
-13848,
1834,
-2444,
-4812,
3096,
2276,
-8205,
-2199,
11251,
-1398,
-5959,
-4050,
-661,
849,
-2982,
2592,
-3068,
-4503,
1736,
4211,
-330,
-562,
-5118,
1928,
3262,
-6794,
-1803,
-2944,
416,
4186,
-807,
-191,
-1389,
-4906,
4400,
-5574,
5308,
-6371,
3006,
-7105,
-2581,
-6661,
7597,
2292,
4420,
-396,
-993,
3410,
95,
4659,
4750,
-6916,
-13983,
5926,
2782,
2543,
-4206,
-2508,
11649,
-1851,
6076,
-4713,
-321,
1124,
-472,
5036,
5300,
-133,
-4960,
5671,
-3361,
1619,
-4052,
-356,
8878,
5015,
2048,
4701,
1995,
-1150,
4150,
-4955,
-1056,
4567,
-1336,
-108,
1288,
-6774,
-1708,
-344,
5134,
575,
-1843,
1501,
2628,
1340,
4002,
-1453,
1644,
269,
3022,
-1497,
-218,
-11272,
14298,
5358,
-6451,
-828,
6155,
-2799,
1833,
2659,
2130,
-8006,
-228,
-1170,
-3676,
-8406,
2468,
-3892,
-10710,
-156,
1795,
-6489,
-949,
-4057,
-73,
11045,
4035,
-4036,
-507,
-7872,
-5183,
7171,
-5049,
-7731,
9480,
7653,
-1289,
-9789,
6607,
4629,
7510,
-6634,
-749,
4606,
4801,
2261,
-1750,
-3322,
473,
4296,
-2517,
-4296,
-1910,
-894,
10733,
2182,
-1264,
4989,
-3047,
-4946,
-6336,
2500,
-63,
536,
-15228,
82,
5255,
-1432,
2481,
-2738,
-3848,
2489,
274,
-3891,
5426,
-3055,
-1114,
3742,
-5031,
81,
2037,
11845,
4223,
1125,
2131,
3938,
-6156,
2007,
-7434,
3211,
6737,
-897,
4774,
-2427,
2398,
-106,
-3641,
-1877,
1948,
2409,
-4709,
-2095,
10030,
4408,
2169,
-5413,
2321,
85,
-2439,
-2964,
-5455,
-3357,
4800,
142,
729,
-1232,
4201,
3556,
-4075,
-3746,
4685,
3381,
4866,
-589,
875,
8336,
654,
-670,
-7426,
-9177,
795,
7271,
5687,
78,
-4520,
-734,
6155,
-10894,
-3809,
-567,
7246,
59,
-1165,
1413,
-1592,
4370,
7035,
1776,
-7629,
10409,
-6047,
469,
-362,
-6342,
-119,
-3362,
8116,
-5950,
-678,
-1450,
-5222,
-3514,
2072,
-5279,
-60,
-2814,
3834,
-643,
449,
-3311,
-6294,
1079,
5832,
-6268,
2762,
-2281,
7402,
-156,
3758,
3431,
5892,
-4622,
-3306,
5267,
-629,
1379,
-2207,
-1125,
5121,
2956,
-146,
-4176,
1299,
-13185,
-2385,
4165,
-5774,
-942,
2541,
-4293,
-3426,
-1084,
-816,
-1224,
4441,
8248,
2368,
-7896,
-7690,
-904,
-4779,
-444,
3362,
-6631,
-3619,
1552,
10125,
2334,
1520,
-609,
-5801,
5526,
-3232,
-4767,
1033,
-736,
1050,
-533,
3233,
1492,
3095,
-1093,
-2810,
-1438,
917,
3416,
1134,
-4002,
6280,
1957,
-6620,
-379,
-643,
-295,
-5046,
3242,
1695,
3183,
-12411,
-9217,
-7185,
-7020,
589,
-2287,
1240,
12829,
-3148,
5211,
1993,
882,
-5187,
-2744,
1021,
2311,
-4980,
3685,
-1948,
2370,
407,
5099,
9486,
444,
37,
6889,
4001,
-131,
5119,
3778,
1491,
1556,
1302,
1121,
1855,
-9119,
-7169,
1203,
-1787,
-7848,
-2101,
-3338,
1098,
4319,
-4174,
-10085,
7236,
-10125,
-3478,
-519,
-2286,
6858,
3201,
3813,
7249,
-2059,
-680,
6014,
4707,
-940,
1252,
-1969,
-318,
-1794,
-2686,
957,
6083,
-7380,
2669,
-2917,
-3,
-1600,
-211,
-1087,
-5167,
279,
7647,
-366,
-814,
-8887,
560,
3010,
2943,
-5443,
13955,
3864,
-1214,
10537,
-2317,
-8306,
3098,
-5673,
4298,
6223,
9698,
-4738,
-875,
-3930,
2502,
4497,
-8108,
-1095,
12945,
2120,
-9217,
-339,
-1356,
-8024,
2636,
3704,
853,
4131,
7276,
-816,
1550,
-5049,
-3060,
1440,
2019,
-6333,
-7523,
4083,
-11939,
6051,
3389,
2696,
-3356,
7483,
-4850,
-5041,
6471,
-1650,
1069,
-3821,
5711,
-403,
-2452,
2315,
-2319,
-885,
10323,
689,
390,
1920,
1179,
-658,
-5613,
9491,
7252,
-386,
-2738,
3927,
5025,
-1996,
-1258,
-8638,
-692,
-800,
25,
-2063,
-6852,
-6166,
2446,
2431,
-4195,
-12762,
4087,
-816,
-4230,
-3951,
3224,
-537,
-9325,
7843,
4781,
-5221,
-730,
-1080,
-1960,
-347,
-213,
-4540,
780,
3046,
-1532,
5964,
322,
-947,
-2364,
-2793,
2912,
5318,
-437,
2034,
2172,
-5523,
2120,
-3282,
-4998,
-2366,
-10613,
-2654,
2284,
1478,
-5226,
4618,
-12363,
-6734,
2590,
2819,
-8013,
-5677,
6771,
-2638,
7186,
161,
3020,
-1096,
-3900,
-3919,
-532,
5578,
-484,
335,
-3052,
-1766,
-5554,
-2159,
3852,
-9375,
-4144,
-4029,
2948,
-562,
-4986,
3997,
5101,
-3244,
1491,
1397,
-8585,
-426,
220,
-1157,
3817,
1738,
-1758,
-2284,
2716,
-1490,
-1818,
-125,
-4096,
-8638,
6482,
-1148,
4061,
-3606,
3373,
-3692,
3926,
-2974,
1115,
-2943,
5260,
1604,
1460,
-4853,
4139,
8772,
-1377,
-451,
3536,
-3336,
-5846,
-2392,
7157,
-3681,
-3443,
5208,
2405,
5165,
305,
-2106,
453,
-4163,
-1977,
3134,
-153,
-4963,
-4776,
-1427,
-1740,
-9848,
7344,
-3916,
3111,
-338,
3631,
1455,
1068,
-10082,
4391,
2188,
-8954,
-3814,
4283,
1346,
-2749,
2610,
-997,
4952,
-2308,
-6810,
-5443,
2029,
1032,
-845,
3472,
-9699,
-6013,
-12220,
3044,
-125,
-407,
2978,
5281,
86,
-8146,
-202,
3716,
-6653,
2292,
8635,
1376,
-8722,
4963,
3686,
-7835,
-82,
-4151,
-11015,
-1859,
7167,
-7389,
2103,
4285,
10596,
3603,
-6513,
7184,
6413,
1877,
6507,
-1124,
5711,
-2450,
4641,
5513,
6069,
-2220,
684,
-11631,
5513,
998,
3558,
-4460,
183,
761,
1113,
-200,
10333,
3494,
-1295,
10246,
-134,
5434,
1740,
-446,
-3378,
6096,
-4203,
2206,
-1830,
-3120,
4571,
-4129,
951,
603,
4774,
-2626,
-7928,
-3884,
4058,
-5062,
-4510,
5598,
-2930,
-6475,
3181,
-2659,
1333,
-134,
-912,
4905,
1688,
2658,
-6382,
2097,
974,
-4863,
2173,
8504,
7632,
3803,
2290,
11531,
-2653,
-2728,
-3395,
-1214,
2317,
716,
-4527,
-8313,
423,
-1361,
-4755,
-8103,
4421,
-2723,
9031,
-7568,
-3682,
4950,
-4164,
-8435,
-490,
-6871,
-2272,
4105,
-6873,
-693,
-10195,
4420,
-1830,
4019,
-7603,
7307,
5945,
4635,
7568,
1501,
-12119,
-2461,
-4353,
-5355,
-3545,
-657,
1011,
-2755,
-9806,
7683,
-5134,
-2517,
-3833,
7322,
2361,
6312,
-410,
-2637,
3291,
-2737,
2999,
3249,
-4103,
-2366,
-982,
5373,
12649,
1964,
-4966,
-7032,
-8553,
8623,
-6402,
4330,
938,
-2542,
-2142,
-45,
5632,
2538,
-1608,
9795,
2126,
7531,
-8510,
792,
6581,
293,
-1022,
1956,
161,
1649,
-2279,
-7058,
-2486,
-11490,
5860,
5558,
1835,
1603,
-3030,
-967,
-6575,
-7934,
5693,
4505,
9116,
1323,
5982,
10114,
4254,
-2912,
-338,
-4267,
-459,
-1457,
1280,
11699,
3081,
-2474,
3639,
-3658,
263,
1446,
-8305,
-7713,
-7280,
523,
-1603,
2568,
4451,
2744,
-1397,
-1110,
7837,
-8954,
-3187,
-1598,
-5830,
352,
-1601,
-178,
-6953,
2927,
9548,
-4507,
-2150,
-5485,
-5337,
845,
137,
6981,
11276,
2710,
5516,
2979,
3222,
-1016,
-78,
1984,
143,
1103,
1351,
3932,
-581,
-7938,
-7091,
-5112,
-11027,
-2509,
-1207,
-4746,
212,
-4558,
-1283,
-3849,
-5400,
-5838,
-1617,
-2884,
2104,
3383,
-2691,
2483,
-2844,
7746,
-4017,
107,
-3616,
-2707,
6189,
4808,
-2328,
-3841,
-1288,
-5756,
-81,
1316,
2602,
-324,
-4193,
3721,
2871,
3088,
-2646,
771,
7255,
4952,
-8404,
-862,
-4320,
-3549,
-4707,
-2128,
148,
-6992,
5708,
5057,
-6150,
10136,
849,
9031,
1123,
607,
-3714,
8794,
1861,
-132,
3859,
-1592,
-790,
288,
5536,
77,
-2943,
5728,
-1501,
11543,
1061,
-3400,
1467,
-5961,
-92,
4102,
-7132,
-3500,
-4146,
-4211,
-5247,
1980,
-4083,
-6698,
-16,
-2924,
146,
-2610,
-7832,
-6159,
594,
-8554,
-2378,
-3062,
-4480,
-6157,
3303,
1307,
-6461,
2538,
-4580,
1451,
-2600,
-3877,
6091,
-807,
-7989,
-112,
960,
-5822,
2114,
4838,
-1502,
-3481,
9884,
-4599,
1873,
9021,
213,
1390,
7619,
-116,
5659,
2387,
8301,
-1286,
-1040,
-8658,
4476,
-1143,
884,
3289,
4141,
6182,
1725,
-155,
-4978,
1125,
-1582,
4095,
2926,
1970,
-5000,
4419,
-1566,
3427,
-6165,
-7781,
-3112,
-794,
825,
1271,
2556,
2040,
4762,
-2359,
2067,
-206,
-5595,
6657,
2950,
590,
-972,
-6030,
-3147,
8365,
720,
-406,
2887,
-3769,
-1708,
6051,
4163,
-4507,
4453,
2915,
-13868,
1858,
-3084,
5481,
-2330,
1214,
-4426,
-5422,
-5690,
-5290,
899,
-1320,
1369,
1620,
-4327,
-3461,
-1905,
7401,
5417,
6324,
-1845,
-3136,
7336,
408,
-242,
4160,
247,
-6606,
-927,
-4447,
3093,
900,
521,
1402,
-7420,
11449,
-246,
666,
8848,
-1296,
-2985,
-1906,
5765,
-4076,
-3729,
-6794,
3826,
3746,
406,
1321,
584,
-4779,
-3476,
1431,
-8120,
-7771,
727,
1410,
-3200,
2688,
-1982,
-2378,
10116,
-2368,
-927,
-2623,
2581,
-8048,
10087,
-8793,
-8443,
-832,
9238,
5312,
-2235,
1346,
1049,
2050,
8332,
2660,
-2232,
719,
-10223,
-2523,
-8471,
-256,
3704,
793,
-2250,
-10467,
-5399,
8015,
-235,
3678,
471,
1636,
-1428,
5094,
3881,
-1709,
7379,
-3233,
-5085,
9824,
1159,
861,
2105,
-613,
-2598,
1378,
-6025,
11528,
-1018,
-5354,
1799,
-2416,
6521,
-22,
-2038,
5787,
-2301,
-2573,
3011,
-5643,
10257,
-3390,
-2903,
-5299,
5483,
3395,
6610,
2197,
7166,
178,
5999,
-3477,
8614,
-934,
1351,
-3599,
2888,
2317,
-7435,
9021,
2072,
-11513,
1293,
-2857,
29,
-5999,
5493,
-704,
3537,
5861,
-82,
3688,
219,
7616,
4907,
357,
2696,
5681,
5324,
-813,
4507,
6248,
2771,
-4970,
2670,
-5414,
732,
-1824,
-4041,
264,
2883,
-8527,
3329,
-647,
7528,
3835,
1288,
-810,
-2581,
13565,
6067,
-2796,
515,
-4456,
8521,
3989,
4953,
-1250,
3602,
904,
-4974,
-10133,
-1917,
5948,
-3196,
106,
916,
7890,
-2557,
-4050,
6009,
-1690,
2462,
6626,
2916,
1149,
-7028,
3092,
-6795,
2915,
-8663,
-7916,
827,
-4641,
3585,
940,
3115,
-815,
-1797,
1435,
-1152,
-3260,
479,
363,
-4407,
927,
-3603,
-3358,
4744,
3387,
5416,
-164,
-6347,
-2873,
-7126,
14083,
-4353,
4423,
4544,
5380,
4578,
-1914,
4939,
1127,
5144,
5879,
6591,
6981,
7208,
1052,
1332,
-4837,
2780,
679,
-969,
-2259,
-2925,
-3550,
-555,
-5803,
258,
491,
1465,
-4789,
-2987,
210,
-4828,
1507,
4244,
-10808,
-1358,
-7241,
-3496,
-3777,
1120,
3147,
8365,
1582,
1980,
2533,
1819,
10374,
-4208,
6104,
-2690,
-565,
-3830,
-2481,
2670,
-8644,
611,
-2008,
7745,
8582,
-7191,
2083,
-341,
-1199,
-11933,
2570,
-4610,
3980,
-4924,
-1587,
-4281,
2528,
845,
4319,
-407,
313,
3081,
7372,
-1476,
410,
-1364,
12195,
-1711,
-8745,
-1525,
-7014,
-8487,
-2680,
-131,
-3222,
824,
4679,
696,
398,
2181,
96,
-1578,
-2773,
-2510,
-2857,
-1670,
-152,
-6477,
1202,
4396,
-3463,
2500,
3648,
-9847,
6649,
6710,
5517,
1174,
-2262,
-5466,
-4175,
-5004,
-2827,
-133,
-8871,
-9156,
-2328,
2168,
4727,
912,
-6619,
1641,
580,
-5684,
-2089,
970,
-1376,
2347,
9056,
-2502,
-8064,
3802,
4463,
9941,
-790,
9925,
-3920,
13098,
-4658,
55,
6015,
-666,
-684,
-4820,
-3546,
-2472,
-2312,
2480,
1260,
-1453,
66,
-8638,
217,
8280,
3285,
-1668,
3790,
-91,
-2314,
2160,
-600,
2684,
753,
976,
2932,
998,
-5616,
-2729,
2523,
-119,
-3980,
-8709,
-4828,
-3240,
-8212,
1807,
-575,
-6206,
-5572,
7343,
2873,
2197,
907,
3232,
5728,
3535,
2327,
3528,
1527,
7013,
-1634,
-5699,
6979,
2094,
-324,
1710,
5556,
-5641,
1421,
6719,
-5986,
3891,
-4419,
4219,
-12,
8853,
534,
3162,
-4367,
434,
3173,
2324,
-918,
4406,
3818,
-3850,
-3069,
-4885,
2006,
-5633,
335,
2979,
1838,
216,
7047,
404,
1595,
9584,
-2234,
-5185,
-582,
-166,
-4528,
-1599,
-1334,
2104,
-662,
-732,
2758,
-6846,
-5272,
-1043,
2217,
-1244,
-832,
2706,
-5194,
-3182,
4512,
4851,
-3042,
-6012,
-452,
-1317,
-2541,
6438,
-2345,
-568,
-10628,
-2357,
-5078,
1546,
4371,
-4435,
-1643,
-10036,
1055,
1263,
4363,
6390,
1557,
-1568,
-3889,
4970,
-2880,
-11467,
-10033,
-4217,
8747,
-5284,
-1109,
8434,
2912,
-1991,
1286,
253,
-6657,
-2571,
-2926,
9220,
3718,
-5195,
3782,
-1860,
2147,
-1221,
1395,
-6795,
-938,
4363,
5717,
1796,
6847,
-2727,
-3486,
4170,
2664,
-537,
2957,
3407,
-10603,
-5175,
6977,
8608,
8831,
1313,
4104,
358,
984,
9742,
-2348,
-2001,
2984,
992,
2880,
3458,
5362,
1551,
2483,
4135,
12511,
1781,
2162,
-383,
2335,
4270,
-2535,
6107,
867,
-5328,
1618,
-2401,
730,
-751,
2997,
-938,
-4035,
8000,
-3291,
-1783,
-6659,
30,
-846,
1177,
1341,
603,
-6805,
-2794,
987,
-5899,
-2409,
6612,
1371,
5173,
9499,
1034,
-3978,
4986,
3891,
-171,
1118,
8566,
-7757,
6291,
-3394,
7529,
-2550,
-89,
-2163,
-1690,
10601,
-9982,
-9801,
4786,
10785,
6540,
-8707,
-3259,
1796,
532,
-4106,
-269,
6223,
4046,
-1772,
-6403,
-1158,
3491,
-1545,
3794,
1292,
5629,
2623,
426,
-4972,
2475,
-907,
7199,
58,
7057,
9123,
-389,
-2708,
1738,
3007,
4300,
2900,
1726,
-1970,
5030,
7487,
3604,
7078,
1159,
2845,
-2045,
3614,
5416,
-5948,
1601,
-137,
-1054,
-507,
-2425,
155,
4879,
354,
-13287,
-762,
-3543,
-225,
-13,
2455,
-242,
-2103,
-1515,
-844,
2155,
-6400,
-342,
-10491,
956,
4679,
-1101,
7589,
-8034,
-3285,
-2478,
-3217,
4622,
-10529,
-4969,
270,
-4647,
1424,
6531,
1611,
4353,
1379,
8390,
-3206,
-3347,
193,
-1179,
2055,
2334,
765,
-3088,
-3112,
5793,
-1982,
81,
-4116,
816,
-3702,
9166,
4963,
-3703,
4474,
-11295,
-3390,
5504,
2467,
-4944,
373,
541,
7370,
2828,
5969,
3891,
-3265,
5486,
-6100,
954,
1625,
3408,
-4935,
-3857,
53,
345,
4961,
8082,
-6246,
8883,
-5911,
1060,
6470,
-1337,
-12168,
-3099,
2491,
-194,
-325,
-5464,
11067,
-3631,
6793,
-3849,
-2984,
-1807,
-4354,
-57,
554,
8261,
11131,
2422,
3292,
-1693,
-375,
1586,
-5379,
-6265,
4162,
-3082,
4514,
-4482,
3635,
5694,
-6523,
-4765,
6124,
-5218,
551,
-3627,
8170,
-785,
-5772,
-1067,
-2705,
1532,
4745,
-1595,
-8475,
-8307,
-2260,
1261,
-2810,
-466,
3396,
-5093,
4273,
-2170,
3793,
-615,
790,
-1834,
-1881,
5175,
-4963,
-2135,
-2582,
-3987,
4971,
770,
-144,
3907,
2533,
-103,
-134,
258,
-7450,
-1782,
11994,
716,
-829,
4567,
9282,
-869,
-4930,
603,
-1466,
4537,
15,
776,
-447,
5337,
-1205,
2845,
-884,
12180,
-2393,
-3048,
7444,
-2672,
708,
-1204,
-8272,
12479,
-3327,
3306,
-6853,
1278,
-885,
8590,
-5235,
2412,
4024,
-3559,
2186,
-2298,
2070,
6325,
-3766,
3406,
5468,
-975,
-1304,
-6928,
8330,
2050,
2662,
3996,
800,
7304,
2187,
-1050,
-3921,
624,
6480,
8255,
42,
5713,
-107,
-770,
3072,
-169,
10223,
-1753,
-1544,
-7494,
-582,
-1545,
993,
-4990,
-5005,
4312,
-1094,
-7680,
872,
504,
-1481,
1101,
-820,
6545,
-6323,
5819,
-130,
-2950,
11350,
-1737,
-985,
2784,
822,
432,
3069,
1869,
4689,
-1481,
-6883,
-2025,
-4155,
-2561,
3529,
-5248,
1362,
11756,
-3048,
1399,
3448,
4409,
2752,
2475,
-808,
-9372,
-7214,
-24,
-7848,
11484,
5557,
-6936,
-158,
-2493,
-13449,
2956,
6577,
2335,
-6306,
-6695,
1010,
-5524,
-2390,
2060,
-5579,
2592,
4466,
2153,
3795,
-3943,
2786,
-956,
3950,
2139,
-742,
2063,
12323,
-10419,
-7587,
-1891,
-407,
3160,
-1778,
817,
3886,
2399,
2310,
-8988,
5722,
-1469,
-194,
1457,
106,
-723,
4921,
7887,
6120,
4177,
-2523,
3789,
618,
2115,
-592,
-3623,
-4233,
3619,
-6524,
-4016,
-867,
-7482,
-1024,
3525,
4649,
10987,
-1767,
-5819,
2853,
2555,
-970,
6357,
5161,
5189,
-801,
-715,
-4780,
-4145,
-8992,
-1446,
2369,
-2088,
172,
-5506,
-1780,
2742,
-681,
4528,
-4482,
-1916,
1913,
-610,
-2500,
2429,
247,
2375,
-3585,
-1654,
346,
-2148,
-1030,
-1348,
4972,
-5958,
632,
2789,
4015,
2786,
4324,
-3124,
-5985,
922,
2533,
-717,
-2392,
-4180,
1490,
-2287,
1186,
-1143,
4475,
-13158,
-1732,
-6189,
-126,
-9025,
-5504,
-6512,
3070,
-1408,
282,
-1714,
-1390,
-253,
-2314,
-2176,
3957,
-765,
-1616,
-3831,
-5043,
10254,
2450,
-6081,
-8374,
4380,
9938,
7272,
10627,
-56,
6436,
-1845,
2451,
738,
2994,
-3973,
-452,
1021,
4227,
-3962,
10372,
-1629,
5728,
924,
3194,
-1789,
-791,
1644,
-2064,
2102,
-1479,
-5939,
-5541,
250,
-500,
-1880,
1430,
-2018,
2968,
5385,
-5586,
1206,
-4868,
-2234,
4926,
-3845,
-1097,
-538,
3538,
-7325,
3150,
-2108,
2037,
-6027,
200,
8479,
4518,
4074,
3677,
9876,
6955,
2656,
-4003,
193,
153,
-2045,
2322,
-1455,
-7095,
-203,
-5095,
5267,
1558,
5804,
-685,
6563,
-119,
-7054,
-1786,
-1738,
-8064,
1967,
7180,
3406,
-3679,
-7469,
-12339,
1617,
-8590,
-2594,
3794,
-705,
-9014,
-2610,
-1817,
-73,
-2917,
-845,
-2643,
838,
1892,
37,
2486,
7883,
-1994,
-7738,
7150,
295,
2446,
3555,
1407,
-5560,
854,
390,
3697,
2382,
-2053,
-7118,
1221,
2476,
9497,
-2242,
-4438,
-1086,
-3802,
-653,
-7712,
5399,
960,
3407,
634,
822,
10330,
4484,
-1593,
-453,
-5628,
1126,
-6313,
-347,
74,
7657,
-2785,
-6767,
9188,
-2894,
-4054,
-1208,
-5115,
-3849,
-7476,
-8969,
4291,
-3523,
6865,
-6365,
4356,
-3025,
-3432,
3145,
1557,
111,
2632,
1073,
3820,
-1243,
757,
4504,
4664,
1724,
-371,
-2299,
-2995,
-9987,
-9287,
5026,
-1609,
2157,
-4949,
-5467,
-6350,
-11,
-4997,
-3244,
2686,
3538,
-12056,
-863,
3731,
3136,
-2365,
-2252,
-8810,
4378,
-1902,
-7378,
2769,
-2141,
4777,
-2402,
-1433,
-523,
2772,
3539,
1932,
-3448,
-972,
-3651,
-4449,
-3533,
-3196,
-7279,
6115,
-3766,
6284,
3078,
-1869,
438,
6466,
-1738,
1572,
-12516,
4893,
6740,
-3766,
5039,
7360,
5139,
8645,
-1500,
-3170,
2977,
-2694,
-234,
-2885,
-3378,
7756,
3866,
2709,
3012,
-5312,
1893,
-4569,
3764,
1582,
-4095,
5670,
-2429,
3252,
-3273,
7717,
6537,
6787,
7255,
-1581,
564,
-639,
4479,
3391,
-461,
5305,
-1011,
-2167,
-1576,
6241,
-1532,
845,
11249,
-4480,
-4,
7129,
578,
3364,
-775,
2841,
1566,
229,
6667,
5368,
8798,
2406,
8909,
675,
6420,
-6685,
378,
-9744,
-1361,
1432,
-1264,
5559,
-1928,
-760,
-11608,
-2540,
2900,
8189,
-8488,
-252,
-3741,
-110,
176,
5294,
-2982,
3781,
-3324,
-820,
2647,
1161,
-7613,
1227,
-1568,
143,
594,
-9878,
-1645,
783,
3053,
2771,
1149,
1171,
-3458,
-6466,
11695,
-8786,
-9824,
3989,
1559,
-3045,
-1655,
935,
-7682,
490,
-3559,
940,
-4428,
1599,
-2948,
-5017,
1450,
7222,
2243,
5161,
2055,
-5247,
253,
3052,
-802,
3860,
-3491,
-607,
-1458,
1019,
4485,
4512,
8245,
1906,
1339,
11247,
2258,
8216,
-5192,
-3428,
-853,
-582,
10168,
3859,
503,
-475,
6954,
-3276,
4048,
776,
-1542,
6897,
-7329,
-330,
3477,
1950,
716,
-4074,
-1115,
88,
-2419,
5750,
-2507,
-7620,
-1490,
8559,
12342,
1759,
-3723,
2238,
1831,
4583,
-265,
-962,
-7561,
-2778,
-1412,
6491,
3647,
-2294,
-755,
-6,
3128,
5719,
-5524,
4637,
-2162,
5071,
512,
-1415,
559,
-1797,
-4645,
-3504,
6878,
-8176,
-7532,
622,
1144,
-1334,
201,
6048,
-5853,
-2122,
7899,
-3984,
-5660,
-4707,
355,
-6759,
-8138,
-6384,
3616,
6363,
-1143,
2962,
-1261,
1002,
-971,
8987,
-7805,
322,
443,
-1686,
-4452,
-1114,
6181,
-3021,
734,
-8148,
579,
607,
5131,
4606,
5881,
-1430,
-6243,
-3028,
-2435,
-841,
-1994,
1631,
-2434,
-6232,
-940,
1240,
2889,
1200,
-198,
6105,
-1156,
4091,
-954,
-2473,
718,
738,
7051,
-3391,
7206,
-1681,
-9162,
1375,
3013,
4211,
-3100,
3598,
13025,
-1024,
-2738,
-2849,
2623,
7004,
2623,
14718,
-2142,
-5125,
1403,
699,
-1537,
-765,
-1703,
-4497,
1906,
-235,
-10036,
-9263,
-1798,
-5768,
3121,
-6599,
-7926,
377,
11639,
-6448,
7862,
-3279,
2980,
1903,
-3787,
5329,
-2635,
-3158,
-558,
-9373,
334,
3485,
6957,
-5735,
-5007,
6814,
-1334,
-3800,
2577,
2371,
-2588,
-722,
-4516,
-6582,
-3388,
-988,
-7870,
-5204,
6244,
2023,
3079,
205,
6236,
2794,
4927,
7027,
2792,
-6105,
-4194,
7800,
-702,
2755,
-5603,
5309,
1287,
-2514,
-6020,
-4958,
-8062,
3800,
3921,
-10118,
-742,
-2022,
-4455,
-797,
-56,
-3093,
7512,
-1387,
-2204,
-1330,
-2614,
448,
7105,
2139,
993,
1081,
-2198,
-2355,
1735,
5103,
-3044,
112,
1605,
4745,
3134,
-4908,
4689,
1305,
5195,
-2163,
2622,
-326,
-5090,
7428,
-4087,
-1829,
4370,
4386,
-3121,
-3449,
-3716,
-1047,
6373,
2576,
3750,
125,
-7096,
-7137,
-9551,
-130,
-4354,
8534,
-361,
2816,
-4694,
-330,
-1222,
-3496,
-4103,
2036,
1529,
-2505,
-6756,
-4496,
4989,
-3777,
4335,
1421,
-7881,
5029,
-3870,
2742,
219,
4473,
-5354,
-283,
5293,
-542,
2127,
8613,
364,
3317,
-3499,
4052,
956,
531,
3284,
231,
2736,
1206,
-3868,
-11238,
-584,
-2211,
1913,
-3914,
-3087,
3217,
-8888,
-3689,
-1145,
5804,
-3171,
1883,
-1214,
-4187,
5377,
2553,
-3274,
-1300,
-5126,
5692,
-4344,
10658,
1230,
-2323,
-1829,
3248,
1926,
5795,
5675,
-924,
-3460,
3528,
-778,
833,
-22,
-277,
-2565,
-6211,
9804,
2755,
-4777,
1076,
355,
-599,
2630,
1644,
5113,
-2054,
260,
-1004,
-1759,
-3911,
7063,
-2380,
1670,
1151,
-7062,
-1850,
8226,
1014,
5660,
1666,
1135,
2390,
-1213,
8397,
-1162,
1219,
5547,
262,
-2488,
-5540,
2760,
-1701,
6726,
-5901,
-8769,
2947,
-6197,
1361,
2212,
3818,
4027,
-12515,
6200,
-5836,
3199,
-2982,
5982,
3655,
-6408,
-506,
-2241,
-2202,
415,
-1701,
-2879,
6140,
2534,
-11649,
-3515,
-4654,
3439,
-582,
-10449,
692,
-2166,
-5130,
2880,
2734,
4373,
-5542,
8981,
-4226,
-995,
4261,
3837,
-4913,
6306,
2046,
-4767,
-6822,
-5280,
2167,
1416,
-3647,
-1388,
-2971,
3084,
1836,
8694,
-1554,
565,
-7546,
1680,
-11090,
-3501,
7628,
4599,
5960,
2719,
4416,
1280,
-1498,
11287,
-6054,
938,
-4801,
5275,
1366,
-577,
-378,
5362,
-6607,
2081,
2121,
-6477,
10732,
-3959,
-3835,
-3898,
-4681,
-3305,
-3316,
-1990,
4263,
-6331,
12224,
-2580,
-319,
5334,
1444,
-752,
1971,
-1731,
3468,
-4084,
-5793,
1973,
477,
5171,
-9197,
5289,
-1666,
-2494,
-3233,
1969,
-3587,
-2771,
2134,
-2809,
-2348,
-938,
-813,
-7375,
30,
9941,
-9224,
6628,
6376,
1708,
1924,
5037,
4351,
-4229,
-8399,
3609,
3711,
-5132,
8692,
2426,
-8862,
-1505,
2044,
11470,
5142,
-3262,
-698,
-9130,
-1673,
-5687,
-1861,
-4482,
-4386,
2994,
2112,
-9884,
-2404,
2086,
1413,
-3121,
296,
-1594,
735,
-830,
-2368,
-1609,
-3167,
4526,
-6060,
1455,
-592,
-2406,
-2934,
-1812,
910,
-8261,
-4317,
3603,
4151,
6524,
-2906,
-115,
8,
-1415,
5427,
2388,
-9573,
-7573,
4070,
1429,
-1121,
-1490,
4540,
-1524,
6136,
-5399,
1815,
3608,
-4490,
-856,
6999,
-3646,
-3719,
-8613,
4100,
4561,
-1843,
-7629,
-1185,
7579,
-3683,
1162,
-1586,
2199,
-4295,
-3937,
-163,
-1319,
-3718,
2177,
157,
-7302,
2328,
2466,
5304,
-470,
-11166,
-2621,
2772,
4809,
7446,
-2174,
-4739,
8390,
158,
-5298,
-2381,
7943,
182,
-677,
3694,
-8625,
-9316,
-2689,
1688,
-1918,
2139,
-7779,
-543,
-329,
5596,
679,
-6825,
4776,
-8117,
-7016,
3893,
4134,
5362,
-8551,
-986,
3477,
4822,
2659,
-8322,
-2507,
-1142,
1961,
-8249,
-3115,
4405,
4240,
5516,
-3393,
2654,
3687,
-4569,
-7001,
-7095,
1495,
8508,
3153,
4089,
-2107,
6240,
2756,
-744,
543,
7481,
699,
-3186,
5664,
-1257,
-1874,
-3993,
-2869,
-739,
4580,
1445,
-2572,
-1646,
3528,
9825,
-1019,
6460,
6210,
7840,
2923,
-1249,
-3973,
2547,
733,
1301,
-3138,
-5006,
1840,
90,
1902,
4350,
1303,
-1087,
5783,
-2400,
2529,
-4536,
683,
-133,
9406,
7266,
4,
3314,
-6131,
-13188,
2695,
3710,
3829,
2039,
1123,
4700,
-1852,
4319,
-3753,
-6983,
962,
-1791,
5881,
1716,
7586,
1140,
-2703,
3288,
-6046,
-1080,
-3428,
-7771,
-3361,
-8018,
130,
-4113,
-676,
-1433,
5343,
-3144,
10182,
-5164,
-3018,
-9153,
-10174,
-3690,
-3508,
8058,
606,
6616,
-3520,
3662,
1062,
-636,
3639,
-4429,
9676,
-1208,
4744,
-5812,
2209,
-1153,
9289,
-4728,
1069,
-5928,
745,
-2845,
-7256,
-5745,
-4723,
6294,
7491,
-4497,
-399,
4277,
8173,
1430,
-2756,
430,
-1529,
-786,
2315,
531,
1114,
-4717,
3595,
554,
-3035,
2579,
-10639,
-9841,
-5300,
3746,
4397,
-2333,
-4208,
-2148,
-5744,
-3346,
2311,
-4024,
-484,
2225,
1093,
577,
-3664,
-1109,
-1448,
1201,
-4831,
1681,
-3725,
80,
28,
7242,
4972,
-3461,
-1956,
5994,
-737,
528,
-3288,
4483,
-8140,
-3143,
4356,
2383,
1460,
-6401,
1909,
6134,
5135,
3879,
-4920,
1936,
-2497,
3817,
3388,
-853,
2414,
-6822,
5220,
2047,
4122,
-6263,
6184,
-1389,
-8053,
-2290,
-508,
189,
-685,
431,
-285,
1242,
-1411,
-788,
1228,
733,
787,
10507,
3810,
271,
-2059,
5228,
9524,
-2838,
8157,
10386,
-1050,
-14695,
-3222,
3707,
4646,
2405,
-875,
-2164,
-1713,
-1964,
11774,
3956,
3640,
16,
-3613,
660,
-6502,
-607,
-4442,
2796,
665,
6688,
140,
-6581,
-1742,
2721,
-3713,
-369,
1522,
-312,
6488,
6806,
-77,
-3889,
3022,
430,
-9133,
-3852,
-1470,
4576,
-4487,
-6023,
-3686,
-2363,
-4285,
-11743,
-1607,
1504,
1902,
3191,
-8275,
-2446,
-4617,
894,
-674,
-3557,
-2076,
-6428,
3320,
1922,
6855,
-7632,
-591,
-6123,
370,
-5050,
-3800,
2990,
-1625,
7029,
5288,
-1177,
3047,
4960,
-768,
-2850,
-770,
5418,
6449,
5288,
-2715,
-780,
7212,
-2404,
11321,
-8712,
5333,
2144,
180,
1194,
-5085,
8141,
-3528,
-1386,
4597,
-3863,
-3462,
-7442,
7516,
7415,
-3612,
-8677,
6865,
10623,
4155,
4317,
-210,
-4418,
2120,
2965,
1247,
-1520,
4620,
1436,
5214,
-269,
6240,
-322,
2450,
-7006,
-1551,
5077,
5948,
-2673,
2869,
7764,
-937,
2652,
-1762,
4569,
-2476,
799,
-5752,
-189,
-2541,
1654,
-3820,
-8243,
847,
1809,
-1264,
-937,
141,
6194,
-321,
-1773,
2472,
-7213,
-337,
-5474,
-4171,
1008,
1734,
-7153,
1388,
-4020,
1655,
-2714,
-1019,
-1395,
-9813,
-3373,
2377,
-10596,
6947,
-932,
-3808,
2659,
-1018,
4490,
3341,
8040,
1626,
-2458,
5948,
13260,
4364,
5038,
1549,
-12982,
5280,
-839,
8428,
-7719,
-4388,
8996,
2758,
-4382,
-4294,
-230,
264,
-1810,
-5863,
747,
2272,
7091,
268,
-5020,
-914,
3917,
1411,
2068,
-5061,
-5386,
3077,
-9557,
2862,
309,
-62,
131,
172,
7041,
2830,
6755,
5150,
4851,
-1800,
3682,
2223,
536,
4996,
8228,
1268,
837,
3909,
-847,
-6135,
-6314,
-726,
12841,
-3076,
6593,
1046,
-6141,
10420,
-1341,
-759,
-4050,
2895,
-487,
8248,
-2760,
-307,
186,
8457,
-4142,
10304,
3199,
1410,
3699,
-11353,
-947,
4658,
-5710,
11299,
-1042,
-3291,
10558,
-6475,
2358,
-1537,
3847,
-1966,
-2590,
803,
3071,
-840,
805,
-830,
3564,
-2735,
6124,
3774,
5117,
2702,
8820,
6354,
943,
-3921,
-148,
-55,
-2769,
-4518,
11290,
-11098,
-11406,
4095,
-3657,
3829,
-7064,
1857,
6047,
6211,
-2655,
-783,
-2748,
-1863,
2974,
10728,
-5810,
2677,
-3518,
5229,
5382,
-1714,
1578,
-12788,
1087,
-4613,
2463,
-1980,
6029,
5438,
4799,
10032,
4947,
-1129,
2008,
-2429,
8516,
1312,
-792,
5881,
-11129,
2257,
7290,
-3584,
100,
-283,
-4557,
-4187,
-3553,
-3215,
-697,
4745,
7772,
-12021,
6486,
1806,
1532,
2278,
1152,
687,
-246,
4032,
-7314,
5,
0,
1555,
120,
7469,
-7812,
49,
3145,
-2751,
3423,
-5239,
2638,
3380,
5675,
-2527,
900,
-2112,
1115,
5504,
361,
-577,
3111,
-9498,
2060,
4013,
12583,
-3000,
-6508,
-320,
-83,
-5631,
10333,
-9190,
-548,
9,
-6971,
1802,
6552,
9558,
-2371,
-7069,
-6754,
11256,
5441,
-6377,
-4631,
18090,
683,
5622,
1683,
2283,
425,
-5771,
5468,
682,
8620,
4422,
-7091,
-1865,
12072,
-1399,
2265,
-3254,
6341,
4067,
6686,
-13110,
7378,
-12224,
-1359,
-92,
13476,
7385,
-1312,
578,
-3737,
-10791,
-919,
-187,
1842,
9232,
-1298,
-1563,
6201,
-1901,
6111,
-12211,
-6436,
-6777,
-5692,
9539,
-5159,
6275,
1137,
-369,
-1230,
-4214,
-1031,
-3134,
6100,
-500,
-2690,
-5392,
2617,
8186,
4380,
6988,
5753,
-10641,
-406,
9150,
1579,
-2636,
3682,
6071,
-1519,
-1496,
9895,
3531,
10549,
4063,
-5303,
97,
4718,
-6563,
5968,
1786,
-4436,
-3418,
407,
-3915,
2929,
-6451,
-1096,
8607,
7198,
-4326,
-6877,
-3518,
2236,
513,
-2279,
-4497,
2829,
-5543,
-953,
-3418,
-6618,
4075,
6897,
-9386,
1935,
7946,
-7692,
-8733,
3658,
1271,
-7680,
2112,
3573,
3585,
7087,
-2536,
-6900,
12450,
-90,
-2282,
4369,
-2261,
6335,
-4897,
-3085,
-2282,
-723,
-6884,
-8005,
866,
-1290,
5786,
15851,
-2879,
6996,
2535,
-5398,
-4438,
-528,
1097,
-19,
-6767,
-1365,
-694,
10816,
-5269,
-1593,
1933,
-6940,
2732,
-10347,
-12929,
6222,
-13062,
2481,
11022,
5874,
-3457,
2599,
-548,
2879,
-14320,
-3093,
1507,
-6748,
698,
-6898,
-1015,
11458,
-4549,
5587,
-949,
-2089,
-4741,
-81,
1264,
5345,
-2415,
7115,
-4230,
-6861,
3543,
-3754,
10504,
4122,
-8117,
-5184,
-7196,
3508,
-6140,
-280,
-8294,
6142,
-3284,
5493,
11421,
-10374,
-1140,
-7581,
-2721,
-685,
-6274,
-9876,
3684,
3284,
619,
-4006,
7071,
1339,
-4387,
-11952,
2817,
-7151,
3794,
3213,
-892,
5796,
4080,
4607,
-3774,
-3380,
4859,
-4023,
-6058,
973,
-7603,
-4200,
4773,
-1823,
10599,
2590,
7494,
7021,
3054,
-407,
11159,
5656,
8179,
-6389,
5689,
3597,
-3255,
12951,
10513,
9708,
-11898,
462,
212,
12537,
10432,
-1828,
-8557,
4718,
-2839,
-7837,
2149,
-1562,
-7070,
7675,
13363,
5037,
6543,
13961,
-10965,
-3567,
-6765,
2680,
-4266,
2437,
-2100,
-7667,
-483,
2041,
-2052,
-1854,
6863,
7971,
-8543,
6258,
9791,
12002,
-1543,
11376,
11402,
-7183,
-2329,
-976,
-6016,
-6516,
-8464,
4836,
-2445,
-3561,
-9968,
-9214,
7635,
-5054,
2402,
-11505,
-3469,
-963,
-1536,
-3843,
-3220,
5161,
-5230,
-10859,
-221,
-3074,
3366,
-4701,
-7150,
252,
1450,
10581,
3566,
-11007,
3846,
-3379,
2235,
2721,
5067,
-9533,
2800,
-1791,
10830,
7934,
1948,
9282,
609,
3498,
10856,
-2050,
13878,
-561,
3634,
2651,
-8256,
9680,
-4270,
-3929,
-12024,
-4688,
435,
2788,
-1151,
2109,
8632,
-14487,
2691,
-13051,
13362,
6409,
-4809,
-11153,
4298,
-4120,
8251,
-3528,
59,
-11394,
8648,
3809,
1508,
-2105,
4224,
-1208,
9058,
2763,
1570,
82,
-5732,
148,
6605,
4101,
4198,
-598,
1222,
-1812,
-4690,
-5474,
-2185,
3593,
-5382,
968,
1196,
-2992,
1684,
6735,
-6942,
8831,
258,
1743,
-147,
-2704,
-2569,
7509,
15136,
-7039,
11687,
2103,
23492,
-2195,
-8676,
-993,
8342,
3250,
-1453,
4663,
16971,
10832,
349,
5277,
-4913,
-3292,
-354,
-9019,
-5268,
-1846,
3140,
3586,
6750,
-4021,
4874,
2200,
-3899,
-7654,
13575,
12917,
4200,
5581,
4803,
-4529,
-595,
14534,
-213,
-986,
7367,
-3578,
-2652,
-10656,
-19085,
6624,
-1918,
3367,
3196,
4758,
-1153,
-2738,
4170,
-11119,
-251,
-3137,
-231,
-8859,
12398,
-6530,
-7459,
-2783,
5642,
-9798,
-7307,
7961,
-4114,
-8978,
-551,
-703,
4186,
9457,
8780,
-4943,
-3125,
-1022,
-4420,
5218,
3354,
5250,
-1700,
-6013,
2513,
2073,
-11468,
-10210,
-17176,
6386,
-5704,
-5384,
3038,
2662,
-7529,
-1743,
20148,
2156,
891,
2817,
-15004,
4322,
580,
3025,
2403,
-4276,
4707,
-122,
3836,
-2128,
-5299,
-6428,
3756,
-3170,
11918,
6443,
-10922,
-4819,
700,
8815,
-5902,
6690,
-1892,
1039,
-9468,
-645,
-6688,
-10996,
11243,
-9144,
3317,
-5239,
12542,
8940,
-166,
-11813,
3101,
4588,
18643,
3297,
8860,
-1383,
9727,
5297,
2282,
-1812,
7208,
-14369,
16840,
-13751,
2722,
10512,
-4927,
-9243,
370,
-6536,
2291,
-7537,
-3617,
-1726,
-619,
-2837,
13629,
-6679,
8948,
-5844,
9362,
-14303,
11430,
7788,
-8879,
882,
-1351,
9075,
-7102,
-505,
8269,
-15192,
112,
7267,
1651,
3614,
1835,
1608,
-1593,
-6004,
15027,
-7784,
-1552,
4355,
-7073,
-2307,
-7177,
-182,
-10390,
-10018,
1170,
11557,
5732,
6175,
-7622,
-7999,
-7116,
10737,
3620,
6378,
452,
-4676,
2637,
7095,
-8318,
611,
3425,
-732,
7611,
-675,
2437,
12823,
139,
-2319,
-7801,
3509,
4940,
-16524,
8671,
10034,
-3010,
-6384,
-1365,
5076,
8393,
-1683,
-2603,
15687,
-2305,
13989,
-9180,
1598,
-12356,
6957,
-8043,
6309,
4424,
-3402,
-9108,
-661,
-4500,
1638,
9939,
-5640,
-3352,
1838,
-7176,
9802,
-664,
-5973,
10599,
-342,
9488,
1896,
-10577,
-1453,
10341,
-18592,
6747,
6498,
-470,
-7823,
4500,
-3116,
833,
10203,
3728,
1972,
14376,
4387,
-10507,
-2801,
-8776,
2071,
-6621,
8037,
-8218,
-7371,
3182,
-4793,
-4860,
-7486,
4214,
264,
-725,
12554,
-2106,
13066,
-2002,
-2569,
8437,
1996,
168,
-2196,
-1210,
535,
-2917,
2359,
4857,
6585,
2108,
2913,
8323,
-3244,
-4651,
3348,
-478,
6081,
3597,
-6242,
2591,
13268,
-6859,
-8755,
3098,
4600,
3919,
6175,
4092,
5594,
-2408,
-5898,
-13649,
5150,
-11076,
4106,
5946,
-3432,
2909,
6928,
3047,
12968,
8825,
4787,
-2783,
-2878,
-7913,
1063,
-2253,
-1539,
8442,
9575,
-8242,
8874,
-4681,
-18721,
-6668,
-11866,
-3030,
4991,
3074,
9683,
-1028,
3898,
4051,
11940,
-6432,
8440,
-1275,
-6862,
-4105,
4430,
-6338,
1025,
-7636,
3643,
-8125,
15071,
-941,
-12356,
-3619,
-7379,
6324,
6723,
4944,
3280,
3939,
-1773,
4037,
2895,
1903,
6319,
1851,
3078,
-2763,
1324,
7516,
7051,
1587,
-1281,
-6463,
-4392,
25,
14815,
-4382,
3748,
7777,
5628,
-3411,
33,
14416,
-10594,
8828,
4159,
-11246,
-7720,
1032,
-5522,
-526,
-10883,
7150,
-11105,
7780,
-726,
1786,
-3694,
-8087,
-695,
-2436,
7926,
1631,
-4520,
-10185,
991,
-230,
3154,
-6882,
-1788,
-1944,
15806,
4458,
-2157,
3393,
-8682,
854,
-1007,
-13037,
-1513,
-6973,
-1143,
-6398,
-9983,
10574,
-1222,
8280,
-3536,
-3279,
-1567,
-12516,
5885,
2183,
1515,
-1674,
407,
8618,
2024,
6626,
294,
3936,
1412,
855,
1879,
-5781,
7060,
-6620,
-5183,
856,
3454,
-14221,
-1435,
-1435,
-3606,
-1185,
-10721,
3699,
1946,
7411,
8767,
-3091,
3189,
6882,
-5059,
-1369,
911,
866,
1817,
-9520,
3179,
-1949,
-3884,
3959,
4258,
15559,
7303,
3799,
-6057,
-4013,
-5482,
-3821,
-1400,
8736,
-10423,
-5559,
-6616,
-6506,
890,
-690,
13196,
278,
-4700,
-10280,
451,
-3073,
-10088,
-1353,
5450,
334,
-7165,
-12962,
-6143,
1395,
-14768,
-5104,
1276,
-1283,
-7857,
-3723,
1644,
1987,
4823,
-909,
1230,
-2281,
12286,
1223,
-4993,
18867,
1560,
3644,
-1289,
-9625,
-1226,
937,
9653,
-1336,
-624,
-2225,
-5372,
-11644,
6472,
3159,
-2694,
-8989,
-1191,
1767,
13546,
-5498,
-5521,
-5868,
-17614,
-7422,
11545,
-4634,
-7681,
3846,
-383,
-2152,
2932,
-11157,
9029,
2257,
3697,
-13933,
4472,
-5491,
-1352,
859,
-2782,
14734,
-9166,
-906,
1421,
-2253,
1992,
-2596,
-11157,
10767,
10602,
8142,
3513,
-4916,
18005,
-9609,
-12706,
115,
85,
1028,
3215,
-3857,
1491,
4639,
-972,
-3188,
-8622,
-161,
9431,
-4634,
1720,
-753,
5224,
7017,
-903,
5463,
5041,
8501,
10429,
116,
-5281,
-5041,
-17610,
-756,
10318,
-3464,
4825,
-3759,
-7676,
-3781,
701,
6197,
2685,
-771,
6561,
906,
-5879,
-6101,
-8556,
-6555,
4185,
-886,
-1668,
292,
2659,
5428,
2998,
-1814,
-11803,
3408,
-4626,
8880,
5721,
18126,
-2262,
1765,
2856,
6400,
7898,
935,
2477,
2143,
-3225,
-224,
-6848,
-1066,
4232,
-496,
-319,
10993,
3956,
6930,
-654,
-850,
-394,
-2537,
-6956,
-15865,
-356,
13775,
9049,
-5503,
9142,
-13584,
10324,
-5171,
-4975,
-8747,
5535,
7837,
-1846,
-1049,
-1593,
-3605,
10117,
666,
4369,
8827,
6622,
-3208,
-5769,
-4100,
9419,
306,
-11707,
10151,
5578,
-1316,
135,
221,
6824,
6937,
6071,
1611,
6058,
1791,
2497,
-238,
6106,
-14320,
-13810,
-3739,
-479,
11770,
3208,
1930,
-7139,
-5381,
14896,
-1057,
-7093,
3995,
740,
2459,
-3369,
-17513,
-2416,
7764,
-1449,
3618,
-5717,
-6053,
-2855,
10692,
-13351,
17258,
4950,
-12834,
10392,
-4393,
15237,
9059,
6794,
9391,
-1149,
2986,
-2311,
-9595,
4093,
3495,
-760,
-7989,
5840,
-5778,
2094,
-1362,
2465,
7638,
-8075,
-8861,
497,
-644,
-1747,
736,
-3345,
-1806,
12404,
-8450,
-9928,
1218,
8506,
-2464,
1901,
-5982,
-1099,
-4031,
-8377,
3374,
-6895,
6645,
-11612,
-6084,
7097,
4988,
4387,
-8067,
-1876,
-7886,
3553,
1655,
14189,
2892,
1028,
4568,
3793,
1041,
-6226,
-14056,
2005,
-4535,
-16116,
10845,
785,
2140,
10344,
10591,
3090,
16530,
-853,
2799,
1357,
-5222,
17483,
-15709,
-22,
13560,
-1338,
5943,
6489,
-10007,
-9879,
117,
5297,
-352,
7769,
-12081,
2942,
-4393,
8392,
12863,
7915,
-865,
-1340,
-2259,
4540,
-4440,
10073,
1151,
-3997,
7850,
-4079,
-209,
-3157,
-11064,
4194,
-1238,
-9756,
-2436,
-1861,
529,
-1105,
-440,
-8040,
9888,
-7737,
8590,
3978,
645,
2571,
5350,
1152,
1635,
8231,
320,
9958,
-170,
-4924,
-7707,
-6372,
-2170,
3507,
-3099,
-2760,
8261,
8675,
16323,
5383,
-1834,
10189,
-5373,
1558,
-8014,
7954,
-7363,
-1750,
-379,
-1200,
-2251,
3071,
70,
637,
-794,
10787,
-5927,
-6263,
9109,
5012,
-2019,
7573,
1310,
12508,
9196,
-1387,
-5563,
8083,
-9749,
5638,
-2265,
-1399,
680,
-2933,
-6067,
5293,
-4914,
-3121,
17151,
-7475,
-7093,
6225,
5642,
-8366,
-2138,
3128,
-6073,
-14707,
8297,
81,
-3309,
8200,
-616,
-711,
-349,
-1825,
2380,
-12596,
-4801,
-1634,
-3564,
-6612,
1590,
-733,
4619,
-14724,
-4138,
4509,
-1682,
8732,
-2017,
1651,
2263,
3346,
3247,
827,
-10260,
-5884,
-9484,
-4695,
-6126,
1326,
12033,
-12310,
-3546,
2063,
-1718,
9160,
-13326,
-11383,
1094,
-4320,
-896,
-7371,
6783,
-15315,
-4886,
794,
10844,
-953,
-14799,
-11253,
-11282,
-3603,
3861,
9125,
929,
1451,
8637,
-2035,
2644,
-978,
5800,
1467,
4464,
-2605,
472,
766,
-4411,
4442,
-1181,
-11793,
-3396,
7799,
5706,
-2448,
7882,
-3458,
3540,
-12381,
8096,
5602,
-776,
10727,
1230,
-10475,
1960,
-5288,
4196,
5284,
2068,
2631,
12183,
3520,
-5294,
-3733,
-2591,
4165,
-14736,
-15092,
-5539,
7946,
-1556,
-10680,
6227,
-1477,
9167,
-1360,
-3947,
517,
-1141,
-103,
1115,
-3465,
-1367,
752,
9875,
-10644,
922,
7258,
3295,
-4150,
-3293,
-6607,
119,
-6157,
-5826,
-5685,
-3422,
6082,
4174,
6618,
2767,
-1319,
-4942,
1347,
810,
8012,
-1429,
6294,
4837,
738,
11595,
-8498,
-10277,
3803,
3157,
5979,
1271,
5724,
-1086,
-6986,
5479,
-7648,
7069,
6478,
3405,
-1636,
8821,
-3415,
-3273,
4340,
-3012,
-9713,
-11976,
3342,
9511,
-8896,
2164,
-2258,
5610,
-3380,
9964,
-1482,
674,
54,
6861,
3881,
12803,
2237,
-2719,
4108,
8498,
-11025,
-7723,
-2312,
9653,
3620,
-7930,
1730,
-6732,
-8044,
-504,
9166,
5518,
3292,
-1797,
-13612,
-15194,
-7501,
4426,
-4114,
5838,
6607,
-7280,
-10001,
-6031,
-3743,
-941,
-2396,
-4572,
-9250,
-2887,
-6497,
-9644,
-6304,
-3323,
-6506,
-9888,
-4213,
4486,
11284,
-10116,
-13239,
2323,
7772,
656,
-13199,
795,
5315,
-4945,
-4551,
-1775,
-1874,
-4400,
-2423,
-1537,
-1576,
1169,
5407,
-3567,
-3341,
2198,
3039,
-5009,
-1139,
6274,
-7413,
6941,
-1807,
-1530,
-7713,
-1459,
9,
1338,
-805,
-8504,
3756,
-5895,
2239,
3862,
627,
4821,
4431,
-12514,
-7,
-15866,
3440,
17885,
832,
-2238,
-2135,
1380,
1813,
-1653,
4010,
-1462,
3536,
15314,
3804,
2726,
6607,
7938,
2576,
7176,
6531,
8606,
-4760,
-5186,
578,
742,
-6975,
701,
2037,
2436,
-136,
12969,
-2624,
-4890,
-376,
676,
-5202,
-6898,
4644,
-8904,
3874,
10162,
3591,
-3596,
7834,
12074,
3418,
4643,
-4962,
-4053,
1780,
3762,
-10010,
-4662,
16456,
1682,
2938,
-10062,
691,
500,
-7193,
-5500,
18409,
5243,
-4718,
4699,
2169,
203,
-4562,
-5751,
-6582,
-7995,
7511,
8406,
1966,
-8110,
-995,
1960,
1994,
7370,
-150,
-2555,
-8532,
-10635,
4437,
-1556,
2411,
-2351,
65,
-2762,
4126,
10103,
-4059,
2614,
-7369,
-7933,
-3554,
10397,
-234,
-4363,
5728,
11611,
2887,
-9341,
7907,
-14234,
-1056,
349,
984,
-1385,
-57,
-5573,
-286,
-8352,
2976,
-1910,
-2410,
2978,
-4788,
-4554,
6512,
182,
-2312,
-14174,
-2588,
-15712,
10500,
835,
4342,
-2655,
-4925,
-2938,
11267,
-7019,
10857,
6283,
-11361,
-1961,
-7155,
10126,
-3170,
6234,
-3204,
-1862,
4365,
8335,
1293,
200,
-13198,
7853,
9259,
6997,
150,
-5527,
-1222,
1845,
7901,
5072,
3652,
599,
-146,
-5593,
369,
-4688,
2156,
3567,
-745,
6840,
427,
1465,
7639,
13594,
-2650,
-8589,
-1266,
7083,
-2900,
-1585,
-14809,
3458,
-3847,
-1961,
7680,
-2707,
-7793,
-329,
7648,
-14268,
1138,
2229,
314,
-10944,
-90,
-13422,
3632,
2255,
4331,
-1113,
5516,
-2510,
-6455,
6355,
1705,
-7591,
1983,
-1378,
-6399,
4525,
-175,
-4891,
-4144,
5963,
-12009,
-3103,
-9678,
4070,
15484,
-3150,
12425,
6912,
10595,
-4759,
-2279,
11199,
-4379,
-2216,
-2952,
-2959,
-13432,
-16053,
-3569,
-15420,
-5463,
8463,
-9976,
-5060,
4379,
-4230,
-12154,
-5357,
-202,
-8523,
-5642,
-2472,
6464,
4516,
-4206,
-4178,
9570,
-1636,
1316,
-94,
1074,
-4841,
-5058,
-4071,
-2119,
525,
-5386,
6707,
-823,
3098,
20141,
687,
-3421,
-3720,
-6879,
-2010,
-1608,
-3494,
492,
-14923,
-11684,
11202,
5845,
-586,
-13491,
-4531,
-3568,
6159,
-3214,
1660,
10681,
-3945,
4996,
-594,
-3295,
3679,
861,
-1705,
-331,
4984,
-2842,
9073,
255,
2970,
-8745,
5303,
-11351,
684,
-2218,
528,
-12294,
6598,
15871,
11066,
3642,
685,
8546,
14067,
7474,
-5728,
3284,
-6807,
550,
9351,
-8929,
8430,
6468,
11931,
-150,
-9443,
-6847,
224,
4957,
445,
3822,
4221,
2502,
2848,
-19984,
9267,
1740,
1976,
642,
641,
498,
15704,
-8284,
-5346,
3160,
794,
5418,
5420,
-476,
7166,
-3288,
-5502,
6804,
-7532,
-2188,
5993,
-7170,
3870,
-1270,
2524,
-4613,
-7815,
-6039,
257,
1071,
-200,
-5592,
3981,
3296,
-4063,
10789,
6523,
5314,
-443,
11273,
7548,
7759,
-5090,
475,
-2789,
8247,
6630,
1414,
4366,
2196,
-6401,
-2926,
-1558,
-51,
13061,
-14543,
-7164,
-1466,
-11383,
-816,
-645,
761,
-4993,
395,
-2891,
8395,
277,
2725,
12461,
8212,
-3335,
-116,
3762,
7040,
10583,
7395,
2258,
-4307,
13430,
-2077,
5006,
2292,
-8969,
-695,
7363,
3278,
-9,
9282,
13215,
904,
23851,
-8336,
12115,
-3022,
-15180,
-12643,
-2465,
-6816,
-3174,
1037,
-9174,
-4279,
4341,
-6704,
-2794,
-1831,
-11616,
721,
-842,
-9183,
10358,
5182,
-2393,
-3618,
-8190,
-5029,
-13106,
-3106,
-7155,
20242,
14502,
-16694,
9660,
1091,
6209,
9816,
-8141,
92,
-8152,
-257,
7092,
7687,
4840,
10644,
7272,
1182,
43,
12701,
-5244,
-4595,
-2600,
-13666,
9093,
14607,
395,
-9078,
2178,
183,
-77,
-925,
6063,
3985,
-5856,
17779,
4257,
525,
925,
1033,
8872,
-3967,
10146,
-1164,
-5214,
-2189,
9581,
11210,
5522,
-16030,
1637,
-6196,
-2162,
-9331,
-4806,
-3106,
3230,
-8778,
7308,
-5541,
-2840,
-2176,
6328,
1532,
-2723,
-2922,
13064,
7567,
-6634,
-5292,
-1182,
-1493,
-11797,
9337,
1806,
1563,
7070,
-12552,
3889,
-7441,
10664,
10272,
8199,
-3483,
-10405,
5954,
-3959,
6957,
-4938,
8007,
-15461,
-4350,
-17911,
4176,
215,
-3862,
-55,
9288,
4966,
1532,
2447,
-2782,
-4419,
-4266,
2987,
-2123,
-5794,
5047,
-4845,
-6303,
1566,
-12101,
9149,
-1001,
-4811,
-1656,
-7464,
3768,
8318,
-1888,
864,
-6016,
-2651,
1806,
-469,
7079,
-765,
6799,
-4752,
-5438,
18762,
8479,
-7125,
-15396,
-2371,
-2489,
956,
-8500,
-2908,
16019,
-400,
-5453,
5297,
2458,
6922,
-554,
126,
7466,
-10635,
9986,
7456,
5427,
-1650,
2497,
11410,
-9370,
5383,
-3196,
-9352,
-2915,
-3358,
1233,
-8962,
-10031,
-4364,
-1846,
1544,
5970,
4654,
5454,
7478,
-5404,
-3895,
-6801,
5015,
5333,
-12587,
-5756,
-4964,
-5388,
-5189,
-5024,
518,
9228,
-7722,
6422,
-6610,
7356,
-2585,
-3884,
6819,
1635,
-6477,
-1455,
239,
-10305,
562,
-3666,
-12721,
2248,
-3847,
5731,
712,
1662,
4867,
3046,
-1795,
-10767,
5715,
6217,
-6150,
-3795,
5298,
-8685,
2685,
1448,
615,
-6307,
3385,
-5536,
7102,
-1783,
9476,
1737,
-1989,
2808,
8532,
8500,
6162,
-2397,
1077,
-100,
1881,
-4444,
4553,
-402,
601,
4838,
-2522,
-4756,
-15719,
2833,
-3235,
-3166,
10515,
2597,
6191,
-2934,
-4829,
-3383,
-3853,
-2873,
7183,
-1609,
-1285,
2533,
3811,
-2683,
1466,
5608,
-9744,
-382,
-3877,
5918,
1073,
4151,
1452,
6210,
8773,
-362,
8828,
11460,
-15569,
-2889,
-12215,
3877,
465,
-3952,
2517,
-5389,
104,
-4009,
-6245,
-4816,
4519,
-255,
-571,
2244,
-9273,
-4774,
839,
16396,
6557,
-1502,
5376,
-4602,
9473,
-11371,
113,
971,
11062,
1388,
4730,
9130,
-7566,
5101,
-1546,
3911,
4162,
-16499,
-8959,
-13820,
3683,
-14198,
1018,
-6543,
-16688,
6111,
187,
7159,
2845,
7425,
504,
4853,
19662,
-1714,
2322,
-649,
15279,
7104,
-8266,
-4513,
411,
-4077,
3920,
-5772,
-2952,
9517,
5468,
-851,
7870,
4603,
-1226,
9672,
11314,
4205,
-4533,
8974,
-920,
1458,
-6088,
-2748,
-5393,
-10704,
161,
5267,
-6269,
-9383,
-2557,
-850,
-3144,
12190,
6569,
6721,
-276,
2049,
11859,
-7801,
-3014,
64,
9568,
3772,
-5822,
-685,
2574,
-2643,
11915,
16055,
-5693,
-4518,
-7819,
7491,
-2920,
208,
-3069,
-1311,
-10117,
-4431,
17174,
3413,
2116,
7662,
-984,
-3096,
-3112,
2537,
-2305,
5246,
4867,
-2098,
-8525,
-1545,
-4421,
7284,
-7358,
8113,
2126,
-2408,
-9887,
-6645,
1344,
-1968,
-6448,
-3193,
-2850,
1153,
2923,
3144,
-5136,
-4901,
12888,
-11553,
-9488,
-5552,
659,
-11392,
8913,
-1101,
3346,
2840,
2247,
-7641,
3897,
-14481,
11491,
2673,
-12398,
-10274,
-4077,
2165,
1833,
-64,
14025,
-2425,
8952,
-3978,
573,
-6669,
-2736,
1740,
6722,
2335,
2136,
1531,
-5868,
700,
6219,
-355,
-11164,
8199,
201,
-4137,
-2714,
-8049,
-2527,
-6520,
7583,
61,
8905,
6011,
-4056,
-778,
-1838,
-5700,
8710,
10870,
8983,
5232,
10462,
-13730,
1936,
-4007,
8556,
1159,
-7406,
4937,
-233,
11636,
8802,
-200,
2757,
-15343,
-5111,
3628,
7381,
-2167,
-10917,
1096,
-1790,
12963,
-746,
9387,
6529,
7759,
-5514,
-3863,
-4908,
11713,
4272,
498,
4455,
6260,
-460,
-7820,
-2101,
-3140,
9122,
-1765,
-2825,
-2743,
-2806,
14016,
-5944,
-1338,
-1050,
-1412,
3828,
3851,
4711,
-206,
3223,
341,
-3931,
-2584,
9675,
-6898,
9264,
1481,
5902,
3001,
12048,
9699,
5758,
6549,
813,
-5903,
2014,
-5274,
553,
11662,
5506,
2347,
2250,
8213,
7934,
2573,
-3459,
-4089,
-7987,
-4018,
1849,
-7537,
-6351,
-155,
6226,
-12268,
8085,
-11076,
2230,
-10588,
1093,
-11630,
6261,
2407,
35,
-1390,
-997,
2149,
7772,
-4486,
-2825,
-14068,
-8615,
-10038,
-2809,
-2253,
-4755,
4764,
-13417,
-4914,
-6531,
1679,
2346,
-8170,
3394,
-1883,
-3290,
4472,
-5866,
-5389,
4246,
-2068,
-3397,
-1250,
1691,
6929,
3080,
-168,
12535,
918,
4266,
-4707,
5939,
-16580,
-7655,
5530,
-11475,
-8471,
4961,
6987,
10404,
-3982,
4399,
1844,
-3546,
-8812,
-2960,
-7097,
7628,
-10491,
10932,
6462,
-7652,
12232,
-1520,
2597,
3567,
3113,
6379,
7116,
-1965,
815,
-5163,
-1878,
1498,
4416,
3251,
-1234,
-1995,
-1987,
-8091,
4777,
994,
-7854,
1408,
-1097,
-5776,
-2130,
982,
-969,
-174,
-6625,
-10530,
12593,
6232,
313,
3647,
1698,
4984,
-4674,
-4720,
-479,
-2088,
-3175,
-3763,
-2902,
2605,
-1912,
5702,
1783,
-4542,
-5811,
-3207,
-4594,
891,
323,
-3804,
-1966,
-8008,
7786,
-3416,
3242,
8581,
-7265,
-7529,
9468,
-8894,
4046,
2334,
5299,
1622,
-4948,
-8508,
1192,
3820,
15378,
-366,
8574,
6132,
14083,
4588,
-529,
-3124,
10800,
-902,
-5737,
2020,
2549,
-8300,
-14921,
-61,
4096,
-28,
-3521,
-3128,
11181,
-5599,
-5521,
6238,
-2405,
4963,
7729,
1726,
282,
5366,
541,
-4373,
5859,
2416,
-7567,
5487,
-5810,
3134,
-6721,
-4619,
-4530,
7145,
-1111,
-2744,
2777,
-6301,
-9804,
8130,
4820,
6782,
-8479,
-494,
-1573,
-2642,
-1405,
-6004,
-706,
839,
-102,
-9334,
-138,
-13841,
-14189,
-8429,
114,
-7486,
-8064,
-11880,
5373,
-2724,
1081,
5204,
4328,
-5024,
-775,
4375,
-7718,
-1723,
-5667,
2095,
-1888,
-10815,
-17301,
-10818,
-6329,
1363,
4671,
-3742,
4171,
-9106,
-3898,
5133,
6155,
1821,
6712,
13718,
-4479,
-2549,
-2545,
-6912,
7807,
-6602,
1178,
19178,
-867,
-5895,
-8998,
1600,
1229,
460,
13249,
6202,
-7015,
-2824,
-4210,
99,
-2281,
6147,
2736,
-2461,
3033,
-4336,
-23338,
-3656,
-4963,
6898,
-7856,
-2404,
8267,
-4244,
-2720,
-702,
-3548,
-1678,
-3084,
9786,
-501,
6967,
-12910,
7632,
2204,
-3776,
5095,
-1597,
-10146,
6391,
12415,
-7582,
16619,
-7088,
2680,
-1503,
-1939,
10499,
-3834,
-6105,
-2439,
-7092,
549,
12065,
1713,
-6344,
-3740,
-937,
3583,
2227,
7371,
5069,
-13043,
1694,
7328,
-2773,
2525,
2175,
12904,
-2837,
2146,
-6531,
3452,
7040,
5767,
-2774,
-5494,
-12639,
4861,
14988,
2735,
10056,
957,
1399,
-1799,
1023,
6274,
-3161,
-8751,
-3706,
5126,
4363,
-948,
9950,
-1925,
11831,
16368,
2055,
-2367,
-11260,
-846,
5919,
-14759,
-675,
8898,
-6854,
-2938,
3486,
9077,
7359,
-2512,
1059,
5179,
-4730,
9762,
-15831,
9529,
-1442,
3020,
-3289,
2206,
-3698,
100,
-481,
-1253,
13786,
-3252,
2080,
5046,
-8934,
-2909,
11287,
4309,
-10295,
6553,
5553,
-2498,
-354,
-6995,
-2451,
-5813,
-8681,
-2562,
-5407,
-3236,
-3939,
-891,
3167,
-10511,
-3062,
2548,
9810,
7678,
660,
2813,
-1711,
3871,
-7573,
11991,
-464,
14441,
871,
-4534,
-6320,
-865,
4220,
1941,
-2203,
2522,
11152,
1708,
-4617,
-3803,
5959,
5241,
210,
1466,
-7831,
-4016,
1539,
-9428,
-2173,
-1168,
8035,
-980,
-899,
5963,
10629,
4125,
-8918,
102,
6033,
4544,
11912,
-11,
5476,
-5394,
2367,
-4171,
10415,
-5617,
-1811,
2611,
8510,
-5119,
1840,
-1568,
2385,
13460,
10727,
2749,
17790,
10233,
-9219,
785,
1622,
-4226,
1649,
-11326,
-4949,
-7044,
7961,
-2211,
280,
-1217,
2024,
-8759,
9322,
6441,
-448,
6596,
-3781,
10883,
-2962,
-298,
-9951,
7435,
-5952,
4200,
-6246,
13129,
41,
6033,
2475,
9278,
2609,
5268,
5222,
7873,
10828,
-4536,
-4764,
6401,
188,
10117,
45,
1788,
-6309,
7359,
-611,
1176,
-17289,
3897,
3434,
-9792,
7958,
6555,
12932,
1106,
-4644,
1150,
3004,
-8387,
638,
7535,
-940,
5423,
2718,
6617,
5066,
9632,
4901,
-6459,
7189,
11881,
3694,
-8747,
-4787,
-8866,
-10922,
3346,
550,
16732,
-8101,
9862,
-2,
3614,
6272,
7573,
4668,
8247,
6433,
5178,
-3812,
1121,
7793,
7249,
-5297,
-247,
2164,
7183,
32,
-8620,
-16435,
4580,
5128,
7646,
5847,
9862,
-4468,
-7925,
5975,
-1319,
-7878,
-4917,
3927,
-506,
-2589,
-6338,
-3106,
-5494,
1602,
-8098,
-1148,
-3344,
-5069,
-3555,
3204,
4699,
-4934,
-7907,
1982,
4482,
2918,
5396,
3951,
5291,
-1437,
3664,
3010,
5107,
-547,
-2130,
-1708,
2939,
5485,
-2811,
-2111,
4844,
-14580,
-12706,
-12147,
-4848,
809,
-1154,
1047,
-7281,
-1303,
-3734,
-2315,
-1121,
12416,
9998,
487,
-9175,
2930,
15430,
-15467,
4689,
-11983,
3432,
2580,
3216,
-272,
3342,
4175,
1572,
4175,
5166,
-7937,
-2348,
1793,
476,
-7059,
-8734,
4381,
421,
-8430,
-5788,
-602,
4683,
1066,
-14241,
1395,
1520,
-5013,
7466,
-7581,
-8207,
1774,
-11230,
6711,
4959,
-1662,
1207,
600,
3929,
10517,
3121,
7252,
-14195,
-6877,
-5646,
4939,
10215,
2542,
6126,
-8608,
2448,
17179,
-1428,
-1715,
-8138,
5618,
3242,
2767,
2852,
3347,
-6031,
-6851,
1986,
-1922,
-4464,
10085,
5749,
13337,
-9700,
-9227,
1951,
2724,
6262,
-1926,
8891,
1772,
14970,
-4501,
2262,
-467,
1967,
4781,
718,
-5939,
-16116,
-10764,
-1100,
6095,
-450,
-10876,
237,
-3992,
-4002,
-8947,
7174,
11603,
7380,
-663,
5492,
3383,
11457,
5264,
-2436,
1722,
3770,
-4020,
-1715,
6190,
-6314,
1185,
-6006,
3833,
7938,
553,
4120,
-6826,
3316,
5055,
5318,
4665,
-6114,
-265,
-79,
3593,
907,
3675,
3079,
-1791,
-1638,
4204,
-9270,
3510,
2933,
9,
2874,
3331,
-4281,
8667,
4787,
-5008,
-3952,
-2303,
11939,
3447,
7938,
-2586,
-1033,
-1150,
3108,
-19298,
-683,
-1098,
-4247,
420,
-4068,
-102,
910,
1335,
2134,
-4886,
983,
330,
-1508,
-7301,
9190,
-10137,
1307,
4757,
-4746,
-7457,
2138,
1172,
12018,
-3409,
7259,
-1048,
-13279,
-668,
3038,
7441,
1515,
2602,
-5693,
-9613,
-2075,
-4390,
1504,
-20059,
-11481,
10442,
-2278,
2770,
6975,
-1967,
3823,
-2429,
-12987,
-7179,
1368,
1989,
-8489,
4895,
-2180,
2137,
3114,
741,
-1758,
-647,
513,
-5181,
1732,
2246,
1249,
2793,
-1485,
-4559,
-7219,
5977,
-7534,
-5653,
-3537,
-11266,
-8253,
5958,
-6219,
2416,
6460,
-274,
-2714,
-1029,
-17253,
-8265,
-6356,
-12044,
3891,
-2027,
-542,
10700,
-6341,
4416,
-15289,
11801,
-6374,
3821,
1715,
-16157,
-3272,
-2179,
-13552,
-3480,
1267,
-10889,
9350,
-4343,
2726,
5947,
546,
11660,
10334,
4417,
11754,
2323,
-7328,
5487,
-486,
-2165,
-302,
-1457,
-9486,
-2068,
-15873,
5667,
-1801,
14909,
-16176,
-3934,
2365,
1340,
6169,
4176,
-7029,
-6318,
-2827,
5140,
3480,
-602,
-5662,
7962,
15030,
-10629,
15278,
-4056,
-4272,
2425,
6711,
-12480,
-5610,
1312,
-132,
9478,
-5562,
-9381,
-1442,
3382,
-1444,
2310,
-12550,
7989,
280,
447,
8777,
-2055,
-2051,
3022,
-720,
1772,
3584,
-1642,
-3093,
-8745,
4598,
-3204,
-5015,
-6684,
-7462,
-4999,
-6122,
-5394,
-2904,
3763,
10664,
5556,
-15067,
-1284,
-3933,
-10133,
-4666,
-7904,
-7224,
-8239,
-6340,
1062,
2796,
8834,
-6817,
-5326,
-11365,
-9836,
1081,
-5242,
953,
-10745,
7023,
5990,
-774,
1100,
6467,
2551,
339,
-5038,
16861,
7518,
-17405,
-5697,
-277,
-1167,
-4913,
-3565,
19269,
-2064,
-1213,
-2303,
-461,
-8496,
-675,
-2704,
8147,
970,
4017,
-7678,
-5862,
11788,
8877,
-362,
-2317,
-14813,
4257,
-1860,
-2770,
-13781,
1955,
2414,
1055,
2717,
12846,
14867,
-3213,
-16907,
4330,
-7654,
-5704,
1360,
3065,
-16159,
-2340,
8359,
-14836,
125,
8789,
2873,
7452,
2699,
479,
-6029,
-3265,
-10491,
5226,
19533,
6127,
2883,
-2011,
4562,
-5111,
-13450,
155,
-6182,
10199,
5989,
-8119,
4783,
-189,
-29,
-6142,
3556,
-6131,
765,
15557,
6351,
-5420,
-8992,
-4776,
7763,
5018,
-4222,
-6626,
-2025,
-5508,
-6349,
-9203,
11241,
2074,
2945,
-7765,
-3129,
-11575,
-8363,
12006,
-8175,
6795,
7099,
-3019,
4636,
2731,
937,
12060,
-7025,
11696,
632,
-4534,
-4300,
-10472,
-2967,
-4523,
-8446,
-6544,
-13101,
4866,
-1361,
5796,
-4812,
6110,
7919,
-6239,
-9459,
-3107,
5376,
-9854,
-6650,
7676,
-2414,
-6345,
8854,
4961,
9215,
4678,
1330,
4047,
-2116,
1606,
1889,
2036,
2088,
3453,
-8605,
8299,
-3934,
-11454,
-4360,
561,
6689,
-3527,
-7219,
-7443,
7344,
-2444,
4781,
-6144,
-5243,
372,
2724,
2537,
-1860,
15876,
1025,
-17520,
8991,
-4180,
4971,
11087,
4426,
2622,
-192,
1518,
-1481,
1766,
1454,
-1839,
-12334,
11249,
154,
-14376,
-12164,
-1492,
3514,
-5935,
-9086,
-11484,
1371,
-427,
12297,
5393,
1727,
-4683,
1708,
310,
3952,
-3189,
5813,
639,
2429,
-10579,
12850,
4737,
-9399,
6492,
5139,
11082,
-2591,
2948,
7784,
402,
-9262,
10477,
708,
-1383,
-3978,
-6842,
-965,
-1769,
-4761,
-8003,
-3230,
-12011,
10358,
-9104,
6670,
3611,
6634,
6510,
2026,
14450,
-2972,
-4573,
3926,
3895,
3048,
-7075,
-14513,
1676,
-4033,
-5827,
2370,
-671,
-9904,
-3098,
9943,
11784,
418,
-10351,
-6680,
-2758,
10,
8180,
-875,
10626,
-1798,
3132,
-10487,
-6783,
5352,
2209,
8618,
1818,
-1205,
-3885,
-3820,
4837,
-5702,
6730,
-1977,
-962,
-6459,
1433,
4052,
12655,
-1074,
15,
556,
1131,
-800,
-8306,
-2439,
-1998,
5567,
-2760,
-6317,
1108,
-15720,
1890,
5046,
-4567,
1662,
8410,
-5446,
6588,
1609,
2179,
-11855,
-1020,
-8552,
-1639,
-5106,
3944,
-7143,
-1168,
13140,
1154,
-8093,
1370,
-757,
9477,
3160,
-5114,
6841,
-649,
-646,
9879,
6281,
5582,
2309,
-9148,
-3653,
-5750,
-6624,
10506,
-4830,
4414,
18,
-6928,
-12199,
-52,
7753,
10872,
2844,
-5575,
-4726,
7631,
-6848,
4309,
2865,
2465,
-5391,
-28,
11088,
394,
-1502,
-2177,
-3808,
-2689,
-4919,
-4603,
-1169,
4582,
-13353,
1229,
1823,
-5245,
1553,
-1051,
-219,
-387,
-3443,
-694,
-11060,
-4733,
-8097,
-4220,
-5479,
-39,
1714,
11400,
-9848,
7040,
7451,
7145,
916,
-482,
-9361,
8524,
1083,
7941,
-4696,
-1806,
-2754,
2868,
-17830,
-416,
6623,
-1918,
588,
313,
-7783,
2537,
4473,
5000,
-3660,
3256,
-6136,
-8996,
778,
-4542,
13344,
-9388,
5220,
-1871,
-3963,
7690,
-2787,
3018,
11977,
13259,
-5736,
11329,
-1407,
-6901,
-8621,
17537,
2750,
10712,
1631,
689,
2157,
4455,
-2901,
638,
-12392,
-6686,
7825,
-2335,
-5375,
9833,
-954,
6267,
-2994,
-8453,
-9165,
-10084,
5255,
13473,
-7506,
-7540,
-20840,
-296,
1089,
-11692,
4137,
10319,
-19,
-4027,
-4617,
-1934,
1791,
6157,
5833,
4,
1278,
-1261,
1747,
-9319,
-1584,
2883,
1514,
-5460,
-4168,
198,
-8828,
294,
-6810,
766,
9085,
4277,
1070,
10611,
-6515,
-3682,
-959,
3106,
-5677,
5579,
-4314,
-1825,
-815,
934,
-1569,
-2724,
-6215,
-5703,
4887,
3023,
-8079,
-2313,
774,
5880,
-6249,
-3074,
3487,
2882,
-1044,
2446,
2004,
-905,
-6104,
275,
2677,
-1904,
-8705,
6139,
1996,
1296,
2264,
-90,
7367,
-9169,
-2029,
-1051,
-4854,
-5333,
-752,
134,
-6345,
-496,
-2646,
-868,
843,
3361,
678,
5671,
3820,
-6457,
1751,
-3532,
1341,
-4521,
-5562,
3393,
-4636,
1320,
4731,
-2212,
6648,
5561,
-1368,
-2405,
6683,
3836,
-438,
8110,
-187,
1179,
-5386,
-10080,
-509,
-4424,
2033,
-7005,
4227,
-10849,
1591,
2685,
4091,
-1057,
-4375,
-1683,
-1673,
-1612,
9173,
-3380,
3015,
-2080,
2734,
3216,
3180,
374,
-945,
-2062,
-6612,
-6199,
1042,
2590,
9826,
-3545,
-887,
-8957,
3455,
-3263,
2758,
3339,
-747,
12054,
555,
5633,
-1650,
-2825,
-6681,
3270,
2327,
2934,
1074,
-3331,
-1771,
539,
1138,
7878,
388,
-1729,
-1112,
1743,
2392,
-6443,
4421,
-1011,
15179,
3854,
-969,
-696,
-537,
7850,
-4521,
1265,
-1957,
-4714,
-4302,
4642,
-6087,
1324,
261,
1848,
4337,
2657,
-3845,
-10486,
-281,
-6736,
-1398,
-3962,
4328,
-1613,
-6598,
4425,
722,
-5019,
-8168,
2059,
1275,
210,
-191,
-3115,
-4894,
5587,
-8202,
-6274,
-10165,
2321,
-252,
2959,
-2606,
-4734,
-3574,
5650,
-5246,
6300,
-1085,
1630,
-3968,
-878,
-1383,
3581,
-1529,
2496,
-2358,
1920,
1106,
5397,
1229,
-6952,
4176,
-5698,
5824,
-1347,
5378,
2445,
413,
3456,
6371,
1142,
-759,
1867,
3327,
-5557,
1298,
7279,
2594,
-201,
-3335,
5556,
-2278,
-1612,
-2482,
3480,
-10655,
-5511,
-6224,
-4196,
-3549,
-970,
2324,
-2364,
-4036,
7684,
2155,
-8536,
3846,
-1135,
1943,
8089,
2460,
1049,
1238,
6885,
5602,
291,
4317,
6420,
-1576,
706,
501,
-858,
9823,
877,
271,
5621,
-1761,
10946,
2622,
4366,
2452,
-6470,
1702,
1874,
-2368,
-4959,
-4572,
3697,
-7198,
6778,
-6016,
-2269,
-6853,
4060,
-3904,
-4,
-274,
1867,
12624,
-2784,
-317,
3098,
-4445,
-11730,
-261,
1006,
2043,
3005,
4846,
-10266,
1836,
3514,
1614,
-2331,
-3602,
6063,
-9343,
130,
-6474,
2720,
1366,
-3606,
5336,
2896,
5365,
2157,
-2004,
1136,
-2973,
4351,
2498,
4525,
1252,
-5378,
-297,
56,
3582,
-221,
-7122,
-3203,
-1757,
-3644,
1156,
1303,
2905,
-8012,
-601,
7383,
6547,
-8138,
6176,
-2754,
-4605,
1586,
1244,
5653,
-72,
1080,
-1760,
269,
3638,
-353,
-1073,
-2930,
2132,
260,
1730,
4291,
6354,
1059,
-2845,
9420,
-1716,
-1539,
-2899,
878,
-4127,
2146,
2923,
-3776,
3324,
5091,
9858,
-2881,
-109,
-475,
3558,
-1928,
7175,
-424,
-2521,
464,
-4363,
-5891,
985,
3175,
-1176,
-3427,
3699,
-636,
-6317,
9524,
-9703,
-10742,
-1092,
803,
-3645,
-9399,
-2936,
-8274,
3910,
5781,
222,
5421,
5585,
-1774,
-10288,
2937,
5742,
-1024,
1736,
-4537,
3114,
-4125,
-2021,
-1963,
2744,
-354,
-2037,
-2680,
-1326,
-3499,
10465,
-3095,
-806,
-3184,
-5257,
-7735,
-1369,
-4721,
4874,
1655,
8755,
280,
2683,
-4685,
-4257,
-3309,
12077,
2636,
-2104,
-3845,
4036,
-7829,
1705,
-1081,
7208,
-1281,
-1345,
2645,
-643,
-3153,
-6202,
3745,
5964,
-4265,
-644,
6386,
-2181,
-7020,
9506,
7760,
-3891,
-5753,
2570,
6747,
1756,
-3281,
3069,
-5173,
7333,
-2499,
-643,
-4576,
-306,
-1546,
-1608,
-1918,
-577,
-953,
-958,
6039,
-2758,
-8859,
2941,
-10213,
-1157,
-43,
-3009,
756,
-3428,
-4074,
-4189,
565,
-1627,
-5432,
3898,
-51,
-965,
3729,
-5654,
-9264,
-701,
7874,
2760,
-81,
-4784,
-876,
-2187,
2867,
-1664,
-3086,
-2050,
631,
-3262,
-3221,
-2321,
1831,
830,
2763,
170,
4380,
944,
-1866,
4672,
2157,
-1245,
1941,
2411,
4305,
-5226,
-1645,
4915,
-1926,
766,
-8731,
-3574,
4775,
7829,
1622,
3982,
2402,
-5947,
-4690,
-8092,
6611,
-1707,
-767,
4671,
9209,
-275,
-3662,
-7265,
1793,
-4497,
1190,
1724,
1059,
-1515,
69,
-3134,
4183,
793,
378,
-4675,
1393,
-4576,
17,
928,
-11493,
2310,
-5600,
-1426,
5125,
-7483,
2610,
-1977,
3563,
4100,
-7776,
-8287,
7320,
-3143,
1598,
870,
183,
-4883,
-6951,
4951,
308,
-8216,
-6590,
-4960,
724,
-1004,
6771,
3066,
2957,
6430,
-976,
1053,
-1671,
2411,
-1278,
-2661,
845,
1288,
-3211,
8400,
9174,
7013,
-5619,
-2377,
7162,
7674,
-4619,
-5386,
-3472,
3493,
6105,
-969,
-4475,
-1653,
-6482,
-3477,
1258,
-4304,
3126,
-4352,
-4172,
231,
3184,
-1685,
-1805,
8322,
1055,
-4914,
4056,
-3487,
-1711,
-6811,
-1333,
-2938,
-575,
925,
1375,
6162,
2986,
722,
5413,
-2621,
-2137,
2627,
3857,
-541,
7809,
8900,
7530,
1306,
9026,
-4424,
-3457,
6049,
1403,
-10293,
-6342,
-2068,
-8791,
-1122,
-3708,
2771,
1429,
2793,
-4758,
230,
-1114,
-2573,
3621,
1591,
-225,
2095,
-2049,
6617,
-5894,
-100,
749,
9253,
-1593,
-4586,
-1317,
-4990,
-3202,
4738,
2351,
753,
5089,
-2007,
-2068,
-2787,
6941,
1754,
3690,
-10055,
2939,
2613,
5644,
-3731,
-4371,
8901,
838,
-214,
-702,
-2378,
2930,
-3778,
8755,
-3639,
1857,
-1770,
1227,
-3653,
-2003,
-1484,
7439,
6000,
767,
-5520,
-3338,
-561,
4151,
5401,
3378,
-2904,
7995,
3652,
-6856,
2407,
2457,
855,
-5375,
558,
-7452,
8188,
12342,
-913,
-1524,
-10541,
-2485,
-9809,
4390,
-268,
-3556,
-1007,
-10482,
-3122,
-7041,
3040,
-1952,
-2192,
3561,
-521,
1360,
-118,
3017,
3813,
2997,
-4540,
532,
8953,
-4865,
-706,
-7640,
-2710,
3186,
-2088,
-3588,
-644,
-7965,
-3768,
3150,
1823,
-7530,
-3325,
1262,
11947,
-6929,
-3440,
-3194,
-1285,
2572,
885,
-518,
-3732,
3633,
-1829,
4206,
3349,
1797,
6569,
4456,
-5133,
2228,
5546,
2675,
-1176,
-2380,
-4474,
-6713,
-8033,
-5790,
-12045,
-3316,
4437,
-378,
-5834,
-4403,
8197,
1053,
1054,
2189,
-11017,
9789,
3180,
-1548,
-3706,
-1952,
-11906,
2161,
2800,
8254,
-10611,
-3890,
6516,
1493,
283,
2200,
5072,
3380,
-294,
8890,
-5733,
-18,
403,
-3866,
1929,
-13172,
1134,
-538,
7438,
-1112,
1369,
9313,
6116,
-182,
-5170,
2104,
-1763,
1838,
8429,
-2522,
5526,
1205,
6276,
-270,
1109,
1114,
-4903,
97,
6080,
4610,
-7456,
-6062,
-995,
-6068,
1306,
-4605,
2334,
-855,
-3590,
2066,
7756,
-5018,
-2489,
-3702,
5946,
-929,
806,
-7816,
2336,
5880,
95,
-6516,
-2177,
-569,
-1011,
796,
-8429,
-6560,
-4857,
-2779,
-2228,
-1624,
1808,
5294,
-7036,
1047,
-1089,
-3232,
5498,
1539,
2792,
1724,
396,
745,
-662,
6120,
-7117,
9172,
7579,
14341,
-6513,
-1663,
-1710,
-1840,
-2567,
6239,
5284,
-3424,
1720,
1912,
1405,
-1601,
4705,
-5647,
5837,
-4986,
-5008,
-2868,
4708,
2724,
-2360,
5222,
2054,
2713,
-913,
-310,
4766,
-1871,
-5432,
-8025,
-2865,
875,
9961,
5075,
-3761,
-1008,
4682,
131,
-46,
-728,
3244,
-4010,
-6569,
-1621,
5453,
-27,
2620,
1056,
7965,
12174,
359,
4555,
-7282,
-8999,
792,
-206,
-5236,
-6350,
2307,
1419,
-994,
-5305,
-3624,
-2083,
2706,
-2552,
8325,
-10191,
-9289,
-9227,
3905,
1071,
-7049,
-4540,
-1396,
-4728,
-2378,
187,
143,
4624,
201,
6113,
2729,
1005,
3233,
-3794,
4874,
997,
-388,
-7615,
-2718,
-13592,
-4491,
-1326,
2576,
-2539,
-4215,
9766,
9793,
-3429,
-144,
-2286,
4210,
7307,
2302,
2139,
-4657,
-3411,
2268,
7698,
-1056,
84,
2678,
1742,
-6537,
-3918,
-4604,
6403,
4832,
994,
-9340,
3845,
-5033,
3677,
-7732,
4517,
-6306,
5651,
-3293,
-2402,
-4279,
-5740,
-2126,
-4466,
-1271,
4341,
-4028,
9417,
1341,
1078,
-4042,
345,
-1406,
-6532,
1523,
7206,
-1227,
-150,
-12051,
-1346,
9466,
1347,
-659,
-1244,
8317,
-9632,
1051,
1884,
-877,
2947,
-4564,
-9291,
5107,
1250,
740,
1524,
4032,
-3943,
3993,
-11130,
-8775,
2465,
7,
-4341,
124,
2695,
-2838,
4169,
-28,
107,
7888,
-6762,
-1185,
2900,
-5066,
-8372,
3828,
3326,
4851,
3406,
9641,
-12426,
-2971,
434,
-2866,
5725,
3155,
-1507,
1714,
-998,
4572,
-6366,
896,
4105,
3275,
-5440,
-7042,
5087,
-2656,
2497,
4918,
1367,
-2235,
-3962,
4163,
-2184,
799,
-7473,
-1951,
-5385,
1215,
-167,
1746,
11633,
-1648,
-2994,
2718,
-3002,
9347,
-3862,
-218,
-865,
4514,
-575,
-1463,
-10088,
-4856,
-1750,
-2007,
7697,
4705,
-4788,
758,
-2257,
-8402,
7168,
2590,
-5089,
8118,
-582,
-2496,
-1006,
182,
-1291,
-1474,
-3383,
2915,
-1523,
-2845,
1090,
3965,
-9873,
3681,
-6353,
3245,
771,
-2267,
5788,
1528,
1160,
435,
6769,
2698,
227,
-3815,
-1125,
-1214,
-1781,
-721,
3485,
-6369,
-623,
235,
5909,
3090,
3686,
-4931,
7530,
5344,
-5824,
3434,
-5358,
1850,
-311,
-2183,
-1007,
842,
-1539,
14073,
-9243,
-3627,
-4237,
109,
-1659,
-781,
-2872,
493,
11819,
-6375,
8005,
5088,
1441,
-5101,
-196,
5072,
-3328,
8403,
-10925,
4728,
-9767,
-601,
6425,
-5315,
70,
-4836,
-4339,
-3580,
466,
680,
-2914,
9023,
3433,
-2651,
4070,
-6728,
-3111,
8534,
-5346,
-1988,
10375,
-1328,
2697,
-8004,
2273,
7530,
-5920,
3192,
-9760,
2752,
-6100,
-4275,
-3928,
-3421,
3693,
-3550,
3960,
4961,
2240,
1330,
-410,
-3022,
2923,
1018,
3614,
-3257,
930,
-5379,
-3895,
1239,
-5511,
7339,
-408,
5161,
-1151,
2064,
8389,
-894,
-9592,
-6523,
-8643,
9293,
27,
-1752,
1004,
-3923,
2796,
2952,
166,
1984,
-2891,
-7061,
-1201,
-970,
-7106,
-6206,
1561,
-1160,
760,
-1988,
-4530,
7483,
4492,
253,
-2113,
-3878,
7998,
-4726,
-750,
-718,
-7232,
2107,
-4382,
1721,
9432,
6879,
-1993,
895,
-8509,
2981,
-105,
-950,
3378,
-3560,
89,
6402,
-1533,
4157,
-2271,
1288,
728,
-2492,
2291,
-1876,
-2624,
3001,
3189,
-797,
4900,
4791,
3294,
441,
1579,
-2524,
3191,
-5415,
-6445,
5857,
3682,
-5493,
610,
7635,
-1902,
1807,
3914,
11020,
4820,
2233,
1050,
-3177,
-7674,
4132,
1134,
3250,
2604,
-3832,
-3210,
4472,
3089,
-3086,
1692,
2813,
-10394,
-4577,
-2714,
2876,
1502,
-2669,
-1044,
4702,
-2425,
3168,
3063,
1909,
-4304,
-6436,
-2901,
-2237,
-1571,
7102,
-4141,
-4431,
5418,
-2979,
-895,
3465,
8953,
1786,
857,
-142,
-4228,
-25,
-3200,
3966,
-3859,
5120,
652,
-5890,
1281,
582,
4420,
5951,
-1994,
804,
-253,
4554,
-4576,
2894,
6655,
4645,
763,
6662,
538,
3497,
-188,
-5204,
-4011,
-2309,
4162,
7195,
1882,
2990,
3013,
-7693,
-1952,
-824,
1440,
2247,
-1737,
4688,
2496,
1955,
12619,
-571,
-380,
-1525,
-2261,
627,
-793,
1318,
4708,
6218,
-3787,
4820,
-8253,
788,
-2088,
7201,
-8419,
8727,
-687,
-290,
8264,
-2337,
-938,
-663,
-1508,
7012,
4392,
-8208,
9895,
-746,
7121,
6611,
-2058,
-1528,
14401,
6798,
2279,
-3026,
-14088,
-3021,
-2451,
-9108,
3428,
4146,
1506,
-1605,
-2068,
-6865,
1843,
-4409,
-3870,
-1923,
-672,
-2835,
3136,
2917,
9878,
1343,
-4470,
5175,
3154,
3079,
3384,
3479,
5582,
-314,
879,
-1296,
3094,
11238,
2087,
9159,
2112,
9074,
2491,
861,
3218,
2842,
1776,
4645,
2912,
1250,
-2228,
85,
3327,
2431,
3885,
-921,
-2329,
-1720,
-2709,
6799,
-2366,
-115,
5794,
2345,
-4243,
-1680,
2813,
919,
75,
8186,
-1612,
-3914,
-8708,
-1310,
-3742,
-3667,
-3183,
4454,
-896,
4016,
4971,
-9025,
-3425,
2695,
5802,
-6818,
-2337,
4617,
4087,
-4279,
-7566,
889,
4595,
1572,
856,
7535,
3326,
-6465,
5573,
-6308,
2760,
-3491,
1615,
2443,
-188,
4547,
-3961,
-11208,
-53,
-1410,
-2800,
427,
4778,
-1858,
-9252,
-318,
-4157,
-2243,
9167,
8013,
-884,
5155,
773,
6875,
-716,
3415,
-5468,
3048,
-1144,
1000,
6196,
16,
2419,
-3663,
-1788,
356,
-1184,
-848,
6271,
-9777,
-3570,
4074,
-4,
1606,
-11067,
-1478,
-5379,
-4360,
2905,
2415,
8738,
1058,
2688,
6568,
-1169,
-1740,
1021,
-2425,
-2695,
3145,
-23,
5493,
2875,
-4974,
5500,
-3815,
1891,
-2140,
1476,
372,
-2375,
2401,
-4952,
2976,
7477,
2357,
8265,
-5971,
1506,
-6934,
3069,
3967,
534,
-5342,
5273,
-7713,
-2723,
3457,
-4897,
1067,
-2497,
-3634,
-4389,
891,
-7787,
2587,
6058,
3251,
-1930,
-417,
3275,
-5082,
8078,
-2555,
-3765,
-4309,
-1503,
-7181,
-339,
-3486,
-3777,
-4903,
4385,
1341,
3462,
-1660,
8145,
-2905,
2777,
-4642,
4711,
2347,
-2766,
5062,
3892,
3642,
1308,
689,
3819,
-240,
-4363,
-4274,
-2758,
-7781,
4970,
-2046,
1211,
4001,
5154,
291,
1668,
-1699,
-5418,
311,
-1807,
1375,
3518,
6027,
-1497,
4475,
4653,
88,
4809,
4891,
-2777,
2618,
-2289,
5023,
4027,
3261,
2925,
-2895,
5373,
711,
1889,
-642,
-3571,
-1413,
-7769,
4682,
-7744,
-6272,
2389,
4293,
9421,
-7086,
-2519,
979,
-7737,
9930,
7632,
4424,
-2604,
1282,
2291,
-5264,
-2790,
1897,
1408,
2867,
-6655,
-11119,
9285,
3330,
3158,
870,
4713,
-5719,
5521,
308,
-948,
-296,
1297,
1638,
8563,
-487,
3244,
-5979,
615,
-5740,
2726,
480,
151,
-2462,
3936,
783,
3091,
11700,
118,
-570,
3439,
-2913,
6118,
2211,
1019,
-1949,
3948,
1367,
-1938,
3646,
-932,
-3467,
-4048,
4211,
-5093,
-3691,
-1518,
5181,
-1828,
-8529,
-7421,
-1356,
884,
-2311,
9836,
6146,
-6853,
-5171,
6641,
-2663,
-3225,
773,
7105,
8391,
7100,
1318,
5666,
-9209,
909,
-1794,
-5188,
-3406,
4293,
1773,
-168,
4728,
-1014,
-2068,
-4418,
4048,
605,
256,
881,
-1747,
7341,
3580,
-4208,
-1989,
5051,
-11171,
2045,
737,
2105,
1977,
-4564,
-3587,
110,
-12776,
-6314,
-5132,
-3756,
8588,
-9792,
-1394,
-192,
-6948,
-2344,
-3596,
4370,
-663,
1924,
-6327,
-2402,
2256,
8603,
354,
-2595,
-2316,
-2457,
2450,
1632,
1438,
-5646,
-2344,
156,
8520,
-3035,
-4178,
5526,
779,
10595,
-2212,
-430,
2571,
157,
-4984,
-2130,
1869,
817,
7499,
4733,
1530,
-1814,
-2522,
-5542,
-4269,
2443,
-10272,
-3824,
-7388,
-1398,
494,
3668,
846,
-10374,
882,
-258,
-3336,
4,
5873,
1233,
2869,
-467,
-9628,
-12501,
-6062,
-5879,
458,
2983,
-1060,
-2018,
-10964,
2273,
5725,
5413,
-8135,
-9073,
7347,
-6037,
1453,
-17282,
-2238,
-659,
2227,
2809,
4142,
-8313,
7133,
-7102,
8087,
-4839,
-3774,
-2813,
-771,
-10703,
-44,
-3680,
5948,
3256,
1894,
156,
-12000,
2596,
-7071,
6976,
-11767,
-1474,
-460,
8119,
-1166,
-1450,
-6410,
1482,
-2953,
-4851,
-9247,
1735,
-813,
-6018,
4367,
7797,
3975,
-384,
4742,
-1588,
3142,
-7917,
4702,
-9878,
3238,
-256,
9536,
662,
11795,
-331,
1070,
559,
-1401,
-5650,
-2239,
4227,
-5456,
-182,
11047,
4806,
671,
-6106,
-5773,
-4276,
-5805,
-3721,
-393,
-1972,
-2406,
8379,
1947,
4709,
695,
-1401,
-1506,
3792,
-1718,
5240,
-1759,
-9227,
2151,
2561,
-5343,
3801,
644,
1094,
-6154,
-3625,
1462,
-4586,
-5320,
459,
3553,
1313,
3735,
-2004,
881,
584,
-934,
9490,
865,
2692,
-375,
4133,
-1599,
-1693,
-1016,
298,
7999,
-3666,
-151,
3571,
2696,
-4355,
-1341,
-1646,
-3215,
12321,
1303,
3610,
-889,
-5024,
-5219,
-658,
-2964,
-182,
2124,
-1947,
-1348,
-169,
-144,
754,
3164,
-62,
-8400,
1127,
1006,
-4879,
-8252,
-1175,
-2618,
2542,
512,
-3261,
-361,
3686,
-3351,
-718,
8692,
2466,
-8077,
1041,
6409,
1178,
7137,
3364,
-4954,
-10899,
2113,
-1615,
4507,
-3253,
2892,
11530,
448,
-259,
-6146,
-2952,
1876,
-6730,
-1538,
401,
6001,
-394,
8907,
2466,
-5320,
-2698,
-865,
-1759,
7868,
-1876,
-9424,
-1218,
-4143,
-6307,
2815,
-7143,
-1071,
709,
-76,
627,
-579,
1219,
5352,
-4163,
2165,
-746,
1938,
3378,
-2583,
7573,
8645,
-5926,
2015,
-4924,
2544,
6720,
-9173,
2005,
-1563,
-8105,
2111,
1527,
2441,
-2738,
1933,
-925,
5766,
401,
-8717,
-4140,
-996,
-2260,
-5978,
-4233,
-1288,
-1484,
-6260,
-10955,
-335,
9441,
-1815,
1880,
-9143,
3912,
5776,
-3589,
-5145,
1445,
6873,
1198,
-4353,
-7961,
232,
299,
-4529,
6900,
899,
-4686,
290,
16016,
-117,
2596,
-2092,
9457,
2475,
2288,
59,
13580,
-10903,
4080,
1757,
-2619,
558,
-3583,
2705,
-1245,
-10876,
11946,
3812,
-2544,
-2010,
-3440,
-1966,
-1811,
-5921,
12385,
-1846,
-2397,
-1918,
-5704,
344,
3245,
-4628,
-8249,
-4903,
4429,
-3668,
9022,
-3976,
14049,
4843,
-185,
-5801,
6990,
-3252,
-594,
6358,
2803,
-7332,
5119,
-7801,
6144,
2444,
7778,
-2771,
361,
6040,
-42,
-2683,
5953,
685,
3415,
1784,
-4874,
-5135,
-4175,
4582,
8355,
-3246,
852,
-1318,
-1186,
-7891,
1022,
1869,
-1820,
-2175,
180,
-2775,
5368,
-2235,
-4466,
-6847,
4605,
339,
1130,
4340,
-7421,
6691,
2296,
-1257,
-2297,
4291,
2704,
2754,
5504,
-4284,
-4026,
6833,
10522,
5012,
-5666,
-4357,
969,
1313,
-5954,
-513,
-7708,
-1328,
2023,
4399,
-2257,
2335,
5969,
-2666,
-4773,
803,
-688,
7629,
7103,
-3167,
-5281,
-1140,
3501,
-587,
-4731,
-2051,
3248,
5543,
-711,
5137,
-1731,
-2045,
9793,
-8609,
-3691,
5663,
-1508,
1490,
8967,
557,
3560,
8363,
4230,
3905,
-188,
7116,
4969,
-2374,
-7928,
2529,
-6246,
-1970,
-9385,
39,
-4103,
7043,
-3475,
2734,
-1092,
10921,
-1367,
-138,
4331,
-9366,
-7416,
2732,
4774,
7780,
8701,
2372,
-6454,
5366,
3,
-2745,
753,
4381,
-7345,
-284,
-490,
-3706,
1192,
-6898,
-1534,
3034,
-3162,
-5160,
3132,
-434,
-3514,
3385,
-2979,
-1712,
839,
1445,
-3150,
-6219,
2800,
7015,
-3578,
250,
2783,
11180,
2817,
4736,
5685,
4291,
382,
-564,
4128,
-6395,
-10298,
1258,
12997,
-548,
3856,
-4334,
-4637,
-2639,
7191,
2156,
-477,
661,
-7661,
-2617,
1126,
2179,
5023,
-3158,
1855,
-8111,
138,
2087,
-1542,
5590,
-4210,
-352,
4471,
897,
-2011,
-6103,
-7164,
-8616,
-4601,
-665,
-7282,
-2182,
2208,
-7979,
-1873,
5656,
1291,
17,
-1904,
-1493,
5280,
-5014,
917,
2723,
91,
-1357,
-2848,
2265,
-1975,
-3411,
-2183,
5346,
1228,
5610,
382,
3014,
3989,
6559,
-4305,
1480,
5483,
-4253,
-339,
-5152,
5350,
4132,
-3966,
-1335,
-3800,
3930,
2740,
2974,
-1491,
-1825,
-10087,
-3072,
301,
7740,
-3326,
1391,
487,
-7110,
7516,
2951,
9486,
4013,
657,
-4292,
7501,
353,
4349,
-739,
-738,
-1779,
2173,
5156,
3854,
2455,
-931,
6457,
2095,
-3074,
-2488,
-1639,
-7283,
-1335,
-6420,
-6004,
1502,
-6986,
-2018,
5111,
1166,
-1757,
2844,
-258,
-2745,
1533,
3113,
4696,
2379,
-2322,
6899,
3056,
6155,
-2151,
-73,
-3748,
-4810,
3271,
3532,
-884,
-10200,
-7112,
-6485,
-3592,
6229,
-943,
2096,
-3818,
3548,
3687,
2127,
367,
-2883,
608,
8748,
11909,
326,
-3229,
626,
-719,
3648,
-3424,
3162,
5751,
-8749,
-3357,
3207,
-3239,
2769,
-4435,
3715,
-3640,
6380,
-7200,
-3710,
-5927,
1252,
1721,
-167,
-484,
-3763,
1480,
-5512,
985,
-5685,
5451,
3146,
-1058,
-12,
-4219,
7228,
1904,
1797,
961,
3182,
574,
-8091,
9067,
1291,
1335,
10204,
7464,
2908,
-5735,
-9968,
-8779,
-3351,
354,
-2947,
-10226,
-3051,
-3877,
1095,
-2748,
-71,
171,
10806,
3248,
9301,
-2074,
-2747,
-978,
-6678,
1248,
7519,
2608,
4164,
3967,
4949,
-3743,
570,
11215,
-97,
-1427,
498,
700,
2379,
-3349,
4621,
1878,
-1955,
2331,
-1405,
-1008,
719,
-168,
4401,
1015,
1227,
-2390,
867,
344,
4716,
-3160,
1469,
848,
-3908,
-7977,
6469,
-11840,
-1969,
-7301,
3416,
347,
-5837,
323,
-4010,
2136,
3724,
2008,
-1221,
1176,
-3672,
2826,
7856,
1197,
-2053,
-4371,
981,
-3687,
-2573,
-5146,
-7607,
-1401,
4775,
-1119,
-3848,
-2625,
-5396,
-3500,
-2235,
828,
503,
4276,
1753,
1715,
7752,
-751,
3170,
-1291,
1239,
-149,
6268,
-1010,
2204,
2992,
1838,
-1319,
2368,
3829,
3049,
8471,
-1031,
-135,
-3223,
-4109,
-9546,
7283,
6724,
-6766,
-6324,
7966,
-1759,
-1996,
-1099,
1940,
-820,
-1797,
1419,
-5026,
4288,
-2968,
124,
-7579,
-981,
-7589,
427,
265,
8327,
6703,
-2070,
1129,
-4538,
9231,
-2806,
551,
-1341,
4622,
-1071,
1504,
-507,
1996,
-1444,
4212,
11453,
6868,
-11123,
-3230,
1797,
-3367,
-2626,
-10496,
-759,
-1813,
1652,
-237,
3435,
1518,
2520,
-4055,
-4168,
-3422,
-7461,
684,
5451,
2189,
420,
-13160,
773,
-1284,
4182,
5693,
4374,
839,
3252,
6917,
5905,
-4738,
8341,
2478,
-4652,
-2515,
3903,
5873,
6188,
-2443,
2265,
-1919,
3695,
-3823,
-1847,
-1658,
-2959,
1984,
-281,
-3151,
-5862,
-5577,
-2441,
-1560,
2600,
-3002,
-1634,
1121,
547,
1202,
-2408,
-1577,
909,
-2578,
-1411,
5228,
1840,
2685,
-1042,
1453,
2861,
2308,
-3487,
2317,
2877,
3003,
-5623,
4401,
2717,
-2791,
-2141,
1527,
-5424,
-3044,
-2935,
-8445,
-1370,
1314,
-727,
-3549,
-6901,
-3143,
-3208,
2118,
-5123,
-6628,
4136,
-2126,
1769,
5509,
2021,
-306,
1776,
831,
4878,
4041,
7533,
1156,
-3163,
-5668,
7255,
3176,
12225,
-2177,
-1702,
-5842,
-3399,
-6202,
2085,
3265,
-5706,
10229,
-4422,
-630,
-2657,
2418,
-1224,
4230,
1653,
1010,
-2931,
-705,
4398,
-1331,
-2789,
5020,
-1332,
8474,
-103,
7490,
-2406,
-4692,
404,
-5317,
-8508,
610,
6361,
-5018,
-6401,
-3742,
7454,
-666,
-7309,
1740,
3933,
-7921,
-3401,
-7710,
1116,
-3540,
8325,
-7863,
-5913,
-6881,
5719,
-1684,
1499,
7642,
1657,
632,
401,
2142,
2294,
-3491,
2944,
285,
-183,
-3501,
-6475,
-1274,
3073,
6870,
-4148,
3208,
3012,
1069,
-1045,
609,
-646,
-4131,
-4057,
2065,
1325,
192,
-6406,
8776,
-5325,
-2833,
-1288,
-5109,
7283,
-4225,
3502,
-3390,
-5129,
-4233,
4905,
-4850,
2262,
8115,
4267,
-8738,
-3284,
93,
-2709,
5119,
3511,
3234,
-10132,
-3508,
1700,
244,
2283,
6538,
347,
-8339,
-2403,
1332,
5116,
-2284,
-4911,
-4653,
6298,
430,
4071,
-1332,
7916,
-4332,
7010,
-4087,
11571,
4923,
6845,
2570,
-2003,
4200,
-3604,
-8785,
-2844,
623,
7084,
3401,
-6414,
-1960,
7354,
6455,
-8643,
6173,
-10409,
-515,
-4741,
1631,
-9911,
1579,
-10084,
4938,
-2814,
-501,
1875,
4778,
-6214,
-3934,
2826,
6295,
308,
7949,
-2308,
-2593,
-789,
1244,
7806,
2400,
-4743,
-1246,
-5867,
-3660,
6420,
-1354,
3451,
-1195,
-5354,
-1472,
6756,
-3005,
272,
153,
2296,
-2391,
-1331,
1532,
-3991,
-7383,
6154,
3580,
-5049,
4369,
3649,
-316,
-2399,
338,
-120,
7149,
4171,
6460,
2551,
-231,
-5702,
-526,
-475,
4983,
-1203,
-3787,
4283,
-5080,
-3868,
-7368,
-6585,
3772,
6104,
-2730,
-2284,
-4552,
-2595,
-2641,
-6982,
-1142,
898,
-745,
-5888,
3304,
1242,
-174,
-12107,
-4945,
3293,
-2629,
5669,
-2027,
1503,
1792,
5065,
-2579,
3073,
6956,
-3719,
-4669,
-1978,
2183,
613,
1427,
-4566,
1730,
548,
5057,
-307,
1590,
-4222,
7199,
1445,
-6719,
8590,
6248,
423,
2809,
4318,
3389,
3835,
-914,
6004,
7823,
-9087,
11557,
4832,
-3528,
-296,
-952,
4671,
-6544,
-6598,
2565,
-1529,
-1799,
-1440,
3928,
-6201,
1236,
-4188,
-3661,
-1603,
-3423,
254,
1906,
-6607,
6731,
2555,
2048,
2626,
-7683,
-6949,
-1729,
-5165,
-6527,
12978,
237,
2050,
-7151,
-5650,
3596,
4354,
-1287,
2944,
-3613,
-8451,
5985,
-7008,
-3136,
-2697,
5175,
-1995,
1083,
4803,
3258,
-1908,
-3648,
1910,
4648,
-2006,
-9923,
-15427,
-1342,
-4447,
-4964,
-7202,
-1436,
3421,
7143,
574,
-3410,
2102,
-1724,
5634,
5452,
266,
-123,
-5819,
-12095,
1280,
1000,
-1656,
-1003,
-1358,
-698,
5087,
-1595,
-5603,
2592,
4058,
2299,
-729,
5913,
-3235,
-5091,
1015,
2808,
5002,
237,
-3468,
-1159,
3710,
-807,
-4425,
3385,
-4819,
-4230,
-6287,
2218,
5708,
1920,
-3593,
2840,
228,
622,
-6325,
2804,
-4278,
731,
7972,
-3676,
5088,
-4535,
1872,
-5649,
6740,
-3132,
2300,
9872,
-3503,
1309,
779,
7874,
3163,
-3783,
-2365,
-2183,
5150,
-2721,
-1582,
3495,
-278,
3064,
-623,
1020,
-2749,
41,
2577,
-2418,
692,
2614,
-1494,
9550,
-1413,
-333,
10554,
-175,
-749,
-630,
3456,
-3277,
6510,
2264,
12302,
4296,
3706,
1139,
2530,
-7089,
11411,
597,
-9619,
-2998,
-1018,
-1923,
-6844,
10772,
3208,
-7833,
-2164,
5510,
1245,
-5341,
-2420,
-3173,
-4317,
5562,
1214,
-319,
-4361,
5011,
7352,
-3355,
2504,
-8714,
-7055,
-2747,
4020,
-3873,
-1331,
1359,
5123,
-2876,
-2493,
-1300,
-47,
5337,
5672,
495,
-3996,
620,
-2498,
15674,
5288,
-4256,
836,
-1291,
6296,
1529,
-2290,
9724,
-7431,
-4856,
3718,
-720,
7254,
-946,
-340,
-5678,
5096,
-2828,
4310,
-58,
-687,
-393,
-4365,
-219,
5116,
-1330,
-222,
-2477,
1360,
1315,
-2575,
-1695,
5905,
-1230,
-1474,
-5078,
2379,
2197,
5025,
4379,
-3343,
-8602,
3278,
642,
7520,
6327,
4434,
-4564,
-2357,
-740,
-1010,
-10698,
4224,
-4235,
-1210,
-2163,
3926,
671,
-1475,
-1306,
-6265,
-2345,
-7376,
-8405,
-1230,
-51,
-10319,
2168,
289,
-1857,
6000,
1468,
-1217,
6803,
1600,
-2687,
539,
-4331,
-5987,
6738,
10334,
1845,
-12229,
-1397,
-2206,
-3527,
245,
-1661,
2005,
-137,
4289,
3966,
2960,
3391,
3439,
-5371,
4260,
2487,
1762,
3362,
4571,
3718,
-4214,
1916,
-3389,
-7629,
-2407,
7687,
1265,
-124,
-4662,
-2472,
5097,
-3000,
-1597,
6688,
-4149,
10668,
-351,
262,
-3695,
5224,
4692,
6901,
-4209,
2501,
-8060,
1934,
569,
9492,
5274,
-96,
6493,
341,
6289,
-5019,
-2158,
-5214,
1270,
-4419,
-11349,
3343,
3481,
-1308,
-5138,
5131,
-4351,
-3420,
-946,
2660,
13397,
3159,
-8388,
940,
-6190,
-7351,
-2698,
-5142,
3139,
6244,
-2742,
-808,
-3493,
14858,
-6014,
-10503,
10656,
2019,
-9099,
-3870,
-1556,
1514,
1766,
4000,
1325,
258,
-5389,
1079,
3608,
-6177,
-3238,
-911,
-7586,
-8184,
3927,
4290,
616,
-6564,
-458,
6961,
767,
444,
947,
-3898,
6351,
-3974,
-2245,
5127,
79,
5021,
-1097,
-4358,
9703,
490,
1999,
-7774,
-245,
3621,
7377,
-7587,
2168,
5204,
8325,
-677,
3624,
5558,
-6901,
1255,
5050,
1968,
2408,
7951,
-1744,
6810,
3664,
-2316,
-162,
-730,
857,
-636,
2105,
-3566,
-7967,
-1724,
-4616,
6552,
1151,
-3094,
-86,
-3025,
720,
-1616,
3230,
2539,
3682,
7643,
7235,
-1090,
-4366,
-4731,
-4018,
-6398,
-1018,
1741,
1324,
-3188,
4187,
-1850,
-8273,
3033,
4353,
-2299,
-3372,
-3464,
-7065,
1491,
742,
-817,
3393,
7675,
3523,
-6313,
-41,
4696,
3403,
-6105,
5778,
-136,
2314,
-3324,
4892,
5780,
4575,
-2832,
-4701,
-6217,
-2285,
-4159,
857,
-5292,
4198,
-7000,
4782,
4465,
-2981,
1447,
654,
-3594,
7442,
8776,
974,
4200,
-3847,
-154,
-1403,
-448,
-3292,
-1469,
-1529,
3543,
736,
-8906,
-2354,
-817,
-1641,
-423,
-5875,
-897,
-272,
6755,
122,
2876,
-4593,
-870,
6992,
1954,
-4989,
197,
2985,
-3116,
-364,
1693,
-2241,
4171,
1318,
4301,
-103,
8457,
-3407,
8469,
1174,
8546,
-1529,
-3292,
-3813,
12164,
-1234,
-3767,
-1518,
2528,
-2597,
5876,
709,
2702,
-3009,
9932,
11761,
5491,
4251,
1112,
-4434,
4659,
-7993,
2788,
5179,
-1114,
-2789,
-2284,
-2895,
2709,
3806,
-2879,
3003,
6953,
-5203,
-11215,
-3644,
-4567,
-590,
-3948,
1785,
-6331,
-6975,
-605,
5857,
1783,
1048,
-6345,
-138,
-3911,
5898,
-8620,
605,
-5605,
-5624,
-490,
-5000,
2959,
3526,
-487,
4876,
-12130,
-2941,
-79,
411,
2997,
838,
-5103,
-2979,
-6103,
6134,
9161,
410,
-8245,
-2227,
4493,
4899,
2392,
851,
-5454,
-838,
6583,
1891,
-5468,
-5344,
1524,
-1629,
3871,
-7556,
8354,
3283,
3861,
-6041,
3789,
77,
4902,
-135,
-3196,
1329,
-9581,
9147,
-4729,
-775,
-1362,
3455,
1855,
5666,
506,
-3345,
1627,
5589,
1155,
-6667,
4375,
-5211,
-585,
6916,
2128,
-4154,
1015,
-1107,
-2475,
901,
-1840,
-2518,
-4894,
-3529,
5258,
5410,
-5833,
7129,
-620,
-4816,
-369,
-2656,
-59,
-10783,
7210,
-68,
-10877,
625,
4327,
-8227,
-5223,
-9172,
2553,
-3520,
4311,
-5598,
4417,
1531,
2643,
-2233,
-1631,
2049,
-1935,
1450,
3957,
-5978,
1434,
5750,
-2703,
-4990,
-1971,
-757,
3406,
1720,
-6395,
5973,
-5518,
-2051,
-2791,
1365,
1550,
-6870,
2444,
3503,
542,
5669,
7087,
-1532,
-1085,
1142,
-5683,
-371,
1015,
-1463,
-1361,
-4310,
7567,
50,
6466,
-2007,
-4933,
4420,
-2374,
2567,
1763,
4505,
4697,
-1331,
3602,
-5385,
5026,
-755,
2485,
1779,
2093,
3653,
2917,
-87,
2963,
-2933,
881,
-1088,
-6804,
-5044,
4179,
1054,
2208,
298,
8146,
-704,
-1143,
-3618,
-1406,
-4920,
1754,
3211,
2722,
243,
4746,
-2241,
5888,
-1132,
1338,
1674,
1757,
532,
-8826,
-2066,
67,
-7471,
5088,
-3596,
-507,
-1537,
6174,
-7526,
748,
5630,
-3211,
3990,
-1151,
844,
-3434,
-3522,
5613,
-11667,
-1239,
-974,
617,
4331,
-611,
-2607,
-3723,
-4720,
5031,
3691,
-879,
-1113,
5671,
-1602,
-1191,
-5142,
-4227,
5284,
-6029,
-260,
-2622,
755,
2263,
-7353,
-1387,
5986,
-4385,
2264,
3240,
9173,
-8048,
4879,
-3545,
6996,
-9584,
3379,
-2235,
-3496,
6698,
-3148,
-6247,
-6981,
3324,
-4276,
-4079,
7344,
2403,
6217,
-3845,
533,
-6305,
1489,
-541,
-1735,
6884,
125,
6134,
-6043,
-3852,
3003,
5579,
-2961,
2895,
-58,
4396,
-68,
-2440,
-2105,
-3696,
-3196,
-4461,
11413,
978,
907,
-4461,
-6437,
-5373,
-4570,
6026,
5849,
9667,
-11641,
-1807,
945,
8114,
820,
2373,
3617,
-54,
470,
3536,
644,
-978,
-3715,
3713,
-2816,
4724,
137,
1459,
3270,
-3468,
4296,
-1721,
1004,
3575,
2971,
-2114,
-5630,
-5587,
3630,
4090,
-7547,
784,
-1129,
5,
0,
1555,
-3003,
-1658,
-7708,
-2544,
-5351,
-7694,
-5634,
6572,
3255,
5948,
1492,
-2966,
396,
150,
-3141,
10272,
-4203,
4348,
-7541,
987,
5087,
-1034,
4145,
-4959,
929,
-16970,
2797,
7192,
10290,
-9008,
-5626,
-1769,
1607,
1467,
4285,
-16812,
4309,
3943,
3173,
-4766,
-3731,
-17660,
-17758,
-3367,
3575,
-2026,
1432,
235,
9682,
-3233,
-7876,
1329,
-10183,
10693,
2516,
984,
4114,
-2752,
-10941,
460,
-6327,
9533,
-12163,
7446,
8977,
-1437,
-3205,
-13110,
8157,
15037,
-12609,
6429,
-1271,
4539,
-4158,
-5632,
-4259,
3625,
694,
-1959,
-8908,
1842,
7346,
5971,
95,
1719,
-2051,
-1610,
853,
5589,
1001,
788,
7005,
1346,
-1743,
1255,
3085,
-2479,
-5045,
7323,
-6210,
8143,
636,
-1262,
-3573,
5822,
-2174,
2635,
1143,
6573,
-11594,
6322,
6821,
6711,
4118,
11767,
-1803,
7859,
-2502,
15646,
-4231,
-513,
97,
1733,
11605,
-197,
3274,
-12382,
1303,
9009,
-911,
-3575,
-11821,
3081,
2130,
-4999,
-1238,
-13884,
3109,
-3064,
11447,
2111,
12492,
1947,
7463,
3746,
6787,
9600,
-1225,
-12593,
8078,
-2906,
-8719,
-1111,
7951,
4701,
-11445,
2113,
-3729,
10836,
-527,
4055,
-3507,
-2760,
-9713,
2646,
8830,
-3980,
-6865,
-1399,
-114,
9020,
16311,
-1440,
9707,
-3959,
-8195,
12719,
9180,
5940,
9861,
4311,
8121,
8201,
2078,
94,
1306,
6741,
1647,
-2812,
-8274,
-4185,
3028,
3559,
-7419,
1329,
-1436,
-1917,
4695,
5097,
4597,
-7912,
-69,
-4128,
-5143,
2577,
2590,
6755,
-6003,
-6140,
-3216,
-9479,
5105,
3321,
-529,
311,
-2043,
1574,
1154,
-4130,
-3314,
-8403,
-8738,
7717,
-1196,
-7143,
9134,
11706,
3672,
1861,
-6025,
7842,
5394,
-104,
1072,
9938,
-2830,
-2943,
790,
-3153,
-5238,
-5079,
-3674,
-10108,
-15006,
-4086,
-3722,
11031,
-11306,
-4361,
7195,
-9087,
-1270,
-8203,
13315,
-10084,
7850,
-9478,
-5099,
-4485,
-3580,
3845,
3288,
-4825,
13219,
6438,
5651,
-12603,
5780,
200,
1349,
2624,
-3325,
-3747,
-1739,
2047,
2269,
-3269,
7680,
5271,
17724,
-8403,
21,
6748,
-2220,
-15831,
1272,
5069,
5453,
-10251,
-3548,
7574,
-6894,
7257,
4529,
-5396,
-2991,
2839,
-5638,
7955,
175,
-2108,
9396,
1656,
-5039,
-2169,
-1254,
-3410,
-1129,
1231,
-4924,
-2752,
2835,
-13087,
-9738,
5257,
10324,
12208,
7492,
-11279,
149,
7619,
14500,
4522,
7513,
-6207,
-1615,
-12155,
-1797,
6898,
-1348,
5477,
4563,
2315,
-1308,
5164,
-4070,
-928,
167,
-6208,
3383,
-1229,
9890,
1865,
-1340,
-9088,
-4669,
9621,
17203,
-313,
7434,
1977,
9420,
-7987,
-2196,
10736,
-3437,
-8441,
4905,
7233,
-3046,
-12727,
2793,
3250,
5608,
1672,
-457,
1768,
-9329,
-8477,
7124,
1204,
3997,
-2652,
3155,
11212,
-1465,
9193,
13144,
9465,
-5886,
10282,
-4513,
460,
14499,
-9165,
-12529,
986,
-3388,
5998,
6503,
-1976,
4347,
4155,
10872,
5957,
9090,
16119,
5632,
4131,
5821,
3073,
-13453,
7592,
9248,
4687,
1979,
-6969,
1773,
-7292,
38,
-626,
1597,
-8047,
-3037,
-4529,
-2879,
-737,
-2478,
4117,
-1557,
5385,
3112,
1812,
6012,
-6711,
7632,
-4783,
-6264,
-19294,
3604,
2383,
-1121,
8490,
1673,
3104,
5961,
-3461,
-3005,
9644,
-1159,
-981,
-3666,
-7327,
1626,
4451,
12745,
6342,
6121,
2519,
-25,
2772,
-6740,
1693,
-1,
-142,
6281,
6853,
-5077,
-9654,
1814,
9061,
-8505,
-11637,
493,
2036,
-1554,
-132,
1269,
4244,
-7544,
314,
7208,
-5435,
9904,
949,
21224,
-8174,
-6496,
8287,
1313,
-10081,
-6833,
8422,
-5562,
-5861,
-2836,
3474,
8542,
-9198,
-338,
-2102,
-14000,
2455,
-5391,
-6393,
-11010,
3558,
-10134,
12318,
-1393,
3040,
4216,
5125,
-421,
6668,
4197,
-18088,
6663,
-8334,
10219,
3020,
5451,
16177,
-520,
-5539,
10783,
-6940,
2108,
-1151,
6671,
3477,
3120,
-5848,
3425,
2786,
2888,
-1497,
-4402,
-7067,
8483,
-10934,
-3041,
-1531,
2303,
378,
-4079,
-6713,
-3006,
-4735,
-151,
-18899,
-1252,
-8458,
-13802,
-7781,
-24,
-3068,
766,
-57,
1429,
-6200,
-1281,
-9984,
5647,
-1480,
3007,
9604,
8250,
-1235,
200,
-825,
4671,
-4537,
-1381,
2551,
3414,
-2342,
6092,
1449,
-11037,
-5954,
156,
14726,
-9376,
-4803,
-799,
-6954,
12990,
4830,
3330,
-7152,
10275,
-9192,
388,
-1958,
-3213,
-2425,
51,
6590,
-8989,
11554,
-449,
1263,
-1031,
10830,
-4644,
-6046,
10220,
-3014,
2728,
3914,
-4273,
226,
-2487,
-9112,
4806,
-323,
-3753,
-3199,
-13505,
-2989,
702,
13676,
8312,
1055,
-1217,
-7429,
6221,
-7596,
1315,
-7525,
-459,
1813,
-1211,
1047,
-1776,
1958,
2618,
-2389,
-3731,
11420,
-5961,
2998,
1497,
-8550,
1464,
4970,
8835,
6058,
10813,
-7181,
-10947,
-2782,
2522,
-10985,
-2742,
883,
-2881,
-4803,
7028,
2013,
-3752,
1680,
-8716,
1993,
4453,
-792,
10324,
8029,
-2543,
2436,
5093,
-5586,
-2375,
3056,
437,
-1718,
187,
-10001,
-10070,
-3967,
2105,
6596,
5936,
714,
-7062,
8123,
-6648,
3524,
200,
-5818,
-7502,
-7133,
4659,
1562,
-6223,
-12880,
-5450,
-1780,
3799,
7420,
5247,
-1233,
10216,
4741,
-110,
2306,
1421,
-2634,
-710,
-3238,
-2567,
-6631,
1801,
6910,
7468,
9602,
-8195,
8205,
5505,
-3207,
-2593,
-424,
-593,
-7455,
-932,
-1280,
-3978,
1336,
12035,
-7004,
10300,
-7289,
5542,
5643,
-6248,
-810,
6934,
-7932,
6339,
-6222,
10522,
-9306,
-6070,
9639,
-5861,
2234,
-5136,
7078,
-15845,
-5744,
-38,
-4139,
6764,
-173,
3247,
-11792,
2426,
9636,
-4121,
3658,
-7035,
7214,
-2511,
15694,
5803,
-262,
-7697,
415,
-4618,
-6851,
-3984,
-2894,
2445,
-4945,
-6782,
4220,
-4575,
2135,
4977,
-841,
2106,
-731,
4067,
-6094,
-3879,
3514,
-4094,
-4145,
-1114,
-2378,
-8922,
15280,
11775,
7387,
-5227,
-4919,
-2164,
-6053,
-3976,
-7110,
-9285,
214,
-1786,
-6003,
8519,
3807,
-8345,
7501,
-2761,
10513,
569,
-7961,
3083,
5415,
239,
-2338,
11725,
3763,
1482,
-1340,
-1057,
-3590,
7979,
-1962,
880,
6226,
2432,
-1451,
-2775,
-3583,
1261,
-637,
-7048,
1822,
1016,
-1785,
-1084,
-5573,
1849,
-8210,
-5191,
8269,
-6477,
5308,
-6685,
-3353,
6286,
5991,
-1216,
8203,
-5534,
11020,
5673,
-7588,
4488,
1628,
2874,
4884,
-9213,
-12009,
2968,
-6492,
4042,
-2219,
-6011,
409,
2327,
10219,
-6620,
4049,
2594,
-2704,
-707,
1692,
6064,
-5184,
-6049,
-9997,
-7729,
1600,
-8995,
978,
2911,
1383,
9649,
4847,
9881,
13072,
-6069,
10894,
6713,
6847,
3790,
7574,
-5346,
6580,
8594,
-7204,
14818,
-8079,
4962,
8064,
-6132,
-3352,
-9767,
4529,
2900,
3010,
-6038,
-4435,
-2590,
-5270,
6921,
-12261,
-2862,
9403,
-540,
2773,
-6124,
-3799,
1786,
-1624,
-7066,
-485,
-4123,
-2848,
-1766,
5997,
-721,
247,
3981,
-5619,
-6263,
-4248,
-5312,
-10007,
12024,
3108,
4627,
-3577,
-1115,
9301,
4569,
10000,
-4239,
3228,
-7136,
2140,
-7121,
7099,
10342,
8614,
16134,
-4187,
-2822,
6079,
-8545,
-4842,
2587,
-4171,
6914,
-10749,
-5667,
9989,
5361,
8876,
8489,
3908,
12561,
-1023,
-6562,
-230,
2448,
1032,
7120,
5848,
-1548,
10646,
-11054,
2353,
4497,
-18366,
-7705,
-14311,
11908,
3998,
7800,
-9319,
-7610,
-10666,
149,
-4597,
-4262,
811,
7774,
-2792,
9346,
-151,
-747,
-11728,
465,
-17502,
16569,
-2779,
9398,
-3787,
3130,
-10658,
5746,
5098,
2295,
6000,
68,
1982,
10264,
5611,
4134,
-7157,
4386,
5266,
2501,
1647,
-8027,
-4778,
-11357,
-8602,
-11849,
14014,
11801,
8759,
3853,
793,
3159,
-13117,
-2307,
-3350,
2406,
2500,
7616,
2623,
-3172,
11891,
226,
-18447,
-19389,
-850,
-2110,
355,
158,
-8297,
-5812,
-1234,
-466,
9826,
-6733,
-1083,
3595,
4989,
8468,
-3543,
3797,
3133,
1414,
-1304,
-5271,
14410,
-368,
-4681,
-1392,
999,
-1833,
-4420,
-5040,
-1055,
-15407,
-3589,
-4509,
-3090,
-11563,
11014,
-7966,
-8276,
1020,
-5499,
-6541,
10077,
15377,
1764,
-362,
7066,
2741,
-241,
6566,
-3844,
18799,
161,
6497,
7966,
-7800,
-163,
4782,
8551,
7063,
-590,
433,
-10329,
-465,
-5585,
7042,
3182,
-4831,
3252,
4659,
-1875,
9308,
717,
-6325,
-809,
-5833,
473,
622,
-15100,
-2058,
-4490,
-2242,
1217,
1667,
-978,
-10515,
1231,
-4855,
-3156,
654,
-2969,
10891,
5176,
-255,
4224,
8971,
12058,
2534,
8431,
6319,
-15380,
-2827,
-883,
-5873,
-5367,
-1250,
780,
-1541,
-9589,
-2495,
-3651,
3178,
3780,
-7662,
-4161,
-9173,
8471,
964,
2613,
9,
3214,
-8372,
2449,
-2357,
-2893,
-3055,
-5390,
-5259,
1621,
-922,
9003,
-6502,
-1764,
-1731,
7917,
3190,
-5400,
2588,
-4661,
-11387,
919,
-6914,
10750,
-2060,
-11024,
-1645,
-5699,
3989,
6105,
6904,
1093,
923,
3263,
-8513,
-371,
6977,
-11291,
-2631,
-3692,
2828,
1669,
4412,
-2597,
10362,
1592,
-3059,
2597,
-6796,
2219,
-10439,
-10672,
-4744,
-3186,
4771,
15420,
13565,
3013,
-6176,
5351,
-6027,
9946,
-6345,
-4436,
488,
-1612,
-9184,
-1162,
-8668,
9772,
-4002,
-6577,
560,
192,
395,
9313,
-8059,
-11298,
4992,
-3882,
-1181,
-1951,
3431,
2706,
10749,
-152,
5878,
-10149,
8139,
3531,
-8958,
-10229,
3625,
11306,
-8030,
-376,
4724,
-9322,
-5331,
-7912,
-2062,
-9151,
-9864,
7973,
3409,
-7273,
3635,
2912,
-7023,
-3329,
3879,
-5861,
3347,
2780,
-8455,
420,
2509,
892,
1211,
-2308,
-8749,
-5669,
3355,
1167,
3753,
1625,
-653,
2325,
-4658,
-10143,
-1385,
-2895,
-1251,
8769,
6153,
5830,
10284,
-11091,
-3101,
-2718,
548,
-3310,
-1270,
-10828,
40,
1301,
-5940,
12008,
8167,
1388,
-785,
1497,
-2149,
-825,
-4712,
-1137,
4780,
-157,
-1532,
5326,
-1589,
-9844,
2154,
-5249,
-7378,
4547,
-3191,
17552,
-3628,
15909,
1353,
2700,
1363,
7459,
12210,
-4812,
-2056,
2777,
1259,
-5814,
-4242,
2169,
-1345,
-8568,
-8207,
5872,
-5588,
-2396,
-9001,
1332,
1203,
-7206,
396,
6088,
3085,
8738,
-9655,
5537,
-1370,
-2565,
3603,
-3709,
10106,
-9870,
-5038,
-4688,
16522,
2382,
12908,
-7792,
-9446,
4835,
-2476,
6436,
10337,
9179,
925,
-5143,
-6761,
-1256,
7958,
350,
-2886,
4430,
-1406,
-10748,
-4431,
-9285,
-4228,
-15835,
13292,
5716,
-5781,
-16027,
-5298,
-4808,
1943,
653,
-1737,
-173,
1714,
9976,
2481,
-4868,
-12209,
-843,
5300,
7236,
-7648,
7421,
-4888,
-9546,
-5040,
-7395,
-3129,
9019,
-1079,
4325,
-9325,
3223,
6474,
3436,
-6998,
-4248,
-9037,
294,
-483,
1021,
2278,
4362,
13205,
5483,
1139,
-14979,
-1569,
-6205,
5535,
3076,
507,
-3731,
-11977,
209,
-12520,
-3494,
3490,
-2235,
-3152,
-4747,
-4703,
-9802,
5039,
-928,
8889,
4952,
4638,
-8509,
-2017,
275,
6106,
-17982,
2367,
2143,
9914,
653,
3293,
-13815,
-15564,
-9792,
4265,
11087,
-7316,
4174,
5207,
-65,
-4979,
4899,
-698,
-20551,
2776,
6110,
10874,
-3462,
10576,
1475,
-5470,
-2340,
-1101,
-3986,
-2323,
-751,
-729,
857,
-4343,
-6967,
7156,
6925,
2571,
-1052,
-4307,
3333,
-4887,
9912,
11289,
9767,
9164,
5308,
2202,
4104,
-31246,
2909,
479,
9051,
5399,
-831,
5847,
4745,
-7443,
2673,
-7312,
-3669,
4964,
-9173,
5678,
5274,
6904,
-4389,
5558,
4164,
-4202,
3294,
6312,
1917,
6743,
1016,
-2871,
2058,
11542,
-1685,
-1572,
5426,
4874,
629,
5562,
-6871,
-3013,
2334,
-1597,
-9486,
12808,
-6541,
-9870,
3975,
5685,
-1439,
1005,
1834,
-11810,
-4877,
2880,
5297,
1045,
-11154,
8884,
3046,
7246,
2564,
-2632,
6770,
-5564,
-99,
-599,
-3530,
1277,
-3096,
1690,
12794,
-2428,
5348,
-5753,
-2981,
1698,
-2928,
-16423,
13706,
781,
1357,
-3000,
1736,
1508,
-2395,
-6843,
6454,
2739,
1557,
7403,
-5478,
-4266,
-1451,
-206,
-3418,
7866,
-5181,
8848,
-3963,
-2011,
-7291,
-6749,
3891,
-7319,
-14712,
10116,
-6641,
7882,
-4202,
-5887,
-2264,
-5733,
3239,
10387,
4702,
7459,
-6289,
-7584,
-412,
17100,
4273,
8447,
6141,
7769,
-7732,
1255,
2023,
3741,
3314,
-6843,
-3943,
-3746,
-14125,
7237,
809,
5511,
4401,
-11371,
-2947,
-9863,
-16273,
19952,
9390,
-5691,
1846,
-5133,
-3340,
-1604,
-4363,
-4682,
5073,
-10954,
1799,
-1129,
694,
11189,
6455,
10920,
13006,
-9181,
-3654,
-10822,
6360,
-5658,
3465,
-11999,
9196,
-1970,
-4289,
2411,
-1546,
-12196,
8291,
-15,
6967,
824,
-4075,
-9621,
4830,
3316,
7432,
4065,
-8400,
14784,
-14130,
5326,
-10795,
1556,
6634,
-2010,
-3851,
366,
-244,
4479,
-2487,
9282,
5094,
-996,
-3646,
-865,
12059,
-1308,
7382,
-5165,
4652,
-10927,
-8159,
-6323,
9784,
3307,
2435,
-8088,
1090,
-3228,
1494,
8524,
556,
-1398,
2922,
-5029,
-5639,
-7300,
-927,
-1649,
-4508,
-5095,
-11061,
5741,
-5832,
-2981,
12665,
-2849,
4571,
-9991,
15205,
-3854,
12902,
-9525,
3046,
-1745,
1975,
-12124,
464,
-2969,
11744,
-8054,
3946,
5581,
-1349,
-982,
4509,
584,
-8102,
7815,
7040,
4237,
1956,
6814,
-5389,
-11057,
1099,
6136,
4303,
1802,
-3268,
3383,
2501,
-3709,
-8803,
6147,
9893,
2511,
7783,
-914,
-2520,
5952,
-8034,
-9587,
-5202,
-6025,
-7349,
-3540,
5345,
-10646,
-3109,
6249,
10205,
934,
-5509,
13848,
10802,
-1910,
-8711,
204,
7448,
4732,
10650,
-7402,
293,
7590,
10846,
-650,
12103,
4322,
2860,
-931,
-3205,
5908,
-2795,
3241,
-8136,
8553,
2416,
8667,
-10246,
8028,
-5598,
-4679,
-3967,
-9879,
-3117,
1391,
645,
-3774,
-11244,
19,
-5492,
-446,
-9109,
-10709,
7851,
-7441,
-747,
-1174,
11197,
4299,
3241,
10061,
-8820,
-8537,
2671,
-15100,
-12857,
-9800,
3884,
1007,
7078,
1694,
7263,
-7908,
-7820,
7426,
-9573,
-1005,
2607,
15358,
304,
1991,
3356,
-374,
-610,
-3810,
-1777,
-6192,
5593,
4501,
-7421,
-4855,
9670,
8417,
12475,
3352,
-8179,
11673,
-8934,
-755,
170,
19376,
-1046,
-1095,
3802,
35,
-1786,
3107,
5446,
-4920,
1514,
9301,
-3331,
-600,
4254,
-14843,
7196,
-4637,
-3872,
9310,
7767,
-5768,
4521,
1542,
5330,
8553,
-8046,
3188,
11618,
2129,
-2056,
-605,
-8203,
-1332,
3322,
8214,
-3081,
-11706,
2982,
-2032,
-17611,
11096,
3630,
-7022,
-612,
378,
4183,
2354,
4188,
5485,
3501,
1999,
6475,
3511,
7207,
6170,
1728,
1082,
-9181,
1860,
-8805,
-7478,
10270,
-1315,
1877,
1416,
-3489,
7453,
672,
-1607,
-10439,
-4427,
5884,
-5808,
6502,
-2070,
-3838,
3539,
53,
11598,
-604,
-1858,
-10368,
-6171,
15110,
-7997,
9424,
-656,
-10600,
1017,
5186,
-4721,
-4947,
2330,
-3966,
-2742,
2035,
558,
2995,
8649,
1501,
-10003,
9445,
-4253,
972,
-3249,
-1475,
-11392,
-8675,
2447,
7060,
4962,
-10586,
-5220,
-2255,
177,
-5942,
-2103,
6333,
-9713,
5774,
4448,
-13072,
2545,
9713,
-11041,
-1142,
3679,
-3346,
15638,
-2228,
-819,
-974,
6098,
-10108,
311,
-1913,
-2061,
-536,
-3232,
5674,
-7109,
4580,
4961,
5547,
-2008,
-5160,
-2965,
1843,
-14,
-6474,
-2929,
-3858,
-6698,
-3511,
8361,
8022,
10694,
3443,
2977,
-2566,
3319,
678,
-7988,
2352,
-2233,
7225,
5087,
-4235,
-3124,
-4364,
-13816,
-2148,
2032,
8502,
-6537,
-7999,
-1268,
1842,
-1903,
-10803,
6260,
-1908,
1106,
7207,
833,
9291,
253,
-2749,
4894,
5422,
-5724,
-10812,
-4497,
17426,
9768,
3048,
-1847,
-1206,
4405,
84,
-5693,
1096,
-3511,
-10251,
-12592,
9750,
-3866,
6422,
1205,
11261,
-37,
-4928,
-10869,
-10190,
5160,
-725,
-2785,
9126,
13780,
-6641,
-4029,
-1767,
2483,
-5336,
-633,
5681,
-11506,
-7339,
3169,
-15005,
6646,
-2821,
7204,
-7824,
-11658,
2617,
8935,
1106,
1024,
-4715,
9304,
-538,
2479,
220,
-4582,
3988,
4768,
12735,
3121,
5407,
-9300,
8471,
-541,
-11937,
-6723,
6042,
9555,
-5719,
5948,
11308,
-317,
2803,
-2137,
-1153,
-3713,
-5011,
4368,
8822,
-4715,
-7159,
-5057,
3326,
-409,
2690,
1092,
15578,
-7920,
-7924,
-5402,
-6426,
3603,
5781,
-1217,
4903,
-3070,
5414,
-7054,
1059,
-686,
-1485,
4441,
1218,
-10712,
1515,
-5785,
-1746,
-6725,
1896,
-3665,
6682,
-3854,
2812,
-3678,
5977,
9978,
9302,
-9235,
-2893,
-344,
10713,
16963,
-2434,
-5833,
714,
-9407,
4161,
56,
-6664,
9018,
-9704,
3367,
1924,
356,
-454,
7471,
-1401,
-4125,
6339,
-1497,
577,
-7329,
-2102,
-9702,
2463,
-1674,
-128,
-9240,
-3844,
-6825,
1446,
706,
-9507,
1015,
-4632,
-6506,
-13513,
6499,
9792,
-273,
3452,
6914,
-389,
1029,
4160,
3777,
6149,
-3415,
2053,
-5143,
-3283,
6403,
3756,
-133,
-4815,
-5723,
8457,
-1353,
1882,
4173,
4211,
8553,
8254,
1252,
2283,
-3973,
3028,
6642,
1689,
-2317,
-2118,
-6254,
4840,
-11195,
-2710,
1291,
2443,
1135,
2301,
7378,
8988,
6526,
4491,
8370,
-6217,
-3417,
13081,
-627,
-126,
1837,
5689,
5942,
-2970,
-5398,
-2657,
-8916,
-46,
10277,
-14278,
7398,
7249,
3543,
1482,
9990,
3990,
5571,
125,
-3391,
8285,
8285,
4081,
-7698,
-9974,
676,
-1481,
-12802,
5752,
-2003,
-2507,
-2307,
1467,
9045,
267,
3979,
3947,
-7942,
3225,
5873,
3379,
-6155,
13460,
2378,
2202,
-7424,
12115,
-10504,
578,
-2610,
-11091,
2704,
-7395,
407,
2885,
1282,
3995,
5111,
-14914,
1872,
1939,
-4437,
15518,
5957,
-939,
1844,
-116,
-945,
5096,
6458,
3217,
2010,
1170,
3571,
4583,
-4019,
1416,
-17295,
1032,
2523,
-10951,
13015,
5433,
-11281,
-12210,
3787,
-4045,
5513,
-6519,
-10570,
-2089,
2220,
8216,
2252,
-13061,
-4676,
-8069,
-237,
-7256,
-10673,
-6000,
-1051,
14470,
6744,
-6248,
-1062,
3269,
-1789,
-5502,
6307,
-2568,
2753,
12463,
5222,
-8209,
-1848,
799,
2739,
-6280,
-1856,
1427,
15872,
7706,
-7494,
-5326,
9550,
10041,
6901,
-1679,
-488,
-7077,
3716,
2024,
11365,
-1256,
-7050,
9214,
-297,
-9269,
1381,
-447,
4443,
-7260,
16412,
-5807,
3048,
-4273,
-1963,
8319,
-1906,
2908,
6248,
2906,
14881,
7305,
4401,
-5787,
-6455,
-4875,
-2323,
-4171,
3243,
2567,
-5613,
2195,
-2424,
-5268,
-190,
35,
832,
4110,
5170,
-7193,
1219,
704,
1710,
5481,
-4426,
3537,
-6028,
6767,
5218,
520,
1015,
3636,
12513,
-10798,
3461,
2638,
-3472,
-5075,
3449,
1193,
2671,
-11353,
5242,
-2168,
-1317,
7369,
4617,
-1484,
6544,
-6512,
-2496,
526,
17812,
-5176,
10304,
2693,
9078,
5365,
-4536,
-4096,
4000,
-5623,
4653,
954,
11923,
8591,
3736,
-5254,
5124,
-4174,
-9384,
6125,
-6419,
809,
-12085,
13408,
-9828,
8609,
-3199,
5357,
4367,
-3367,
588,
4401,
6212,
-7040,
-908,
-890,
-3691,
-9604,
-3794,
-623,
4306,
-13263,
5407,
-9214,
1899,
-1766,
-4619,
-9736,
-3723,
4705,
3298,
14295,
6341,
3595,
7410,
14190,
-13883,
6668,
338,
3173,
-9343,
-11306,
-1357,
5508,
7311,
-3021,
-4614,
-7443,
14311,
-5357,
-8392,
-4658,
3077,
-2774,
-7218,
6718,
3462,
-4957,
5200,
-5048,
-6877,
4369,
7215,
2680,
-20782,
-14563,
-1180,
-13864,
-7452,
-8383,
2104,
-2809,
-3745,
-1928,
6296,
-441,
-8684,
-3610,
6438,
11091,
2461,
8056,
-10676,
-4772,
10614,
3283,
-776,
-14577,
6344,
-8759,
4373,
-7244,
7560,
5884,
10409,
5687,
-8190,
7655,
-4181,
313,
3643,
1605,
1834,
159,
9275,
5671,
14794,
-4263,
-5096,
-5555,
-8859,
-2436,
-2999,
3598,
-4896,
16603,
3960,
-5700,
1232,
-1374,
-5965,
1226,
-10811,
5660,
10499,
-44,
-1279,
6961,
7804,
-5285,
-8811,
-12256,
14257,
5851,
5047,
-2587,
-4803,
3204,
1895,
-2806,
3610,
9651,
7178,
-3438,
-10130,
11561,
8239,
-832,
12567,
3273,
7388,
12335,
1445,
-4937,
754,
6343,
6443,
-8501,
-2262,
-6526,
-10473,
4641,
13706,
-15111,
-1201,
-2066,
-9988,
6622,
5382,
-5317,
3413,
-2528,
738,
-895,
1784,
1213,
640,
-42,
-5930,
2980,
2120,
-9226,
5593,
-7689,
-4551,
-269,
-9417,
-5108,
-2285,
-14322,
-11469,
-5946,
2377,
-5712,
5156,
2241,
-783,
-381,
8091,
-3610,
1831,
-3644,
6625,
6486,
4689,
-752,
-16646,
-364,
11339,
-5024,
2745,
-1806,
-8507,
2989,
7637,
2440,
1503,
7611,
-14058,
2212,
916,
-4706,
-1879,
-2010,
2742,
5404,
-177,
3555,
2323,
-8946,
-1304,
-5534,
11272,
2239,
480,
7909,
-2617,
4992,
7428,
2280,
-1349,
3532,
1905,
1321,
-5006,
9556,
-4122,
158,
-5933,
5369,
1579,
3824,
8519,
-4675,
-7335,
8178,
8619,
2936,
-1614,
3945,
1816,
6893,
-5857,
252,
-5641,
693,
-98,
6193,
-3440,
-4962,
-4305,
7353,
68,
1409,
-11468,
-13560,
5378,
-5484,
5554,
-321,
-7374,
507,
-9509,
7872,
-10061,
2645,
-1441,
-8572,
-2729,
-1011,
7392,
-2467,
2103,
-10135,
-6281,
5263,
-6751,
7370,
-2035,
10939,
3743,
3015,
-7508,
-10755,
-5085,
-436,
-12206,
5024,
10635,
7232,
9050,
5939,
1477,
-4724,
-4442,
5444,
3793,
-226,
-11930,
-7083,
-12252,
2224,
-3228,
9872,
4605,
3292,
-3602,
-5213,
-1376,
6192,
5160,
3733,
1344,
579,
-2111,
-358,
-5258,
-4736,
-11988,
6589,
7588,
8302,
77,
4429,
1731,
2726,
-387,
12385,
-3009,
-876,
4954,
-3196,
-332,
-3220,
-4186,
-7968,
-920,
2153,
-6921,
-6663,
5514,
8827,
10734,
2635,
-7070,
2213,
-9417,
16964,
-2869,
3353,
8243,
-5102,
-1765,
1170,
3989,
15724,
11310,
3640,
3652,
-10847,
-8260,
-483,
4873,
2590,
-6528,
6280,
4132,
-11029,
-1714,
3202,
-13399,
-1056,
7982,
2860,
3139,
-3327,
3678,
305,
237,
12608,
2690,
-9140,
11417,
-6357,
-4974,
-1035,
8324,
12495,
3092,
-5040,
-9380,
5007,
4935,
5442,
-4030,
4231,
-737,
3386,
-2648,
7641,
5234,
3253,
1115,
4166,
-13893,
-9902,
-1135,
8327,
3158,
2332,
1787,
-3252,
5763,
-8655,
4744,
-10619,
-7219,
12517,
-8788,
7090,
-5242,
5029,
14425,
-6066,
-1483,
-2069,
-8793,
-7484,
-1611,
3354,
5520,
-9913,
6326,
4126,
27,
2502,
2466,
-6540,
13736,
6709,
7490,
-2500,
-296,
-1794,
-3521,
7620,
6275,
3314,
-2605,
-3383,
9019,
3584,
-5706,
5909,
2243,
-10329,
-817,
7766,
-3979,
-181,
-3606,
-4716,
-3183,
-65,
-836,
-8911,
-11098,
15785,
8479,
4781,
-4621,
14540,
-1664,
5796,
-5851,
6159,
-1827,
-8489,
4733,
5664,
974,
-3565,
5758,
-5378,
5335,
-315,
6506,
-9965,
7157,
-6308,
2907,
736,
-8038,
12371,
2675,
-1495,
5056,
9857,
2043,
13456,
909,
-2322,
-2532,
-1692,
7199,
10701,
-6118,
5684,
-1137,
-3240,
2128,
899,
8417,
-9843,
-155,
-99,
-3194,
2441,
-8400,
3925,
-9005,
-5266,
-17307,
-6592,
-7792,
-6385,
9790,
100,
-8006,
-4958,
-2628,
-8071,
-989,
-2686,
-2241,
1544,
-5021,
10563,
-5826,
1860,
1897,
-512,
-5197,
373,
-5527,
-12782,
-13195,
3009,
7334,
3881,
-3283,
-9622,
13292,
192,
-3754,
2242,
-6302,
-3098,
9284,
3196,
-2121,
99,
-927,
-8893,
3779,
1259,
12301,
-1351,
13649,
2454,
15049,
9043,
11436,
-21725,
-933,
-7778,
10725,
6516,
-8174,
-6462,
6672,
-9283,
3002,
-168,
7300,
5140,
2912,
6497,
-106,
13989,
5851,
4372,
3191,
-5646,
-1661,
8159,
2269,
-6371,
-1839,
9156,
-15268,
-2548,
-13922,
7497,
-6928,
-5803,
-2722,
732,
-7733,
855,
-9363,
-11591,
-2114,
5805,
-2690,
-3021,
1111,
6639,
787,
4854,
3406,
-2151,
-5661,
-577,
8437,
582,
10041,
5275,
465,
-2327,
-9140,
-3221,
15861,
93,
7057,
-4313,
241,
-3412,
-1434,
-4743,
-3777,
2073,
-2424,
-12605,
-1066,
-757,
-5419,
11700,
3581,
3019,
1773,
-4490,
5642,
-5896,
16532,
-8988,
-7015,
-5079,
6555,
3610,
2244,
449,
10503,
5990,
7684,
14304,
1634,
1604,
2520,
4123,
3133,
-1153,
4109,
576,
-5798,
-5744,
-6870,
9639,
-7758,
-11108,
3252,
-5134,
14835,
-5355,
-3590,
-8620,
-14296,
-989,
-3473,
-6146,
6729,
7264,
-5960,
7281,
-3128,
4016,
-4294,
5612,
24455,
-1690,
-2768,
-5742,
-13623,
-7986,
-17232,
-8302,
-15554,
-9662,
-6855,
2350,
-3888,
11803,
-4913,
2305,
2658,
-9103,
9286,
-5369,
17312,
-2244,
5835,
-11185,
-1101,
-7527,
-6138,
8129,
2679,
-5805,
-5998,
3794,
-9792,
-5864,
-3654,
5112,
216,
6510,
6923,
95,
-3087,
-5030,
425,
4554,
-5229,
-1506,
1656,
9642,
3176,
2769,
4886,
-2719,
-11262,
-1504,
192,
-2688,
3367,
1858,
-7560,
3259,
-273,
-1304,
-1218,
-739,
2623,
-3300,
1210,
6512,
-1850,
1652,
-4140,
12320,
4453,
-6119,
3072,
-2417,
3840,
152,
2316,
5162,
-3760,
2362,
8406,
-5307,
-2806,
-9355,
3866,
11366,
-613,
1839,
3619,
-2115,
-10168,
-713,
5803,
-9029,
-1748,
5354,
7441,
2124,
10917,
9100,
6266,
16644,
-2553,
-6102,
2536,
1669,
2996,
-5355,
-8565,
8389,
353,
-2037,
1973,
-12591,
4472,
-827,
-6043,
-405,
8431,
-6554,
-3537,
9451,
-2629,
-7861,
4932,
-4489,
1681,
-4731,
1392,
10130,
-12439,
-3775,
10869,
1359,
-13216,
-40,
1528,
9439,
-3929,
2895,
4496,
-7161,
-8609,
1489,
-3239,
-7959,
-6251,
10706,
2369,
-5698,
2618,
7339,
-928,
-10950,
-6958,
-4098,
6747,
-11233,
-2135,
580,
-6912,
3023,
572,
-2778,
-5569,
-3781,
-2277,
-1392,
-3740,
4042,
-2721,
-2048,
-8563,
-686,
7460,
3222,
1754,
7145,
-7396,
-10549,
-4082,
-10236,
-4392,
-7082,
240,
-5456,
-4412,
-706,
1905,
-1035,
-3938,
-4296,
-13674,
10841,
-15716,
-619,
-1019,
9006,
-2363,
-4562,
-2335,
-4746,
-14627,
3047,
-2035,
-516,
-387,
-12357,
8192,
-2288,
-2967,
-10882,
-7222,
-2027,
3005,
-1646,
-99,
-2926,
-950,
3835,
-10243,
-3595,
2264,
2765,
2293,
5325,
2114,
1015,
2485,
3409,
-5674,
-5064,
1996,
7044,
-493,
1773,
-16894,
-2637,
5670,
-15186,
-2528,
534,
4318,
-4825,
2922,
12413,
4281,
13218,
-12916,
2773,
14017,
-7872,
139,
-7930,
6688,
4050,
-6073,
-5231,
3254,
3392,
11418,
-98,
-2105,
-2417,
7336,
-8699,
-6938,
-5308,
-2590,
5506,
-352,
19,
-3454,
-5361,
5535,
98,
-1410,
1909,
3578,
3931,
5661,
-6608,
-3178,
763,
-7987,
4429,
3048,
814,
-8998,
-5088,
993,
-10305,
-4838,
-2067,
7555,
-13668,
998,
-6331,
-398,
4425,
-7507,
13508,
4815,
-2110,
-2744,
810,
566,
393,
-9388,
177,
12190,
5584,
5505,
-2826,
-12751,
3280,
4773,
-8146,
6075,
-6177,
7185,
-9970,
-4945,
-90,
5844,
-7863,
-8503,
4387,
-3033,
5515,
5680,
-1094,
-7252,
9261,
-14000,
561,
-8694,
5482,
1753,
1834,
-9982,
-6234,
-967,
-7305,
-2000,
-2701,
3156,
12571,
-6204,
2295,
9585,
11023,
2525,
-2125,
6455,
3691,
4246,
12097,
-3713,
-2678,
1978,
6831,
5043,
-6370,
1592,
-1956,
-2530,
2836,
6131,
2767,
-7357,
-5529,
-3167,
-1982,
-11245,
596,
6380,
7256,
-7276,
8099,
1522,
1704,
3983,
-9926,
-5645,
68,
-3118,
3101,
3381,
-9827,
2579,
4210,
1511,
1876,
7543,
1414,
1796,
3587,
165,
1266,
-4081,
-4867,
730,
-8379,
4182,
-1150,
-616,
3032,
13705,
13014,
-3388,
7771,
-4739,
4652,
-1142,
7413,
8573,
2377,
2610,
-5319,
-2763,
7513,
557,
-4191,
-6352,
1269,
-2161,
2917,
3098,
583,
-942,
4407,
8502,
-2149,
3975,
-4839,
4939,
-407,
-9385,
-4244,
-5160,
4399,
11306,
-2189,
-2820,
3046,
-11951,
5464,
4147,
942,
-9656,
-6103,
-5914,
-5495,
4343,
2181,
5135,
15067,
1395,
6754,
-3272,
-5958,
17821,
-341,
2516,
-1683,
-9387,
5407,
-9232,
-2595,
-4128,
3346,
6856,
5819,
1583,
-1531,
-4454,
-7542,
-5624,
8022,
1376,
-1231,
-4153,
7413,
3251,
453,
12727,
-812,
-7587,
8539,
-2315,
-2363,
-5061,
25218,
83,
-8154,
-3597,
2287,
-2156,
-5027,
-15644,
-1620,
-10052,
1227,
3371,
97,
5578,
8687,
-2517,
1111,
2977,
493,
-14215,
1946,
-6377,
-8082,
2620,
-3323,
-2185,
-706,
-1371,
-1686,
7645,
-5313,
-230,
-6312,
-3720,
2922,
140,
9559,
-5738,
-161,
11014,
4238,
5900,
8158,
4158,
4776,
4370,
3574,
7384,
-160,
13682,
-571,
-10262,
16453,
-8021,
-5419,
13201,
-8870,
-2016,
-2072,
-5374,
-381,
7836,
-10685,
347,
-4914,
-3278,
3281,
-1395,
-4226,
10985,
1324,
-3720,
8019,
-4013,
707,
-4472,
-14286,
-1306,
-5505,
2751,
3837,
-1866,
5950,
-4537,
10462,
-9360,
10770,
13922,
-1943,
-1832,
4382,
8178,
-6376,
9744,
-7229,
6579,
-11936,
-290,
-2593,
-1508,
-1322,
4455,
-8989,
-9636,
12704,
3282,
6427,
-6694,
3785,
1762,
8225,
3477,
-12900,
-610,
121,
10069,
2195,
-384,
-2869,
-5771,
-1176,
3406,
-6146,
4615,
-10384,
-2062,
-13208,
1329,
-2851,
-4906,
-6626,
8444,
4222,
8635,
-4810,
-2733,
-1333,
-2074,
-14093,
4308,
-6336,
6599,
2384,
-11164,
2287,
6045,
7554,
-6344,
6023,
3374,
-6401,
-6151,
3695,
-5099,
-8857,
366,
-157,
-6146,
-9134,
-6664,
707,
-10636,
5956,
6225,
-4097,
-7726,
-4443,
-1174,
-2062,
-4769,
-7925,
923,
-8033,
-1759,
5194,
8807,
-3825,
-3878,
1840,
2543,
-8417,
1605,
1183,
3403,
2633,
5583,
-4288,
-4199,
5123,
-3825,
-11397,
-2721,
-6924,
-8632,
10852,
-6861,
-5929,
-6381,
5276,
3847,
-1367,
2362,
-3447,
-6871,
-5697,
4697,
-4079,
-9404,
-580,
11440,
6654,
2085,
2626,
9621,
-1709,
7143,
7826,
-1760,
4603,
13154,
-3631,
2255,
-3795,
7100,
-6273,
10653,
5616,
8943,
4895,
14281,
-6318,
-8765,
9667,
-4435,
-7409,
5222,
2522,
3966,
-3154,
6445,
1877,
12804,
-7310,
-8639,
2757,
-3334,
9017,
-893,
8496,
-1037,
-2535,
3927,
-2265,
5810,
-3166,
-1973,
-3528,
-4203,
-1202,
3362,
2469,
294,
974,
9572,
-5281,
6054,
4695,
-52,
-3000,
-6451,
2739,
3374,
2491,
11055,
-13680,
-2358,
-2345,
471,
-18057,
-7846,
2459,
-4900,
-1701,
3398,
-1282,
3556,
10030,
7319,
-2157,
5537,
-1258,
2643,
-688,
-6517,
-1140,
-2996,
-5923,
-5141,
-6787,
-8521,
-12666,
5388,
9601,
1688,
7357,
12204,
-8554,
-3407,
2833,
-3790,
11849,
7343,
-12307,
-2859,
2416,
11762,
-943,
-6520,
3991,
-3032,
-4814,
7760,
4457,
1605,
-6818,
8648,
2681,
4559,
-4785,
13057,
-388,
1076,
5518,
-3793,
-460,
-3412,
-5643,
7004,
4199,
5910,
2680,
762,
-1728,
2942,
-2873,
4436,
-4187,
-8910,
-9292,
-6436,
-5862,
-4926,
2214,
7816,
7881,
-7777,
5494,
-6477,
3647,
-1497,
-20041,
10656,
60,
-7542,
-5668,
9219,
-9404,
-1187,
2963,
-2907,
-4944,
-3801,
1153,
-1184,
3036,
1251,
-947,
4155,
7209,
7132,
-241,
156,
8865,
-2718,
-6212,
5,
673,
-13174,
-12038,
860,
1695,
944,
12849,
357,
3600,
2098,
-5739,
-2301,
4989,
-3000,
2204,
-2257,
1274,
-5150,
10172,
6778,
33,
-6224,
5821,
-6711,
2099,
2970,
1092,
1451,
-3270,
8795,
-3562,
8432,
-995,
1406,
1503,
3830,
-2180,
-3767,
-705,
2418,
1228,
1541,
-3564,
2573,
3449,
-2501,
-3908,
1511,
8828,
139,
4431,
2267,
-11063,
-3038,
-1172,
827,
-706,
-2032,
8434,
-5683,
-2455,
2161,
-3990,
-7863,
1649,
-1909,
2506,
148,
-1511,
1711,
-767,
-878,
7372,
93,
6076,
-4915,
-5179,
704,
-1363,
2534,
-6574,
4088,
3336,
2059,
-5007,
4815,
-2042,
1261,
-452,
2780,
-4526,
4031,
6793,
-1449,
1495,
3611,
-1896,
4338,
-1949,
-458,
-5809,
-3604,
7174,
1612,
-4254,
4003,
4985,
3881,
-3866,
9767,
-4642,
-2202,
-1453,
8879,
-1489,
-1494,
2131,
-2889,
-6520,
-1069,
730,
-342,
4195,
-1996,
-1472,
342,
-7023,
-188,
-3886,
-9411,
-1655,
-3720,
-1789,
4109,
4015,
746,
-6356,
3410,
-7217,
2834,
641,
-4473,
-3618,
5357,
-210,
-558,
-821,
4472,
-1215,
8217,
457,
-1374,
2841,
-2663,
-1214,
665,
-4770,
-2615,
-7202,
-5098,
4258,
-1013,
4109,
1199,
-6562,
-4858,
-5574,
5080,
278,
-3271,
-3326,
-5573,
-7989,
-4963,
1002,
-152,
-3585,
4328,
-1559,
1732,
5315,
-4585,
-3351,
2804,
-1334,
87,
-1946,
4137,
1125,
-1820,
-1745,
-5968,
-461,
1290,
-2815,
679,
-1803,
-1747,
-2525,
-6205,
-5073,
-852,
-6949,
-2184,
-2849,
9267,
-8137,
-5861,
-1217,
1851,
3384,
5084,
-9182,
-6328,
5389,
1852,
8114,
-5297,
79,
4976,
-2135,
2507,
-6751,
8652,
-2535,
2091,
-1221,
382,
931,
-3373,
2570,
-2251,
1186,
-5552,
8236,
-3405,
-2120,
1664,
941,
-1697,
2703,
-5185,
-3753,
4155,
-2984,
4850,
2623,
-3981,
2312,
-2756,
-9466,
-4406,
7782,
-2569,
-5271,
4284,
-3221,
2550,
-1505,
3159,
1925,
1113,
-3737,
2559,
2402,
-11436,
4781,
2490,
3075,
-4012,
-2792,
-1759,
4164,
7077,
-2720,
-4096,
-3684,
-1559,
-2601,
-2184,
-4003,
-1905,
2438,
-133,
4631,
-8539,
720,
1211,
4147,
8951,
-5218,
-764,
2509,
-727,
10007,
3231,
-2212,
650,
-164,
1077,
1281,
-10577,
1035,
-1146,
10605,
-3260,
-10570,
-4761,
-795,
4213,
1786,
-5330,
-5300,
4871,
200,
-4598,
-5581,
1388,
1162,
-100,
-297,
-540,
-9462,
1965,
-6204,
4178,
2630,
2605,
-4129,
5481,
-7387,
-9307,
1711,
-1729,
-9293,
3177,
308,
-8068,
2566,
-5646,
-2331,
2629,
8706,
-579,
1562,
14072,
-1469,
3310,
1330,
2193,
5544,
-1151,
-610,
-6155,
-293,
-1911,
4641,
2692,
2325,
1067,
-2880,
-3192,
-553,
-6867,
5190,
681,
-3953,
2402,
2964,
3786,
-2100,
-3294,
-4740,
-8641,
820,
1603,
2604,
-728,
109,
9560,
5300,
5729,
-1937,
-4115,
2850,
5744,
7942,
-5431,
-2737,
641,
221,
-3711,
9341,
-3815,
-634,
-1101,
-2067,
-7179,
573,
1091,
1890,
-3129,
5876,
-821,
5940,
-4363,
-731,
8,
5780,
-4875,
11955,
-4364,
-6916,
4361,
-2032,
2412,
3854,
-2976,
-5232,
5635,
-9899,
-4644,
4565,
-7569,
-7787,
1899,
-5337,
7487,
-502,
-6809,
2747,
4257,
14530,
-5551,
-4771,
1560,
1726,
-7322,
-201,
-3539,
5357,
3872,
1025,
7001,
3707,
-202,
-1052,
1011,
-5127,
2442,
6627,
3709,
-53,
-9036,
-6297,
4008,
-9605,
-4859,
9199,
810,
2838,
-6684,
-2469,
-12528,
-948,
4056,
-4034,
-829,
-4268,
-3783,
-734,
-936,
7118,
5619,
-454,
-396,
13090,
2568,
-1034,
-5490,
-4806,
-3159,
3792,
-317,
-1321,
431,
8830,
-3295,
-80,
1467,
2531,
4101,
-3746,
-2347,
2067,
-1011,
-4303,
-2795,
-7832,
-2669,
3769,
-8659,
3088,
-4865,
2713,
-4615,
-3691,
-2690,
2544,
-361,
9733,
5047,
-2045,
9047,
-4500,
-1414,
-4667,
3496,
-14,
5412,
-318,
2606,
4171,
-505,
5553,
-11251,
-4025,
1658,
-6697,
2524,
582,
2406,
3467,
379,
3098,
1126,
-5365,
2517,
-599,
624,
-1681,
-827,
-804,
-4144,
8954,
445,
3316,
4877,
1302,
-243,
12267,
-643,
-1939,
-2877,
-668,
-6013,
456,
-778,
-2282,
-3974,
4032,
-3032,
-5654,
9003,
5835,
1743,
-7511,
9277,
683,
1188,
8198,
3241,
-8449,
8185,
-1073,
9830,
-4729,
-1409,
6189,
-602,
-4400,
12957,
-3143,
1680,
5762,
3351,
5747,
-1626,
-4047,
-859,
-2879,
1293,
3108,
-1135,
-2199,
318,
3478,
9601,
-1148,
3232,
4227,
-2484,
-3840,
-3565,
-5199,
-959,
-2703,
7642,
-2202,
-7349,
-710,
-1335,
486,
-3285,
2978,
7194,
-14,
-2043,
306,
385,
-2783,
706,
-1868,
-4555,
-1839,
-2863,
2555,
420,
535,
2342,
-586,
-5202,
5519,
3838,
-4371,
-4339,
-2822,
2277,
3666,
-2549,
5572,
-2215,
2585,
6099,
5276,
6046,
6677,
-537,
10518,
-1946,
3088,
-1415,
4871,
-943,
316,
7065,
-27,
-2597,
3019,
-3107,
-10172,
6733,
4302,
-800,
259,
8233,
-4579,
774,
1965,
3793,
2108,
-5558,
-2612,
867,
10403,
2632,
-2102,
940,
-5854,
4472,
-2088,
4532,
-1,
1817,
-924,
-2509,
747,
577,
-4913,
-5715,
2737,
90,
12352,
-1787,
-777,
643,
195,
1549,
6111,
5588,
846,
-301,
3821,
1630,
-6792,
-5528,
4848,
121,
7864,
-1357,
406,
5001,
-3390,
2165,
909,
-4891,
-3799,
5452,
-2545,
-2083,
1985,
8246,
2217,
-2901,
2650,
-2160,
-2073,
-1163,
-72,
-2500,
-5019,
539,
686,
-2905,
-3212,
-2194,
-284,
-2316,
-1646,
3607,
318,
577,
-7033,
4701,
6516,
-6317,
321,
6262,
2522,
5982,
7270,
-3576,
-711,
-4531,
-4105,
-5842,
1956,
6515,
-4648,
927,
873,
-10575,
8881,
789,
-3709,
-6684,
5088,
2444,
2900,
1015,
6719,
1905,
11014,
-6180,
972,
5250,
3780,
-173,
5527,
-3338,
-493,
2845,
-7160,
-4000,
-3322,
-5654,
2803,
-6566,
-5529,
908,
574,
2500,
-5253,
-6112,
-1314,
10205,
-2065,
-3345,
1183,
4477,
-3395,
-6010,
-8014,
-2787,
4163,
454,
7120,
5410,
-1458,
1920,
849,
-6974,
-1613,
2529,
-4669,
-4300,
-4000,
-4119,
5183,
1853,
1242,
-6074,
-6335,
1780,
-202,
9457,
2781,
1258,
2181,
-6135,
-4484,
9136,
-2470,
-872,
-2704,
-5244,
-6549,
2089,
2306,
7838,
4033,
-5328,
-8157,
-1320,
4227,
4472,
-5996,
1255,
-9061,
1638,
2393,
-3603,
-2150,
-2402,
191,
-1488,
4671,
-5425,
6105,
-993,
-6250,
5227,
-5774,
-5009,
-6331,
2892,
-1383,
593,
-1325,
177,
2282,
733,
2766,
7525,
-1578,
-5578,
11820,
-1613,
4144,
-10380,
4370,
6320,
4168,
-3086,
-6965,
8104,
3296,
4525,
5525,
-9565,
-1465,
-4001,
-452,
1311,
4055,
-7981,
-690,
2594,
922,
713,
3148,
4922,
-5581,
120,
-3682,
-5283,
-246,
-540,
-2421,
4109,
-673,
-1197,
1443,
-3337,
-5255,
-3618,
-980,
4842,
6504,
1966,
-1823,
243,
4670,
6134,
-1760,
6938,
2944,
-4637,
1256,
7300,
-10,
-4075,
-5383,
-1401,
-1280,
-3526,
-7292,
612,
5480,
-4409,
4904,
3757,
-3663,
-1095,
-2335,
-3037,
-5496,
-3132,
2223,
-3008,
553,
-1731,
-7635,
-3530,
2015,
379,
-7828,
-2414,
618,
-2257,
2962,
6450,
948,
-1966,
-886,
6267,
5220,
-4989,
-4683,
1256,
-2681,
-2017,
4414,
-3477,
-2962,
3323,
-4596,
-2210,
4811,
2222,
-3605,
5154,
-6317,
-1450,
-3044,
228,
830,
-136,
-1474,
104,
-4205,
4006,
-4267,
573,
1021,
7110,
3256,
2934,
-3112,
4452,
2198,
-2902,
951,
-5886,
-347,
-5449,
3281,
-4101,
381,
-2626,
-1716,
-2637,
-4681,
-3654,
-6047,
-5788,
3081,
9368,
5128,
-2693,
-8770,
-4544,
-2287,
-7844,
-6260,
-3266,
-1141,
170,
2687,
3037,
1716,
-2139,
-4688,
3320,
-4202,
3858,
1541,
-3229,
782,
-890,
-5237,
-2360,
-3665,
-5236,
-2518,
1606,
2395,
-5597,
7411,
209,
-4650,
-4550,
1215,
-4222,
-2651,
2417,
6957,
-4864,
3107,
1475,
275,
4562,
-984,
2623,
-5308,
2820,
-2806,
2388,
5446,
6067,
4258,
-10866,
-4981,
-9116,
-3877,
-4932,
-90,
2076,
2142,
2917,
-3029,
5562,
745,
3111,
3303,
-7488,
3296,
-1737,
-417,
2603,
7917,
4395,
-15195,
-8551,
5355,
-4665,
899,
-2961,
-844,
-3511,
-7664,
-3049,
-2518,
-3335,
87,
6672,
-4011,
-5567,
2265,
-6652,
-6585,
-7346,
-3577,
-3743,
-7949,
3290,
6081,
2408,
446,
-2122,
3821,
448,
-6676,
997,
519,
8416,
748,
8063,
-1761,
422,
3606,
-934,
-4247,
8691,
2018,
2683,
3550,
-6289,
-2288,
3153,
-4085,
-1077,
-1065,
-7894,
2037,
-4149,
-7477,
2600,
4732,
-222,
-1969,
-9374,
6457,
-2837,
-12080,
-4245,
-1495,
-4876,
801,
1650,
1665,
9188,
1760,
-3007,
-3022,
1436,
903,
-4333,
4530,
1636,
7488,
-60,
-6668,
1914,
1232,
2340,
1366,
2191,
5288,
-1400,
1160,
5056,
-2245,
6762,
-1388,
-762,
-5175,
-1774,
-3624,
-2889,
-3401,
-873,
5628,
218,
4401,
8278,
5202,
-9009,
899,
2770,
1969,
-8281,
-8604,
-5189,
-4058,
-653,
-2744,
-6004,
1532,
2506,
5112,
945,
2900,
-2739,
-8537,
59,
5196,
-4616,
7169,
-3011,
-2691,
9324,
7547,
7048,
-2591,
10010,
-3580,
6346,
876,
-153,
168,
584,
986,
-813,
-2087,
3086,
-1496,
-581,
856,
1971,
-1213,
-2510,
-4469,
3863,
-6219,
1743,
588,
2737,
-5826,
3450,
-2059,
-949,
-137,
2619,
873,
1118,
-4404,
-2309,
-749,
-2243,
2030,
-4444,
-78,
776,
2302,
-1618,
-3975,
2985,
-1956,
-3512,
-2957,
6600,
-4250,
733,
-618,
-6306,
-1941,
3353,
-2168,
5762,
-5153,
1486,
-1571,
-1182,
-1370,
-7153,
1264,
877,
-3125,
4267,
3183,
-2587,
184,
-3051,
-1792,
-2170,
2448,
1713,
354,
263,
-2964,
-7115,
1677,
-1264,
-2557,
-1652,
2645,
-2950,
2666,
-3108,
674,
-1457,
-7615,
8495,
4352,
2958,
362,
3552,
-4253,
3218,
-5194,
865,
2670,
3190,
9730,
-10423,
-1738,
266,
2748,
7241,
-4695,
-1280,
5973,
216,
5055,
1842,
341,
8327,
-4471,
-5121,
32,
-901,
-1880,
-2028,
1773,
10083,
354,
1774,
5143,
8514,
-2094,
-1479,
9394,
-1670,
8223,
2409,
-1148,
-4378,
506,
-8377,
-2510,
-4924,
3638,
1130,
840,
1386,
1219,
-3419,
1918,
-1561,
3532,
-4243,
-2337,
-869,
2989,
187,
732,
-4587,
1010,
5815,
-10739,
-994,
-4371,
-7273,
-2410,
455,
1719,
10795,
-6331,
2850,
6895,
-168,
-5880,
-25,
209,
47,
2050,
-1625,
-4877,
112,
-4566,
-1682,
1460,
2690,
172,
1353,
1436,
-5957,
-4884,
-2125,
2565,
-7272,
-7928,
-324,
-7565,
-3207,
1626,
-573,
4353,
4041,
-1799,
1560,
5260,
-3341,
-2558,
-4953,
2141,
6713,
-635,
5946,
1193,
5455,
1306,
-7866,
5091,
913,
5979,
-9895,
-4501,
4167,
2923,
3834,
-2765,
-2977,
-240,
22,
-849,
5747,
857,
-5964,
-2795,
3375,
-3778,
-6898,
-1746,
-5357,
6943,
-4022,
2810,
10196,
-1505,
-3032,
1756,
6203,
5966,
-3371,
-6714,
-3791,
-3176,
-509,
3691,
2711,
-7085,
-11123,
-4268,
3601,
2675,
1601,
5022,
3571,
4306,
-3659,
-7404,
-6308,
-9027,
-2218,
-3367,
1224,
61,
-224,
3489,
-889,
4653,
-8433,
-25,
1404,
-13543,
-2353,
-8885,
-535,
2414,
-6571,
-3870,
-2547,
-5593,
2554,
-5320,
4591,
1882,
4676,
1411,
2317,
317,
1161,
10686,
-4930,
-3221,
-1323,
178,
5956,
6745,
-744,
4370,
-3113,
1811,
1148,
-14704,
251,
6012,
2627,
-5717,
2875,
-2000,
5963,
1027,
7073,
6953,
6261,
6712,
-7338,
3650,
-2045,
-1905,
-2545,
4606,
5422,
7689,
-6229,
5384,
1442,
-3031,
-6797,
-1430,
692,
-3897,
10467,
-6998,
-1946,
-1204,
8013,
4792,
-2555,
-3399,
5793,
2720,
3817,
-3427,
2168,
-137,
-1985,
-3211,
-1877,
4932,
3639,
188,
-6901,
-1518,
-979,
-2134,
4015,
2406,
-4711,
772,
2296,
-525,
-232,
-5429,
-7665,
-2588,
10718,
-438,
7873,
4155,
-1437,
-446,
2853,
1567,
5318,
-5689,
2233,
3536,
6190,
-105,
820,
6574,
-2875,
-1925,
-4719,
3164,
-1702,
-1608,
4775,
-3338,
-4632,
2004,
-5099,
2496,
-974,
169,
-1362,
3108,
12021,
5514,
-8145,
-8713,
126,
-3043,
2211,
11,
5413,
3541,
2211,
-2273,
-3751,
5705,
-7540,
-4573,
-7334,
7129,
2343,
301,
4187,
4993,
-2683,
5718,
3290,
1034,
662,
3772,
8963,
-9372,
-3732,
-109,
-188,
1960,
-2503,
1424,
-5671,
72,
5997,
268,
2245,
1383,
4943,
727,
8031,
-6252,
2120,
-2023,
-8338,
-8557,
4181,
-4240,
1817,
-1822,
4275,
-6196,
3469,
558,
-8476,
1401,
-250,
-689,
1541,
-1797,
6289,
-7940,
4143,
-5910,
-499,
-2620,
4196,
8588,
1054,
2523,
-1262,
-2277,
4458,
5710,
-8207,
-4503,
1139,
892,
3167,
3834,
-3500,
-312,
-2069,
-609,
-500,
2367,
-2630,
-5496,
-618,
902,
-4648,
3198,
9822,
-9766,
3131,
-1760,
-1867,
3132,
2526,
-3299,
-6160,
3904,
3893,
-2852,
9257,
3678,
12272,
-5006,
1193,
3737,
2922,
695,
-4707,
4582,
8247,
-7567,
6725,
1977,
9113,
922,
-6501,
-2664,
-4183,
7621,
-2125,
887,
-6135,
-1885,
1438,
713,
-2856,
854,
1283,
4618,
-9629,
-4429,
1027,
1680,
-2298,
-7587,
-1976,
3455,
-1287,
81,
-5135,
493,
-2304,
2365,
-1438,
8407,
-1400,
-2607,
453,
8666,
1463,
-1145,
-3965,
-3009,
-6623,
1834,
-2965,
5035,
3363,
5874,
-5320,
2306,
716,
3097,
-300,
-787,
-1088,
6523,
-1316,
-264,
552,
-231,
5917,
-363,
-790,
5843,
-339,
-5691,
2679,
5614,
-2058,
-9378,
2185,
5892,
614,
-562,
-701,
-312,
-2358,
-2862,
-1527,
-9200,
-400,
1425,
3396,
-1731,
1733,
-11039,
803,
-4666,
355,
-7072,
2190,
505,
-645,
-2519,
3164,
6042,
-3070,
-2394,
7761,
-4999,
2596,
6037,
-8038,
1792,
-1180,
-3665,
5406,
-2513,
7309,
3722,
7543,
-7304,
-4641,
75,
1470,
-1561,
-824,
7906,
-6887,
1835,
4356,
461,
-3362,
6018,
-9347,
8,
1843,
-6818,
-2023,
3574,
-651,
-5688,
993,
-350,
-5571,
3554,
8777,
-1655,
-581,
5608,
1057,
-2762,
2515,
1277,
-2479,
-4528,
5001,
4788,
-7403,
6780,
-7172,
39,
-1342,
4938,
-1344,
-8192,
-2589,
-6050,
4754,
-1613,
2529,
6209,
2481,
1553,
1057,
-1207,
2471,
-2670,
2776,
6462,
-5156,
-3052,
-3167,
-3930,
-1389,
3688,
1579,
-10975,
75,
4278,
1107,
1839,
1861,
-1577,
5381,
2052,
-2256,
-6059,
-3024,
-3547,
1115,
41,
-7242,
8117,
-2132,
-5767,
7037,
-6457,
5043,
-7113,
2024,
5924,
-2866,
-861,
-5905,
5927,
-124,
-5498,
-6180,
2313,
1871,
-4397,
3653,
-7077,
5529,
-2664,
-4556,
-2648,
275,
-829,
-3631,
3908,
820,
3447,
-1227,
1902,
-8625,
-4103,
-1737,
7807,
8010,
-198,
-10297,
1419,
2148,
5982,
-2207,
1895,
8303,
-891,
6106,
-4428,
4070,
-5343,
-2200,
5618,
-1342,
414,
2165,
-1699,
-5134,
8835,
2400,
-3256,
-1827,
6000,
-5146,
-916,
-3841,
5471,
-541,
-172,
1106,
2730,
4848,
2872,
9420,
-723,
4792,
-3771,
67,
-64,
2215,
-879,
-3049,
1184,
789,
1168,
-232,
-1542,
-5454,
181,
716,
-5889,
4060,
-5393,
2299,
211,
-3593,
-5568,
4898,
3108,
-1367,
1743,
5136,
-432,
3437,
1327,
6696,
-3849,
-2826,
160,
4776,
1573,
-843,
-9648,
3574,
-298,
6019,
509,
5523,
3614,
-4046,
2220,
-4190,
1368,
-1100,
-3681,
-1134,
2139,
5296,
-2855,
-2254,
-4284,
1334,
5826,
2355,
-1699,
-1594,
-3534,
-6907,
152,
-2166,
-4778,
-4433,
2625,
-1813,
2884,
-1661,
-9373,
4084,
4211,
-161,
1984,
-4796,
-6614,
4293,
-119,
6599,
3711,
-5295,
4055,
4685,
9929,
-2748,
-1036,
1621,
-1464,
6377,
-4755,
-2065,
-6202,
-4803,
1075,
2228,
-2742,
8880,
8388,
-6953,
-271,
-2360,
-2940,
-1230,
-5199,
5826,
-10209,
-10995,
-1549,
1253,
-1447,
5440,
17,
1573,
-9337,
-721,
-2228,
-2261,
5448,
1552,
9,
-8517,
-1567,
2454,
-366,
2202,
-1982,
2572,
-2332,
-7418,
6463,
1507,
410,
-6906,
1153,
-897,
2990,
282,
-6423,
-2416,
-1553,
-569,
2629,
-11049,
-1446,
2762,
3650,
7781,
1479,
3269,
3684,
3250,
-9597,
1030,
2255,
-1507,
-4822,
-12630,
-2327,
-3277,
6759,
4032,
-4778,
3486,
3920,
3873,
-3323,
9796,
1901,
3829,
7173,
298,
3193,
1881,
3632,
-3343,
914,
5647,
-3837,
-5056,
-5783,
-7076,
10185,
-4242,
-3623,
-1107,
2792,
-3589,
-213,
7493,
-3975,
3847,
833,
1044,
1254,
2908,
2537,
4827,
2,
245,
3484,
8796,
1363,
-5525,
-2719,
4945,
-2736,
-5696,
-4670,
-4712,
-5157,
3783,
-5431,
-5436,
-6587,
1548,
807,
11329,
-2461,
-8012,
-1336,
-4037,
-4343,
-7422,
-2280,
-69,
12884,
-8856,
-1325,
-3484,
-4970,
5588,
5089,
-479,
857,
1846,
-2812,
1934,
39,
2792,
209,
-1729,
1723,
-2609,
-3906,
-2334,
-2368,
-3827,
6623,
-3348,
4547,
3269,
-3461,
1723,
4665,
2567,
-1927,
-599,
1127,
-229,
-8206,
984,
2167,
-4172,
-1550,
3790,
6154,
-1402,
3352,
454,
3403,
-949,
-5379,
1551,
-3459,
5902,
-1595,
-2308,
1800,
-885,
5873,
-1125,
-344,
-300,
3387,
6273,
-101,
-5678,
-1201,
-6218,
-1295,
4968,
6471,
-1752,
4486,
-4692,
5987,
1192,
1578,
-3528,
4408,
-3777,
2207,
6357,
-1832,
6855,
-1290,
5301,
736,
-6112,
632,
-1959,
-5895,
2960,
-859,
-281,
753,
-1579,
-9991,
7986,
1522,
4170,
3840,
2554,
-5899,
11471,
-515,
-28,
2458,
-924,
-10012,
-4539,
11420,
-494,
3008,
-3428,
2213,
-8282,
4240,
4836,
3856,
-8426,
6314,
5049,
-3712,
4123,
1553,
-3279,
-6635,
7322,
-471,
1304,
2580,
-1092,
1664,
2789,
-4728,
4442,
1445,
1238,
4023,
-2845,
4845,
869,
-71,
-1152,
-10250,
3276,
-3399,
-146,
10231,
-1511,
2853,
-914,
5015,
-1839,
1597,
-2707,
-4336,
-6994,
4613,
2820,
2169,
-760,
-5477,
-5430,
-5014,
1428,
-583,
-3125,
-3511,
3627,
-1087,
-7997,
3467,
-2912,
-1906,
3106,
6015,
-1706,
-3390,
-2750,
-3429,
4418,
1434,
6301,
1123,
1174,
6722,
3791,
-2861,
-8082,
-2557,
-1294,
4885,
653,
-1454,
4525,
7124,
8920,
5678,
-489,
-2526,
1556,
4299,
3991,
-6750,
301,
-3596,
-1731,
-3431,
-2354,
-2258,
-6779,
-126,
-12196,
1668,
2806,
-7707,
-3385,
1586,
-2636,
-4909,
304,
858,
2517,
-5226,
3286,
2840,
8985,
1046,
1612,
4014,
-1811,
3269,
2088,
-1529,
-598,
2301,
-2174,
-5746,
268,
-3741,
5292,
-384,
-2554,
-9223,
-6993,
1248,
-1108,
8399,
2445,
7173,
-1249,
-656,
-2158,
-774,
5521,
5718,
3094,
744,
5527,
1003,
7745,
-6646,
-4512,
-934,
-1489,
670,
-5960,
2446,
-6042,
-733,
-3953,
-3239,
4324,
217,
-2120,
-1362,
-4086,
380,
-12423,
5632,
2222,
1235,
1243,
-1883,
5940,
-1633,
1570,
400,
6490,
1205,
6142,
2414,
-2255,
6017,
-2770,
2771,
-3693,
1431,
-2948,
10511,
-8730,
-6773,
-4408,
-5256,
1456,
-2199,
-1835,
-1571,
781,
-8118,
155,
4621,
47,
3699,
6716,
7192,
-598,
-1054,
-554,
-6531,
2809,
-2038,
-5935,
-3750,
-1995,
-4415,
146,
1,
-3756,
-5024,
7833,
2955,
1105,
761,
-176,
6199,
-3951,
4531,
3193,
-4887,
-4222,
-1587,
2425,
-1643,
-5689,
1494,
-2441,
11908,
9979,
2946,
4387,
-4748,
-2718,
2272,
4499,
4914,
-2236,
6524,
2199,
-12114,
3581,
-3562,
-4586,
-1801,
395,
549,
-687,
-3212,
-5719,
-11277,
-1024,
-6639,
887,
1914,
10,
-4031,
-3651,
-2451,
-5164,
7762,
4833,
259,
-634,
-2522,
50,
-4694,
-320,
-12369,
-747,
-2773,
-5696,
1559,
1371,
4705,
-9685,
-5871,
-5927,
144,
-4419,
694,
2866,
5292,
-1055,
6562,
-285,
2192,
-7242,
-12157,
-609,
389,
2280,
158,
5432,
-3192,
4247,
5303,
-8817,
-8803,
-2977,
107,
1812,
5786,
7311,
-1323,
8223,
-2545,
1517,
-1050,
7576,
-1751,
2529,
-1677,
3513,
7508,
5062,
2938,
3848,
10071,
-2686,
7626,
181,
1539,
-3843,
-797,
-5645,
-5152,
3635,
-6081,
-5389,
-2736,
5769,
5082,
-5445,
2526,
1673,
3965,
1808,
-2066,
2254,
-1875,
1290,
4098,
-11450,
-1714,
-7483,
5622,
7440,
5738,
-3969,
1344,
-1282,
180,
10755,
-1164,
7645,
-3935,
-1218,
1769,
-3404,
-5193,
3780,
5015,
-8363,
8004,
-3282,
947,
7075,
-249,
-3229,
1390,
-4282,
-1205,
-92,
7159,
-8704,
-1832,
-3403,
-1883,
2631,
377,
7398,
-2789,
1982,
2719,
944,
8228,
-10932,
-3696,
-8067,
3955,
-3559,
2665,
5187,
6928,
6092,
-1099,
4050,
-3609,
-2008,
839,
-5751,
-4870,
-778,
-8100,
-4746,
5708,
-1716,
-9178,
4559,
-523,
2291,
411,
1806,
-1573,
-593,
1868,
-789,
5618,
1840,
-29,
3004,
-709,
-4713,
4080,
59,
13779,
2338,
700,
-5956,
-2281,
-1540,
-6901,
-6111,
3114,
9653,
5560,
547,
-5377,
-815,
375,
2329,
-7448,
-1040,
-4990,
-10595,
-58,
-5572,
-1372,
-3762,
-8280,
-4184,
4849,
-2295,
-2701,
-3511,
-1057,
2190,
1380,
-292,
-8688,
-9674,
244,
392,
-11078,
-6169,
4941,
3326,
-53,
264,
-1819,
-801,
-283,
1984,
1827,
3311,
3852,
4024,
3288,
6348,
8042,
-2627,
3455,
-3029,
178,
-398,
-1870,
-396,
1697,
-5507,
9417,
-3391,
-1126,
-2898,
-1021,
-2061,
-3982,
-8105,
-2646,
3352,
10763,
-1206,
-4899,
-8025,
13656,
6562,
-6942,
-1642,
-1786,
6907,
3822,
-2633,
3889,
-2046,
-737,
1388,
-3102,
-2775,
-4941,
1152,
-6905,
-11141,
619,
777,
835,
791,
-5792,
-5164,
-271,
664,
-445,
288,
-3053,
-3443,
620,
-2145,
-6028,
-4698,
9283,
-4601,
-3531,
2139,
1812,
7438,
1000,
-1305,
-510,
7232,
-7466,
2634,
2871,
6561,
3952,
1264,
-1259,
-2315,
2071,
1830,
11959,
-3050,
1641,
2709,
5319,
7312,
-1888,
3141,
-4925,
-1929,
-6955,
2868,
1200,
-3771,
3439,
4014,
1182,
-4524,
5428,
-3320,
1748,
-8325,
2808,
-340,
3151,
375,
1424,
2141,
6692,
6179,
6233,
4329,
-5169,
220,
1589,
757,
-1583,
924,
7634,
-3322,
-6692,
3501,
-9114,
1902,
2384,
621,
2226,
-384,
-6062,
-4196,
-5187,
5759,
-1890,
787,
-3664,
-4373,
2019,
4454,
-1115,
6017,
283,
-7130,
3489,
303,
-2617,
3028,
-5590,
-1099,
1929,
-10309,
-4649,
8050,
1637,
-5133,
1085,
-4492,
-1446,
6470,
-3424,
2092,
-6168,
-3359,
810,
7773,
3584,
-2293,
5131,
-5267,
5567,
-3088,
1241,
5044,
-2528,
-1976,
-6002,
-2065,
-6572,
1348,
144,
-2755,
-4172,
58,
2920,
-2977,
-6497,
-4592,
-1465,
-751,
365,
5975,
45,
242,
7088,
-3119,
-6087,
7984,
-5169,
2076,
-1013,
-6937,
5200,
980,
3572,
5420,
4511,
4016,
-452,
-5438,
-322,
-8185,
1078,
3548,
-133,
-3228,
3182,
3042,
-438,
2216,
-4128,
-4791,
4549,
5084,
1220,
-5849,
-1479,
2192,
-1075,
-1781,
3525,
5533,
-3111,
28,
2607,
1878,
1689,
-3632,
-4422,
-708,
-2366,
-162,
2254,
-9734,
-1536,
-4307,
5511,
-2769,
-2682,
4192,
-3139,
5115,
-1008,
-215,
4724,
-1330,
-13123,
3437,
7491,
-4158,
-7713,
-1475,
-4664,
-2609,
-9382,
-1538,
4918,
-5101,
-3548,
2044,
-1838,
433,
9345,
-2475,
3021,
-7237,
11054,
3760,
-3453,
-4028,
-6026,
8394,
3654,
-127,
1169,
789,
-2088,
-3013,
-2413,
1541,
-3553,
1709,
3944,
-1420,
1652,
-3202,
10353,
-4396,
3794,
-4509,
8350,
8283,
682,
6694,
4440,
713,
-12615,
739,
185,
4326,
968,
-528,
1964,
-564,
9079,
9491,
-6421,
-3819,
-445,
-8671,
-1564,
6155,
2239,
234,
-7131,
2698,
-2044,
-488,
-4184,
-1160,
-3038,
-2421,
3817,
-3890,
-6417,
1991,
-3303,
-3598,
1992,
1155,
-372,
3602,
-1313,
3011,
-13175,
-4424,
2813,
-6429,
1152,
-1950,
7716,
3404,
-7820,
1359,
3508,
1653,
-2811,
4242,
-7927,
-3081,
-1516,
-432,
3276,
-837,
672,
672,
8222,
4893,
7661,
-7698,
1395,
5175,
110,
-4140,
233,
-2720,
-1861,
4515,
1786,
3873,
-779,
1303,
-6554,
1744,
-240,
-104,
-3679,
4486,
-5158,
-1810,
-1442,
-242,
1440,
3418,
-5596,
272,
1664,
-475,
-403,
13509,
156,
2330,
192,
-4079,
3031,
5830,
-4567,
-3730,
1258,
-3319,
-1595,
-1306,
5655,
4868,
-1034,
-1521,
-792,
-431,
6041,
8988,
7688,
3418,
-4074,
-890,
-4572,
-6466,
3186,
762,
-1896,
-3217,
-15849,
1226,
-257,
1329,
1073,
3428,
-4604,
334,
323,
9770,
1922,
-7078,
-2335,
-1103,
-4329,
-478,
-848,
-10272,
-4256,
-9587,
3276,
5152,
-120,
-1258,
3457,
6271,
1920,
2890,
-147,
2905,
-451,
-598,
-1250,
-535,
-2830,
7629,
-3802,
221,
6378,
-3854,
11347,
-1634,
-321,
-6531,
4712,
-4068,
3741,
13004,
-28,
-419,
2298,
2776,
1844,
5724,
-2095,
-4072,
-1685,
537,
2286,
3740,
-5635,
-1596,
4453,
10806,
-4264,
-1236,
-1904,
-3764,
-4304,
3179,
3092,
-18,
1252,
393,
5961,
554,
3040,
6268,
-4053,
7174,
-3796,
-1906,
600,
-3508,
-2076,
32,
-523,
1204,
-5160,
2717,
5557,
4081,
-2966,
1872,
-4731,
2498,
1658,
6338,
1121,
1747,
911,
-2199,
-11127,
2678,
-3485,
3997,
-4597,
-2116,
-2978,
-3688,
3874,
-122,
1620,
4511,
-7235,
2222,
-2890,
-398,
-2241,
-893,
-1904,
-899,
3683,
-1899,
-3237,
-5273,
-3178,
986,
-4999,
5344,
613,
1824,
-712,
3869,
-3322,
-2867,
-3122,
2308,
-408,
3704,
-813,
-2510,
-5177,
1169,
9404,
-4236,
2826,
-2403,
-190,
1125,
-10378,
1036,
4121,
-5770,
6084,
-1112,
-5805,
-3776,
-11035,
-1042,
2224,
-1815,
-108,
4457,
-307,
-2856,
-4844,
-4110,
-2995,
10777,
1690,
157,
-2186,
-6876,
-1485,
-8404,
-2493,
-2401,
-6177,
3939,
1448,
-7901,
-808,
-4082,
436,
17892,
1391,
-5677,
-1262,
7710,
2380,
15,
-702,
7320,
-8730,
485,
-1559,
-1476,
1040,
1344,
-4119,
-3676,
5295,
-5717,
8926,
-478,
2378,
-12227,
2132,
-1901,
988,
-3719,
10215,
615,
-2785,
-6546,
-173,
-4819,
-3098,
-9668,
-7679,
-3572,
-1906,
-1526,
10648,
4259,
-6533,
-5172,
8460,
-14665,
1015,
1216,
796,
345,
-2857,
1611,
-635,
2942,
2864,
-2284,
-2727,
-7306,
4617,
3963,
1545,
4121,
842,
-990,
-1244,
9171,
8967,
-6277,
-5560,
-9408,
1777,
3940,
-691,
-1580,
454,
-4024,
-1088,
622,
760,
-1838,
3285,
-7404,
3216,
-4314,
2355,
3151,
-1936,
-1762,
-2804,
-6363,
-791,
-318,
1598,
-5111,
794,
-8052,
7209,
743,
-6723,
570,
2040,
8414,
4454,
6515,
-4252,
-2427,
4102,
-4621,
9759,
-402,
3322,
-5547,
336,
-6889,
-6887,
-3392,
3919,
1564,
-7910,
2969,
3000,
384,
5849,
-6883,
5047,
2630,
1118,
3567,
1233,
-2156,
-6940,
3434,
-5912,
746,
944,
-100,
-736,
4011,
-6522,
2441,
-6331,
5071,
-4224,
-991,
-3269,
5310,
-797,
-10465,
-2286,
7681,
4201,
5330,
-44,
-4135,
-1366,
3466,
-2228,
3697,
-2642,
429,
3157,
-4917,
7540,
-8051,
-1436,
11607,
1304,
-5530,
363,
-646,
-4311,
4705,
-3859,
-2767,
1698,
-4131,
1662,
-3578,
-1817,
-2830,
-3852,
6030,
-3093,
5458,
1166,
2713,
3993,
-3616,
1095,
-2928,
-1283,
3058,
8355,
1583,
-3607,
-789,
2005,
-512,
283,
3509,
-2062,
463,
-7173,
13752,
-1048,
1363,
-3842,
-7027,
-848,
8145,
-1556,
-4214,
3352,
-2244,
-3612,
4529,
10282,
3227,
-1073,
-2037,
-4123,
1273,
-2470,
2117,
-802,
1818,
-2852,
3101,
3558,
-4344,
4926,
1634,
7723,
370,
-1645,
1114,
4141,
4278,
7911,
-5761,
746,
4045,
-5092,
3509,
-3534,
-3431,
330,
-7981,
8632,
-3736,
-54,
-1335,
979,
-12406,
2273,
-6633,
3771,
6572,
5697,
-3088,
-3994,
-3946,
-1624,
-4321,
-6234,
-3999,
-740,
4086,
-84,
828,
-3813,
3190,
7870,
7194,
-2296,
1918,
-1251,
-7467,
1731,
-5325,
2284,
1999,
-1814,
-1365,
6367,
-449,
9501,
6110,
482,
4999,
2449,
-8647,
4289,
1280,
-2466,
2670,
-5982,
1216,
3499,
9716,
5428,
1741,
-1232,
-2459,
2681,
-6056,
-4403,
-1614,
-2719,
-2790,
-6923,
-1179,
1335,
13241,
-1765,
-3251,
-3589,
283,
1668,
-1926,
8675,
-7768,
-1560,
-5482,
-2606,
2825,
-1363,
765,
272,
-5751,
880,
-224,
-11303,
-3245,
-2130,
-1222,
2933,
12125,
4266,
2892,
-671,
5053,
-176,
7625,
-4559,
6962,
3565,
-3687,
-1713,
-1292,
3027,
-6085,
10884,
-661,
3468,
1471,
-6391,
-3800,
2156,
1382,
-2730,
-2689,
2846,
9232,
-2969,
6466,
-3379,
1116,
2604,
1960,
-7523,
-61,
5504,
-1919,
-2912,
1092,
-5927,
-1394,
-1742,
3875,
2333,
-3671,
5504,
-7613,
1252,
6783,
4065,
5373,
-7721,
-5212,
1672,
2463,
3661,
-429,
-2793,
-1541,
6561,
8008,
2469,
-7252,
-1937,
-2082,
-1607,
4617,
757,
1986,
1710,
-4472,
-4028,
-777,
-964,
2394,
-83,
431,
-703,
5918,
9584,
-1666,
5136,
-4854,
-4164,
-5007,
4682,
15700,
-5090,
1235,
2131,
-1067,
1998,
972,
2176,
8349,
7776,
4541,
1771,
393,
-2035,
-4256,
-495,
-5975,
-3073,
-3037,
-125,
-4245,
-5222,
-4340,
253,
3715,
5211,
2759,
4545,
1248,
-4678,
3250,
-5630,
-797,
-1962,
-463,
-1320,
4147,
5287,
-124,
-6982,
9152,
2220,
-3706,
1542,
-784,
-1464,
-4822,
-2379,
7766,
3959,
6531,
-756,
2279,
-21,
-3724,
-3612,
829,
-4199,
4677,
-6008,
-4431,
-2455,
-4661,
3014,
3106,
-13456,
700,
-5326,
2100,
-2292,
-3033,
-226,
-146,
1334,
-3206,
-4501,
-634,
-3686,
6581,
-3012,
3356,
-1316,
-7753,
-6568,
690,
-1829,
5020,
3329,
-3821,
-5722,
990,
-1226,
-3739,
6022,
-843,
990,
-3449,
6267,
5239,
-4982,
-2922,
4476,
-7653,
8270,
8261,
1907,
-860,
-1218,
-4408,
250,
-4887,
-6454,
-6717,
1733,
2522,
-4684,
-2228,
-8980,
1849,
-3613,
2060,
-2314,
-1780,
10197,
-6267,
-12256,
8106,
2208,
-2262,
2300,
-4927,
-3905,
-6375,
-8516,
3351,
2432,
509,
5631,
6010,
3226,
2395,
-6507,
4196,
1792,
-10173,
3656,
3406,
9073,
4404,
-10656,
1967,
2713,
3064,
4799,
-450,
2853,
-481,
2057,
-3086,
1098,
-359,
-5599,
2980,
-526,
-4954,
4562,
721,
4299,
5437,
2712,
-3420,
-6324,
-633,
2211,
-3738,
6203,
-1379,
-3416,
3487,
-1892,
770,
1385,
6515,
-377,
3274,
-322,
4294,
4807,
-1988,
-846,
4800,
-7192,
4704,
-11851,
-2024,
1673,
-2589,
-2417,
-6917,
-3201,
-4178,
5140,
4775,
-6532,
5278,
-866,
4867,
2009,
-3526,
5153,
274,
-2890,
-1094,
5454,
1508,
-3155,
3154,
-6511,
-5228,
4616,
-362,
-275,
-1979,
7984,
-994,
8868,
-107,
-5816,
-167,
1932,
-4013,
5,
0,
1555,
-5590,
2519,
-8727,
6164,
5921,
4651,
-739,
7453,
-2348,
-2381,
454,
-8834,
-3299,
-5410,
-2297,
126,
-339,
6848,
1888,
-2023,
2086,
7764,
-7665,
2305,
207,
2357,
-9211,
6945,
-4207,
1665,
-14374,
5271,
2147,
-4746,
-8820,
4666,
-16436,
-4510,
1356,
-11094,
17742,
-4655,
4164,
1010,
2224,
-3724,
-3859,
5134,
-9227,
-6263,
-7851,
12719,
-4177,
6773,
879,
-2095,
7545,
4310,
3654,
-2013,
3127,
143,
-5515,
-4070,
1137,
-1293,
5525,
-1960,
-5655,
2253,
5212,
-1751,
5430,
-15646,
-1280,
-10087,
8502,
8001,
-10592,
-2161,
1962,
-1723,
2078,
-8364,
3760,
3077,
-6369,
2281,
1624,
-5220,
-5616,
5466,
-4590,
-10961,
-13822,
-520,
15176,
-47,
-4208,
-8919,
10337,
-5948,
-12998,
-588,
-5673,
-10610,
-989,
-5562,
-2599,
3291,
-5932,
2326,
2503,
2865,
-9413,
5461,
-2487,
11538,
-749,
-1945,
5824,
-6178,
-9248,
-6204,
-4688,
1656,
841,
11282,
-10906,
4438,
-778,
-10974,
1378,
-2555,
1943,
-4514,
5591,
2632,
-1546,
6273,
-10598,
-1914,
1287,
10657,
6755,
-6870,
-1785,
11637,
4115,
1975,
6325,
-8206,
3018,
8090,
4380,
4580,
7837,
-7499,
-6204,
-7335,
-13268,
-2968,
-2776,
-6053,
-16009,
3911,
1680,
2766,
4387,
1293,
-16555,
-53,
-1264,
14823,
-6488,
1262,
11640,
-11011,
-2994,
3584,
5275,
-2925,
-536,
-897,
-9159,
1443,
13071,
-3734,
-8080,
5365,
4068,
-3474,
-88,
-8358,
1420,
1905,
7796,
-303,
-7798,
-3992,
2644,
2966,
-2989,
1283,
-13182,
7683,
-5605,
3236,
-4518,
8732,
6586,
11914,
-1089,
9677,
-8881,
-5875,
7964,
3074,
6139,
7821,
4507,
-2537,
-10994,
-12506,
8807,
-656,
-8608,
-9783,
4331,
5661,
-6601,
-1921,
-2066,
1608,
-7839,
8069,
6657,
9390,
-4147,
625,
-254,
-4713,
-450,
-88,
-1662,
5880,
-2232,
2041,
-2885,
-15526,
163,
19959,
-1203,
1819,
2240,
7175,
15663,
4356,
-1356,
-1343,
-2351,
-10317,
-1685,
4354,
616,
3241,
6562,
-1965,
1283,
530,
-314,
-11025,
-9028,
-6020,
-7336,
-4442,
8010,
4514,
-3012,
-12648,
-175,
4447,
-845,
-7920,
9887,
-6209,
-3406,
6198,
5392,
-1214,
5392,
9520,
2046,
7512,
-3807,
-362,
-8347,
-12126,
876,
-3815,
-1320,
576,
-14046,
6873,
-5439,
764,
853,
-7756,
5807,
7500,
8148,
-11813,
1927,
1604,
-8281,
5389,
5235,
6027,
5924,
-596,
2511,
-2844,
17733,
2037,
-9097,
-2813,
-3602,
5615,
7539,
-16619,
5688,
-6639,
395,
7393,
3620,
13046,
1134,
-1919,
-7493,
-3557,
18264,
6430,
427,
12393,
3102,
938,
-8599,
-3861,
-3190,
-1320,
13882,
-9305,
-2035,
-4190,
-5986,
-2086,
4086,
7015,
1881,
-3518,
4228,
1722,
3297,
-7241,
5316,
5965,
-219,
5223,
-10432,
2505,
2496,
7756,
10640,
-1942,
2531,
-2348,
-5108,
-733,
1545,
841,
-2464,
-4196,
10021,
2684,
7032,
12048,
531,
-22282,
6456,
-5544,
-2279,
6945,
-8182,
955,
-7516,
1358,
-7816,
2263,
1642,
3562,
-2754,
-945,
3440,
-1879,
5754,
-2250,
6120,
307,
-10194,
-6818,
8376,
3748,
-7902,
-2073,
1124,
4980,
-4942,
5218,
-106,
6225,
-5741,
7699,
3609,
-6406,
-8976,
6470,
12125,
-1513,
-1174,
-583,
-809,
5015,
2648,
-7221,
-14613,
-3966,
877,
-753,
3619,
-4085,
-11195,
5103,
-1562,
4772,
9622,
-5288,
10599,
-3857,
3430,
5847,
1710,
6440,
2119,
-11812,
-6009,
1284,
744,
-4238,
6208,
8228,
3435,
303,
2414,
-324,
5167,
3168,
3698,
-6944,
1906,
9392,
-829,
5727,
2295,
-2372,
-4348,
-3693,
33,
6746,
302,
3619,
9765,
621,
-7622,
3527,
1584,
12343,
4712,
-6216,
-4555,
-705,
10520,
-8192,
-7631,
-10374,
4045,
7220,
-11663,
-767,
3811,
-5187,
-213,
4978,
3589,
9393,
18662,
8506,
-6289,
977,
4790,
-330,
-2762,
211,
-4991,
1423,
3968,
-10657,
-1848,
-883,
4507,
-2220,
-4067,
7547,
3735,
6838,
-2911,
-2084,
10306,
-3365,
5594,
7849,
9306,
-4047,
538,
9298,
-14725,
5261,
2987,
-2154,
-4151,
-839,
-2872,
-4225,
-2777,
11405,
-11526,
9749,
-2208,
-848,
-1934,
8394,
-5600,
1206,
-7018,
1677,
2527,
-6896,
2331,
4006,
-722,
6065,
20386,
-608,
8517,
9372,
6212,
-9805,
-2188,
-625,
-4163,
1608,
-3819,
-3025,
-11791,
-3293,
1386,
2964,
-11517,
561,
-1504,
-25,
9292,
-1774,
6784,
-10453,
-8246,
-4052,
12559,
-5175,
12277,
-4500,
3229,
-6041,
-3470,
-1709,
7200,
770,
-9074,
-3196,
1252,
10978,
7145,
1127,
-1935,
-834,
-6666,
-48,
5665,
11774,
-5536,
-4333,
1412,
-3505,
-6302,
-5004,
-1162,
4379,
-6306,
1319,
2635,
-6910,
3779,
7872,
5909,
11040,
11227,
3372,
1483,
-3403,
-4237,
9818,
-8439,
-1381,
-8298,
-2269,
238,
6819,
12999,
308,
-4821,
-1804,
958,
4080,
13012,
7240,
10181,
-2474,
-6370,
6370,
1578,
-1321,
10062,
-3517,
-3565,
9523,
6861,
-482,
6273,
5327,
5040,
5700,
6291,
-565,
-5277,
-7195,
-3903,
-2379,
7820,
728,
980,
7677,
7299,
-9740,
1274,
9018,
9673,
1181,
-5987,
-4814,
4380,
-2989,
11794,
-6930,
-2717,
2446,
3341,
12535,
-7524,
-1079,
-2018,
3208,
18179,
4916,
-3339,
-4272,
8028,
-678,
-307,
3411,
-5137,
7733,
7843,
11041,
-1521,
7215,
-2857,
-5822,
4859,
-2076,
6502,
3069,
-10774,
-6007,
-1599,
-1879,
6233,
4636,
18932,
-8331,
-636,
1665,
-7077,
1534,
-1205,
8388,
-4469,
3082,
-9427,
-1237,
-8161,
-5185,
14822,
-4081,
6878,
8075,
15161,
3164,
10232,
10307,
-2469,
7705,
-1022,
6529,
-4800,
9490,
-2617,
-3541,
-3633,
-3409,
-7870,
-5462,
-14900,
2414,
8676,
-1518,
12677,
7765,
-2390,
-5603,
12483,
1727,
-13933,
4645,
-505,
-5688,
-189,
-5549,
14670,
-179,
7332,
-1222,
-4339,
10659,
11853,
-4909,
6808,
3118,
-7418,
8044,
5178,
5514,
-1367,
-2161,
-7738,
-4528,
-6048,
-11310,
1729,
3934,
-12090,
-6725,
7554,
-8308,
-5846,
2314,
-5025,
3696,
4364,
4342,
758,
-417,
-3089,
12288,
-1433,
6133,
2166,
573,
5717,
5301,
-5491,
-3009,
-4944,
742,
5822,
-1594,
6423,
-823,
-2575,
-821,
-17669,
-8951,
-677,
8583,
-3928,
3136,
-3000,
-2191,
14250,
-4674,
6022,
10563,
1916,
8015,
1277,
-4794,
4805,
-1813,
-5864,
-14634,
-11522,
866,
-2900,
4092,
-11165,
-2379,
11120,
-11351,
-12209,
8952,
3783,
-2352,
-12383,
-896,
8916,
-8657,
13281,
3204,
6236,
-4361,
450,
-631,
-3556,
6288,
-3095,
-548,
-9894,
5566,
-2180,
-6482,
-939,
1162,
-12961,
-1308,
-8867,
6142,
730,
-11704,
-993,
3529,
-11021,
-4568,
9734,
2267,
4220,
5492,
4592,
914,
3847,
328,
-1829,
-3355,
-208,
932,
5482,
4513,
5842,
-7701,
-7097,
-3633,
-1844,
4146,
4223,
-1804,
13051,
9431,
-10892,
-6116,
11837,
3413,
7801,
-1844,
6046,
-5247,
-7673,
-15394,
1115,
-6145,
5669,
814,
2651,
-103,
503,
1941,
-8119,
9613,
-10795,
-7619,
7827,
6764,
-2994,
-2125,
10502,
-7046,
-18122,
-5301,
-2685,
8365,
10451,
-7774,
-1788,
11104,
2729,
-2789,
-1775,
6404,
5391,
-392,
10444,
-1126,
6238,
1341,
-7446,
-3585,
-5126,
-9900,
7850,
3008,
186,
-4254,
9469,
-2,
-13171,
1102,
7511,
1523,
-1289,
-12258,
-8590,
-3575,
-3247,
198,
-4350,
8787,
-2175,
6195,
-6793,
-6586,
3280,
1014,
2902,
-3586,
-4873,
-8078,
-415,
19001,
-4437,
4066,
1055,
4648,
5128,
9679,
8981,
-4191,
5387,
-2805,
18140,
-5343,
-5807,
1758,
-1045,
-6191,
3927,
4576,
-1485,
4982,
-6503,
-7463,
-9433,
-3671,
7337,
-835,
23,
-7198,
-2029,
3298,
-5923,
2164,
8741,
4802,
-1403,
5450,
749,
-10457,
-1261,
-6883,
-3572,
1951,
-7638,
5958,
-795,
-2468,
10310,
-5522,
-3678,
12592,
2389,
-11064,
-5159,
1952,
10280,
2436,
4302,
-6360,
-13291,
-609,
-3042,
-732,
1510,
3729,
8637,
2596,
6297,
-4915,
-1179,
13306,
2132,
-7805,
182,
3546,
-3634,
7717,
3010,
-1646,
-10165,
-9023,
-5677,
1786,
4356,
-1122,
-2234,
5156,
-7991,
-166,
3543,
1289,
-7258,
-445,
1111,
-1685,
-3284,
-3403,
1561,
3641,
-12497,
-3728,
1899,
5962,
-10921,
-2920,
-7661,
6994,
-996,
-1188,
-5308,
-3312,
2889,
-5251,
-12162,
-181,
9449,
-10421,
-41,
16829,
-3613,
4828,
3622,
-2614,
1166,
-12123,
-2888,
-3961,
16419,
-8747,
-3808,
6124,
4023,
-569,
-4874,
31,
-20,
-11925,
7031,
-11463,
2579,
757,
6778,
1468,
-5083,
3546,
7255,
13426,
1535,
2199,
-7212,
-722,
15221,
-1149,
-8297,
-1318,
-3770,
14294,
4422,
2446,
-12648,
-315,
7391,
-413,
3528,
-7382,
3488,
-3383,
-5027,
-2713,
6594,
-5731,
1053,
-4732,
-7819,
6389,
2411,
-4473,
-3027,
-754,
2748,
6507,
6434,
3832,
1337,
7651,
-3896,
5623,
4377,
-7144,
-15579,
-8529,
-12478,
4598,
-10615,
-999,
-1488,
-8827,
7427,
9911,
5554,
-12380,
5519,
14477,
799,
1097,
8736,
-3441,
-8125,
-5349,
3372,
-3082,
5885,
-1291,
6276,
4762,
3395,
-7169,
-519,
9550,
4045,
7865,
7570,
5634,
700,
1893,
-909,
-2158,
-6346,
2189,
2777,
-5835,
-2520,
4189,
-6788,
-13622,
-3648,
-7748,
-4332,
-3064,
2672,
-5539,
2229,
-2182,
-5354,
4168,
743,
4780,
-3465,
-5361,
-8660,
2665,
-1273,
-13266,
-6100,
-7080,
6176,
5550,
-6872,
10220,
6736,
-6091,
-5319,
-8365,
-4867,
-3892,
8135,
-7478,
-4220,
-3446,
1048,
982,
3504,
5635,
-5020,
-1699,
-5782,
6565,
1861,
2791,
9869,
-2016,
6201,
-1600,
-1996,
-6926,
437,
3730,
-2808,
-22,
-8747,
15760,
-4316,
682,
965,
2610,
-4066,
7272,
-978,
-516,
931,
-5511,
2893,
-1696,
-1545,
-8004,
-7418,
6017,
3257,
-11334,
-7165,
6687,
3676,
-3544,
-1880,
-9215,
7096,
-5935,
9749,
7181,
-4525,
-9681,
12555,
7497,
890,
862,
6205,
-1908,
5195,
-7475,
-11186,
3967,
-2606,
-10727,
5163,
-2751,
-3546,
9171,
-1257,
-1882,
8354,
8860,
-4574,
6164,
321,
-4369,
1269,
5804,
2557,
1235,
1331,
-3922,
4909,
4138,
-3020,
-5866,
9395,
4818,
-2507,
-8979,
11452,
1510,
1663,
4213,
1855,
1511,
4415,
-4981,
5329,
4001,
-4918,
-2995,
-7546,
4338,
3139,
4938,
-7707,
9807,
1621,
-10325,
-5478,
6900,
960,
-2299,
1663,
-11503,
-5250,
-3338,
-1196,
-4864,
-4971,
1502,
6439,
-2218,
1380,
25,
-7969,
4184,
2964,
1065,
9081,
-1649,
-891,
-1214,
87,
2178,
5329,
2508,
3444,
-13497,
-2893,
1770,
267,
2345,
-10262,
11614,
-10903,
-2318,
10742,
15207,
3166,
6804,
-1842,
-15551,
4428,
3722,
-4827,
-4417,
-2039,
8728,
-5992,
11781,
1428,
266,
6614,
6652,
1390,
-457,
-3270,
-10604,
-2209,
13187,
-5273,
-5367,
6717,
-4968,
-9774,
2078,
-7170,
-6681,
-5524,
-2112,
3595,
4781,
-401,
1090,
2314,
-8159,
-11185,
-1726,
-2875,
-889,
-4613,
19370,
-1892,
-5245,
7946,
9325,
2958,
1937,
1757,
3228,
-2365,
-13158,
-4583,
4743,
8133,
-6402,
-7358,
-5849,
-1985,
10860,
2494,
-10374,
6980,
-9162,
3754,
2152,
1785,
4310,
5211,
-1481,
7259,
1029,
-6718,
3885,
-1726,
10268,
-3072,
7104,
-10308,
-7419,
3231,
8114,
5719,
-2794,
8076,
2640,
-4435,
-4197,
-1311,
-6974,
5765,
1895,
-4336,
-2056,
3674,
3030,
-505,
-4104,
-5154,
-3551,
9035,
-8238,
1096,
376,
-4636,
5945,
-811,
7294,
-2204,
-5070,
969,
-13402,
256,
-6099,
-7910,
12225,
10750,
530,
-7921,
-3321,
5291,
2128,
1957,
3053,
-1438,
-1795,
346,
-1801,
-1273,
2347,
-6248,
626,
1808,
3398,
-3717,
1285,
980,
9260,
-12543,
3708,
5075,
-1600,
651,
-2376,
1664,
-1775,
9892,
-8202,
5296,
-2499,
-5401,
-2903,
2810,
-10237,
-2563,
6057,
-4114,
892,
359,
2577,
-420,
2055,
-12287,
8332,
6886,
3329,
11931,
4690,
-4490,
7050,
-3148,
10439,
-2266,
13193,
-1544,
5851,
2588,
-821,
2432,
7576,
10455,
-4364,
-6544,
-246,
-5716,
-8642,
-10165,
11885,
7376,
-1119,
-9687,
575,
2596,
16387,
-8214,
-1450,
-10271,
144,
13897,
259,
-2670,
4135,
-11234,
1901,
1267,
2182,
1548,
9930,
888,
768,
-9268,
-1434,
5901,
-3162,
5957,
-8440,
-1562,
7770,
-3265,
7128,
-1029,
11245,
7637,
-1064,
13175,
-2320,
14713,
707,
-4674,
6567,
6421,
-6639,
-285,
-1675,
798,
256,
2076,
-6155,
-9767,
2827,
4375,
5484,
4193,
3465,
4631,
17575,
10930,
2925,
-3926,
5787,
-9695,
2055,
9669,
2680,
77,
7132,
-8438,
-529,
-2335,
-9044,
5381,
3600,
-5929,
-825,
-5964,
5393,
-9896,
-11451,
2388,
-2537,
8233,
-3237,
-10217,
8556,
5141,
6359,
9678,
329,
-6066,
386,
4779,
-8868,
-2745,
5640,
-1426,
-7505,
-7622,
4026,
1696,
3100,
-3815,
1079,
3820,
-1054,
3888,
-3517,
-11363,
692,
8346,
-2160,
15596,
-2159,
-124,
-96,
12268,
7385,
2663,
10474,
-6207,
-337,
-7486,
-110,
-765,
7776,
5074,
-4338,
-915,
6927,
-1242,
-4097,
-6347,
-6097,
1935,
8030,
2897,
-5827,
-1702,
-7801,
-92,
-12091,
9545,
-2851,
-6607,
1899,
-1293,
-11395,
7505,
10723,
-6147,
10618,
-1314,
-2072,
5201,
6208,
-2531,
-5568,
6461,
7760,
3042,
9534,
-4556,
5902,
-6305,
11160,
1554,
-7315,
-2228,
-7287,
258,
-5336,
-1212,
620,
8499,
-5185,
5899,
-4776,
-3593,
-1073,
4876,
-7704,
-5590,
5299,
-1303,
-4061,
3511,
-3115,
2145,
-3775,
1249,
-4321,
8866,
5492,
-9893,
3391,
3384,
1332,
-7748,
181,
-13465,
-4882,
3260,
-5111,
-1445,
-4662,
1348,
18625,
-5865,
-12559,
6264,
6465,
2527,
-10444,
-4035,
10488,
5716,
-9354,
-7649,
2540,
-9424,
-6796,
1964,
-1021,
6695,
-6515,
-2002,
-659,
-245,
-2439,
428,
-4245,
-3234,
-6896,
-9372,
-6275,
1641,
10966,
-6127,
-7963,
7310,
6975,
-8850,
5896,
-4843,
-9278,
-10263,
-5327,
2585,
496,
-6125,
5085,
-8201,
-5251,
-2849,
11449,
-3042,
-7279,
15535,
-4705,
2751,
3045,
-4754,
-836,
-2105,
3463,
-14545,
-5749,
-2280,
-858,
-18138,
-2307,
3744,
5290,
1323,
1830,
2091,
-8849,
-1265,
-2776,
-5184,
1922,
-6409,
60,
2809,
-5990,
-3593,
-8219,
-7,
2820,
-3414,
-1302,
2984,
-6861,
-819,
3045,
-5607,
3557,
-9019,
5793,
-6048,
-8924,
-12307,
-3989,
-2839,
12550,
-4974,
-1613,
-4435,
-2707,
2023,
-18389,
2036,
5419,
-1227,
-4333,
212,
6573,
-9946,
9282,
-8253,
-3046,
9832,
-45,
302,
-4104,
582,
129,
-6168,
3296,
-5428,
-3107,
-1247,
1467,
-470,
3557,
-7330,
-118,
-4832,
-1618,
-11926,
1254,
5401,
3389,
3952,
8742,
-12564,
-3305,
-668,
-4867,
-1940,
7320,
-723,
-13410,
6583,
5446,
-1237,
1180,
-1002,
2076,
-2049,
815,
4942,
2221,
-3302,
6518,
-3076,
6934,
-7962,
3259,
1174,
8630,
-5290,
2003,
-191,
-14038,
-3429,
2287,
-11936,
-1141,
2075,
1558,
-10251,
4652,
-123,
-6732,
-6025,
2889,
-563,
-8862,
-12518,
-4364,
11785,
-8627,
-5305,
-13218,
10382,
3993,
915,
-5371,
4870,
2606,
-2852,
1460,
-884,
-7877,
-267,
5598,
-7782,
14157,
4613,
-1,
-10017,
-7262,
10639,
-1830,
-1616,
-810,
3154,
-1831,
4346,
8323,
-6766,
1748,
1357,
-1446,
2674,
-149,
1615,
-828,
1101,
3488,
-3411,
3465,
-3000,
-6733,
-1557,
-1613,
-6031,
-4691,
-616,
227,
858,
-4569,
7452,
6742,
-7019,
11393,
858,
-853,
8640,
7390,
-2945,
7984,
-8034,
3181,
-5006,
1342,
6871,
-2075,
-8670,
3377,
4928,
-745,
-1327,
4299,
3850,
-8888,
-6093,
4157,
3108,
5142,
3386,
1728,
2609,
-3404,
-5341,
10928,
-16434,
-457,
-5151,
-7894,
1083,
3249,
7285,
-2635,
-14148,
2292,
-1833,
4668,
17265,
-3659,
-5224,
-6285,
3339,
426,
351,
10025,
-1302,
7161,
1829,
6594,
-30,
2846,
3266,
-11304,
9916,
1069,
-1734,
12314,
6618,
5046,
-6129,
-9290,
-8240,
-6172,
4384,
-3369,
-441,
13031,
-1042,
9379,
2102,
-7003,
-5484,
2537,
3313,
-3353,
-18423,
7552,
7707,
-566,
2831,
-1171,
6267,
9641,
-1776,
-2854,
1943,
-7205,
-3258,
1709,
-1149,
-5877,
-3265,
2064,
-2034,
10616,
3805,
2345,
4039,
-3166,
-4481,
376,
-1928,
2290,
-1116,
286,
-4212,
2869,
2908,
-11033,
2240,
1227,
3523,
-2735,
-7622,
9692,
1952,
6992,
-12677,
-403,
8868,
-10801,
2538,
4908,
-18999,
-5419,
-4424,
14047,
-9093,
-10144,
72,
115,
5656,
11020,
190,
-3203,
-3129,
5795,
5232,
9792,
-4969,
3132,
6045,
-9897,
-4937,
-2539,
-15734,
-5241,
9019,
1167,
-4063,
964,
-13,
3694,
17370,
-6432,
43,
-11999,
-5565,
-1939,
2687,
-78,
-8788,
-580,
-447,
8354,
-2542,
-1854,
413,
-7550,
2538,
5569,
6848,
-1710,
18,
115,
4042,
-3038,
-1663,
-3022,
3381,
3135,
-1260,
-9346,
709,
2396,
-1966,
419,
-645,
3618,
-12937,
2536,
2138,
6877,
-10897,
3331,
-2051,
8556,
517,
2301,
9419,
3045,
-646,
18049,
-1248,
-7153,
2477,
7209,
-9274,
5060,
-3305,
3685,
75,
-53,
3326,
-6110,
5627,
10111,
-1585,
3904,
11765,
3148,
-1122,
-6332,
1105,
9279,
4315,
3228,
-5392,
-4886,
-1848,
17395,
-5924,
-939,
5245,
6231,
-3541,
2427,
-14352,
-8823,
-2568,
-1558,
6903,
10776,
7368,
7717,
4229,
9668,
-8176,
7324,
4543,
-7873,
-2664,
13125,
-3735,
5166,
1776,
-5852,
2875,
-367,
-4879,
-380,
998,
-6964,
3252,
35,
-8890,
3046,
289,
-9013,
-1780,
5,
1809,
6308,
-529,
-5628,
-3099,
-3832,
5356,
398,
1725,
-1203,
-1010,
-8485,
10534,
7234,
3007,
8561,
7765,
16073,
-10560,
-3235,
-6157,
2534,
5724,
-2973,
5831,
-5088,
653,
-2765,
-1465,
8742,
-3464,
7201,
8126,
-4840,
-459,
558,
7150,
2140,
-6548,
-2112,
-1542,
2085,
-1757,
-383,
13517,
-830,
11707,
-12578,
5395,
2779,
5647,
3163,
-814,
-1247,
-3046,
12311,
2486,
-15488,
-8696,
14232,
331,
-925,
1363,
-5003,
-13680,
-2060,
9118,
-3957,
-994,
9421,
7908,
2913,
-6774,
16867,
-2841,
-9365,
-8734,
3028,
-2868,
-13783,
1895,
10992,
-1098,
4193,
7900,
-5040,
8046,
15196,
-6844,
-1151,
52,
9047,
5798,
-3043,
-8617,
-6050,
-1680,
10095,
4035,
-10022,
8766,
2467,
3951,
-8998,
-1038,
-3837,
-6246,
1747,
2180,
975,
-4421,
-16002,
1974,
1242,
3087,
-7507,
-17075,
-10385,
-5858,
520,
4080,
-4563,
12530,
-9217,
-5635,
-4642,
-8015,
-5364,
1503,
5406,
6458,
5967,
-8018,
-5168,
8180,
11671,
13757,
-2511,
-8898,
3728,
3211,
-1239,
-46,
7316,
-5985,
6010,
-1717,
7551,
8078,
-394,
1729,
9289,
-4395,
-5653,
-1154,
4326,
-5147,
1462,
-233,
1893,
33,
-6992,
-4864,
-6271,
-8355,
-5677,
-7882,
-652,
8718,
2942,
14674,
-2882,
-2929,
-673,
3203,
-3877,
-5248,
-1759,
8052,
-19792,
-4543,
1935,
3704,
-46,
-447,
6454,
2612,
4760,
-870,
4858,
13169,
-3111,
-2801,
-5279,
3151,
-7054,
-1331,
4211,
4023,
8386,
-5358,
567,
-7596,
5879,
-54,
137,
3447,
-7156,
-3471,
-7056,
2270,
5361,
-4933,
-142,
-3165,
-3738,
5241,
2166,
11533,
9938,
4398,
5469,
-387,
5602,
3867,
1504,
-2086,
2915,
-1083,
-3093,
-2468,
-884,
-14002,
-4107,
8326,
-8313,
15380,
9517,
-7325,
-2145,
-2421,
6630,
1511,
12378,
-281,
6037,
111,
7682,
-4110,
5243,
-5937,
-3474,
562,
-3463,
1042,
3712,
-1617,
10766,
-2539,
-334,
-4149,
9504,
2881,
7961,
-5947,
13243,
-8314,
-6885,
-3052,
6374,
-11202,
12848,
3628,
3361,
-4928,
-8383,
797,
-4348,
-3637,
-6622,
610,
10557,
13303,
11854,
-551,
6251,
-5897,
-193,
5797,
2931,
8065,
7871,
381,
-6996,
-4103,
5956,
-2160,
2987,
-2914,
8241,
-1203,
-9223,
1309,
-5453,
860,
-7426,
-1339,
-1807,
2223,
-1857,
-808,
-7136,
9601,
-292,
-2962,
-5447,
3621,
-6356,
-1229,
-5917,
-185,
-4472,
-485,
12710,
-4833,
-1221,
871,
1278,
-180,
-2943,
4604,
6478,
9080,
1581,
-5514,
-119,
-3962,
-5750,
-962,
-11746,
-4729,
-3327,
1555,
3951,
1033,
800,
-6622,
6439,
5883,
-4287,
-17434,
-3057,
519,
3706,
-4770,
-4960,
2436,
20795,
4302,
11777,
6431,
-10527,
-4008,
2775,
2693,
-4019,
-667,
6279,
-1781,
-375,
13364,
10671,
5250,
-2864,
-5502,
-2980,
6132,
-423,
2558,
3631,
-4514,
-1046,
7750,
-17559,
14090,
5394,
320,
1108,
-7672,
14105,
4945,
-893,
-8908,
-4799,
4551,
-2105,
-3204,
16792,
-3728,
458,
8841,
11154,
3190,
813,
6212,
-220,
7778,
-6323,
-5562,
2542,
3428,
1521,
-3194,
-3619,
361,
11930,
-422,
1931,
905,
11847,
-7390,
11738,
1651,
2830,
11861,
-6731,
-4730,
-4194,
5341,
3540,
7870,
2858,
-5741,
2018,
-9084,
-1200,
-3545,
2578,
-9577,
13644,
3279,
-592,
-6458,
-9783,
-2669,
6179,
3143,
-5669,
-15035,
-190,
5053,
889,
-3164,
-408,
-256,
-66,
8513,
-3836,
-2639,
-1710,
1171,
2616,
-4027,
-4671,
5000,
4390,
-1390,
2948,
5714,
21545,
-2181,
-428,
-544,
4368,
13891,
-7231,
1631,
1872,
3519,
-7340,
-1827,
-4612,
-7587,
6462,
399,
1401,
1378,
-8739,
1653,
3405,
8742,
2007,
-4881,
1873,
5601,
-2728,
1108,
737,
4856,
-1253,
390,
-3023,
-5455,
6401,
-7669,
878,
-3391,
-1687,
-2288,
-10284,
11345,
-11551,
-6434,
1703,
1423,
1553,
4008,
-1250,
2000,
-6168,
-6552,
-5145,
-4548,
1311,
7350,
7074,
-9678,
632,
6802,
11147,
-6162,
7420,
-546,
-930,
-5155,
11098,
5761,
6317,
-2457,
2329,
13921,
-14138,
-15680,
-11085,
-4186,
-8919,
278,
13816,
9008,
-9193,
9070,
-1041,
-4642,
-7498,
6066,
-5105,
-4298,
-14638,
-2074,
7822,
12810,
-5029,
6276,
3760,
7088,
-7116,
11117,
7894,
13093,
772,
-4527,
-6445,
-8994,
7872,
14495,
3164,
4488,
-12770,
5738,
3527,
2508,
8515,
-3658,
-6873,
-3522,
12998,
-5164,
-234,
-4062,
-2752,
1651,
1051,
-8398,
-7022,
-6285,
-8203,
-2964,
-11631,
-2777,
5509,
-5826,
-1467,
9998,
-5663,
-3761,
-4736,
-1758,
-7680,
-6949,
-6366,
-4720,
-7596,
-7103,
-683,
9737,
-659,
-3165,
-2245,
9088,
-3965,
9118,
6,
1571,
433,
2920,
-1713,
-7262,
4242,
1226,
6335,
-3829,
-4397,
-2688,
6914,
-9775,
-5158,
1483,
-680,
10017,
7741,
8928,
1311,
-3748,
-6057,
-2371,
-5631,
8965,
2498,
-733,
-31,
-6788,
-1853,
-1854,
5454,
8368,
-4973,
-4809,
4068,
16815,
564,
-2929,
5069,
-6804,
-1577,
-7403,
-11335,
-3194,
-3753,
14630,
3028,
-5,
-6381,
-10158,
-9399,
1956,
-7370,
-139,
-12861,
-6330,
2744,
8960,
1093,
3063,
-1259,
4675,
-2367,
1767,
-604,
16381,
9822,
-5209,
2818,
-907,
-617,
13792,
-982,
-2131,
-999,
-13499,
1800,
6822,
-2374,
5443,
96,
-10526,
-3368,
-4222,
6224,
4735,
873,
10797,
2139,
-8898,
8209,
9083,
7743,
-5094,
11041,
7248,
-12118,
3679,
5778,
-2490,
6720,
-3488,
5476,
6954,
7500,
-1103,
3830,
2704,
-7615,
-2617,
8693,
298,
6224,
3318,
6118,
8083,
6712,
3861,
-6745,
4212,
2544,
100,
-1350,
9346,
-4848,
-3079,
845,
11588,
-6408,
-5189,
-3095,
-6925,
4090,
-144,
-5642,
-4523,
5582,
10449,
1148,
-2318,
4549,
-394,
-3229,
7531,
-4980,
-7106,
-4218,
-10226,
11358,
2827,
152,
8599,
-3026,
-2304,
2252,
-7073,
-4057,
-6302,
13440,
7252,
-6362,
-4724,
-1724,
5070,
860,
2326,
3092,
1533,
-6890,
-4753,
-6367,
-15281,
872,
-1347,
-5662,
5567,
-4427,
-2119,
-4277,
4081,
-3925,
710,
-3,
2846,
-8805,
-5615,
-8529,
-1430,
6352,
1543,
23,
-480,
-3369,
793,
918,
15317,
-7218,
-8330,
-3811,
6663,
2716,
-3709,
-4421,
-2280,
-3148,
-5471,
-5888,
472,
1267,
4996,
2628,
5596,
-495,
7061,
3210,
7876,
-2384,
7691,
1191,
-538,
-3553,
7095,
10496,
-718,
1844,
-5719,
1548,
-6357,
4944,
-14168,
11593,
7907,
-10199,
-4521,
3382,
3284,
4157,
-1110,
318,
-4229,
421,
-11889,
3430,
-2370,
-9965,
9507,
-14179,
1796,
-13831,
-882,
-13612,
-7980,
-2583,
2015,
5489,
-795,
-4915,
452,
-11689,
-7602,
-4888,
9094,
2095,
-6092,
-457,
-1417,
1652,
-1969,
1752,
11895,
-8361,
-3273,
-1732,
-5472,
3653,
9942,
-9877,
-4606,
6440,
-1404,
-2752,
-7078,
-2845,
-1664,
4723,
-2305,
1044,
9819,
-4742,
-12330,
-1324,
8531,
-10505,
2100,
-13187,
-9269,
-2961,
-1623,
-2437,
1782,
4398,
3455,
-264,
-513,
3131,
2782,
10043,
3475,
-1646,
-14897,
5046,
-2120,
-1182,
2653,
4949,
-2382,
-1068,
-1645,
13485,
8034,
-8110,
2410,
-4044,
2218,
-672,
-4857,
-8568,
1512,
-2533,
7659,
-1653,
7058,
-8965,
-4814,
371,
-799,
7086,
-1461,
-4360,
-12337,
-3269,
1528,
174,
-4777,
-3410,
253,
3910,
4102,
791,
5802,
-4966,
1835,
1652,
-9537,
1665,
5917,
-253,
-2223,
-1530,
-4046,
2546,
6727,
-5330,
1389,
-3476,
-7771,
7988,
3751,
-2284,
13739,
-5,
-1332,
5552,
4769,
9915,
9357,
-3637,
10054,
-3216,
-4601,
-3005,
-2475,
-11032,
3533,
-1422,
2725,
-2339,
-1371,
2801,
-1643,
3833,
4340,
-11037,
1602,
5244,
5517,
705,
-495,
8929,
5602,
-16594,
-17830,
453,
801,
8703,
8224,
-3371,
737,
-1736,
-3131,
-4076,
7559,
628,
2363,
1895,
10693,
-9384,
16457,
-7270,
-3267,
-801,
-5845,
8278,
3337,
-7394,
3655,
2376,
3903,
-3495,
113,
-484,
-9144,
-171,
11254,
-5049,
3599,
2790,
15959,
-2219,
-7208,
-4192,
-8437,
-6798,
-14438,
3066,
5845,
7264,
-5712,
14717,
-3028,
-1911,
-10903,
-757,
137,
-7852,
1284,
-3077,
-6390,
1165,
6539,
3995,
6189,
2527,
4482,
2820,
3069,
9337,
-120,
-662,
-3415,
-6057,
14142,
17710,
5967,
7493,
-14629,
17080,
635,
2307,
5772,
6300,
5543,
-4473,
-3328,
-373,
4680,
-968,
-18885,
-235,
-9536,
4506,
2755,
3383,
5821,
7181,
5607,
5691,
7010,
-13784,
-1524,
-10692,
-1273,
132,
7199,
10642,
-2850,
-6552,
-4623,
-2176,
-8389,
-348,
-4082,
-2059,
-1300,
-4189,
-100,
1993,
1030,
-7049,
-14130,
-3841,
-14338,
3528,
-2024,
-11969,
1571,
6668,
-15009,
680,
-4577,
-4825,
-5183,
-1508,
1470,
2954,
-2552,
943,
-1311,
6067,
3160,
-2471,
-4074,
-4737,
-1385,
-3471,
14069,
-1884,
15,
-12173,
6679,
-566,
-5587,
126,
9130,
1070,
-1031,
-9322,
-4269,
-5180,
-9130,
-7277,
-1810,
-2465,
-1317,
300,
-9661,
-968,
4687,
-4136,
540,
3096,
11530,
8920,
10418,
334,
7560,
-3908,
15028,
-3282,
5433,
-4455,
-8974,
1181,
444,
368,
-5934,
9589,
5274,
3713,
-3611,
4500,
-7537,
-6711,
7653,
-9810,
-13155,
-2257,
3438,
-10966,
4069,
8634,
-592,
143,
-796,
-2141,
5399,
7975,
93,
3583,
-2073,
8899,
171,
-5513,
520,
-1233,
-2491,
-5958,
4975,
-10291,
-1292,
4562,
-4990,
-127,
-6657,
972,
-2306,
367,
-10588,
10622,
2517,
-3703,
-8311,
1007,
-1552,
-423,
-20726,
-2087,
-7449,
-3549,
-3429,
-8864,
1084,
-2234,
11351,
-3935,
-9315,
2427,
878,
-11543,
-6251,
12109,
-3683,
2267,
-525,
2912,
-2850,
-5226,
5993,
713,
-4760,
-11896,
-1266,
-6364,
5320,
7870,
-62,
8992,
-4466,
-433,
-6241,
4517,
-1025,
-3838,
6525,
351,
-591,
-6850,
3834,
-682,
-4624,
4938,
-1332,
-3899,
8281,
-6135,
1976,
-259,
4831,
-5034,
5071,
-9431,
-1997,
-4774,
-698,
3870,
-3926,
10441,
9133,
1264,
-3092,
7196,
-6107,
-2504,
1289,
-10408,
5618,
6433,
-5672,
-5444,
2334,
-8535,
-7201,
-18414,
7627,
1893,
-5595,
5051,
-1246,
1388,
10501,
-11479,
4687,
-388,
-3358,
3032,
-8680,
-1431,
-5538,
-6768,
1941,
-7720,
-5899,
-3228,
-7329,
12399,
-8022,
3340,
-3172,
-3791,
-2274,
-7130,
-1023,
12240,
16018,
9288,
-7446,
-11553,
-2272,
-11702,
-9947,
4203,
5220,
-5397,
-2668,
2927,
-7873,
5204,
6114,
-7733,
2565,
-2883,
-5655,
-2449,
-17157,
17768,
-3579,
-4395,
-4465,
5418,
-1474,
-6177,
7808,
-8462,
-4532,
342,
4540,
3868,
5989,
-781,
-589,
-1884,
-15068,
-10391,
-3715,
360,
-733,
-582,
-2528,
4916,
2211,
342,
1156,
9590,
-2395,
4217,
-4638,
11246,
-5215,
4322,
-3740,
11040,
2423,
8135,
9976,
5409,
667,
-1396,
-2733,
4493,
-4771,
4453,
-1684,
1821,
-4025,
-1610,
5825,
10139,
2275,
1994,
2422,
55,
2975,
12634,
3142,
7506,
-13535,
6045,
-9025,
1831,
-6618,
6679,
5588,
-10096,
-4366,
1452,
-13687,
1324,
2517,
1515,
14243,
-4716,
1293,
-15240,
8801,
4097,
-2154,
403,
-2691,
5223,
-5388,
2795,
1087,
3536,
45,
-8357,
-14477,
2258,
-10385,
919,
-7555,
-2288,
3167,
1409,
6350,
-2779,
-1736,
-7436,
-1789,
-9494,
1705,
-823,
-83,
9946,
5914,
4923,
2165,
-8494,
3367,
42,
976,
-2674,
-3094,
-2581,
-9112,
-4449,
-4286,
-3181,
-7571,
-6756,
-5997,
-4595,
-2766,
-4688,
10360,
-1954,
-2249,
-1139,
8512,
-3719,
13335,
4075,
-215,
-171,
-6817,
1110,
1942,
8088,
-2450,
-10451,
-6778,
4317,
-1183,
-5072,
796,
-2116,
-1244,
1308,
-4748,
6936,
-1785,
1484,
-6490,
-7441,
3245,
2840,
7873,
-8500,
4003,
-431,
-1404,
2493,
379,
-3953,
2994,
12545,
-3357,
391,
-426,
417,
-4359,
2497,
-1327,
-5512,
7807,
-3909,
840,
-7489,
3089,
-4919,
4279,
-13288,
7667,
-6204,
2830,
-3524,
-1499,
-346,
5246,
-10866,
-9397,
1447,
5699,
-8333,
3912,
-6305,
-7317,
9120,
2212,
-1131,
-3213,
-13458,
4765,
-854,
-2212,
-1344,
2845,
-3149,
-2245,
10154,
-934,
3534,
-11600,
-727,
160,
5072,
-6771,
2812,
4389,
-4542,
133,
-3895,
-351,
-3458,
-8905,
264,
39,
6367,
-868,
-11022,
5630,
-10178,
426,
-3905,
5507,
-4799,
489,
5675,
2993,
-1046,
-4808,
-6974,
1254,
1804,
-3828,
9753,
-3761,
-69,
3915,
9129,
13953,
-8613,
-3704,
14414,
2885,
-10599,
-479,
12251,
-940,
4827,
7715,
11747,
-1042,
3190,
-4781,
-4514,
-1626,
-2847,
-5886,
-9450,
-2873,
-7191,
5078,
-4113,
4082,
2214,
-5989,
4830,
3611,
-2463,
5222,
119,
-7018,
-2161,
-3476,
4735,
-1237,
1729,
-8411,
5052,
-5217,
7455,
4735,
1846,
12058,
6426,
2771,
18142,
-2098,
137,
-6954,
-1807,
1047,
-2019,
-987,
3639,
-9791,
-2995,
14780,
7935,
9456,
-3731,
1724,
-3785,
-54,
-152,
-1023,
-107,
-533,
-3076,
-10598,
-7683,
3641,
3302,
1431,
-13530,
3790,
1954,
-2864,
-6361,
-2136,
-1521,
1940,
-3919,
-6182,
90,
-3921,
-4607,
4975,
4548,
7837,
-2469,
1022,
6804,
12322,
2763,
-2640,
-7129,
-4519,
9703,
6175,
-6633,
-6081,
-6186,
2528,
2394,
-2613,
7953,
4305,
6913,
6325,
3165,
5521,
-5070,
-3459,
13840,
-3895,
-4524,
-3136,
13969,
-5340,
470,
6453,
-8401,
-3081,
-3018,
1303,
-3304,
-620,
3410,
4967,
10799,
-4554,
4101,
991,
-10230,
-6849,
2758};

#endif
